@@ -29,18 +29,18 @@
       use smallbox_gvec, only: ngb
       use gvecw, only: ngw
       use gvect, only: gstart
-      use ions_base, only: na, nat, nax, nsp, rcmax
+      use ions_base, only: na, nat, nax, nsp, rcmax, ityp
       use cell_base, only: omega, alat, tpiba2
       use local_pseudo, only: vps, rhops
       use io_global,                ONLY : stdout, ionode, ionode_id
-      use mp_global,                ONLY : intra_bgrp_comm, np_ortho, me_ortho, ortho_comm
+      use mp_global,                ONLY : intra_bgrp_comm
       use dener
       use constants,                only : pi, au_gpa
       USE io_files,                 ONLY : tmp_dir, prefix
-      use uspp,                     only : nhsa=> nkb, nhsavb=> nkbus, &
+      use uspp,                     only : nkb, nkbus, &
                                            betae => vkb, rhovan => becsum, &
-                                           deeq, qq_nt, nlcc_any
-      use uspp_param,               only : nh, nvb, ish
+                                           deeq, qq_nt, nlcc_any, indv_ijkb0
+      use uspp_param,               only : nh, upf
       use cg_module,                only : ene_ok,  maxiter,niter_cg_restart, &
                                            conv_thr, passop, enever, itercg,c0old
       use ions_positions,           only : tau0
@@ -53,9 +53,7 @@
       use orthogonalize_base,       ONLY : calphi_bgrp
       use cp_interfaces,            ONLY : rhoofr, dforce, compute_stress, vofrho, nlfl_bgrp, prefor
       use cp_interfaces,            ONLY : nlsm2_bgrp, calbec, caldbec_bgrp, nlfq_bgrp
-      use cp_interfaces,            ONLY : collect_lambda, distribute_lambda
-      USE cp_main_variables,        ONLY : descla, drhor, drhog
-      USE descriptors,              ONLY : la_descriptor, ldim_cyclic
+      USE cp_main_variables,        ONLY : idesc, drhor, drhog
       USE mp_global, ONLY:  me_image, my_image_id, nbgrp
       USE fft_base,  ONLY: dffts, dfftp
 
@@ -71,8 +69,8 @@
       integer :: nfi, nlam
       logical :: tfirst , tlast
       complex(dp) :: eigr(ngw,nat)
-      real(dp) :: bec(nhsa,nbspx)
-      real(dp) :: becdr(nhsa,nbspx,3)
+      real(dp) :: bec(nkb,nbspx)
+      real(dp) :: becdr(nkb,nbspx,3)
       integer irb(3,nat)
       complex(dp) :: eigrb(ngb,nat)
       real(dp) :: rhor(dfftp%nnr,nspin)
@@ -91,8 +89,9 @@
       complex(dp) :: c0( ngw, nbspx )
       complex(dp) :: cm( ngw, nbspx )
       complex(dp) :: phi( ngw, nbspx )
-      real(dp) :: dbec(nhsa,nbspx,3,3)
+      real(dp) :: dbec(nkb,nbspx,3,3)
 !
+      include 'laxlib.fh'
 !
       integer :: i, j, ig, k, is, iss,ia, iv, jv, il, ii, jj, kk, ip, nrlx
       integer :: inl, jnl, niter, istart, nss, nrl, me_rot, np_rot , comm
@@ -130,9 +129,9 @@
       INTEGER :: i_max
       REAL(kind=DP) :: max_sca
    
-      nrlx = MAXVAL(descla(:)%nrlx)
+      nrlx = MAXVAL(idesc(LAX_DESC_NRLX,:))
 
-      allocate(bec0(nhsa,nbspx),becm(nhsa,nbspx), becdrdiag(nhsa,nbspx,3))
+      allocate(bec0(nkb,nbspx),becm(nkb,nbspx), becdrdiag(nkb,nbspx,3))
       allocate (ave_ene(nbspx))
       allocate(c2(ngw),c3(ngw))
 
@@ -177,16 +176,16 @@
       !orthonormalize c0
 
       call calbec(1,nsp,eigr,c0,bec)
-      CALL gram_bgrp( betae, bec, nhsa, c0, ngw )
+      CALL gram_bgrp( betae, bec, nkb, c0, ngw )
 
       !calculates phi for pcdaga
 
-      CALL calphi_bgrp( c0, SIZE(c0,1), bec, nhsa, betae, phi, nbsp )
+      CALL calphi_bgrp( c0, SIZE(c0,1), bec, nkb, betae, phi, nbsp )
 
       !calculates the factors for S and K inversion in US case
-      if(nvb.gt.0) then
-         allocate( s_minus1(nhsavb,nhsavb))
-         allocate( k_minus1(nhsavb,nhsavb))
+      if(nkbus>0) then
+         allocate( s_minus1(nkb,nkb))
+         allocate( k_minus1(nkb,nkb))
         call  set_x_minus1(betae,s_minus1,dumm,.false.)
         call  set_x_minus1(betae,k_minus1,ema0bg,.true.)
       else
@@ -199,7 +198,7 @@
       numok = 0
 
 !initialize  z0t
-      call id_matrix_init( descla, nspin )
+      call id_matrix_init( idesc, nspin )
 
       allocate(hpsi(ngw,nbspx),hpsi0(ngw,nbspx),gi(ngw,nbspx),hi(ngw,nbspx))
       do while ( itercg .lt. maxiter .and. (.not.ltresh) )
@@ -231,7 +230,7 @@
               call rotate( nrlx, z0t, c0(:,:), bec, c0diag, becdiag )
               c0(:,:)=c0diag(:,:)
               bec(:,:)=becdiag(:,:)
-              call id_matrix_init( descla, nspin )
+              call id_matrix_init( idesc, nspin )
            endif
         
 
@@ -381,18 +380,19 @@
            
            call mp_sum( gamma, intra_bgrp_comm )
            
-           if (nvb.gt.0) then
+           if (nkbus.gt.0) then
               do i=1,nbsp
-                 do is=1,nvb
-                    do iv=1,nh(is)
-                       do jv=1,nh(is)
-                          do ia=1,na(is)
-                             inl=ish(is)+(iv-1)*na(is)+ia
-                             jnl=ish(is)+(jv-1)*na(is)+ia
+                 do ia=1,nat
+                    is=ityp(ia)
+                    IF(upf(is)%tvanp) THEN
+                       do iv=1,nh(is)
+                          do jv=1,nh(is)
+                             inl = indv_ijkb0(ia) + iv
+                             jnl = indv_ijkb0(ia) + jv
                              gamma=gamma+ qq_nt(iv,jv,is)*becm(inl,i)*bec0(jnl,i)
                           end do
                        end do
-                    end do
+                    END IF
                  end do
               enddo
            endif
@@ -402,8 +402,8 @@
            do iss=1,nspin
               nss=nupdwn(iss)
               istart=iupdwn(iss)
-              me_rot = descla( iss )%mype
-              np_rot = descla( iss )%npc * descla( iss )%npr
+              me_rot = idesc( LAX_DESC_MYPE, iss )
+              np_rot = idesc( LAX_DESC_NPC, iss ) * idesc( LAX_DESC_NPR, iss )
               allocate( fmat_ ( nrlx, nudx ) )
               do ip = 1, np_rot
                  if( me_rot == ( ip - 1 ) ) then
@@ -426,12 +426,12 @@
               enddo
               deallocate( fmat_ )
            enddo
-           if(nvb.gt.0) then
+           if(nkbus.gt.0) then
               do iss=1,nspin
                  nss=nupdwn(iss)
                  istart=iupdwn(iss)
-                 me_rot = descla( iss )%mype
-                 np_rot = descla( iss )%npc * descla( iss )%npr
+                 me_rot = idesc( LAX_DESC_MYPE, iss )
+                 np_rot = idesc( LAX_DESC_NPC, iss ) * idesc( LAX_DESC_NPR, iss)
                  allocate( fmat_ ( nrlx, nudx ) )
                  do ip = 1, np_rot
                     if( me_rot == ( ip - 1 ) ) then
@@ -443,16 +443,17 @@
                     do i=1,nss
                        jj = ip 
                        do j=1,nrl
-                          do is=1,nvb
-                             do iv=1,nh(is)
-                                do jv=1,nh(is)
-                                   do ia=1,na(is)
-                                      inl=ish(is)+(iv-1)*na(is)+ia
-                                      jnl=ish(is)+(jv-1)*na(is)+ia
+                          do ia=1,nat
+                             is=ityp(ia)
+                             IF( upf(is)%tvanp ) THEN
+                                do iv=1,nh(is)
+                                   do jv=1,nh(is)
+                                      inl = indv_ijkb0(ia) + iv
+                                      jnl = indv_ijkb0(ia) + jv
                                       gamma=gamma+ qq_nt(iv,jv,is)*becm(inl,i+istart-1)*bec0(jnl,jj+istart-1)*fmat_(j,i)
                                    end do
                                 end do
-                             end do
+                             END IF
                           enddo
                           jj = jj + np_rot
                        enddo
@@ -520,8 +521,8 @@
          do iss = 1, nspin
             nss    = nupdwn(iss)
             istart = iupdwn(iss)
-            me_rot = descla( iss )%mype
-            np_rot = descla( iss )%npc * descla( iss )%npr
+            me_rot = idesc( LAX_DESC_MYPE, iss )
+            np_rot = idesc( LAX_DESC_NPC, iss ) * idesc( LAX_DESC_NPR, iss)
             allocate( fmat_ ( nrlx, nudx ) )
             do ip = 1, np_rot
                if( me_rot == ( ip - 1 ) ) then
@@ -565,7 +566,7 @@
         !orthonormalize
 
         call calbec(1,nsp,eigr,cm,becm)
-        CALL gram_bgrp( betae, becm, nhsa, cm, ngw )
+        CALL gram_bgrp( betae, becm, nkb, cm, ngw )
                
         !calculate energy
         if(.not.tens) then
@@ -628,7 +629,7 @@
         endif
 
         call calbec(1,nsp,eigr,cm,becm)
-        CALL gram_bgrp( betae, becm, nhsa, cm, ngw )
+        CALL gram_bgrp( betae, becm, nkb, cm, ngw )
 
         !test on energy: check the energy has really diminished
 
@@ -699,7 +700,7 @@
           c0=c0+spasso*passov*hi
           restartcg=.true.
           call calbec(1,nsp,eigr,c0,bec)
-          CALL gram_bgrp( betae, bec, nhsa, c0, ngw )
+          CALL gram_bgrp( betae, bec, nkb, c0, ngw )
           ene_ok=.false.
           !if  ene1 << energy <  ene0; go to  ene1
         else if( (enever.ge.ene0).and.(ene0.gt.ene1)) then
@@ -709,7 +710,7 @@
           c0=c0+spasso*passov*hi
           restartcg=.true.!ATTENZIONE
           call calbec(1,nsp,eigr,c0,bec)
-          CALL gram_bgrp( betae, bec, nhsa, c0, ngw )
+          CALL gram_bgrp( betae, bec, nkb, c0, ngw )
           !if ene > ene0,en1 do a steepest descent step
           ene_ok=.false.
         else if((enever.ge.ene0).and.(ene0.le.ene1)) then
@@ -725,7 +726,7 @@
             ! chenge the searching direction
             spasso=spasso*(-1.d0)
             call calbec(1,nsp,eigr,cm,becm)
-            CALL gram_bgrp( betae, bec, nhsa, cm, ngw )
+            CALL gram_bgrp( betae, bec, nkb, cm, ngw )
             call calbec(1,nsp,eigr,cm,becm)
             if(.not.tens) then
               call rhoofr(nfi,cm(:,:),irb,eigrb,becm,dbec,rhovan,rhor,drhor,rhog,drhog,rhos,enl,denl,ekin,dekin6)
@@ -778,7 +779,7 @@
         if(.not. ene_ok) call calbec (1,nsp,eigr,c0,bec)
 
         !calculates phi for pc_daga
-        CALL calphi_bgrp( c0, SIZE(c0,1), bec, nhsa, betae, phi, nbsp )
+        CALL calphi_bgrp( c0, SIZE(c0,1), bec, nkb, betae, phi, nbsp )
   
         !=======================================================================
         !
@@ -810,14 +811,14 @@
        if(tpre) then!if pressure is need the following is written because of caldbec
           call  calbec(1,nsp,eigr,c0,bec)
           if(.not.tens) then
-            call caldbec_bgrp( eigr, c0, dbec, descla )
+            call caldbec_bgrp( eigr, c0, dbec, idesc )
             call rhoofr(nfi,c0(:,:),irb,eigrb,bec,dbec,rhovan,rhor,drhor,rhog,drhog,rhos,enl,denl,ekin,dekin6)
           else
 
             !     calculation of the rotated quantities
             call rotate( nrlx, z0t, c0(:,:), bec, c0diag, becdiag )
             !     calculation of rho corresponding to the rotated wavefunctions
-            call caldbec_bgrp( eigr, c0diag, dbec, descla )
+            call caldbec_bgrp( eigr, c0diag, dbec, idesc )
             call rhoofr(nfi,c0diag,irb,eigrb,becdiag,dbec,rhovan,rhor, &
                         drhor,rhog,drhog,rhos,enl,denl,ekin,dekin6)
           endif
@@ -918,11 +919,11 @@
            enddo
            CALL mp_sum( lambda_repl, intra_bgrp_comm )
                      !
-           CALL distribute_lambda( lambda_repl, lambda( :, :, is ), descla( is ) )
+           CALL distribute_lambda( lambda_repl, lambda( :, :, is ), idesc( :, is ) )
            !
         end do
         
-        if(l_cprestart .and. .not.tens .and. nspin==1 .and. nvb<1) then
+        if(l_cprestart .and. .not.tens .and. nspin==1 .and. nkbus<1) then
 
 !if required project c0 on previous manifold of occupied states                                                                                    
 !NOT IMPLEMENTED YET FOR ENSEMBLE DFT AND NSPIN==2
@@ -955,7 +956,7 @@
             enddo
           
             call calbec (1,nsp,eigr,c0,bec)
-            CALL gram_bgrp( betae, bec, nhsa, c0, ngw )
+            CALL gram_bgrp( betae, bec, nkb, c0, ngw )
             call calbec(1,nsp,eigr,c0,bec)
           
 
@@ -995,7 +996,7 @@
             enddo
             
             CALL mp_sum( lambda_repl, intra_bgrp_comm )
-            CALL distribute_lambda( lambda_repl, lambda( :, :, 1 ), descla( 1 ) )
+            CALL distribute_lambda( lambda_repl, lambda( :, :, 1 ), idesc( :, 1 ) )
             cm(:,:)=c0(:,:)
             call calbec (1,nsp,eigr,cm,becm)
 
@@ -1014,13 +1015,13 @@
               !
               lambdap(:,:,iss) = 0.0d0
               !
-              CALL cyc2blk_redist( nss, fmat0(1,1,iss), nrlx, SIZE(fmat0,2), &
-                                  lambda_dist, nlam, nlam, descla(iss) )
+              CALL cyc2blk_redist( nss, fmat0(:,:,iss), nrlx, SIZE(fmat0,2), &
+                                  lambda_dist, nlam, nlam, idesc(:,iss) )
               !
               ! Perform lambdap = lambda * fmat0
               !
-              CALL sqr_mm_cannon( 'N', 'N', nss, 1.0d0, lambda(1,1,iss), nlam, lambda_dist, nlam, &
-                                  0.0d0, lambdap(1,1,iss), nlam, descla(iss) )
+              CALL sqr_mm_cannon( 'N', 'N', nss, 1.0d0, lambda(:,:,iss), nlam, lambda_dist, nlam, &
+                                  0.0d0, lambdap(:,:,iss), nlam, idesc(:,iss) )
               !
               lambda_dist      = lambda(:,:,iss)
               lambda(:,:,iss)  = lambdap(:,:,iss)
@@ -1030,13 +1031,13 @@
            !
            DEALLOCATE( lambda_dist )
            !
-           call nlsm2_bgrp( ngw, nhsa, eigr, c0, becdr, nbspx, nbsp )
+           call nlsm2_bgrp( ngw, nkb, eigr, c0, becdr, nbspx, nbsp )
            !
         endif
         !
   
         !
-        CALL nlfl_bgrp( bec, becdr, lambda, descla, fion ) 
+        CALL nlfl_bgrp( bec, becdr, lambda, idesc, fion ) 
           
         ! bforceion adds the force term due to electronic berry phase
         ! only in US-case
