@@ -60,17 +60,12 @@
       USE fft_interfaces,     ONLY: fwfft, invfft
       USE fft_base,           ONLY: dffts, dfftp
       USE cp_interfaces,      ONLY: checkrho, ennl, calrhovan, dennl
-      USE cp_main_variables,  ONLY: iprint_stdout, descla
+      USE cp_main_variables,  ONLY: iprint_stdout, idesc
       USE wannier_base,       ONLY: iwf
       USE exx_module,         ONLY: rhopr 
       USE input_parameters,   ONLY: tcpbo ! BS
-      ! USE scatter_mod,        ONLY: fft_scatter_tg_opt, maps_sticks_to_3d
-#if defined (__OLDXML)
-      USE xml_io_base,        ONLY: read_rho, restart_dir
-#else
       USE io_base,            ONLY: read_rhog
-#endif      
-      USE io_files,           ONLY: tmp_dir, prefix, postfix
+      USE io_files,           ONLY: restart_dir
       USE fft_rho
       USE fft_helper_subroutines, ONLY: c2psi_gamma
       !
@@ -105,7 +100,7 @@
 #endif
       COMPLEX(DP), ALLOCATABLE :: psis(:)
       REAL(DP), ALLOCATABLE :: drhovan(:,:,:,:,:)
-      CHARACTER(LEN=256) :: dirname
+      CHARACTER(LEN=320) :: filename
 
       LOGICAL, SAVE :: first = .TRUE.
       LOGICAL :: ttstress
@@ -163,7 +158,7 @@
          !
          ALLOCATE( drhovan( nhm*(nhm+1)/2, nat, nspin, 3, 3 ) )
          !
-         CALL dennl( bec_bgrp, dbec, drhovan, denl, descla ) 
+         CALL dennl( bec_bgrp, dbec, drhovan, denl, idesc ) 
          !
          IF( nbgrp > 1 ) THEN
             CALL mp_sum( denl, inter_bgrp_comm )
@@ -186,17 +181,19 @@
          ! FIXME: the beginning, the potential computed and no longer updated.
          !
          IF( first ) THEN
-#if defined (__OLDXML)
-            dirname = restart_dir( tmp_dir, ndr )
-            CALL read_rho( dirname, rhor, nspin )
-#else
             CALL errore('rhoofr','option trhor unverified, please report',1)
-            WRITE(dirname,'(A,A,"_",I2,A)') &
-                 TRIM(tmp_dir), TRIM(prefix), ndr,postfix
-            CALL read_rhog ( dirname, root_bgrp, intra_bgrp_comm, &
+            filename = TRIM( restart_dir(ndr) ) // "charge-density"
+            CALL read_rhog ( filename, root_bgrp, intra_bgrp_comm, &
                  ig_l2g, nspin, rhog )
+            !
+            !^^ ... TEMPORARY FIX  (newlsda) ...
+            IF ( nspin==2 ) THEN
+               rhog(:,1) = ( rhog(:,1) + rhog(:,2) )*0.5d0
+               rhog(:,2) = rhog(:,1) - rhog(:,2)
+            ENDIF
+            !^^.......................
+            !
             CALL rho_g2r ( dfftp, rhog, rhor )
-#endif
             rhopr = rhor
             first = .FALSE.
          ELSE
@@ -503,11 +500,12 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
 !
       USE kinds,                    ONLY: DP
       USE control_flags,            ONLY: iprint
-      USE ions_base,                ONLY: na, nsp, nat
-      USE uspp_param,               ONLY: nhm, nh, nvb
+      USE ions_base,                ONLY: na, nsp, nat, ityp
+      USE uspp_param,               ONLY: nhm, nh, upf
+      USE uspp,                     ONLY: nkbus
       USE electrons_base,           ONLY: nspin
       USE smallbox_gvec,            ONLY: ngb
-      USE smallbox_subs,            ONLY: fft_oned2box
+      USE smallbox_subs,            ONLY: fft_oned2box, box2grid
       USE cell_base,                ONLY: ainv
       USE qgb_mod,                  ONLY: qgb, dqgb
       USE fft_interfaces,           ONLY: fwfft, invfft
@@ -527,8 +525,7 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
       REAL(DP),    INTENT(OUT) :: drhor(dfftp%nnr,nspin,3,3)
       COMPLEX(DP), INTENT(OUT) :: drhog(dfftp%ngm,nspin,3,3)
 ! local
-      INTEGER i, j, isup, isdw, nfft, ifft, iv, jv, ig, ijv, is, iss,   &
-     &     isa, ia, ir, ijs
+      INTEGER i, j, isup, isdw, iv, jv, ig, ijv, is, iss, ia, ir, ijs
       REAL(DP) :: asumt, dsumt
       COMPLEX(DP) fp, fm, ci
 #if defined(__INTEL_COMPILER)
@@ -573,7 +570,9 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
       END DO
 !$omp end parallel
 
-      IF ( nvb <= 0 ) RETURN
+      IF ( nkbus <= 0 ) THEN
+         GO TO 1000
+      END IF
 
       ALLOCATE( v( dfftp%nnr ) )
 
@@ -589,9 +588,9 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
                v(:) = (0.d0, 0.d0)
 
 !$omp parallel default(none) &
-!$omp          shared(nvb, na, ngb, nh, eigrb, dfftb, irb, v, &
-!$omp                 ci, i, j, dqgb, qgb, nhm, rhovan, drhovan ) &
-!$omp          private(mytid, ntids, is, ia, nfft, ifft, iv, jv, ijv, ig, iss, isa, &
+!$omp          shared(nat, ityp, ngb, nh, eigrb, dfftb, irb, v, &
+!$omp                 ci, i, j, dqgb, qgb, nhm, rhovan, drhovan, upf ) &
+!$omp          private(mytid, ntids, is, ia, iv, jv, ijv, ig, iss, &
 !$omp                  qv, fg1, fg2, itid, dqgbt, dsumt, asumt )
 
                ALLOCATE( qv( dfftb%nnr ) )
@@ -606,28 +605,19 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
 #endif
 
                iss=1
-               isa=1
 
-               DO is=1,nvb
+               DO ia=1,nat
+                  is=ityp(ia)
+                  IF( upf(is)%tvanp ) THEN
+
 #if defined(__MPI)
-                  DO ia=1,na(is)
-                     nfft=1
-                     IF ( ( dfftb%np3( isa ) <= 0 ) .OR. ( dfftb%np2( isa ) <= 0 ) ) THEN
-                        isa = isa + nfft
+                     IF ( ( dfftb%np3( ia ) <= 0 ) .OR. ( dfftb%np2( ia ) <= 0 ) ) THEN
                         CYCLE
                      END IF
-#else
-                  DO ia=1,na(is),2
-                     !
-                     !  nfft=2 if two ffts at the same time are performed
-                     !
-                     nfft=2
-                     IF (ia.EQ.na(is)) nfft=1
 #endif
 
 #if defined(_OPENMP)
                      IF ( mytid /= itid ) THEN
-                        isa = isa + nfft
                         itid = MOD( itid + 1, ntids )
                         CYCLE
                      ELSE
@@ -636,49 +626,37 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
 #endif
 
                      dqgbt(:,:) = (0.d0, 0.d0) 
-                     DO ifft=1,nfft
-                        DO iv=1,nh(is)
-                           DO jv=iv,nh(is)
-                              ijv = (jv-1)*jv/2 + iv
-                              IF(iv.NE.jv) THEN
-                                 asumt = 2.0d0 *  rhovan( ijv, isa+ifft-1, iss )
-                                 dsumt = 2.0d0 * drhovan( ijv, isa+ifft-1, iss, i, j )
-                              ELSE
-                                 asumt =  rhovan( ijv, isa+ifft-1, iss )
-                                 dsumt = drhovan( ijv, isa+ifft-1, iss, i, j )
-                              ENDIF
-                              DO ig=1,ngb
-                                 dqgbt(ig,ifft)=dqgbt(ig,ifft) + asumt*dqgb(ig,ijv,is,i,j)
-                                 dqgbt(ig,ifft)=dqgbt(ig,ifft) + dsumt*qgb(ig,ijv,is)
-                              END DO
+                     DO iv=1,nh(is)
+                        DO jv=iv,nh(is)
+                           ijv = (jv-1)*jv/2 + iv
+                           IF(iv.NE.jv) THEN
+                              asumt = 2.0d0 *  rhovan( ijv, ia, iss )
+                              dsumt = 2.0d0 * drhovan( ijv, ia, iss, i, j )
+                           ELSE
+                              asumt =  rhovan( ijv, ia, iss )
+                              dsumt = drhovan( ijv, ia, iss, i, j )
+                           ENDIF
+                           DO ig=1,ngb
+                              dqgbt(ig,1)=dqgbt(ig,1) + asumt*dqgb(ig,ijv,is,i,j)
+                              dqgbt(ig,1)=dqgbt(ig,1) + dsumt*qgb(ig,ijv,is)
                            END DO
                         END DO
                      END DO
                      !     
                      ! add structure factor
                      !
-                     IF(nfft.EQ.2) THEN
-                        fg1 = eigrb(1:ngb,isa   )*dqgbt(1:ngb,1)
-                        fg2 = eigrb(1:ngb,isa+1 )*dqgbt(1:ngb,2)
-                        CALL fft_oned2box( qv, fg1, fg2 )
-                     ELSE
-                        fg1 = eigrb(1:ngb,isa   )*dqgbt(1:ngb,1)
-                        CALL fft_oned2box( qv, fg1 )
-                     ENDIF
+                     fg1 = eigrb(1:ngb,ia   )*dqgbt(1:ngb,1)
+                     CALL fft_oned2box( qv, fg1 )
                      !
-                     CALL invfft( qv, dfftb, isa )
+                     CALL invfft( qv, dfftb, ia )
                      !
                      !  qv = US contribution in real space on box grid
                      !       for atomic species is, real(qv)=atom ia, imag(qv)=atom ia+1
                      !
                      !  add qv(r) to v(r), in real space on the dense grid
                      !
-                     CALL box2grid( irb(1,isa), 1, qv, v )
-                     IF (nfft.EQ.2) CALL box2grid(irb(1,isa+1),2,qv,v)
-
-                     isa = isa + nfft
-!
-                  END DO
+                     CALL box2grid( irb(:,ia), 1, qv, v )
+                  END IF
                END DO
 
                DEALLOCATE( fg1 )
@@ -714,19 +692,19 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
                ALLOCATE( dqgbt( ngb, 2 ) )
                ALLOCATE( fg1( ngb ) )
                ALLOCATE( fg2( ngb ) )
-               isa=1
-               DO is=1,nvb
-                  DO ia=1,na(is)
+               DO ia=1,nat
+                  is=ityp(ia)
+                  IF( upf(is)%tvanp ) THEN
 #if defined(__MPI)
-                     IF ( ( dfftb%np3( isa ) <= 0 ) .OR. ( dfftb%np2( isa ) <= 0 ) ) go to 25
+                     IF ( ( dfftb%np3( ia ) <= 0 ) .OR. ( dfftb%np2( ia ) <= 0 ) ) CYCLE
 #endif
                      DO iss=1,2
                         dqgbt(:,iss) = (0.d0, 0.d0)
                         DO iv= 1,nh(is)
                            DO jv=iv,nh(is)
                               ijv = (jv-1)*jv/2 + iv
-                              asumt=rhovan(ijv,isa,iss)
-                              dsumt =drhovan(ijv,isa,iss,i,j)
+                              asumt=rhovan(ijv,ia,iss)
+                              dsumt =drhovan(ijv,ia,iss,i,j)
                               IF(iv.NE.jv) THEN
                                  asumt =2.d0*asumt
                                  dsumt=2.d0*dsumt
@@ -742,22 +720,20 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
                      !     
                      ! add structure factor
                      !
-                     fg1 = eigrb(1:ngb,isa)*dqgbt(1:ngb,1)
-                     fg2 = eigrb(1:ngb,isa)*dqgbt(1:ngb,2)
+                     fg1 = eigrb(1:ngb,ia)*dqgbt(1:ngb,1)
+                     fg2 = eigrb(1:ngb,ia)*dqgbt(1:ngb,2)
                      CALL fft_oned2box( qv, fg1, fg2 )
 
-                     CALL invfft(qv, dfftb, isa )
+                     CALL invfft(qv, dfftb, ia )
                      !
                      !  qv is the now the US augmentation charge for atomic species is
                      !  and atom ia: real(qv)=spin up, imag(qv)=spin down
                      !
                      !  add qv(r) to v(r), in real space on the dense grid
                      !
-                     CALL box2grid2(irb(1,isa),qv,v)
+                     CALL box2grid(irb(:,ia),qv,v)
                      !
-  25                 isa = isa + 1
-                     !
-                  END DO
+                  END IF
                END DO
 
                DEALLOCATE( dqgbt )
@@ -779,6 +755,7 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
 
       DEALLOCATE( v )
 !
+1000  CONTINUE
       RETURN
 END SUBROUTINE drhov
 
@@ -793,15 +770,15 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
 !     routine makes use of c(-g)=c*(g)  and  beta(-g)=beta*(g)
 !
       USE kinds,                    ONLY: dp
-      USE ions_base,                ONLY: nat, na, nsp
+      USE ions_base,                ONLY: nat, na, nsp, ityp
       USE io_global,                ONLY: stdout
       USE mp_global,                ONLY: intra_bgrp_comm
       USE mp,                       ONLY: mp_sum
-      USE uspp_param,               ONLY: nh, nhm, nvb
-      USE uspp,                     ONLY: deeq
+      USE uspp_param,               ONLY: nh, nhm, upf
+      USE uspp,                     ONLY: deeq, nkbus
       USE electrons_base,           ONLY: nspin
       USE smallbox_gvec,            ONLY: ngb
-      USE smallbox_subs,            ONLY: fft_oned2box
+      USE smallbox_subs,            ONLY: fft_oned2box, box2grid
       USE cell_base,                ONLY: omega
       USE small_box,                ONLY: omegab
       USE control_flags,            ONLY: iprint, iverbosity, tpre
@@ -840,7 +817,9 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
 
       !  Quick return if this sub is not needed
       !
-      IF ( nvb == 0 ) RETURN
+      IF ( nkbus <= 0 ) THEN
+         GO TO 1000
+      END IF
 
       CALL start_clock( 'rhov' )
       ci=(0.d0,1.d0)
@@ -865,8 +844,8 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
          !
 
 !$omp parallel default(none) &
-!$omp          shared(nvb, na, ngb, nh, rhovan, qgb, eigrb, dfftb, iverbosity, omegab, irb, v, &
-!$omp                 stdout, ci, rhor, dfftp ) &
+!$omp          shared(na, ngb, nh, rhovan, qgb, eigrb, dfftb, iverbosity, omegab, irb, v, &
+!$omp                 stdout, ci, rhor, dfftp, upf, nsp, ityp, nat ) &
 !$omp          private(mytid, ntids, is, ia, nfft, ifft, iv, jv, ijv, sumrho, qgbt, ig, iss, isa, ca, &
 !$omp                  qv, fg1, fg2, itid, ir )
 
@@ -889,39 +868,34 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
          ALLOCATE( fg2( ngb ) )
 
 
-         DO is = 1, nvb
+         DO ia = 1, nat
+
+            is = ityp(ia)
+
+            IF( upf(is)%tvanp ) THEN
+
+               nfft = 1
 
 #if defined(__MPI)
-            DO ia = 1, na(is)
-               nfft = 1
-               IF ( ( dfftb%np3( isa ) <= 0 ) .OR. ( dfftb%np2( isa ) <= 0 ) ) THEN
-                  isa = isa + nfft
+               IF ( ( dfftb%np3( ia ) <= 0 ) .OR. ( dfftb%np2( ia ) <= 0 ) ) THEN
                   CYCLE
                END IF
-#else
-            DO ia = 1, na(is), 2
-               !
-               !  nfft=2 if two ffts at the same time are performed
-               !
-               nfft = 2
-               IF( ia .EQ. na(is) ) nfft = 1
 #endif
-
 #if defined(_OPENMP)
                IF ( mytid /= itid ) THEN
-                  isa = isa + nfft
                   itid = MOD( itid + 1, ntids )
                   CYCLE
                ELSE
                   itid = MOD( itid + 1, ntids )
                END IF
 #endif
+
                DO ifft=1,nfft
                   qgbt(:,ifft) = (0.d0, 0.d0)
                   DO iv= 1,nh(is)
                      DO jv=iv,nh(is)
                         ijv = (jv-1)*jv/2 + iv
-                        sumrho=rhovan(ijv,isa+ifft-1,iss)
+                        sumrho=rhovan(ijv,ia+ifft-1,iss)
                         IF(iv.NE.jv) sumrho=2.d0*sumrho
                         DO ig=1,ngb
                            qgbt(ig,ifft)=qgbt(ig,ifft) + sumrho*qgb(ig,ijv,is)
@@ -933,15 +907,15 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
                ! add structure factor
                !
                IF(nfft.EQ.2)THEN
-                  fg1 = eigrb(1:ngb,isa   )*qgbt(1:ngb,1)
-                  fg2 = eigrb(1:ngb,isa+1 )*qgbt(1:ngb,2)
+                  fg1 = eigrb(1:ngb,ia   )*qgbt(1:ngb,1)
+                  fg2 = eigrb(1:ngb,ia+1 )*qgbt(1:ngb,2)
                   CALL fft_oned2box( qv, fg1, fg2 )
                ELSE
-                  fg1 = eigrb(1:ngb,isa   )*qgbt(1:ngb,1)
+                  fg1 = eigrb(1:ngb,ia   )*qgbt(1:ngb,1)
                   CALL fft_oned2box( qv, fg1 )
                ENDIF
 
-               CALL invfft( qv, dfftb, isa )
+               CALL invfft( qv, dfftb, ia )
                !
                !  qv = US augmentation charge in real space on box grid
                !       for atomic species is, real(qv)=atom ia, imag(qv)=atom ia+1
@@ -960,12 +934,10 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
                !
                !  add qv(r) to v(r), in real space on the dense grid
                !
-               CALL  box2grid(irb(1,isa),1,qv,v)
-               IF (nfft.EQ.2) CALL  box2grid(irb(1,isa+1),2,qv,v)
-
-               isa = isa + nfft
+               CALL  box2grid(irb(:,ia),1,qv,v)
+               IF (nfft.EQ.2) CALL  box2grid(irb(:,ia+1),2,qv,v)
 !
-            END DO
+            END IF
          END DO
 
          DEALLOCATE( fg1 )
@@ -1023,17 +995,19 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
          ALLOCATE( fg2( ngb ) )
 
          isa=1
-         DO is=1,nvb
-            DO ia=1,na(is)
+         DO ia=1,nat
+            is = ityp(ia)
 #if defined(__MPI)
-               IF ( ( dfftb%np3( isa ) <= 0 ) .OR. ( dfftb%np2( isa ) <= 0 ) ) go to 25
+               IF ( ( dfftb%np3( ia ) <= 0 ) .OR. ( dfftb%np2( ia ) <= 0 ) ) CYCLE
 #endif
+               IF( upf(is)%tvanp ) THEN
+
                DO iss=1,2
                   qgbt(:,iss) = (0.d0, 0.d0)
                   DO iv=1,nh(is)
                      DO jv=iv,nh(is)
                         ijv = (jv-1)*jv/2 + iv
-                        sumrho=rhovan(ijv,isa,iss)
+                        sumrho=rhovan(ijv,ia,iss)
                         IF(iv.NE.jv) sumrho=2.d0*sumrho
                         DO ig=1,ngb
                            qgbt(ig,iss)=qgbt(ig,iss)+sumrho*qgb(ig,ijv,is)
@@ -1044,11 +1018,11 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
 !     
 ! add structure factor
 !
-               fg1 = eigrb(1:ngb,isa)*qgbt(1:ngb,1)
-               fg2 = eigrb(1:ngb,isa)*qgbt(1:ngb,2)
+               fg1 = eigrb(1:ngb,ia)*qgbt(1:ngb,1)
+               fg2 = eigrb(1:ngb,ia)*qgbt(1:ngb,2)
                CALL fft_oned2box( qv, fg1, fg2 )
 !
-               CALL invfft( qv,dfftb,isa)
+               CALL invfft( qv,dfftb,ia)
 !
 !  qv is the now the US augmentation charge for atomic species is
 !  and atom ia: real(qv)=spin up, imag(qv)=spin down
@@ -1067,10 +1041,10 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
 !
 !  add qv(r) to v(r), in real space on the dense grid
 !
-               CALL box2grid2(irb(1,isa),qv,v)
-  25           isa=isa+1
-!
-            END DO
+               CALL box2grid(irb(:,ia),qv,v)
+
+               END IF
+
          END DO
 !
          DO ir=1,dfftp%nnr
@@ -1114,6 +1088,8 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
       DEALLOCATE( v )
 
       CALL stop_clock( 'rhov' )
+
+1000  CONTINUE
 !
       RETURN
 END SUBROUTINE rhov
