@@ -12,18 +12,19 @@ SUBROUTINE from_scratch( )
     USE control_flags,        ONLY : tranp, trane, iverbosity, tpre, tv0rd, &
                                      tfor, thdyn, &
                                      lwf, tprnfor, tortho, amprp, ampre,  &
-                                     tsde, ortho_eps, ortho_max, &
-                                     force_pairing
-    USE ions_positions,       ONLY : taus, tau0, tausm, vels, velsm, fion, fionm
-    USE ions_base,            ONLY : na, nsp, randpos, zv, ions_vel, vel_srt
+                                     tsde, force_pairing, tcap
+    USE ions_positions,       ONLY : taus, tau0, tausm, vels, velsm, fion, fionm, &
+                                     taum 
+    USE ions_base,            ONLY : na, nsp, randpos, zv, ions_vel, vel, ityp, &
+                                     amass, randvel
     USE ions_base,            ONLY : cdmi, nat, iforce
-    USE ions_nose,            ONLY : xnhp0, xnhpm, vnhp
+    USE ions_nose,            ONLY : xnhp0, xnhpm, vnhp, tempw
     USE cell_base,            ONLY : ainv, h, s_to_r, ibrav, omega, press, &
                                      hold, r_to_s, deth, wmass, iforceh,   &
                                      cell_force, velh, at, alat
     USE cell_nose,            ONLY : xnhh0, xnhhm, vnhh
     USE electrons_nose,       ONLY : xnhe0, xnhem, vnhe
-    use electrons_base,       ONLY : nbsp, f, nspin, nupdwn, iupdwn, nbsp_bgrp, nbspx_bgrp, nbspx
+    use electrons_base,       ONLY : nbsp, f, nspin, nupdwn, iupdwn, nbsp_bgrp, nbspx_bgrp, nbspx, nudx
     USE electrons_module,     ONLY : occn_info, distribute_c, collect_c, distribute_b, collect_b
     USE energies,             ONLY : entropy, eself, enl, ekin, enthal, etot, ekincm
     USE energies,             ONLY : dft_energy_type, debug_energies
@@ -34,7 +35,6 @@ SUBROUTINE from_scratch( )
     USE gvecw,                ONLY : ngw
     USE gvect,                ONLY : gg
     USE gvect,                ONLY : gstart, mill, eigts1, eigts2, eigts3
-    USE uspp_param,           ONLY : nvb
     USE cp_electronic_mass,   ONLY : emass
     USE efield_module,        ONLY : tefield, efield_berry_setup, berry_energy, &
                                      tefield2, efield_berry_setup2, berry_energy2
@@ -44,23 +44,25 @@ SUBROUTINE from_scratch( )
                                      strucf, phfacs, nlfh, vofrho, nlfl_bgrp, prefor
     USE cp_interfaces,        ONLY : rhoofr, ortho, wave_rand_init, elec_fakekine
     USE cp_interfaces,        ONLY : compute_stress, dotcsc, calbec_bgrp, caldbec_bgrp
-    USE cp_interfaces,        ONLY : print_lambda, nlfq_bgrp, setval_lambda
+    USE cp_interfaces,        ONLY : nlfq_bgrp
     USE printout_base,        ONLY : printout_pos
     USE orthogonalize_base,   ONLY : updatc, calphi_bgrp
     USE wave_base,            ONLY : wave_steepest
-    USE wavefunctions_module, ONLY : c0_bgrp, cm_bgrp, phi_bgrp
+    USE wavefunctions, ONLY : c0_bgrp, cm_bgrp, phi_bgrp
     USE fft_base,             ONLY : dfftp, dffts
     USE time_step,            ONLY : delt
-    USE cp_main_variables,    ONLY : descla, bephi, becp_bgrp, nfi, &
+    USE cp_main_variables,    ONLY : idesc, bephi, becp_bgrp, nfi, &
                                      sfac, eigr, taub, irb, eigrb, bec_bgrp, &
                                      lambda, lambdam, lambdap, ema0bg, rhog, rhor, rhos, &
                                      vpot, ht0, edft, becdr_bgrp, dbec, drhor, drhog
-    USE mp_global,            ONLY : np_ortho, me_ortho, ortho_comm, inter_bgrp_comm, nbgrp, me_bgrp
+    USE mp_global,            ONLY : inter_bgrp_comm, nbgrp, me_bgrp
     USE mp,                   ONLY : mp_sum
     USE matrix_inversion
     !
     IMPLICIT NONE
     !
+    include 'laxlib.fh'
+    !
     REAL(DP),    ALLOCATABLE :: emadt2(:), emaver(:)
     REAL(DP)                 :: verl1, verl2
     REAL(DP)                 :: bigr, dum
@@ -102,9 +104,9 @@ SUBROUTINE from_scratch( )
        !
        CALL invmat( 3, h, ainv, deth )
        !
-       CALL randpos( taus, na, nsp, tranp, amprp, ainv, iforce )
+       CALL randpos( taus, nat, ityp, tranp, amprp, ainv, iforce )
        !
-       CALL s_to_r( taus, tau0, na, nsp, h )
+       CALL s_to_r( taus, tau0, nat, h )
        !
     END IF
     !
@@ -149,7 +151,7 @@ SUBROUTINE from_scratch( )
     velh = 0.0d0
     fion = 0.0d0
     !
-    IF ( tv0rd .AND. tfor ) THEN
+    IF ( tv0rd .AND. tfor .AND. .NOT. tcap ) THEN
        !
        ! ... vel_srt=starting velocities, read from input, are brough to
        ! ... scaled axis and copied into array vels. Since velocites are
@@ -157,9 +159,16 @@ SUBROUTINE from_scratch( )
        ! ... to tausm=tau(t)-v*delta t so that the Verlet algorithm will 
        ! ... start with the correct velocity
        !
-       CALL r_to_s( vel_srt, vels, na, nsp, ainv )
+
+       CALL r_to_s( vel, vels, nat, ainv )
        tausm(:,:) =  taus(:,:) - vels(:,:)*delt
        velsm(:,:) =  vels(:,:)
+    ELSE IF (tcap .AND. tfor ) THEN
+       WRITE( stdout, '(" Randomizing ions velocities according to tempw (INPUT VELOCITIES DISCARDED) ")')
+       CALL randvel( tempw, tau0 , taum, nat, ityp, iforce, amass, delt )
+       CALL r_to_s( taum, tausm, nat, ainv )
+       vels(:,:) = (taus(:,:)-tausm(:,:))/delt
+       velsm(:,:) = vels(:,:)
     ELSE
        vels = 0.D0
        tausm = taus
@@ -181,7 +190,7 @@ SUBROUTINE from_scratch( )
        !
        CALL calbec_bgrp ( 1, nsp, eigr, cm_bgrp, bec_bgrp )
        !
-       if ( tstress ) CALL caldbec_bgrp( eigr, cm_bgrp, dbec, descla )
+       if ( tstress ) CALL caldbec_bgrp( eigr, cm_bgrp, dbec, idesc )
        !
        CALL rhoofr( nfi, cm_bgrp, irb, eigrb, bec_bgrp, dbec, becsum, rhor, drhor, rhog, drhog, rhos, enl, denl, ekin, dekin6 )
        !
@@ -218,7 +227,7 @@ SUBROUTINE from_scratch( )
       CALL compute_stress( stress, detot, h, omega )
 
       if( iverbosity > 1 ) &
-             CALL printout_pos( stdout, fion, nat, head = ' fion ' )
+             CALL printout_pos( stdout, fion, nat, ityp, head = ' fion ' )
 
       CALL newd( vpot, irb, eigrb, becsum, fion )
       !
@@ -227,7 +236,7 @@ SUBROUTINE from_scratch( )
          CALL runcp_uspp_force_pairing( nfi, fccc, ccc, ema0bg, dt2bye, rhos,&
                     bec_bgrp, cm_bgrp, c0_bgrp, ei_unp, fromscra = .TRUE. )
          !
-         CALL setval_lambda( lambda(:,:,2), nupdwn(1), nupdwn(1), 0.d0, descla(1) )
+         CALL setval_lambda( lambda(:,:,2), nupdwn(1), nupdwn(1), 0.d0, idesc(:,1) )
          !
       ELSE
          !
@@ -250,22 +259,23 @@ SUBROUTINE from_scratch( )
          &   phi_bgrp( :, iupdwn(2):(iupdwn(2)+nupdwn(2)-1) ) =    phi_bgrp( :, 1:nupdwn(2))
 
       if( tortho ) then
-         CALL ortho( eigr, c0_bgrp, phi_bgrp, lambda, descla, bigr, iter, ccc, bephi, becp_bgrp )
+         CALL ortho( eigr, c0_bgrp, phi_bgrp, lambda, idesc, bigr, iter, ccc, bephi, becp_bgrp )
       else
          CALL gram_bgrp( vkb, bec_bgrp, nkb, c0_bgrp, ngw )
       endif
       !
       IF ( ttforce ) THEN
-         CALL nlfl_bgrp( bec_bgrp, becdr_bgrp, lambda, descla, fion )
+         CALL nlfl_bgrp( bec_bgrp, becdr_bgrp, lambda, idesc, fion )
       END IF
 
-      if ( iverbosity > 1 ) CALL print_lambda( lambda, descla, nbsp, 9, ccc )
+      if ( iverbosity > 1 ) &
+         CALL laxlib_print_matrix( lambda, idesc, nbsp, 9, nudx, ccc, ionode, stdout )
 
       !
-      if ( tstress ) CALL nlfh( stress, bec_bgrp, dbec, lambda, descla )
+      if ( tstress ) CALL nlfh( stress, bec_bgrp, dbec, lambda, idesc )
       !
       IF ( tortho ) THEN
-         CALL updatc( ccc, lambda, phi_bgrp, bephi, becp_bgrp, bec_bgrp, c0_bgrp, descla )
+         CALL updatc( ccc, lambda, phi_bgrp, bephi, becp_bgrp, bec_bgrp, c0_bgrp, idesc )
       END IF
       !
       IF( force_pairing ) THEN
@@ -277,9 +287,9 @@ SUBROUTINE from_scratch( )
       ENDIF
       !
       !
-      CALL calbec_bgrp ( nvb+1, nsp, eigr, c0_bgrp, bec_bgrp )
+      CALL calbec_bgrp ( 1, nsp, eigr, c0_bgrp, bec_bgrp, 1 )
       !
-      if ( tstress ) CALL caldbec_bgrp( eigr, cm_bgrp, dbec, descla )
+      if ( tstress ) CALL caldbec_bgrp( eigr, cm_bgrp, dbec, idesc )
 
       if ( iverbosity > 1 ) CALL dotcsc( eigr, c0_bgrp, ngw, nbsp_bgrp )
       !
@@ -287,8 +297,9 @@ SUBROUTINE from_scratch( )
       xnhpm = 0.0d0
       vnhp  = 0.0d0
       fionm = 0.0d0
-      !
-      CALL ions_vel( vels, taus, tausm, na, nsp, delt )
+
+      ! Is this call useless and wrong?
+      CALL ions_vel( vels, taus, tausm, delt )
       !
       xnhh0(:,:) = 0.0d0
       xnhhm(:,:) = 0.0d0
