@@ -176,7 +176,7 @@ MODULE input
      USE control_flags, ONLY : remove_rigid_rot_ => remove_rigid_rot
      USE control_flags, ONLY : iesr_ => iesr
      USE control_flags, ONLY : textfor
-     USE control_flags, ONLY : do_makov_payne, twfcollect
+     USE control_flags, ONLY : do_makov_payne
      USE control_flags, ONLY : lwf, lwfnscf, lwfpbe0nscf
      USE control_flags, ONLY : smallmem
      USE control_flags, ONLY : tconvthrs
@@ -199,8 +199,6 @@ MODULE input
                                     epol2_       => epol2,     &
                                     efield2_     => efield2
      !
-     USE uspp_param,         ONLY : nvb
-     !
      USE input_parameters,   ONLY: &
         electron_dynamics, electron_damping, electron_temperature,   &
         ion_dynamics, ekin_conv_thr, etot_conv_thr, forc_conv_thr, ion_maxstep,&
@@ -241,7 +239,7 @@ MODULE input
      ! ... define memory- and disk-related internal switches
      !
      smallmem = ( TRIM( memory ) == 'small' )
-     twfcollect = wf_collect
+     IF (smallmem) CALL errore('init', "memory='small' no longer implemented",1)
      !
      ! Options for isolated system
      SELECT CASE( TRIM( assume_isolated ) )
@@ -728,7 +726,7 @@ MODULE input
            ecutrho, ecfixed, qcutz, q2sigma, tk_inp, wmass,                   &
            ion_radius, emass, emass_cutoff, temph, fnoseh, nr1b, nr2b, nr3b,  &
            tempw, fnosep, nr1, nr2, nr3, nr1s, nr2s, nr3s, ekincw, fnosee,    &
-           outdir, prefix, nkstot, xk, vdw_table_name,                        &
+           outdir, prefix, nkstot, xk,                                        &
            occupations, n_inner, fermi_energy, rotmass, occmass,              &
            rotation_damping, occupation_damping, occupation_dynamics,         &
            rotation_dynamics, degauss, smearing, nhpcl, nhptyp, ndega,        &
@@ -755,6 +753,7 @@ MODULE input
                                   exx_ps_rcut_s=>exx_ps_rcut_self,&
                                   exx_me_rcut_s=>exx_me_rcut_self,&
                                   exx_ps_rcut_p=>exx_ps_rcut_pair,&
+                                  texx_cube=>exx_use_cube_domain,&
                                   exx_me_rcut_p=>exx_me_rcut_pair
 !===============================================================
      !
@@ -793,9 +792,7 @@ MODULE input
      USE ensemble_dft,     ONLY : ensemble_initval,tens
      USE wannier_base,     ONLY : wannier_init
      USE efield_module,    ONLY : tefield
-     USE funct,            ONLY : dft_is_nonlocc, get_inlc
-     USE kernel_table,     ONLY : vdw_table_name_ => vdw_table_name, &
-                                  initialize_kernel_table
+     USE funct,            ONLY : dft_is_nonlocc
      USE control_flags,    ONLY : llondon, ts_vdw_ => ts_vdw
      USE london_module,    ONLY : init_london, scal6, lon_rcut
      USE tsvdw_module,     ONLY : vdw_isolated, vdw_econv_thr
@@ -804,7 +801,7 @@ MODULE input
      !
      REAL(DP) :: alat_ , massa_totale
      ! ...   DIIS
-     INTEGER :: ia, iss, inlc
+     INTEGER :: ia, iss
      LOGICAL :: ltest
      !
      !   Subroutine Body
@@ -919,8 +916,8 @@ MODULE input
      CALL wannier_init( wf_efield, wf_switch, sw_len, efx0, efy0, efz0, &
                         efx1, efy1, efz1, wfsd, wfdt, neigh, poisson_eps,&
                         dis_cutoff, exx_ps_rcut_s, exx_me_rcut_s,&
-                        exx_ps_rcut_p, exx_me_rcut_p, vnbsp,&
-                        maxwfdt, wf_q, &
+                        exx_ps_rcut_p, exx_me_rcut_p, texx_cube, &
+                        vnbsp, maxwfdt, wf_q, &
                         wf_friction, nit, nsd, nsteps, tolw, adapt,     &
                         calwf, nwf, wffort, writev, wannier_index,      &
                         restart_mode )
@@ -978,14 +975,6 @@ MODULE input
         vdw_econv_thr= ts_vdw_econv_thr
      END IF
      !
-     ! ... initialize kernel table for nonlocal functionals
-     !
-     IF ( dft_is_nonlocc( ) ) THEN
-        vdw_table_name_ = vdw_table_name
-        inlc = get_inlc()
-        call initialize_kernel_table(inlc)
-     ENDIF
-     !
      RETURN
      !
   END SUBROUTINE modules_setup
