@@ -5,8 +5,6 @@
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
-
-
 !=----------------------------------------------------------------------------=!
    SUBROUTINE printout_new_x   &
      ( nfi, tfirst, tfilei, tprint, tps, h, stress, tau0, vels, &
@@ -21,11 +19,12 @@
       USE energies,          ONLY : print_energies, dft_energy_type
       USE printout_base,     ONLY : printout_base_open, printout_base_close, &
                                     printout_pos, printout_cell, printout_stress, &
-                                    printout_vefftsvdw, printout_wfc
+                                    printout_vefftsvdw, printout_wfc, &
+                                    save_print_counter
       USE constants,         ONLY : au_gpa, bohr_radius_cm, amu_au, &
                                     BOHR_RADIUS_ANGS, pi
-      USE ions_base,         ONLY : na, nsp, nat, ind_bck, atm, amass, cdmi, &
-                                    ions_cofmass, ions_displacement, label_srt
+      USE ions_base,         ONLY : na, nsp, nat, ityp, atm, amass, cdmi, &
+                                    ions_cofmass, ions_displacement
       USE cell_base,         ONLY : s_to_r, get_volume, ainv
       USE efield_module,     ONLY : tefield, pberryel, pberryion, &
                                     tefield2, pberryel2, pberryion2
@@ -34,9 +33,7 @@
       USE electrons_module,  ONLY : print_eigenvalues
       USE pres_ai_mod,      ONLY : P_ext, Surf_t, volclu, surfclu, abivol, &
                                    abisur, pvar, n_ele
-      USE xml_io_base,       ONLY : save_print_counter
       USE cp_main_variables, ONLY : nprint_nfi, iprint_stdout
-      USE io_files,          ONLY : tmp_dir
       USE control_flags,     ONLY : ndw
       USE io_global,         ONLY : ionode, ionode_id, stdout
       USE control_flags,     ONLY : lwf, lwfpbe0nscf  ! exx_wf related
@@ -48,6 +45,7 @@
       USE funct,             ONLY : dft_is_hybrid, exx_is_active
       USE wannier_module,    ONLY : wfc
       USE electrons_base,    ONLY : nbsp, nspin, nupdwn, iupdwn
+      USE wrappers,          ONLY : memstat
       !
       IMPLICIT NONE
       !
@@ -65,12 +63,11 @@
       REAL(DP), INTENT(IN) :: atot! enthalpy of system for c.g. case
       REAL(DP), INTENT(IN) :: ekin
       REAL(DP), INTENT(IN) :: epot ! ( epseu + eht + exc )
-      LOGICAL, INTENT(IN) :: print_forces, print_stress, tstdout
-   
-   !
+      LOGICAL, INTENT(IN) :: print_forces, print_stress, tstdout   
+      !
       REAL(DP) :: stress_gpa( 3, 3 )
       REAL(DP) :: cdm0( 3 )
-      REAL(DP) :: dis( nsp )
+      REAL(DP) :: dis( SIZE(na) )
       REAL(DP) :: out_press, volume
       REAL(DP) :: totalmass
       INTEGER  :: isa, is, ia, kilobytes
@@ -81,8 +78,6 @@
       ! avoid double printing to files by refering to nprint_nfi
       !
       tfile = tfilei .and. ( nfi .gt. nprint_nfi )
-      !
-     
       !
       CALL memstat( kilobytes )
       !
@@ -93,10 +88,10 @@
       IF( tprint ) THEN
          IF ( tfile ) THEN
             ! we're writing files, let's save nfi
-            CALL save_print_counter( nfi, tmp_dir, ndw )
+            CALL save_print_counter( nfi, ndw )
          ELSE IF ( tfilei ) then
             ! not there yet, save the old nprint_nfi
-            CALL save_print_counter( nprint_nfi, tmp_dir, ndw )
+            CALL save_print_counter( nprint_nfi, ndw )
          END IF
       END IF
       !
@@ -149,12 +144,12 @@
             !
             ! Compute Center of mass displacement since the initialization of step counter
             !
-            CALL ions_cofmass( tau0, amass, na, nsp, cdm0 )
+            CALL ions_cofmass( tau0, amass, nat, ityp, cdm0 )
             !
             IF(tstdout) &
                WRITE( stdout,1000) SUM( ( cdm0(:)-cdmi(:) )**2 ) 
             !
-            CALL ions_displacement( dis, tau0 )
+            CALL ions_displacement( dis, tau0, nsp, nat, ityp )
             !
             IF( print_stress ) THEN
                !
@@ -168,46 +163,38 @@
             ! ... write out a standard XYZ file in angstroms
             !
             IF(tstdout) &
-               CALL printout_pos( stdout, tau0, nat, what = 'pos', &
-                                  label = label_srt, sort = ind_bck )
+               CALL printout_pos( stdout, tau0, nat, ityp, what = 'pos', label = atm )
             !
             IF( tfile ) then
                if (.not.nice_output_files) then
-                  CALL printout_pos( 35, tau0, nat, nfi = nfi, tps = tps )
+                  CALL printout_pos( 35, tau0, nat, ityp, nfi = nfi, tps = tps )
                else
-                  CALL printout_pos( 35, tau0, nat, what = 'xyz', &
-                               nfi = nfi, tps = tps, label = label_srt, &
-                               fact= BOHR_RADIUS_ANGS ,sort = ind_bck )
+                  CALL printout_pos( 35, tau0, nat, ityp, what = 'xyz', &
+                               nfi = nfi, tps = tps, label = atm, &
+                               fact= BOHR_RADIUS_ANGS )
                endif
             END IF
             !
             ALLOCATE( tauw( 3, nat ) )
             !
-            isa = 0
-            !
-            DO is = 1, nsp
+            DO ia = 1, nat
                !
-               DO ia = 1, na(is)
-                  !
-                  isa = isa + 1
-                  CALL s_to_r( vels(:,isa), tauw(:,isa), h )
-                  !
-               END DO
+               CALL s_to_r( vels(:,ia), tauw(:,ia), h )
                !
             END DO
             !
             IF(tstdout) WRITE( stdout, * )
             !
             IF(tstdout) &
-               CALL printout_pos( stdout, tauw, nat, &
-                               what = 'vel', label = label_srt, sort = ind_bck )
+               CALL printout_pos( stdout, tauw, nat, ityp, &
+                               what = 'vel', label = atm )
             !
             IF( tfile ) then
                if (.not.nice_output_files) then
-                  CALL printout_pos( 34, tauw, nat, nfi = nfi, tps = tps )
+                  CALL printout_pos( 34, tauw, nat, ityp, nfi = nfi, tps = tps )
                else
-                  CALL printout_pos( 34, tauw, nat, nfi = nfi, tps = tps, &
-                               what = 'vel', label = label_srt, sort = ind_bck )
+                  CALL printout_pos( 34, tauw, nat, ityp, nfi = nfi, tps = tps, &
+                               what = 'vel', label = atm )
                endif
             END IF
             !
@@ -216,15 +203,15 @@
                IF(tstdout) WRITE( stdout, * )
                !
                IF(tstdout) &
-                  CALL printout_pos( stdout, fion, nat, &
-                                  what = 'for', label = label_srt, sort = ind_bck )
+                  CALL printout_pos( stdout, fion, nat, ityp, &
+                                  what = 'for', label = atm )
                !
                IF( tfile ) then
                   if (.not.nice_output_files) then
-                     CALL printout_pos( 37, fion, nat, nfi = nfi, tps = tps )
+                     CALL printout_pos( 37, fion, nat, ityp, nfi = nfi, tps = tps )
                   else
-                     CALL printout_pos( 37, fion, nat, nfi = nfi, tps = tps, &
-                          what = 'for', label = label_srt, sort = ind_bck )
+                     CALL printout_pos( 37, fion, nat, ityp, nfi = nfi, tps = tps, &
+                          what = 'for', label = atm )
                   endif
                END IF
                !
@@ -476,11 +463,11 @@
 
 29471 FORMAT( '#',3X,'nfi',4X,'time(ps)',8X,'ekinc',8X,'T_cell(K)',5X,'Tion(K)',10X,'etot',15X,'enthal',15X,'econs', &
            & 15X,'econt',10X,'Volume',8X,'Pressure(GPa)',8X,'EXX',15X,'EVDW' )
-29481 FORMAT( I7,4(2X,ES12.6),4(2X,F18.8),2X,ES12.6,2X,F12.5,2(2X,F15.8) )
-29482 FORMAT( I7,4(2X,ES12.6),4(2X,F18.8),2X,ES12.6,2X,F12.5,2X,F15.8 )
-29483 FORMAT( I7,4(2X,ES12.6),4(2X,F18.8),2X,ES12.6,2X,F12.5 )
-29484 FORMAT( I7,2(2X,ES12.6),6(2X,F18.8) )
-2949  FORMAT( I7,2X,ES12.6,6(1X,F15.8) )
+29481 FORMAT( I7,4(1X,ES13.6),4(2X,F18.8),1X,ES13.6,2X,F12.5,2(2X,F15.8) )
+29482 FORMAT( I7,4(1X,ES13.6),4(2X,F18.8),1X,ES13.6,2X,F12.5,2X,F15.8 )
+29483 FORMAT( I7,4(1X,ES13.6),4(2X,F18.8),1X,ES13.6,2X,F12.5 )
+29484 FORMAT( I7,2(1X,ES13.6),6(2X,F18.8) )
+2949  FORMAT( I7,1X,ES13.6,6(1X,F15.8) )
       !
       RETURN
    END SUBROUTINE printout_new_x
