@@ -11,29 +11,23 @@
 
    SUBROUTINE writefile_x                                         &
      &     ( h,hold,nfi,c0,cm,taus,tausm,vels,velsm,acc,           &
-     &       lambda,lambdam,descla,xnhe0,xnhem,vnhe,xnhp0,xnhpm,vnhp,nhpcl,nhpdim,ekincm,&
+     &       lambda,lambdam,idesc,xnhe0,xnhem,vnhe,xnhp0,xnhpm,vnhp,nhpcl,nhpdim,ekincm,&
      &       xnhh0,xnhhm,vnhh,velh, fion, tps, mat_z, occ_f, rho )
 !-----------------------------------------------------------------------
 !
       USE kinds,            ONLY: DP
       USE ions_base,        ONLY: nsp, na, cdmi, taui
       USE cell_base,        ONLY: s_to_r
-#if defined (__OLDXML)
-      USE cp_restart,       ONLY: cp_writefile
-#else
       USE cp_restart_new,   ONLY: cp_writefile
-#endif
       USE cp_restart_new,   ONLY: cp_write_zmat
       USE cp_interfaces,    ONLY: set_evtot, set_eitot, c_bgrp_expand, &
            c_bgrp_pack
       USE electrons_base,   ONLY: nspin, nbnd, nbsp, iupdwn, nupdwn, nbspx
       USE electrons_module, ONLY: ei
-      USE io_files,         ONLY: tmp_dir
       USE ensemble_dft,     ONLY: tens
       USE mp,               ONLY: mp_bcast
-      USE control_flags,    ONLY: tksw, ndw, io_level, twfcollect
+      USE control_flags,    ONLY: tksw, ndw, io_level
       USE electrons_module, ONLY: collect_c
-      USE descriptors,      ONLY: la_descriptor
       USE gvecw,            ONLY: ngw
       USE wannier_module,   ONLY : wfc ! BS
 
@@ -53,11 +47,11 @@
       REAL(DP), INTENT(in) :: rho(:,:)
       REAL(DP), INTENT(in) :: occ_f(:)
       REAL(DP), INTENT(in) :: mat_z(:,:,:)
-      TYPE(la_descriptor), INTENT(IN) :: descla(:)
+      INTEGER, INTENT(IN) :: idesc(:,:)
 
       REAL(DP) :: ht(3,3), htm(3,3), htvel(3,3), gvel(3,3)
       INTEGER  :: nk = 1, ispin, i, ib, ierr
-      REAL(DP) :: xk(3,1) = 0.0d0, wk(1) = 2.0d0
+      REAL(DP) :: xk(3,2)=0.0_dp, wk(2)=1.0_dp
       COMPLEX(DP), ALLOCATABLE :: ctot(:,:)
       REAL(DP),    ALLOCATABLE :: eitot(:,:)
       INTEGER  :: nupdwn_tot( 2 ), iupdwn_tot( 2 )
@@ -91,7 +85,7 @@
          !
          ALLOCATE( ctot( SIZE( c0, 1 ), nupdwn_tot(1) * nspin ) )
          !
-         CALL set_evtot( c0, ctot, lambda, descla, iupdwn_tot, nupdwn_tot )
+         CALL set_evtot( c0, ctot, lambda, idesc, iupdwn_tot, nupdwn_tot )
          !
       END IF
       !
@@ -125,15 +119,10 @@
 ! read from file and distribute data calculated in preceding iterations
 !
       USE kinds,          ONLY : DP
-      USE io_files,       ONLY : tmp_dir
       USE electrons_base, ONLY : nbnd, nbsp, nspin, nupdwn, iupdwn, keep_occ, nbspx
       USE gvecw,          ONLY : ngw
       USE ions_base,      ONLY : nsp, na, cdmi, taui
-#if defined (__OLDXML)
-      USE cp_restart,       ONLY: cp_readfile, cp_read_cell, cp_read_wfc
-#else
       USE cp_restart_new,   ONLY: cp_readfile, cp_read_cell, cp_read_wfc
-#endif
       USE cp_restart_new, ONLY : cp_read_zmat
       USE ensemble_dft,   ONLY : tens
       USE autopilot,      ONLY : event_step, event_index, max_event_step
@@ -160,13 +149,13 @@
       !
       REAL(DP) :: ht(3,3), htm(3,3), htvel(3,3), gvel(3,3)
       integer :: nk = 1, ispin, i, ib, ierr
-      REAL(DP) :: xk(3,1) = 0.0d0, wk(1) = 2.0d0
+      REAL(DP) :: xk(3,2), wk(2)
       REAL(DP), ALLOCATABLE :: occ_ ( : )
       REAL(DP) :: b1(3) , b2(3), b3(3)
 
 
       IF( flag == -1 ) THEN
-        CALL cp_read_cell( ndr, tmp_dir, .TRUE., ht, htm, htvel, gvel, xnhh0, xnhhm, vnhh )
+        CALL cp_read_cell( ndr, .TRUE., ht, htm, htvel, gvel, xnhh0, xnhhm, vnhh )
         h     = TRANSPOSE( ht )
         hold  = TRANSPOSE( htm )
         velh  = TRANSPOSE( htvel )
@@ -175,7 +164,7 @@
  
       IF ( flag == 0 ) THEN
         DO ispin = 1, nspin
-          CALL cp_read_wfc( ndr, tmp_dir, 1, 1, ispin, nspin, c2 = cm(:,:), tag = 'm' )
+          CALL cp_read_wfc( ndr, 1, 1, ispin, nspin, c2 = cm(:,:), tag = 'm' )
         END DO
         CALL c_bgrp_pack( cm )
         RETURN
@@ -241,21 +230,22 @@
 
 
 !------------------------------------------------------------------------------!
-   SUBROUTINE set_evtot_x( c0, ctot, lambda, descla, iupdwn_tot, nupdwn_tot )
+   SUBROUTINE set_evtot_x( c0, ctot, lambda, idesc, iupdwn_tot, nupdwn_tot )
 !------------------------------------------------------------------------------!
       USE kinds,             ONLY: DP
       USE electrons_base,    ONLY: nupdwn, nspin, iupdwn, nudx
       USE electrons_module,  ONLY: ei
-      USE cp_interfaces,     ONLY: crot, collect_lambda
-      USE descriptors,       ONLY: la_descriptor
+      USE cp_interfaces,     ONLY: crot
       !
       IMPLICIT NONE
       !
+      include 'laxlib.fh'
+      !
       COMPLEX(DP), INTENT(IN)  :: c0(:,:)
       COMPLEX(DP), INTENT(OUT) :: ctot(:,:)
       REAL(DP),    INTENT(IN)  :: lambda(:,:,:)
       INTEGER,     INTENT(IN)  :: iupdwn_tot(2), nupdwn_tot(2)
-      TYPE(la_descriptor), INTENT(IN) :: descla(:)
+      INTEGER, INTENT(IN) :: idesc(:,:)
       !
       REAL(DP),    ALLOCATABLE :: eitmp(:)
       REAL(DP),    ALLOCATABLE :: lambda_repl(:,:)
@@ -265,12 +255,12 @@
       !
       ctot = 0.0d0
       !
-      CALL collect_lambda( lambda_repl, lambda(:,:,1), descla(1) )
+      CALL collect_lambda( lambda_repl, lambda(:,:,1), idesc(:,1) )
       !
       CALL crot( ctot, c0, SIZE( c0, 1 ), nupdwn(1), iupdwn_tot(1), iupdwn(1), lambda_repl, nudx, eitmp )
       !
       IF( nspin == 2 ) THEN
-         CALL collect_lambda( lambda_repl, lambda(:,:,2), descla(2) )
+         CALL collect_lambda( lambda_repl, lambda(:,:,2), idesc(:,2) )
          CALL crot( ctot, c0, SIZE( c0, 1 ), nupdwn(2), iupdwn_tot(2), iupdwn(2), lambda_repl, nudx, eitmp )
       END IF
       !
