@@ -13,17 +13,18 @@ SUBROUTINE from_restart( )
                                      tsde, tzeroe, tzerop, nbeg, tranp, amprp,&
                                      thdyn, tzeroc, force_pairing, trhor, &
                                      ampre, trane, tpre, dt_old, tv0rd, &
-                                     trescalee
-   USE wavefunctions_module,  ONLY : c0_bgrp, cm_bgrp
+                                     trescalee, tcap
+   USE wavefunctions,  ONLY : c0_bgrp, cm_bgrp
    USE electrons_module,      ONLY : occn_info
    USE electrons_base,        ONLY : nspin, iupdwn, nupdwn, f, nbsp, nbsp_bgrp
    USE io_global,             ONLY : ionode, ionode_id, stdout
    USE cell_base,             ONLY : ainv, h, hold, deth, r_to_s, s_to_r, &
                                      velh, at, alat
-   USE ions_base,             ONLY : na, nsp, iforce, vel_srt, nat, randpos
+   USE ions_base,             ONLY : na, nsp, iforce, vel, nat, ityp, &
+                                     randpos, randvel, amass
    USE time_step,             ONLY : tps, delt
    USE ions_positions,        ONLY : taus, tau0, tausm, taum, vels, fion, fionm, set_velocities, velsm
-   USE ions_nose,             ONLY : xnhp0, xnhpm
+   USE ions_nose,             ONLY : xnhp0, xnhpm, tempw
    USE gvect,    ONLY : mill, eigts1, eigts2, eigts3 
    USE printout_base,         ONLY : printout_pos
    USE gvecw,                 ONLY : ngw
@@ -34,7 +35,7 @@ SUBROUTINE from_restart( )
                                      efield_berry_setup2, tefield2
    USE uspp,                  ONLY : okvan, vkb, nkb, nlcc_any
    USE cp_main_variables,     ONLY : ht0, htm, lambdap, lambda, lambdam, eigr, &
-                                     sfac, taub, irb, eigrb, edft, bec_bgrp, dbec, descla
+                                     sfac, taub, irb, eigrb, edft, bec_bgrp, dbec, idesc
    USE time_step,             ONLY : delt
    USE fft_base,              ONLY : dfftp, dffts
    USE matrix_inversion
@@ -59,7 +60,7 @@ SUBROUTINE from_restart( )
       ! ... Input positions read from input file and stored in tau0
       ! ... in readfile, only scaled positions are read
       !
-      CALL r_to_s( tau0, taus, na, nsp, ainv )
+      CALL r_to_s( tau0, taus, nat, ainv )
       !
    END IF
    !
@@ -72,7 +73,7 @@ SUBROUTINE from_restart( )
       ! ... to tausm=tau(t)-v*delta t so that the Verlet algorithm will 
       ! ... start with the correct velocity
       !
-      CALL r_to_s( vel_srt, vels, na, nsp, ainv )
+      CALL r_to_s( vel, vels, nat, ainv )
       tausm(:,:) =  taus(:,:) - vels(:,:)*delt
       velsm(:,:) =  vels(:,:)
       !
@@ -82,24 +83,33 @@ SUBROUTINE from_restart( )
       !
       ! ... Input positions are randomized
       !
-      CALL randpos( taus, na, nsp, tranp, amprp, ainv, iforce )
+      CALL randpos( taus, nat, ityp, tranp, amprp, ainv, iforce )
       !
    END IF
    !
    IF ( tzerop .AND. tfor ) THEN
       !
-      vel_srt(:,:) = 0.0_dp
+      vel(:,:) = 0.0_dp
       vels(:,:) = 0.0_dp
       CALL set_velocities( tausm, taus, vels, iforce, nat, delt )
       WRITE( stdout, '(" Ionic velocities set to zero")' )
       !
    END IF
    !
-   CALL s_to_r( taus,  tau0, na, nsp, h )
+   CALL s_to_r( taus,  tau0, nat, h )
    !
    !CALL s_to_r( tausm, taum, na, nsp, h )
    !BS: tausm to taum conversion should use hold in variable cell calculations...
-   CALL s_to_r( tausm, taum, na, nsp, hold )
+   CALL s_to_r( tausm, taum, nat, hold )
+
+   IF ( tfor .AND. tcap ) THEN
+        WRITE( stdout, '(" Randomizing ions velocities according to tempw (OLD VELOCITIES DISCARDED)")' )
+     CALL  randvel( tempw, tau0 , taum, nat, ityp, iforce, amass, delt )
+     CALL r_to_s( taum, tausm, nat, ainv )  
+     vels(:,:) = (taus(:,:)-tausm(:,:))/delt
+     velsm(:,:) = vels(:,:)
+      
+   END IF
    !
    IF ( tzeroc ) THEN
       !
@@ -179,7 +189,7 @@ SUBROUTINE from_restart( )
    !
    CALL calbec_bgrp( 1, nsp, eigr, c0_bgrp, bec_bgrp )
    !
-   IF ( tpre     ) CALL caldbec_bgrp( eigr, c0_bgrp, dbec, descla )
+   IF ( tpre     ) CALL caldbec_bgrp( eigr, c0_bgrp, dbec, idesc )
    !
    IF ( tefield  ) CALL efield_berry_setup( eigr, tau0 )
    IF ( tefield2 ) CALL efield_berry_setup2( eigr, tau0 )
