@@ -55,7 +55,7 @@ MODULE efcalc
   END SUBROUTINE clear_nbeg
   !
   !--------------------------------------------------------------------------
-  SUBROUTINE ef_force( fion, na, nsp, zv )
+  SUBROUTINE ef_force( fion, ityp, nat, zv )
     !--------------------------------------------------------------------------
     !
     ! ... Electric Feild for ions here
@@ -63,24 +63,16 @@ MODULE efcalc
     IMPLICIT NONE
     !
     REAL(DP) :: fion(:,:), zv(:)
-    INTEGER        :: na(:), nsp
-    INTEGER        :: is, ia, isa
+    INTEGER        :: ityp(:), nat
+    INTEGER        :: ia
     !
     IF ( wf_efield ) THEN
        !
-       isa = 0
-       !
-       DO is =1, nsp
+       DO ia = 1, nat
           !
-          DO ia = 1, na(is)
-             !
-             isa = isa + 1
-             !
-             fion(1,isa) = fion(1,isa) + efx * zv(is)
-             fion(2,isa) = fion(2,isa) + efy * zv(is)
-             fion(3,isa) = fion(3,isa) + efz * zv(is)
-             !
-          END DO
+          fion(1,ia) = fion(1,ia) + efx * zv(ityp(ia))
+          fion(2,ia) = fion(2,ia) + efy * zv(ityp(ia))
+          fion(3,ia) = fion(3,ia) + efz * zv(ityp(ia))
           !
        END DO
        !
@@ -385,7 +377,7 @@ MODULE wannier_subroutines
        !
        CALL write_rho_g( rhog )
        !
-       CALL stop_run( .TRUE. )
+       CALL stop_cp_run()
        !
     END IF
     !
@@ -436,7 +428,7 @@ MODULE wannier_subroutines
           CALL rhoofr (nfi,cm, irb, eigrb,bec,dbec,rhovan,rhor,drhor,rhog,drhog,rhos,enl,denl,ekin,dekin6,.false.,j)
        END DO
        !
-       CALL stop_run( .TRUE. )
+       CALL stop_cp_run()
        !
     END IF
     !
@@ -448,7 +440,7 @@ MODULE wannier_subroutines
        !
        CALL wf (calwf,cm,bec,eigr,eigrb,taub,irb,b1,b2,b3,utwf,what1,wfc,jwf,ibrav)
        !
-       CALL stop_run( .TRUE. )
+       CALL stop_cp_run( )
        !
     END IF
     !
@@ -457,7 +449,7 @@ MODULE wannier_subroutines
        jwf=iplot(1)
        CALL wf (calwf,cm,bec,eigr,eigrb,taub,irb,b1,b2,b3,utwf,what1,wfc,jwf,ibrav)
        !
-       CALL stop_run( .TRUE. )
+       CALL stop_cp_run( )
        !
     END IF
     !
@@ -618,14 +610,14 @@ MODULE wannier_subroutines
     USE wannier_module,        ONLY : wfx, wfy, wfz, ionx, iony, ionz, wfc
     USE electrons_base,        ONLY : nbsp, f
     USE cell_base,             ONLY : ainv, alat, at
-    USE ions_base,             ONLY : na, nsp, zv
+    USE ions_base,             ONLY : nsp, zv, ityp, nat
     USE io_global,             ONLY : ionode
     !
     IMPLICIT NONE
     !
     REAL(DP) :: enthal, tau0(:,:)
     REAL(DP) :: a1(3), a2(3), a3(3)
-    INTEGER        :: i, is, ia, isa
+    INTEGER        :: i, is, ia
     !
     a1(:) = at(:,1)/alat ; a2(:) = at(:,2)/alat ; a3(:) = at(:,3)/alat
     IF(wf_efield) THEN
@@ -649,18 +641,15 @@ MODULE wannier_subroutines
        iony=0.d0
        ionz=0.d0
        efe_ion=0.d0
-       isa = 0
-       DO is=1,nsp
-          DO ia=1,na(is)
-             isa = isa + 1
-             tt(1)=tau0(1,isa)
-             tt(2)=tau0(2,isa)
-             tt(3)=tau0(3,isa)
-             CALL pbc(tt,a1,a2,a3,ainv,tt)
-             ionx=ionx+zv(is)*tt(1)
-             iony=iony+zv(is)*tt(2)
-             ionz=ionz+zv(is)*tt(3)
-          END DO
+       DO ia=1,nat
+          is = ityp(ia)
+          tt(1)=tau0(1,ia)
+          tt(2)=tau0(2,ia)
+          tt(3)=tau0(3,ia)
+          CALL pbc(tt,a1,a2,a3,ainv,tt)
+          ionx=ionx+zv(is)*tt(1)
+          iony=iony+zv(is)*tt(2)
+          ionz=ionz+zv(is)*tt(3)
        END DO
        efe_ion=efe_ion+efx*ionx+efy*iony+efz*ionz
        IF( ionode ) THEN
@@ -678,7 +667,7 @@ MODULE wannier_subroutines
   !--------------------------------------------------------------------------
   SUBROUTINE wf_closing_options( nfi, c0, cm, bec, eigr, eigrb, taub,  &
                                  irb, ibrav, b1, b2, b3, taus, tausm, vels,   &
-                                 velsm, acc, lambda, lambdam, descla, xnhe0, xnhem,   &
+                                 velsm, acc, lambda, lambdam, idesc, xnhe0, xnhem,   &
                                  vnhe, xnhp0, xnhpm, vnhp, nhpcl,nhpdim,ekincm,&
                                  xnhh0, xnhhm, vnhh, velh, ecut, ecutw, delt, &
                                  celldm, fion, tps, mat_z, occ_f, rho )
@@ -691,9 +680,8 @@ MODULE wannier_subroutines
     USE gvecw,          ONLY : ngw
     USE control_flags,  ONLY : ndw
     USE cell_base,      ONLY : h, hold
-    USE uspp_param,     ONLY : nvb
+    USE uspp,           ONLY : nkbus
     USE cp_interfaces,  ONLY : writefile
-    USE descriptors,    ONLY : la_descriptor
     !
     IMPLICIT NONE
     !
@@ -709,7 +697,7 @@ MODULE wannier_subroutines
     REAL(DP)    :: taus(:,:), tausm(:,:), vels(:,:), velsm(:,:)
     REAL(DP)    :: acc(:)
     REAL(DP)    :: lambda(:,:,:), lambdam(:,:,:)
-    TYPE(la_descriptor), INTENT(IN) :: descla(:)
+    INTEGER, INTENT(IN) :: idesc(:,:)
     REAL(DP)    :: xnhe0, xnhem, vnhe, xnhp0(:), xnhpm(:), vnhp(:), ekincm
     INTEGER           :: nhpcl, nhpdim
     REAL(DP)    :: velh(:,:)
@@ -729,7 +717,7 @@ MODULE wannier_subroutines
        CALL wf( calwf, c0, bec, eigr, eigrb, taub, irb, &
                 b1, b2, b3, utwf, what1, wfc, jwf, ibrav )
        !
-       IF ( nvb == 0 ) THEN
+       IF ( nkbus <= 0 ) THEN
           !
           CALL wf( calwf, cm, bec, eigr, eigrb, taub, irb, &
                    b1, b2, b3, utwf, what1, wfc, jwf, ibrav )
@@ -741,12 +729,12 @@ MODULE wannier_subroutines
        END IF
        !
        CALL writefile( h, hold, nfi, c0, cm, taus, &
-                       tausm, vels, velsm,acc, lambda, lambdam, descla, xnhe0, xnhem, &
+                       tausm, vels, velsm,acc, lambda, lambdam, idesc, xnhe0, xnhem, &
                        vnhe, xnhp0, xnhpm, vnhp,nhpcl,nhpdim,ekincm, xnhh0, xnhhm,&
                        vnhh, velh, fion, tps, mat_z, occ_f, rho )
        !
        CALL stop_clock('wf_close_opt')
-       CALL stop_run( .TRUE. )
+       CALL stop_cp_run( )
        !
     END IF
     !
