@@ -6,14 +6,30 @@
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
+MODULE fft_param
+
+#if defined(__MPI)
+#if defined(__MPI_MODULE)
+  USE mpi
+#else
+  INCLUDE 'mpif.h'
+#endif
+#else
+  INTEGER, PARAMETER :: MPI_COMM_WORLD =  0
+  INTEGER, PARAMETER :: MPI_COMM_NULL  = -1
+  INTEGER, PARAMETER :: MPI_COMM_SELF  = -2
+#endif
   
-  INTEGER, PARAMETER :: nfftx = 2049
+  INTEGER, PARAMETER :: ndims = 10
   !! Number of different FFT tables that the module
   !!could keep into memory without reinitialization
 
-  INTEGER, PARAMETER :: ndims = 10
+  INTEGER, PARAMETER :: nfftx = 2049
   !!Max allowed fft dimension
 
   INTEGER, PARAMETER :: DP = selected_real_kind(14,200)
-
   INTEGER, PARAMETER :: stdout = 6    ! unit connected to standard output
+
+  REAL(DP), PARAMETER :: eps8  = 1.0E-8_DP
+
+END MODULE fft_param
