@@ -5,34 +5,27 @@
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
-!
-!--------------------------------------------------------------------------!
-! FFT scalar drivers Module - contains machine-dependent routines for      !
-! FFTW, FFTW3, ESSL (both 3d for serial execution and 1d+2d FFTs for       !
-! parallel execution; NEC ASL libraries (3d only, no parallel execution)   !
-! Written by Carlo Cavazzoni, modified by P. Giannozzi, contributions      !
-! by Martin Hilgemans, Guido Roma, Pascal Thibaudeau, Stephane Lefranc,    !
-! Nicolas Lacorne, Filippo Spiga, Nicola Varini - Last update Jul 2015     !
-!--------------------------------------------------------------------------!
 
 #if defined(__ARM_LIB)
 
+#if defined(_OPENMP) && defined(__FFT_SCALAR_THREAD_SAFE)
+! thread safety guard
+#error ARM_LIB is not compatiable with __FFT_SCALAR_THREAD_SAFE
+#endif
+
 !=----------------------------------------------------------------------=!
-   MODULE fft_scalar
+    MODULE fft_scalar_arm
 !=----------------------------------------------------------------------=!
 
-       USE, intrinsic ::  iso_c_binding
+       USE iso_c_binding
+       USE fft_param
+       USE fftw_interfaces
 
        IMPLICIT NONE
        SAVE
-
        PRIVATE
        PUBLIC :: cft_1z, cft_2xy, cfft3d, cfft3ds
 
-! ...   Local Parameter
-
-#include "fft_param.f90"
-
 !=----------------------------------------------------------------------=!
    CONTAINS
 !=----------------------------------------------------------------------=!
@@ -73,7 +66,7 @@
 
      INTEGER :: tid
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
      INTEGER :: offset, ldz_t
      INTEGER :: omp_get_max_threads
      EXTERNAL :: omp_get_max_threads
@@ -192,7 +185,7 @@ END IF
      LOGICAL :: dofft( nfftx ), found
      INTEGER, PARAMETER  :: stdout = 6
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
      INTEGER :: offset
      INTEGER :: nx_t, ny_t, nzl_t, ldx_t, ldy_t
      INTEGER  :: itid, mytid, ntids
@@ -240,7 +233,7 @@ END IF
 #endif
 
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
 
    IF( isign < 0 ) THEN
       tscale = 1.0_DP / ( nx*ny )
@@ -309,7 +302,7 @@ END IF
      SUBROUTINE init_plan()
          tscale = 1.0_DP / ( nx * ny )
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
          CALL ZFFT1MX(0, 1.0_DP, .TRUE., nx, ny, r(1), ldx, 1, r(1), ldx, 1,fw_tabley(1, icurrent), INFO)
          CALL ZFFT1MX(0, 1.0_DP, .TRUE., nx, ny, r(1), ldx, 1, r(1), ldx, 1, bw_tabley(1, icurrent), INFO)
          CALL ZFFT1MX(0, tscale, .TRUE., ny, nx, r(1), 1, ldx, r(1), 1, ldx, fw_tablex(1, icurrent), INFO)
@@ -372,8 +365,8 @@ END IF
      COMPLEX (DP), SAVE :: bw_table(ltabl,ndims)
 #else
      ! FFTW
-     C_POINTER, save :: fw_plan(ndims) = 0
-     C_POINTER, save :: bw_plan(ndims) = 0
+     TYPE(C_PTR), save :: fw_plan(ndims) = C_NULL_PTR
+     TYPE(C_PTR), save :: bw_plan(ndims) = C_NULL_PTR
 #endif
 
 
@@ -450,8 +443,8 @@ END IF
          CALL ZFFT3DY (0,1.0_DP,.TRUE.,  nx,ny,nz,f(1),1,ldx,ldx*ldy,f(1),1,ldx, ldx*ldy, bw_table(1, icurrent),ltabl,INFO)
 #else
          ! FFTW
-         IF( fw_plan(icurrent) /= 0 ) CALL DESTROY_PLAN_3D( fw_plan(icurrent) )
-         IF( bw_plan(icurrent) /= 0 ) CALL DESTROY_PLAN_3D( bw_plan(icurrent) )
+         IF( C_ASSOCIATED(fw_plan(icurrent)) ) CALL DESTROY_PLAN_3D( fw_plan(icurrent) )
+         IF( C_ASSOCIATED(bw_plan(icurrent)) ) CALL DESTROY_PLAN_3D( bw_plan(icurrent) )
          idir = -1; CALL CREATE_PLAN_3D( fw_plan(icurrent), nx, ny, nz, idir)
          idir =  1; CALL CREATE_PLAN_3D( bw_plan(icurrent), nx, ny, nz, idir)
 #endif
@@ -531,7 +524,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
      END IF
 
 
-     IF ( isign > 0 ) THEN (G -> R)
+     IF ( isign > 0 ) THEN 
 
         !
         !  k-direction ...
@@ -645,9 +638,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
      END SUBROUTINE init_plan
 
    END SUBROUTINE cfft3ds
-
 !=----------------------------------------------------------------------=!
-   END MODULE fft_scalar
+ END MODULE fft_scalar_arm
 !=----------------------------------------------------------------------=!
-
 #endif
