@@ -6,32 +6,27 @@
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
-!--------------------------------------------------------------------------!
-! FFT scalar drivers Module - contains machine-dependent routines for      !
-! FFTW, FFTW3, ESSL (both 3d for serial execution and 1d+2d FFTs for       !
-! parallel execution; NEC ASL libraries (3d only, no parallel execution)   !
-! Written by Carlo Cavazzoni, modified by P. Giannozzi, contributions      !
-! by Martin Hilgemans, Guido Roma, Pascal Thibaudeau, Stephane Lefranc,    !
-! Nicolas Lacorne, Filippo Spiga, Nicola Varini - Last update Jul 2015     !
-!--------------------------------------------------------------------------!
 
 #if defined(__LINUX_ESSL)
 
+#if defined(_OPENMP) && defined(__FFT_SCALAR_THREAD_SAFE)
+! thread safety guard
+#error ESSL is not compatiable with __FFT_SCALAR_THREAD_SAFE
+#endif
+
 !=----------------------------------------------------------------------=!
-   MODULE fft_scalar
+   MODULE fft_scalar_essl
 !=----------------------------------------------------------------------=!
 
-       USE, intrinsic ::  iso_c_binding
+       USE fft_param
        
        IMPLICIT NONE
-        SAVE
-
-        PRIVATE
-        PUBLIC :: cft_1z, cft_2xy, cfft3d, cfft3ds
+       SAVE
+       PRIVATE
+       PUBLIC :: cft_1z, cft_2xy, cfft3d, cfft3ds
 
 ! ...   Local Parameter
 
-#include "fft_param.f90"
 
         !   Workspace that is statically allocated is defined here
         !   in order to avoid multiple copies of the same workspace
@@ -88,7 +83,7 @@
      !   ltabl   Dimension of the tables of factors calculated at the
      !           initialization stage
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
      INTEGER :: offset, ldz_t
      INTEGER :: omp_get_max_threads
      EXTERNAL :: omp_get_max_threads
@@ -213,7 +208,7 @@
      LOGICAL :: dofft( nfftx ), done
      INTEGER, PARAMETER  :: stdout = 6
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
      INTEGER :: offset
      INTEGER :: nx_t, ny_t, nzl_t, ldx_t, ldy_t
      INTEGER  :: itid, mytid, ntids
@@ -259,7 +254,7 @@
 
        ! WRITE( stdout, fmt="('DEBUG cft_2xy, reinitializing tables ', I3)" ) icurrent
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
 
        tscale = 1.0_DP / ( nx * ny )
        CALL DCFT ( 1, r(1), ldx, 1, r(1), ldx, 1, ny, nx,  1, 1.0_DP, &
@@ -301,7 +296,7 @@
 #endif
 
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
 
    IF( isign < 0 ) THEN
       tscale = 1.0_DP / ( nx * ny )
@@ -649,9 +644,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
      END IF
      RETURN
    END SUBROUTINE cfft3ds
-
 !=----------------------------------------------------------------------=!
-   END MODULE fft_scalar
+ END MODULE fft_scalar_essl
 !=----------------------------------------------------------------------=!
-
 #endif
