@@ -5,34 +5,27 @@
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
-!
-!--------------------------------------------------------------------------!
-! FFT scalar drivers Module - contains machine-dependent routines for      !
-! FFTW, FFTW3, ESSL (both 3d for serial execution and 1d+2d FFTs for       !
-! parallel execution; NEC ASL libraries (3d only, no parallel execution)   !
-! Written by Carlo Cavazzoni, modified by P. Giannozzi, contributions      !
-! by Martin Hilgemans, Guido Roma, Pascal Thibaudeau, Stephane Lefranc,    !
-! Nicolas Lacorne, Filippo Spiga, Nicola Varini - Last update Jul 2015     !
-!--------------------------------------------------------------------------!
 
 #if defined(__FFTW)
 
+#if defined(_OPENMP) && defined(__FFT_SCALAR_THREAD_SAFE)
+! thread safety guard
+#error FFTW is not compatiable with __FFT_SCALAR_THREAD_SAFE
+#endif
+
 !=----------------------------------------------------------------------=!
-   MODULE fft_scalar
+   MODULE fft_scalar_FFTW
 !=----------------------------------------------------------------------=!
+       USE fft_param
+!! iso_c_binding provides C_PTR, C_NULL_PTR, C_ASSOCIATED
+       USE iso_c_binding
+       USE fftw_interfaces
 
-       USE, intrinsic ::  iso_c_binding
-       
        IMPLICIT NONE
        SAVE
-
        PRIVATE
        PUBLIC :: cft_1z, cft_2xy, cfft3d, cfft3ds
 
-! ...   Local Parameter
-
-#include "fft_param.f90"
-
 !=----------------------------------------------------------------------=!
    CONTAINS
 !=----------------------------------------------------------------------=!
@@ -66,26 +59,19 @@
      COMPLEX (DP) :: c(:), cout(:)
 
      REAL (DP)  :: tscale
-     INTEGER    :: i, err, idir, ip, void
+     INTEGER    :: i, err, idir, ip
      INTEGER, SAVE :: zdims( 3, ndims ) = -1
      INTEGER, SAVE :: icurrent = 1
      LOGICAL :: found
 
-     INTEGER :: tid
-
-#if defined(__OPENMP)
+#if defined(_OPENMP)
      INTEGER :: offset, ldz_t
-     INTEGER :: omp_get_max_threads
-     EXTERNAL :: omp_get_max_threads
 #endif
 
      !   Pointers to the "C" structures containing FFT factors ( PLAN )
-     !   C_POINTER is defined in include/fft_defs.h
-     !   for 32bit executables, C_POINTER is integer(4)
-     !   for 64bit executables, C_POINTER is integer(8)
 
-     C_POINTER, SAVE :: fw_planz( ndims ) = 0
-     C_POINTER, SAVE :: bw_planz( ndims ) = 0
+     TYPE(C_PTR), SAVE :: fw_planz( ndims ) = C_NULL_PTR
+     TYPE(C_PTR), SAVE :: bw_planz( ndims ) = C_NULL_PTR
 
      IF( nsl < 0 ) THEN
        CALL fftx_error__(" fft_scalar: cft_1z ", " nsl out of range ", nsl)
@@ -113,13 +99,12 @@
      CALL start_clock( 'cft_1z' )
 #endif
 
-
-#if defined(__OPENMP)
+#if defined(_OPENMP)
 
      ldz_t = ldz
      !
      IF (isign < 0) THEN
-!$omp parallel default(none) private(tid,offset,i,tscale) shared(c,isign,nsl,fw_planz,ip,nz,cout,ldz) &
+!$omp parallel default(none) private(offset,i,tscale) shared(c,isign,nsl,fw_planz,ip,nz,cout,ldz) &
 !$omp &        firstprivate(ldz_t)
 !$omp do
        DO i=1, nsl
@@ -131,7 +116,7 @@
        tscale = 1.0_DP / nz
        cout( 1 : ldz * nsl ) = c( 1 : ldz * nsl ) * tscale
      ELSE IF (isign > 0) THEN
-!$omp parallel default(none) private(tid,offset,i) shared(c,isign,nsl,bw_planz,ip,cout,ldz) &
+!$omp parallel default(none) private(offset,i) shared(c,isign,nsl,bw_planz,ip,cout,ldz) &
 !$omp &        firstprivate(ldz_t)
 !$omp do
        DO i=1, nsl
@@ -145,7 +130,6 @@
 !$omp end parallel
      END IF
 
-
 #else
 
      IF (isign < 0) THEN
@@ -177,8 +161,8 @@
      END SUBROUTINE lookup
 
      SUBROUTINE init_plan()
-       IF( fw_planz( icurrent) /= 0 ) CALL DESTROY_PLAN_1D( fw_planz( icurrent) )
-       IF( bw_planz( icurrent) /= 0 ) CALL DESTROY_PLAN_1D( bw_planz( icurrent) )
+       IF( C_ASSOCIATED(fw_planz( icurrent)) ) CALL DESTROY_PLAN_1D( fw_planz( icurrent) )
+       IF( C_ASSOCIATED(bw_planz( icurrent)) ) CALL DESTROY_PLAN_1D( bw_planz( icurrent) )
        idir = -1; CALL CREATE_PLAN_1D( fw_planz( icurrent), nz, idir)
        idir =  1; CALL CREATE_PLAN_1D( bw_planz( icurrent), nz, idir)
        zdims(1,icurrent) = nz; zdims(2,icurrent) = nsl; zdims(3,icurrent) = ldz;
@@ -219,31 +203,28 @@
      INTEGER, INTENT(IN) :: isign, ldx, ldy, nx, ny, nzl
      INTEGER, OPTIONAL, INTENT(IN) :: pl2ix(:)
      COMPLEX (DP) :: r( : )
-     INTEGER :: i, k, j, err, idir, ip, kk, void
+     INTEGER :: i, k, j, err, idir, ip, kk
      REAL(DP) :: tscale
      INTEGER, SAVE :: icurrent = 1
      INTEGER, SAVE :: dims( 4, ndims) = -1
      LOGICAL :: dofft( nfftx ), found
-     INTEGER, PARAMETER  :: stdout = 6
 
-#if defined(__OPENMP)
+#if defined(_OPENMP)
      INTEGER :: offset
      INTEGER :: nx_t, ny_t, nzl_t, ldx_t, ldy_t
      INTEGER  :: itid, mytid, ntids
-     INTEGER  :: omp_get_thread_num, omp_get_num_threads,omp_get_max_threads
-     EXTERNAL :: omp_get_thread_num, omp_get_num_threads, omp_get_max_threads
+     INTEGER  :: omp_get_thread_num, omp_get_num_threads
+     EXTERNAL :: omp_get_thread_num, omp_get_num_threads
 #endif
 
-
 #if defined(__FFTW_ALL_XY_PLANES)
-     C_POINTER, SAVE :: fw_plan_2d( ndims ) = 0
-     C_POINTER, SAVE :: bw_plan_2d( ndims ) = 0
+     TYPE(C_PTR), SAVE :: fw_plan_2d( ndims ) = C_NULL_PTR
+     TYPE(C_PTR), SAVE :: bw_plan_2d( ndims ) = C_NULL_PTR
 #else
-     C_POINTER, SAVE :: fw_plan( 2, ndims ) = 0
-     C_POINTER, SAVE :: bw_plan( 2, ndims ) = 0
+     TYPE(C_PTR), SAVE :: fw_plan( 2, ndims ) = C_NULL_PTR
+     TYPE(C_PTR), SAVE :: bw_plan( 2, ndims ) = C_NULL_PTR
 #endif
 
-
      dofft( 1 : nx ) = .TRUE.
      IF( PRESENT( pl2ix ) ) THEN
        IF( SIZE( pl2ix ) < nx ) &
@@ -276,7 +257,6 @@
      CALL start_clock( 'cft_2xy' )
 #endif
 
-
 #if defined(__FFTW_ALL_XY_PLANES)
 
      IF( isign < 0 ) THEN
@@ -292,12 +272,12 @@
         !
      END IF
 
-#elif defined(__OPENMP)
+#elif defined(_OPENMP)
 
      nx_t  = nx
      ny_t  = ny
      nzl_t = nzl
-     ldx_t = ldx 
+     ldx_t = ldx
      ldy_t = ldy
      !
      IF( isign < 0 ) THEN
@@ -331,7 +311,7 @@
         end do
 
 !$omp barrier
- 
+
 !$omp workshare
         r = r * tscale
 !$omp end workshare
@@ -406,7 +386,6 @@
 
 #endif
 
-
 #if defined(__FFT_CLOCKS)
      CALL stop_clock( 'cft_2xy' )
 #endif
@@ -427,18 +406,18 @@
 
      SUBROUTINE init_plan()
 #if defined __FFTW_ALL_XY_PLANES
-       IF( fw_plan_2d( icurrent) /= 0 )  CALL DESTROY_PLAN_2D(fw_plan_2d(icurrent) )
-       IF( bw_plan_2d( icurrent) /= 0 )  CALL DESTROY_PLAN_2D(bw_plan_2d(icurrent) )
+       IF( C_ASSOCIATED(fw_plan_2d( icurrent)) )  CALL DESTROY_PLAN_2D(fw_plan_2d(icurrent) )
+       IF( C_ASSOCIATED(bw_plan_2d( icurrent)) )  CALL DESTROY_PLAN_2D(bw_plan_2d(icurrent) )
        idir = -1; CALL CREATE_PLAN_2D( fw_plan_2d(icurrent), nx, ny, idir)
        idir =  1; CALL CREATE_PLAN_2D( bw_plan_2d(icurrent), nx, ny, idir)
 #else
-       IF( fw_plan( 2,icurrent) /= 0 )   CALL DESTROY_PLAN_1D( fw_plan( 2,icurrent) )
-       IF( bw_plan( 2,icurrent) /= 0 )   CALL DESTROY_PLAN_1D( bw_plan( 2,icurrent) )
+       IF( C_ASSOCIATED(fw_plan( 2,icurrent)) )   CALL DESTROY_PLAN_1D( fw_plan( 2,icurrent) )
+       IF( C_ASSOCIATED(bw_plan( 2,icurrent)) )   CALL DESTROY_PLAN_1D( bw_plan( 2,icurrent) )
        idir = -1; CALL CREATE_PLAN_1D( fw_plan( 2,icurrent), ny, idir)
        idir =  1; CALL CREATE_PLAN_1D( bw_plan( 2,icurrent), ny, idir)
 
-       IF( fw_plan( 1,icurrent) /= 0 ) CALL DESTROY_PLAN_1D( fw_plan( 1,icurrent) )
-       IF( bw_plan( 1,icurrent) /= 0 ) CALL DESTROY_PLAN_1D( bw_plan( 1,icurrent) )
+       IF( C_ASSOCIATED(fw_plan( 1,icurrent)) ) CALL DESTROY_PLAN_1D( fw_plan( 1,icurrent) )
+       IF( C_ASSOCIATED(bw_plan( 1,icurrent)) ) CALL DESTROY_PLAN_1D( bw_plan( 1,icurrent) )
        idir = -1; CALL CREATE_PLAN_1D( fw_plan( 1,icurrent), nx, idir)
        idir =  1; CALL CREATE_PLAN_1D( bw_plan( 1,icurrent), nx, idir)
 #endif
@@ -450,7 +429,6 @@
 
    END SUBROUTINE cft_2xy
 
-
 !
 !=----------------------------------------------------------------------=!
 !
@@ -485,8 +463,8 @@
      INTEGER, SAVE :: icurrent = 1
      INTEGER, SAVE :: dims(3,ndims) = -1
 
-     C_POINTER, save :: fw_plan(ndims) = 0
-     C_POINTER, save :: bw_plan(ndims) = 0
+     TYPE(C_PTR), save :: fw_plan(ndims) = C_NULL_PTR
+     TYPE(C_PTR), save :: bw_plan(ndims) = C_NULL_PTR
 
      IF ( nx < 1 ) &
          call fftx_error__('cfft3d',' nx is less than 1 ', 1)
@@ -528,7 +506,7 @@
 
      RETURN
 
-   CONTAINS 
+   CONTAINS
 
      SUBROUTINE lookup()
      ip = -1
@@ -547,8 +525,8 @@
      SUBROUTINE init_plan()
        IF ( nx /= ldx .or. ny /= ldy .or. nz /= ldz ) &
          call fftx_error__('cfft3','not implemented',1)
-       IF( fw_plan(icurrent) /= 0 ) CALL DESTROY_PLAN_3D( fw_plan(icurrent) )
-       IF( bw_plan(icurrent) /= 0 ) CALL DESTROY_PLAN_3D( bw_plan(icurrent) )
+       IF( C_ASSOCIATED (fw_plan(icurrent)) ) CALL DESTROY_PLAN_3D( fw_plan(icurrent) )
+       IF( C_ASSOCIATED (bw_plan(icurrent)) ) CALL DESTROY_PLAN_3D( bw_plan(icurrent) )
        idir = -1; CALL CREATE_PLAN_3D( fw_plan(icurrent), nx, ny, nz, idir)
        idir =  1; CALL CREATE_PLAN_3D( bw_plan(icurrent), nx, ny, nz, idir)
        dims(1,icurrent) = nx; dims(2,icurrent) = ny; dims(3,icurrent) = nz
@@ -600,8 +578,8 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
   INTEGER, SAVE :: icurrent = 1
   INTEGER, SAVE :: dims(3,ndims) = -1
 
-  C_POINTER, SAVE :: fw_plan ( 3, ndims ) = 0
-  C_POINTER, SAVE :: bw_plan ( 3, ndims ) = 0
+  TYPE(C_PTR), SAVE :: fw_plan ( 3, ndims ) = C_NULL_PTR
+  TYPE(C_PTR), SAVE :: bw_plan ( 3, ndims ) = C_NULL_PTR
 
   tscale = 1.0_DP
   ldh = ldx * ldy * ldz
@@ -622,7 +600,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
 
      END IF
 
-
      IF ( isign > 0 ) THEN
 
         DO h = 0, howmany - 1
@@ -646,7 +623,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
            !
 
            incx1 = ldx;  incx2 = ldx*ldy;  m = nz
-   
+
            do i = 1, nx
               if ( do_fft_y( i ) == 1 ) then
                 call FFTW_INPLACE_DRV_1D( bw_plan( 2, ip), m, f( i + h*ldh ), incx1, incx2 )
@@ -691,7 +668,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
            !
 
            incx1 = ldx * ny;  incx2 = 1;  m = 1
- 
+
            do i = 1, nx
               do j = 1, ny
                  ii = i + ldx * (j -1)
@@ -723,12 +700,12 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
      END SUBROUTINE lookup
 
      SUBROUTINE init_plan()
-       IF( fw_plan( 1, icurrent) /= 0 ) CALL DESTROY_PLAN_1D( fw_plan( 1, icurrent) )
-       IF( bw_plan( 1, icurrent) /= 0 ) CALL DESTROY_PLAN_1D( bw_plan( 1, icurrent) )
-       IF( fw_plan( 2, icurrent) /= 0 ) CALL DESTROY_PLAN_1D( fw_plan( 2, icurrent) )
-       IF( bw_plan( 2, icurrent) /= 0 ) CALL DESTROY_PLAN_1D( bw_plan( 2, icurrent) )
-       IF( fw_plan( 3, icurrent) /= 0 ) CALL DESTROY_PLAN_1D( fw_plan( 3, icurrent) )
-       IF( bw_plan( 3, icurrent) /= 0 ) CALL DESTROY_PLAN_1D( bw_plan( 3, icurrent) )
+       IF( C_ASSOCIATED (fw_plan( 1, icurrent)) ) CALL DESTROY_PLAN_1D( fw_plan( 1, icurrent) )
+       IF( C_ASSOCIATED (bw_plan( 1, icurrent)) ) CALL DESTROY_PLAN_1D( bw_plan( 1, icurrent) )
+       IF( C_ASSOCIATED (fw_plan( 2, icurrent)) ) CALL DESTROY_PLAN_1D( fw_plan( 2, icurrent) )
+       IF( C_ASSOCIATED (bw_plan( 2, icurrent)) ) CALL DESTROY_PLAN_1D( bw_plan( 2, icurrent) )
+       IF( C_ASSOCIATED (fw_plan( 3, icurrent)) ) CALL DESTROY_PLAN_1D( fw_plan( 3, icurrent) )
+       IF( C_ASSOCIATED (bw_plan( 3, icurrent)) ) CALL DESTROY_PLAN_1D( bw_plan( 3, icurrent) )
        idir = -1; CALL CREATE_PLAN_1D( fw_plan( 1, icurrent), nx, idir)
        idir =  1; CALL CREATE_PLAN_1D( bw_plan( 1, icurrent), nx, idir)
        idir = -1; CALL CREATE_PLAN_1D( fw_plan( 2, icurrent), ny, idir)
@@ -742,9 +719,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
      END SUBROUTINE init_plan
 
    END SUBROUTINE cfft3ds
-
 !=----------------------------------------------------------------------=!
-   END MODULE fft_scalar
+ END MODULE fft_scalar_FFTW
 !=----------------------------------------------------------------------=!
-
 #endif
