@@ -13,28 +13,30 @@
 ! Written by Carlo Cavazzoni, modified by P. Giannozzi, contributions      !
 ! by Martin Hilgemans, Guido Roma, Pascal Thibaudeau, Stephane Lefranc,    !
 ! Nicolas Lacorne, Filippo Spiga, Nicola Varini - Last update Jul 2015     !
+! stick and plane revision - Stefano de Gironcoli - September 2016
 !--------------------------------------------------------------------------!
 
-
 #if defined(__FFTW3)
 
-!=----------------------------------------------------------------------=!
-   MODULE fft_scalar
-!=----------------------------------------------------------------------=!
+#if defined(_OPENMP) && defined(__FFT_SCALAR_THREAD_SAFE)
+! thread safety guard
+#error FFTW3 is not compatible with __FFT_SCALAR_THREAD_SAFE
+#endif
 
-       USE, intrinsic ::  iso_c_binding
-       
+!=----------------------------------------------------------------------=!
+   MODULE fft_scalar_fftw3
+!=----------------------------------------------------------------------=!
+!! iso_c_binding provides C_PTR, C_NULL_PTR, C_ASSOCIATED
+       USE, intrinsic :: iso_c_binding
+       USE fft_param
        IMPLICIT NONE
        SAVE
-
        PRIVATE
        PUBLIC :: cft_1z, cft_2xy, cfft3d, cfft3ds
 
 ! ...   Local Parameter
 
-#include "fft_param.f90"
-
-#if defined(__OPENMP)
+#if defined(_OPENMP)
 #include "fftw3.f03"
 #else
 #include "fftw3.f"
@@ -73,25 +75,15 @@
      COMPLEX (DP) :: c(:), cout(:)
 
      REAL (DP)  :: tscale
-     INTEGER    :: i, err, idir, ip, void
+     INTEGER    :: i, err, idir, ip
      INTEGER, SAVE :: zdims( 3, ndims ) = -1
      INTEGER, SAVE :: icurrent = 1
      LOGICAL :: done
-     INTEGER :: tid
-
-#if defined(__OPENMP)
-     INTEGER :: offset, ldz_t
-     INTEGER :: omp_get_max_threads
-     EXTERNAL :: omp_get_max_threads
-#endif
-
+ 
      !   Pointers to the "C" structures containing FFT factors ( PLAN )
-     !   C_POINTER is defined in include/fft_defs.h
-     !   for 32bit executables, C_POINTER is integer(4)
-     !   for 64bit executables, C_POINTER is integer(8)
 
-     C_POINTER, SAVE :: fw_planz( ndims ) = 0
-     C_POINTER, SAVE :: bw_planz( ndims ) = 0
+     TYPE(C_PTR), SAVE :: fw_planz( ndims ) = C_NULL_PTR
+     TYPE(C_PTR), SAVE :: bw_planz( ndims ) = C_NULL_PTR
 
      IF( nsl < 0 ) THEN
        CALL fftx_error__(" fft_scalar: cft_1z ", " nsl out of range ", nsl)
@@ -100,14 +92,14 @@
      !
      !   Here initialize table only if necessary
      !
-     
+
      CALL lookup()
 
      IF( .NOT. done ) THEN
 
        !   no table exist for these parameters
        !   initialize a new one
-      
+
        CALL init_plan()
 
      END IF
@@ -137,7 +129,7 @@
    CONTAINS
 
      SUBROUTINE lookup()
-        ! lookup for stored plan 
+        ! lookup for stored plan
         DO ip = 1, ndims
            !   first check if there is already a table initialized
            !   for this combination of parameters
@@ -149,14 +141,17 @@
      END SUBROUTINE lookup
 
      SUBROUTINE init_plan()
-#if defined(__OPENMP)
-       CALL dfftw_cleanup_threads() 
+#if defined(_OPENMP)
+       INTEGER :: void
+       INTEGER, EXTERNAL :: omp_get_max_threads
+       !
+       CALL dfftw_cleanup_threads()
        void = fftw_init_threads()
-       CALL dfftw_plan_with_nthreads(omp_get_max_threads())      
+       CALL dfftw_plan_with_nthreads(omp_get_max_threads())
 #endif
 
-       IF( fw_planz( icurrent) /= 0 ) CALL dfftw_destroy_plan( fw_planz( icurrent) )
-       IF( bw_planz( icurrent) /= 0 ) CALL dfftw_destroy_plan( bw_planz( icurrent) )
+       IF( C_ASSOCIATED(fw_planz( icurrent)) ) CALL dfftw_destroy_plan( fw_planz( icurrent) )
+       IF( C_ASSOCIATED(bw_planz( icurrent)) ) CALL dfftw_destroy_plan( bw_planz( icurrent) )
        idir = -1
        CALL dfftw_plan_many_dft( fw_planz( icurrent), 1, nz, nsl, c, &
             (/SIZE(c)/), 1, ldz, cout, (/SIZE(cout)/), 1, ldz, idir, FFTW_ESTIMATE)
@@ -171,7 +166,6 @@
 
    END SUBROUTINE cft_1z
 
-!
 !
 !=----------------------------------------------------------------------=!
 !
@@ -183,8 +177,6 @@
 !
 !=----------------------------------------------------------------------=!
 !
-!
-
    SUBROUTINE cft_2xy(r, nzl, nx, ny, ldx, ldy, isign, pl2ix)
 
 !     driver routine for nzl 2d complex fft's of lengths nx and ny
@@ -202,23 +194,14 @@
      INTEGER, INTENT(IN) :: isign, ldx, ldy, nx, ny, nzl
      INTEGER, OPTIONAL, INTENT(IN) :: pl2ix(:)
      COMPLEX (DP) :: r( : )
-     INTEGER :: i, k, j, err, idir, ip, kk, void
+     INTEGER :: i, k, j, err, idir, ip, kk
      REAL(DP) :: tscale
      INTEGER, SAVE :: icurrent = 1
      INTEGER, SAVE :: dims( 4, ndims) = -1
      LOGICAL :: dofft( nfftx ), done
-     INTEGER, PARAMETER  :: stdout = 6
 
-#if defined(__OPENMP)
-     INTEGER :: offset
-     INTEGER :: nx_t, ny_t, nzl_t, ldx_t, ldy_t
-     INTEGER  :: itid, mytid, ntids
-     INTEGER  :: omp_get_thread_num, omp_get_num_threads,omp_get_max_threads
-     EXTERNAL :: omp_get_thread_num, omp_get_num_threads, omp_get_max_threads
-#endif
-
-     C_POINTER, SAVE :: fw_plan( 2, ndims ) = 0
-     C_POINTER, SAVE :: bw_plan( 2, ndims ) = 0
+     TYPE(C_PTR), SAVE :: fw_plan( 2, ndims ) = C_NULL_PTR
+     TYPE(C_PTR), SAVE :: bw_plan( 2, ndims ) = C_NULL_PTR
 
      dofft( 1 : nx ) = .TRUE.
      IF( PRESENT( pl2ix ) ) THEN
@@ -232,7 +215,7 @@
      !
      !   Here initialize table only if necessary
      !
- 
+
      CALL lookup()
 
      IF( .NOT. done ) THEN
@@ -311,16 +294,18 @@
      END SUBROUTINE lookup
 
      SUBROUTINE init_plan()
-
-#if defined(__OPENMP)
-       CALL dfftw_cleanup_threads() 
+#if defined(_OPENMP)
+       INTEGER :: void
+       INTEGER, EXTERNAL :: omp_get_max_threads
+       !
+       CALL dfftw_cleanup_threads()
        void = fftw_init_threads()
-       CALL dfftw_plan_with_nthreads(omp_get_max_threads())      
+       CALL dfftw_plan_with_nthreads(omp_get_max_threads())
 #endif
 
        IF ( ldx /= nx .OR. ldy /= ny ) THEN
-          IF( fw_plan(2,icurrent) /= 0 )  CALL dfftw_destroy_plan( fw_plan(2,icurrent) )
-          IF( bw_plan(2,icurrent) /= 0 )  CALL dfftw_destroy_plan( bw_plan(2,icurrent) )
+          IF( C_ASSOCIATED(fw_plan(2,icurrent)) )  CALL dfftw_destroy_plan( fw_plan(2,icurrent) )
+          IF( C_ASSOCIATED(bw_plan(2,icurrent)) )  CALL dfftw_destroy_plan( bw_plan(2,icurrent) )
           idir = -1
           CALL dfftw_plan_many_dft( fw_plan(2,icurrent), 1, ny, 1, r(1:), &
                (/ldx*ldy/), ldx, 1, r(1:), (/ldx*ldy/), ldx, 1, idir, &
@@ -330,8 +315,8 @@
                (/ldx*ldy/), ldx, 1, r(1:), (/ldx*ldy/), ldx, 1, idir, &
                FFTW_ESTIMATE)
 
-          IF( fw_plan(1,icurrent) /= 0 ) CALL dfftw_destroy_plan( fw_plan(1,icurrent) )
-          IF( bw_plan(1,icurrent) /= 0 ) CALL dfftw_destroy_plan( bw_plan(1,icurrent) )
+          IF( C_ASSOCIATED(fw_plan(1,icurrent)) ) CALL dfftw_destroy_plan( fw_plan(1,icurrent) )
+          IF( C_ASSOCIATED(bw_plan(1,icurrent)) ) CALL dfftw_destroy_plan( bw_plan(1,icurrent) )
           idir = -1
           CALL dfftw_plan_many_dft( fw_plan(1,icurrent), 1, nx, ny, r(1:), &
                (/ldx*ldy/), 1, ldx, r(1:), (/ldx*ldy/), 1, ldx, idir, &
@@ -341,8 +326,8 @@
                (/ldx*ldy/), 1, ldx, r(1:), (/ldx*ldy/), 1, ldx, idir, &
                FFTW_ESTIMATE)
        ELSE
-          IF( fw_plan( 1, icurrent) /= 0 ) CALL dfftw_destroy_plan( fw_plan( 1, icurrent) )
-          IF( bw_plan( 1, icurrent) /= 0 ) CALL dfftw_destroy_plan( bw_plan( 1, icurrent) )
+          IF( C_ASSOCIATED(fw_plan( 1, icurrent)) ) CALL dfftw_destroy_plan( fw_plan( 1, icurrent) )
+          IF( C_ASSOCIATED(bw_plan( 1, icurrent)) ) CALL dfftw_destroy_plan( bw_plan( 1, icurrent) )
           idir = -1
           CALL dfftw_plan_many_dft( fw_plan( 1, icurrent), 2, (/nx, ny/), nzl,&
                r(1:), (/nx, ny/), 1, nx*ny, r(1:), (/nx, ny/), 1, nx*ny, idir,&
@@ -361,7 +346,6 @@
 
    END SUBROUTINE cft_2xy
 
-
 !
 !=----------------------------------------------------------------------=!
 !
@@ -391,13 +375,13 @@
 
      INTEGER, INTENT(IN) :: nx, ny, nz, ldx, ldy, ldz, howmany, isign
      COMPLEX (DP) :: f(:)
-     INTEGER :: i, k, j, err, idir, ip
+     INTEGER :: i, k, j, err, idir, ip, void
      REAL(DP) :: tscale
      INTEGER, SAVE :: icurrent = 1
      INTEGER, SAVE :: dims(3,ndims) = -1
 
-     C_POINTER, save :: fw_plan(ndims) = 0
-     C_POINTER, save :: bw_plan(ndims) = 0
+     TYPE(C_PTR), save :: fw_plan(ndims) = C_NULL_PTR
+     TYPE(C_PTR), save :: bw_plan(ndims) = C_NULL_PTR
 
      IF ( nx < 1 ) &
          call fftx_error__('cfft3d',' nx is less than 1 ', 1)
@@ -456,10 +440,18 @@
      END SUBROUTINE lookup
 
      SUBROUTINE init_plan()
+#if defined(_OPENMP)
+       INTEGER :: void
+       INTEGER, EXTERNAL :: omp_get_max_threads
+       !
+       CALL dfftw_cleanup_threads()
+       void = fftw_init_threads()
+       CALL dfftw_plan_with_nthreads(omp_get_max_threads())
+#endif
        IF ( nx /= ldx .or. ny /= ldy .or. nz /= ldz ) &
             call fftx_error__('cfft3','not implemented',3)
-       IF( fw_plan(icurrent) /= 0 ) CALL dfftw_destroy_plan( fw_plan(icurrent) )
-       IF( bw_plan(icurrent) /= 0 ) CALL dfftw_destroy_plan( bw_plan(icurrent) )
+       IF( C_ASSOCIATED(fw_plan(icurrent)) ) CALL dfftw_destroy_plan( fw_plan(icurrent) )
+       IF( C_ASSOCIATED(bw_plan(icurrent)) ) CALL dfftw_destroy_plan( bw_plan(icurrent) )
        idir = -1
        CALL dfftw_plan_dft_3d ( fw_plan(icurrent), nx, ny, nz, f(1:), &
             f(1:), idir, FFTW_ESTIMATE)
@@ -500,7 +492,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
   !----------------------------------------------------------------------
   !
   implicit none
-     INTEGER, PARAMETER  :: stdout = 6
 
   integer :: nx, ny, nz, ldx, ldy, ldz, howmany, isign
   !
@@ -517,9 +508,8 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
   INTEGER, SAVE :: icurrent = 1
   INTEGER, SAVE :: dims(3,ndims) = -1
 
-
-  C_POINTER, SAVE :: fw_plan ( 3, ndims ) = 0
-  C_POINTER, SAVE :: bw_plan ( 3, ndims ) = 0
+  TYPE(C_PTR), SAVE :: fw_plan ( 3, ndims ) = C_NULL_PTR
+  TYPE(C_PTR), SAVE :: bw_plan ( 3, ndims ) = C_NULL_PTR
 
   tscale = 1.0_DP
 
@@ -539,7 +529,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
 
      END IF
 
-
      IF ( isign > 0 ) THEN
 
         !
@@ -604,7 +593,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
         !
 
         incx1 = ldx * ny;  incx2 = 1;  m = 1
- 
+
         do i = 1, nx
            do j = 1, ny
               ii = i + ldx * (j-1)
@@ -621,7 +610,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
 
    CONTAINS
 
-
      SUBROUTINE lookup()
      ip = -1
      DO i = 1, ndims
@@ -636,17 +624,25 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
      END SUBROUTINE lookup
 
      SUBROUTINE init_plan()
-       IF( fw_plan( 1, icurrent) /= 0 ) &
+#if defined(_OPENMP)
+       INTEGER :: void
+       INTEGER, EXTERNAL :: omp_get_max_threads
+       !
+       CALL dfftw_cleanup_threads()
+       void = fftw_init_threads()
+       CALL dfftw_plan_with_nthreads(omp_get_max_threads())
+#endif
+       IF( C_ASSOCIATED(fw_plan( 1, icurrent)) ) &
             CALL dfftw_destroy_plan( fw_plan( 1, icurrent) )
-       IF( bw_plan( 1, icurrent) /= 0 ) &
+       IF( C_ASSOCIATED(bw_plan( 1, icurrent)) ) &
             CALL dfftw_destroy_plan( bw_plan( 1, icurrent) )
-       IF( fw_plan( 2, icurrent) /= 0 ) &
+       IF( C_ASSOCIATED(fw_plan( 2, icurrent)) ) &
             CALL dfftw_destroy_plan( fw_plan( 2, icurrent) )
-       IF( bw_plan( 2, icurrent) /= 0 ) &
+       IF( C_ASSOCIATED(bw_plan( 2, icurrent)) ) &
             CALL dfftw_destroy_plan( bw_plan( 2, icurrent) )
-       IF( fw_plan( 3, icurrent) /= 0 ) &
+       IF( C_ASSOCIATED(fw_plan( 3, icurrent)) ) &
             CALL dfftw_destroy_plan( fw_plan( 3, icurrent) )
-       IF( bw_plan( 3, icurrent) /= 0 ) &
+       IF( C_ASSOCIATED(bw_plan( 3, icurrent)) ) &
             CALL dfftw_destroy_plan( bw_plan( 3, icurrent) )
        idir = -1
        CALL dfftw_plan_many_dft( fw_plan( 1, icurrent), &
@@ -679,9 +675,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, howmany, isign, &
      END SUBROUTINE init_plan
 
    END SUBROUTINE cfft3ds
-
 !=----------------------------------------------------------------------=!
-   END MODULE fft_scalar
+ END MODULE fft_scalar_fftw3
 !=----------------------------------------------------------------------=!
-
 #endif
