@@ -54,14 +54,14 @@ program gwl_punch
   !    
 
   USE kinds,     ONLY : i4b
-  USE gvect,  ONLY : mill
+  USE gvecs,     ONLY : doublegrid
+  USE gvect,     ONLY : mill
   use pwcom
-  USE constants,            ONLY : rytoev
+  USE constants, ONLY : rytoev
   use io_global, ONLY : stdout, ionode, ionode_id
   use io_files,  ONLY : psfile, pseudo_dir
   use io_files,  ONLY : prefix, tmp_dir
   use ions_base, ONLY : ntype => nsp
-  use iotk_module
   use mp_pools, ONLY : kunit
   use mp, ONLY: mp_bcast
   use mp_world, ONLY: world_comm, mpime
@@ -159,7 +159,6 @@ program gwl_punch
   integer :: i, kunittmp, ios
 
   character(len=200) :: pp_file
-  character(len=iotk_attlenx) :: attr
   logical :: found, uspp_spsi, ascii, single_file, raw
 !  INTEGER(i4b), EXTERNAL :: C_MKDIR
  CHARACTER(LEN=256), EXTERNAL :: trimcheck
@@ -432,8 +431,6 @@ program gwl_punch
 
 !
 ! init some quantities ...
-!
-  CALL hinit0()
 !
   if(lda_plus_u) then 
     CALL init_ns()
@@ -511,11 +508,9 @@ end program gwl_punch
 subroutine read_export (pp_file,kunit,uspp_spsi, ascii, single_file, raw)
   !-----------------------------------------------------------------------
   !
-  use iotk_module
-
-
   use kinds,          ONLY : DP 
   use pwcom  
+  USE gvect,          ONLY : ngm, ngm_g, mill, ig_l2g, g
   use gvecw,          ONLY : gcutw
   use control_flags,  ONLY : gamma_only  
   use becmod,         ONLY : bec_type, becp, calbec, &
@@ -525,11 +520,12 @@ subroutine read_export (pp_file,kunit,uspp_spsi, ascii, single_file, raw)
 !  use char,           ONLY : sname
 ! occhio sname is in symme which is now outside pwcom
   use  uspp,          ONLY : nkb, vkb
-  use wavefunctions_module,  ONLY : evc
-  use io_files,       ONLY : nd_nmbr, prefix, iunwfc, nwordwfc, iunsat, nwordatwfc
+  use wavefunctions,  ONLY : evc
+  use io_files,       ONLY : prefix, iunwfc, nwordwfc, iunsat, nwordatwfc
   use io_files,       ONLY : pseudo_dir, psfile
   use io_global,      ONLY : ionode, stdout
   USE ions_base,      ONLY : atm, nat, ityp, tau, nsp
+  use cell_base,      ONLY : bg
   use mp_pools,       ONLY : nproc_pool, my_pool_id, intra_pool_comm, &
                              inter_pool_comm
   use mp,             ONLY : mp_sum, mp_max
@@ -562,7 +558,6 @@ subroutine read_export (pp_file,kunit,uspp_spsi, ascii, single_file, raw)
 
   real(DP) :: wfc_scal 
   logical :: twf0, twfm
-  character(iotk_attlenx) :: attr
   complex(DP), allocatable :: sevc (:,:)
 
   write(stdout,*) "nkstot=", nkstot
@@ -740,9 +735,6 @@ subroutine read_export (pp_file,kunit,uspp_spsi, ascii, single_file, raw)
        ALLOCATE( sevc(npwx,nbnd), STAT=ierr )
        IF (ierr/=0) CALL errore( ' read_export ',' Unable to allocate SEVC ', ABS(ierr) )
 
-       CALL init_us_1
-       CALL init_at_1
-
        CALL allocate_bec_type (nkb,nbnd,becp)
 
        do ik = 1, nkstot
