@@ -10,7 +10,7 @@ subroutine gen_beta_simple (qk, npw_max, dvkb)
   USE kinds,      ONLY : DP
   USE constants,  ONLY : tpi
   USE ions_base,  ONLY : nat, ntyp => nsp, ityp, tau
-  USE cell_base,  ONLY : tpiba
+  USE cell_base,  ONLY : tpiba, omega
   USE klist,      ONLY : ngk
   USE gvect,      ONLY : mill, eigts1, eigts2, eigts3, g
   USE uspp,       ONLY : nkb, indv, nhtol, nhtolm
@@ -83,7 +83,7 @@ subroutine gen_beta_simple (qk, npw_max, dvkb)
   do nt = 1, ntyp
      do nb = 1, upf(nt)%nbeta
         if ( upf(nt)%is_gth ) then
-           call mk_dffnl_gth( nt, nb, npw_max, q, djl(1,nb,nt) )
+           call mk_dffnl_gth( nt, nb, npw_max, omega, tpiba, q, djl(1,nb,nt) )
            cycle
         endif
         do ig = 1, npw_max
@@ -334,7 +334,7 @@ subroutine commutator_Hx_psi_simple (qk, npw_max, nbnd_occ, becp1, becp2, ipol,
   USE io_global,       ONLY : stdout
   USE gvect,           ONLY : g
   USE wvfct,           ONLY : nbnd, et
-  USE wavefunctions_module, ONLY: evc
+  USE wavefunctions, ONLY: evc
   USE lsda_mod,        ONLY : nspin
   USE noncollin_module,ONLY : noncolin, npol
   USE becmod,          ONLY : becp, bec_type, calbec
