@@ -9,22 +9,9 @@
 
 MODULE dspev_module
 
-
     IMPLICIT NONE
-
-#include "la_param.f90"
-
     SAVE
 
-    PRIVATE
-
-    PUBLIC :: pdspev_drv, dspev_drv
-    PUBLIC :: diagonalize_parallel, diagonalize_serial
-
-#if defined __SCALAPACK
-    PUBLIC :: pdsyevd_drv
-#endif
-
 
 CONTAINS
 
@@ -94,14 +81,11 @@ CONTAINS
 !              this vector is equal on all processors. 
 !
 !
+      USE laxlib_parallel_include
 
       IMPLICIT NONE
 
-#if defined (__MPI)
-   !
-   include 'mpif.h'
-   !
-#endif
+      include 'laxlib_kinds.fh'
 
       LOGICAL, INTENT(IN) :: tv
       INTEGER, intent(in) :: N, NRL, LDA, LDV
@@ -424,14 +408,11 @@ CONTAINS
 !
 !
 !
+      USE laxlib_parallel_include
 
       IMPLICIT NONE
 
-#if defined (__MPI)
-   !
-   include 'mpif.h'
-   !
-#endif
+      include 'laxlib_kinds.fh'
 
       LOGICAL, INTENT(IN)  :: tv
       INTEGER, INTENT(IN)  :: n, nrl, ldz, mpime, comm
@@ -556,6 +537,7 @@ CONTAINS
 !
 
       IMPLICIT NONE
+      include 'laxlib_kinds.fh'
       LOGICAL, INTENT(IN) :: tv
       INTEGER, INTENT (IN) :: n,ldv,nrl
       REAL(DP), INTENT(INOUT) :: d(n),v(ldv,n)
@@ -595,6 +577,7 @@ CONTAINS
    !-------------------------------------------------------------------------
    FUNCTION pythag(a,b)
       IMPLICIT NONE
+      include 'laxlib_kinds.fh'
       REAL(DP) :: a, b, pythag
       REAL(DP) :: absa, absb
       absa=abs(a)
@@ -613,63 +596,21 @@ CONTAINS
    !
 !==----------------------------------------------==!
 
-   SUBROUTINE pdspev_drv( jobz, ap, lda, w, z, ldz, &
-                          nrl, n, nproc, mpime, comm )
-     IMPLICIT NONE
-     CHARACTER, INTENT(IN) :: JOBZ
-     INTEGER, INTENT(IN) :: lda, ldz, nrl, n, nproc, mpime
-     INTEGER, INTENT(IN) :: comm
-     REAL(DP) :: ap( lda, * ), w( * ), z( ldz, * )
-     REAL(DP), ALLOCATABLE :: sd( : )
-     LOGICAL :: tv
-     !
-     IF( n < 1 ) RETURN
-     !
-     tv = .false.
-     IF( jobz == 'V' .OR. jobz == 'v' ) tv = .true.
-
-     ALLOCATE ( sd ( n ) )
-     CALL ptredv( tv, ap, lda, w, sd, z, ldz, nrl, n, nproc, mpime, comm)
-     CALL ptqliv( tv, w, sd, n, z, ldz, nrl, mpime, comm)
-     DEALLOCATE ( sd )
-     CALL peigsrtv( tv, w, z, ldz, n, nrl)
-
-     RETURN
-   END SUBROUTINE pdspev_drv
- 
-!==----------------------------------------------==!
-
-      SUBROUTINE dspev_drv( JOBZ, UPLO, N, AP, W, Z, LDZ )
-        IMPLICIT NONE
-        CHARACTER ::       JOBZ, UPLO
-        INTEGER   ::       IOPT, INFO, LDZ, N
-        REAL(DP) ::  AP( * ), W( * ), Z( LDZ, * )
-        REAL(DP), ALLOCATABLE :: WORK(:)
-
-        IF( n < 1 ) RETURN
-
-        ALLOCATE( work( 3*n ) )
-
-        CALL DSPEV(jobz, uplo, n, ap(1), w(1), z(1,1), ldz, work, INFO)
-        IF( info .NE. 0 ) THEN
-           CALL lax_error__( ' dspev_drv ', ' diagonalization failed ',info )
-        END IF
-
-        DEALLOCATE( work )
- 
-        RETURN
-      END SUBROUTINE dspev_drv
-
-
 #if defined __SCALAPACK
 
   SUBROUTINE pdsyevd_drv( tv, n, nb, s, lds, w, ortho_cntx, ortho_comm )
      !
-#if defined(__ELPA) || defined(__ELPA_2016) || defined(__ELPA_2015)
+#if  defined(__ELPA_2015)|| defined(__ELPA_2016)|| defined(__ELPA_2017)
      use elpa1
+
+#elif defined(__ELPA)|| defined(__ELPA_2018) || defined(__ELPA_2019)
+     use elpa
+
 #endif
      IMPLICIT NONE
      !
+     include 'laxlib_kinds.fh'
+     !
      LOGICAL, INTENT(IN)  :: tv  
        ! if tv is true compute eigenvalues and eigenvectors (not used)
      INTEGER, INTENT(IN)  :: nb, n, ortho_cntx, ortho_comm 
@@ -690,11 +631,12 @@ CONTAINS
      INTEGER     :: LWORK, LIWORK, info
      CHARACTER   :: jobv
      INTEGER     :: i, ierr
-#if defined(__ELPA) || defined(__ELPA_2016) || defined(__ELPA_2015)     
-     INTEGER     :: nprow,npcol,my_prow, my_pcol,mpi_comm_rows, mpi_comm_cols
-#if defined(__ELPA_2016)
+#if defined(__ELPA)   || defined(__ELPA_2015) || defined(__ELPA_2016)|| defined(__ELPA_2017)  || defined(__ELPA_2018) || defined(__ELPA_2019)   
+     INTEGER     :: nprow,npcol,my_prow,my_pcol,mpi_comm_rows,mpi_comm_cols
      LOGICAL     :: success
 #endif
+#if defined(__ELPA) || defined(__ELPA_2018) || defined(__ELPA_2019)
+     class(elpa_t), pointer :: elpa_s
 #endif 
 
      IF( SIZE( s, 1 ) /= lds ) &
@@ -716,28 +658,67 @@ CONTAINS
      itmp = 0
      rtmp = 0.0_DP
 
-#if defined(__ELPA) || defined(__ELPA_2016) || defined(__ELPA_2015)
+#if defined(__ELPA)   || defined(__ELPA_2015) || defined(__ELPA_2016)|| defined(__ELPA_2017)  || defined(__ELPA_2018) || defined(__ELPA_2019)
      CALL BLACS_Gridinfo(ortho_cntx,nprow, npcol, my_prow,my_pcol)
 
-#if defined(__ELPA_2016)
-     ierr = get_elpa_row_col_comms(ortho_comm, my_prow, my_pcol,mpi_comm_rows, mpi_comm_cols)
-     success = solve_evp_real_1stage(n,  n,   s, lds,    w,  vv, lds,SIZE(s,2),nb  ,mpi_comm_rows, mpi_comm_cols)
+#if defined(__ELPA)  || defined(__ELPA_2018) || defined(__ELPA_2019)
+  ! => from elpa-2018.11.001 to 2019.xx.xx
+  if (elpa_init(20181101) /= ELPA_OK) then        
+    print *, "ELPA API version in use not supported"
+    stop
+    endif
+  elpa_s => elpa_allocate(ierr)
+  if (ierr /= ELPA_OK) then
+    print *, "ELPA API version in use is not supported"
+    stop
+  endif 
+
+  call elpa_s%set("debug",1,ierr)
+
+! set parameters describing the matrix and it's MPI distribution
+  call elpa_s%set("na", n, ierr)
+  call elpa_s%set("nev", n, ierr)
+  call elpa_s%set("nblk", SIZE(s,2), ierr)                  
+  call elpa_s%set("local_nrows", lds,ierr)     
+  call elpa_s%set("local_ncols", nb,ierr) 
+  call elpa_s%set("mpi_comm_parent", ortho_comm, ierr) 
+  call elpa_s%set("process_row", my_prow, ierr)         
+  call elpa_s%set("process_col", my_pcol, ierr)
+
+  ierr = elpa_s%setup()
+  if (ierr .ne. ELPA_OK) then
+     print *,"Problem setting up options. Aborting ..."
+     stop
+  endif
+
+  call elpa_s%set("solver", ELPA_SOLVER_1STAGE, ierr)
+  call elpa_s%eigenvectors(s, w, vv, ierr)
+  
+  call elpa_deallocate(elpa_s)
+  call elpa_uninit()
+
+#elif defined(__ELPA_2016) || defined(__ELPA_2017)
+     ! -> from ELPA 2016.11.001_pre thru 2017.XX.XX to elpa-2018.05.001
+     ierr = elpa_get_communicators(ortho_comm, my_prow, my_pcol,mpi_comm_rows, mpi_comm_cols)
+     success = solve_evp_real_1stage(n,  n,   s, lds,    w,  vv, lds,SIZE(s,2),nb  ,mpi_comm_rows, mpi_comm_cols, ortho_comm)
+     ! -> ELPA 2016.05.003
+     !ierr = get_elpa_row_col_comms(ortho_comm, my_prow, my_pcol,mpi_comm_rows, mpi_comm_cols)
+     !success = solve_evp_real_1stage(n,  n,   s, lds,    w,  vv, lds,SIZE(s,2),nb  ,mpi_comm_rows, mpi_comm_cols)
 #elif defined(__ELPA_2015)
      ierr = get_elpa_row_col_comms(ortho_comm, my_prow, my_pcol,mpi_comm_rows, mpi_comm_cols)
      ierr = solve_evp_real(n,  n,   s, lds,    w,  vv, lds,SIZE(s,2),nb  ,mpi_comm_rows, mpi_comm_cols)
-#elif defined(__ELPA)
-     CALL get_elpa_row_col_comms(ortho_comm, my_prow, my_pcol,mpi_comm_rows, mpi_comm_cols)
-     CALL solve_evp_real(n,  n,   s, lds,    w,  vv, lds     ,nb  ,mpi_comm_rows, mpi_comm_cols)
 #endif
 
      IF( tv )  s = vv
      IF( ALLOCATED( vv ) ) DEALLOCATE( vv )
 
-#if defined __MPI
+
+#if defined(__MPI) && (defined(__ELPA_2015) || defined(__ELPA_2016) || defined(__ELPA_2017)) 
      CALL mpi_comm_free( mpi_comm_rows, ierr )
      CALL mpi_comm_free( mpi_comm_cols, ierr )
 #endif
 
+
 #else
      CALL PDSYEVD( jobv, 'L', n, s, 1, 1, desch, w, vv, 1, 1, desch, rtmp, lwork, itmp, liwork, info )
 
@@ -765,89 +746,55 @@ CONTAINS
 
 #endif
 
-
-!   ----------------------------------------------
-!   Simplified driver 
-
-SUBROUTINE diagonalize_parallel( n, rhos, rhod, s, desc )
-
-      USE descriptors
-
-      IMPLICIT NONE
-      REAL(DP), INTENT(IN)  :: rhos(:,:) !  input symmetric matrix
-      REAL(DP)              :: rhod(:)   !  output eigenvalues
-      REAL(DP)              :: s(:,:)    !  output eigenvectors
-      INTEGER,   INTENT(IN) :: n         !  size of the global matrix
-      TYPE(la_descriptor), INTENT(IN) :: desc
-
-      IF( n < 1 ) RETURN
-
-      !  Matrix is distributed on the same processors group
-      !  used for parallel matrix multiplication
-      !
-      IF( SIZE(s,1) /= SIZE(rhos,1) .OR. SIZE(s,2) /= SIZE(rhos,2) ) &
-         CALL lax_error__( " diagonalize_parallel ", " inconsistent dimension for s and rhos ", 1 )
-
-      IF ( desc%active_node > 0 ) THEN
-         !
-         IF( SIZE(s,1) /= desc%nrcx ) &
-            CALL lax_error__( " diagonalize_parallel ", " inconsistent dimension ", 1)
-         !
-         !  Compute local dimension of the cyclically distributed matrix
-         !
-         s = rhos
-         !
-#if defined(__SCALAPACK)
-         CALL pdsyevd_drv( .true. , n, desc%nrcx, s, SIZE(s,1), rhod, desc%cntx, desc%comm )
-#else
-         CALL qe_pdsyevd( .true., n, desc, s, SIZE(s,1), rhod )
-#endif
-         !
-      END IF
-
-      RETURN
-
-END SUBROUTINE diagonalize_parallel
-
-
-SUBROUTINE diagonalize_serial( n, rhos, rhod )
-      IMPLICIT NONE
-      INTEGER,  INTENT(IN)  :: n
-      REAL(DP)              :: rhos(:,:)
-      REAL(DP)              :: rhod(:)
-      !
-      ! inputs:
-      ! n     size of the eigenproblem
-      ! rhos  the symmetric matrix
-      ! outputs:
-      ! rhos  eigenvectors
-      ! rhod  eigenvalues
-      !
-      REAL(DP), ALLOCATABLE :: aux(:)
-      INTEGER :: i, j, k
-
-      IF( n < 1 ) RETURN
-
-      ALLOCATE( aux( n * ( n + 1 ) / 2 ) )
-
-      !  pack lower triangle of rho into aux
-      !
-      k = 0
-      DO j = 1, n
-         DO i = j, n
-            k = k + 1
-            aux( k ) = rhos( i, j )
-         END DO
-      END DO
-
-      CALL dspev_drv( 'V', 'L', n, aux, rhod, rhos, SIZE(rhos,1) )
-
-      DEALLOCATE( aux )
-
-      RETURN
-
-END SUBROUTINE diagonalize_serial
-
-
-
 END MODULE dspev_module
+
+!==----------------------------------------------==!
+
+
+   SUBROUTINE pdspev_drv_x ( jobz, ap, lda, w, z, ldz, nrl, n, nproc, mpime, comm )
+     use dspev_module
+     IMPLICIT NONE
+     include 'laxlib_kinds.fh'
+     CHARACTER, INTENT(IN) :: JOBZ
+     INTEGER, INTENT(IN) :: lda, ldz, nrl, n, nproc, mpime
+     INTEGER, INTENT(IN) :: comm
+     REAL(DP) :: ap( lda, * ), w( * ), z( ldz, * )
+     REAL(DP), ALLOCATABLE :: sd( : )
+     LOGICAL :: tv
+     !
+     IF( n < 1 ) RETURN
+     !
+     tv = .false.
+     IF( jobz == 'V' .OR. jobz == 'v' ) tv = .true.
+
+     ALLOCATE ( sd ( n ) )
+     CALL ptredv( tv, ap, lda, w, sd, z, ldz, nrl, n, nproc, mpime, comm)
+     CALL ptqliv( tv, w, sd, n, z, ldz, nrl, mpime, comm)
+     DEALLOCATE ( sd )
+     CALL peigsrtv( tv, w, z, ldz, n, nrl)
+     RETURN
+   END SUBROUTINE pdspev_drv_x
+ 
+!==----------------------------------------------==!
+
+   SUBROUTINE dspev_drv_x( JOBZ, UPLO, N, AP, W, Z, LDZ )
+     use dspev_module
+     IMPLICIT NONE
+     include 'laxlib_kinds.fh'
+     CHARACTER ::       JOBZ, UPLO
+     INTEGER   ::       IOPT, INFO, LDZ, N
+     REAL(DP) ::  AP( * ), W( * ), Z( LDZ, * )
+     REAL(DP), ALLOCATABLE :: WORK(:)
+
+     IF( n < 1 ) RETURN
+
+     ALLOCATE( work( 3*n ) )
+
+     CALL DSPEV(jobz, uplo, n, ap(1), w(1), z(1,1), ldz, work, INFO)
+     IF( info .NE. 0 ) THEN
+        CALL lax_error__( ' dspev_drv ', ' diagonalization failed ',info )
+     END IF
+     DEALLOCATE( work )
+     RETURN
+   END SUBROUTINE dspev_drv_x
+
