@@ -12,10 +12,8 @@ SUBROUTINE lax_error__( calling_routine, message, ierr )
   !
   ! ... This is a simple routine which writes an error message to output: 
   !
+  USE laxlib_parallel_include
   IMPLICIT NONE
-#if defined(__MPI)
-  INCLUDE 'mpif.h'
-#endif
   !
   CHARACTER(LEN=*), INTENT(IN) :: calling_routine, message
     ! the name of the calling calling_routine
