@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2004 PWSCF group
+! Copyright (C) 2001-2018 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -86,7 +86,7 @@ subroutine cgsolve_all (ch_psi, cg_psi, e, d0psi, dpsi, h_diag, &
   !
   !  here the local variables
   !
-  integer, parameter :: maxter = 200
+  integer, parameter :: maxter = 400
   ! the maximum number of iterations
   integer :: iter, ibnd, ibnd_, lbnd
   ! counters on iteration, bands
@@ -102,7 +102,6 @@ subroutine cgsolve_all (ch_psi, cg_psi, e, d0psi, dpsi, h_diag, &
   complex(DP) ::  dcgamma, dclambda
   !  the ratio between rho
   !  step length
-  complex(DP), external :: zdotc
   REAL(kind=dp), EXTERNAL :: ddot
   !  the scalar product
   real(DP), allocatable :: rho (:), rhoold (:), eu (:), a(:), c(:)
@@ -168,7 +167,7 @@ subroutine cgsolve_all (ch_psi, cg_psi, e, d0psi, dpsi, h_diag, &
                  rho(lbnd)=rho(lbnd)-DBLE(h(1,ibnd_))*DBLE(g(1,ibnd_))
               ENDIF
            ELSE
-              rho(lbnd) = zdotc (ndmx*npol, h(1,ibnd_), 1, g(1,ibnd_), 1)
+              rho(lbnd) = ddot (2*ndmx*npol, h(1,ibnd_), 1, g(1,ibnd_), 1)
            ENDIF
         endif
      enddo
@@ -232,8 +231,8 @@ subroutine cgsolve_all (ch_psi, cg_psi, e, d0psi, dpsi, h_diag, &
                  c(lbnd)=c(lbnd)-DBLE(h(1,ibnd_))*DBLE(t(1,lbnd))
               ENDIF
            ELSE
-              a(lbnd) = zdotc (ndmx*npol, h(1,ibnd_), 1, g(1,ibnd_), 1)
-              c(lbnd) = zdotc (ndmx*npol, h(1,ibnd_), 1, t(1,lbnd), 1)
+              a(lbnd) = ddot (2*ndmx*npol, h(1,ibnd_), 1, g(1,ibnd_), 1)
+              c(lbnd) = ddot (2*ndmx*npol, h(1,ibnd_), 1, t(1,lbnd), 1)
            ENDIF
         end if
      end do
