@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2008 Quantum ESPRESSO group
+! Copyright (C) 2001-2018 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -17,6 +17,7 @@ subroutine newdq (dvscf, npe)
   !
   USE kinds,                ONLY : DP
   USE ions_base,            ONLY : nat, ityp, ntyp => nsp
+  USE cell_base,            ONLY : tpiba
   USE noncollin_module,     ONLY : noncolin, nspin_mag
   USE cell_base,            ONLY : omega
   USE fft_base,             ONLY : dfftp
@@ -25,13 +26,11 @@ subroutine newdq (dvscf, npe)
   USE uspp,                 ONLY : okvan
   USE uspp_param,           ONLY : upf, lmaxq, nh, nhm
   USE paw_variables,        ONLY : okpaw
-
-  USE mp_bands,  ONLY: intra_bgrp_comm
-  USE mp,        ONLY: mp_sum
-
-  USE lrus,       ONLY : int3, int3_paw
-  USE qpoint,     ONLY : xq, eigqts
-  USE control_lr, ONLY : lgamma
+  USE mp_bands,             ONLY: intra_bgrp_comm
+  USE mp,                   ONLY: mp_sum
+  USE lrus,                 ONLY : int3, int3_paw
+  USE qpoint,               ONLY : xq, eigqts
+  USE control_lr,           ONLY : lgamma
 
   implicit none
   !
@@ -53,15 +52,13 @@ subroutine newdq (dvscf, npe)
   ! the values of q+G
   ! the spherical harmonics
 
-  complex(DP), external :: zdotc
-  ! the scalar product function
-
   complex(DP), allocatable :: aux1 (:), aux2 (:,:), veff (:), qgm(:)
   ! work space
 
   if (.not.okvan) return
+  !
   call start_clock ('newdq')
-
+  !
   int3 (:,:,:,:,:) = (0.d0, 0.0d0)
   allocate (aux1 (ngm))
   allocate (aux2 (ngm , nspin_mag))
@@ -69,7 +66,7 @@ subroutine newdq (dvscf, npe)
   allocate (ylmk0(ngm , lmaxq * lmaxq))
   allocate (qgm  (ngm))
   allocate (qmod (ngm))
-
+  !
   if (.not.lgamma) allocate (qg (3,  ngm))
   !
   !    first compute the spherical harmonics
@@ -78,12 +75,12 @@ subroutine newdq (dvscf, npe)
      call setqmod (ngm, xq, g, qmod, qg)
      call ylmr2 (lmaxq * lmaxq, ngm, qg, qmod, ylmk0)
      do ig = 1, ngm
-        qmod (ig) = sqrt (qmod (ig) )
+        qmod (ig) = sqrt (qmod (ig) ) * tpiba
      enddo
   else
      call ylmr2 (lmaxq * lmaxq, ngm, g, gg, ylmk0)
      do ig = 1, ngm
-        qmod (ig) = sqrt (gg (ig) )
+        qmod (ig) = sqrt (gg (ig) ) * tpiba
      enddo
   endif
   !
@@ -118,7 +115,7 @@ subroutine newdq (dvscf, npe)
                        enddo
                        do is = 1, nspin_mag
                           int3(ih,jh,na,is,ipert) = omega * &
-                                             zdotc(ngm,aux1,1,aux2(1,is),1)
+                                             dot_product(aux1(:),aux2(:,is))
                        enddo
                     endif
                  enddo
@@ -145,9 +142,14 @@ subroutine newdq (dvscf, npe)
 #if defined(__MPI)
   call mp_sum ( int3, intra_bgrp_comm )
 #endif
+  !
   IF (noncolin) CALL set_int3_nc(npe)
-  IF (okpaw) int3=int3+int3_paw
-
+  !
+  ! Sum of the USPP and PAW terms 
+  ! (see last two terms in Eq.(12) in PRB 81, 075123 (2010))
+  !
+  IF (okpaw) int3 = int3 + int3_paw
+  !
   if (.not.lgamma) deallocate (qg)
   deallocate (qmod)
   deallocate (qgm)
@@ -155,7 +157,9 @@ subroutine newdq (dvscf, npe)
   deallocate (veff)
   deallocate (aux2)
   deallocate (aux1)
-
+  !
   call stop_clock ('newdq')
+  !
   return
+  !
 end subroutine newdq
