@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2018 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -7,35 +7,35 @@
 !
 !
 !-----------------------------------------------------------------------
-subroutine setqmod (ngm, xq, g, qmod, qpg)
+SUBROUTINE setqmod (ngm, xq, g, qmod, qpg)
   !-----------------------------------------------------------------------
   !
   ! This subroutine puts in qmod the modulus of q+G for the interpolation
   ! table used to compute qgm
   !
-  USE kinds, only : DP
-  
-  implicit none
-
-  integer :: ngm
+  USE kinds, ONLY : DP
+  ! 
+  IMPLICIT NONE
+  !
+  INTEGER :: ngm
   ! input: the number of G vectors
 
-  real(DP) :: xq (3), g (3, ngm), qmod (ngm), qpg (3, ngm)
+  REAL(DP) :: xq (3), g (3, ngm), qmod (ngm), qpg (3, ngm)
   ! input: the q vector
   ! input: the G vectors
   ! output: the modulus of the G vectors
   ! output: the q+G vectors
 
-  integer :: ig
+  INTEGER :: ig
   ! counter on G vectors
-  
-  do ig = 1, ngm
+  ! 
+  DO ig = 1, ngm
      qpg (1, ig) = xq (1) + g (1, ig)
      qpg (2, ig) = xq (2) + g (2, ig)
      qpg (3, ig) = xq (3) + g (3, ig)
      qmod (ig) = qpg (1, ig) **2 + qpg (2, ig) **2 + qpg (3, ig) **2
-  enddo
-
-  return
-
-end subroutine setqmod
+  ENDDO
+  !
+  RETURN
+  !
+END SUBROUTINE setqmod
