@@ -39,12 +39,10 @@ MODULE control_flags
             tnosee, tnosep, tnoseh, tcp, tcap,                               &
             tconvthrs, tolp, convergence_criteria, tionstep, nstepe,         &
             tscreen, gamma_only, force_pairing, lecrpa, tddfpt, smallmem,    &
-            tfirst, tlast, tprint, trescalee
+            tfirst, tlast, tprint, trescalee, max_xml_steps, dfpt_hub  
   !
   PUBLIC :: fix_dependencies, check_flags
   PUBLIC :: tksw, trhor, thdyn, trhow
-  PUBLIC :: twfcollect
-  PUBLIC :: lkpoint_dir
   !
   ! ...   declare execution control variables
   !
@@ -76,10 +74,10 @@ MODULE control_flags
   LOGICAL :: timing        = .FALSE. ! print out timing information
   LOGICAL :: memchk        = .FALSE. ! check for memory leakage
   LOGICAL :: tscreen       = .FALSE. ! Use screened coulomb potentials for cluster calculations
-  LOGICAL :: twfcollect    = .FALSE. ! Collect wave function in the restart file at the end of run.
-  LOGICAL :: lkpoint_dir   = .TRUE.  ! save each k point in a different directory
   LOGICAL :: force_pairing = .FALSE. ! Force pairing
   LOGICAL :: lecrpa        = .FALSE. ! RPA correlation energy request
+  LOGICAL :: dfpt_hub      = .FALSE. ! If .true. perform the SCF calculation of U (and V)
+                                     ! and let PW rotuines to know about this
   LOGICAL :: tddfpt        = .FALSE. ! use TDDFPT specific tweaks when using the Environ plugin
   LOGICAL :: smallmem      = .FALSE. ! the memory per task is small
   !
@@ -101,6 +99,7 @@ MODULE control_flags
   INTEGER :: ndr    = 0 !
   INTEGER :: nomore = 0 !
   INTEGER :: iprint =10 ! print output every iprint step
+  INTEGER  :: max_xml_steps =0 ! max number of dynamics included in xml file if 0 all steps are included. 
   INTEGER :: isave  = 0 ! write restart to ndr unit every isave step
   !
   ! ... .TRUE. if only gamma point is used
@@ -169,6 +168,7 @@ MODULE control_flags
     ldftd3 =.FALSE., & ! if .TRUE. compute Grimme D3 dispersion corrections
     ts_vdw  =.FALSE., & ! as above for Tkatchenko-Scheffler disp.corrections
     lxdm    =.FALSE., & ! if .TRUE. compute XDM dispersion corrections
+    lensemb =.FALSE., &! if .TRUE. compute ensemble energies
     restart =.FALSE.   ! if .TRUE. restart from results of a preceding run
   !
   ! ... pw self-consistency
@@ -202,9 +202,10 @@ MODULE control_flags
   REAL(DP), PUBLIC  :: &
     ethr               ! the convergence threshold for eigenvalues
   INTEGER, PUBLIC :: &
-    isolve,           &! Davidson or CG or ParO diagonalization
+    isolve,           &! index selecting Davidson,  CG , PPCG or ParO diagonalization
     david,            &! max dimension of subspace in Davidson diagonalization
-    max_cg_iter        ! maximum number of iterations in a CG call
+    max_cg_iter,      &! maximum number of iterations in a CG call
+    max_ppcg_iter      ! maximum number of iterations in a PPCG call
   LOGICAL, PUBLIC :: &
     diago_full_acc = .FALSE. ! if true,  empty eigenvalues have the same
                              ! accuracy of the occupied ones
@@ -271,6 +272,9 @@ MODULE control_flags
   !
   LOGICAL,          PUBLIC :: textfor = .FALSE.
 
+
+  LOGICAL,          PUBLIC :: treinit_gvecs = .FALSE.
+
   !
   ! ...  end of module-scope declarations
   !
