@@ -17,14 +17,13 @@ MODULE generate_function
   USE kinds, ONLY: DP
 
   IMPLICIT NONE
- 
+
 CONTAINS
 !----------------------------------------------------------------------
       SUBROUTINE planar_average( nnr, naxis, axis, shift, reverse, f, f1d )
 !----------------------------------------------------------------------
       !
       USE kinds,            ONLY : DP
-      USE io_global,        ONLY : stdout
       USE fft_base,         ONLY : dfftp
       USE mp,               ONLY : mp_sum
       USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
@@ -41,23 +40,13 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end
-      INTEGER                   :: idx, idx0, narea
-      !
-      REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
-      !
-      inv_nr1 = 1.D0 / DBLE( dfftp%nr1 )
-      inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
-      inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
-      !
-      idx0 = 0
-      ir_end = nnr
+      INTEGER                   :: idx, narea
       !
 #if defined (__MPI)
-      DO i = 1, me_bgrp
-        idx0 = idx0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
-      END DO
-      ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
-#endif  
+      ir_end = MIN(nnr,dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p)
+#else
+      ir_end = nnr
+#endif
       !
       narea = dfftp%nr1*dfftp%nr2*dfftp%nr3 / naxis
       !
@@ -71,31 +60,39 @@ CONTAINS
          !
          ! ... find the index along the selected axis
          !
-         i = idx0 + ir - 1
-         idx = i / (dfftp%nr1x*dfftp%nr2x)
-         IF ( axis .LT. 3 ) THEN 
-           i = i - (dfftp%nr1x*dfftp%nr2x)*idx
-           idx = i / dfftp%nr1x
-         END IF 
-         IF ( axis .EQ. 1 ) idx = i - dfftp%nr1x*idx
+         idx = ir -1
+         i   = idx / (dfftp%nr1x*dfftp%my_nr2p)
+         idx = idx - (dfftp%nr1x*dfftp%my_nr2p)*i
+         i   = i + dfftp%my_i0r3p
+         IF ( idx .GE. dfftp%nr3 ) CYCLE
+         IF ( axis .LT. 3 ) THEN
+            i   = idx / dfftp%nr1x
+            idx = idx - dfftp%nr1x * i
+            i   = i + dfftp%my_i0r2p
+           IF ( idx .GE. dfftp%nr2 ) CYCLE
+         END IF
+         IF ( axis .EQ. 1 ) THEN
+            i   = idx
+            IF ( idx .GE. dfftp%nr1 ) CYCLE
+         END IF
          !
          idx = idx + 1 + shift
          !
-         IF ( idx .GT. naxis ) THEN 
+         IF ( idx .GT. naxis ) THEN
            idx = idx - naxis
          ELSE IF (idx .LE. 0 ) THEN
            idx = idx + naxis
-         ENDIF           
+         ENDIF
          !
          IF ( reverse ) THEN
            f(ir) = f1d(idx)
          ELSE
            f1d(idx) = f1d(idx) + f(ir)
-         END IF 
-         !      
+         END IF
+         !
       END DO
       !
-      IF ( .NOT. reverse ) THEN 
+      IF ( .NOT. reverse ) THEN
         CALL mp_sum( f1d(:), intra_bgrp_comm )
         f1d = f1d / DBLE(narea)
       END IF
@@ -114,6 +111,7 @@ CONTAINS
       USE io_global,        ONLY : stdout
       USE cell_base,        ONLY : at, bg, alat, omega
       USE fft_base,         ONLY : dfftp
+      USE fft_types,        ONLY : fft_index_to_3d
       USE mp,               ONLY : mp_sum
       USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
@@ -129,7 +127,7 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: idx0
+      LOGICAL                   :: offrange
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
       REAL( DP )                :: scale, spr2, dist, length
@@ -140,31 +138,23 @@ CONTAINS
       inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
       inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
       !
-      idx0 = 0
-      !
 #if defined (__MPI)
-      DO i = 1, me_bgrp
-        idx0 = idx0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
-      END DO
-#endif  
-      !
-#if defined (__MPI)
-      ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
+      ir_end = MIN(nnr,dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p)
 #else
       ir_end = nnr
-#endif  
+#endif
       !
       IF (axis.LT.1.OR.axis.GT.3) &
            WRITE(stdout,*)'WARNING: wrong axis in generate_gaussian'
       IF ( dim .EQ. 0 ) THEN
         scale = charge / ( sqrtpi * spread )**3
       ELSE IF ( dim .EQ. 1 ) THEN
-        length = at(axis,axis) * alat
+        length = ABS( at(axis,axis) * alat )
         scale = charge / length / ( sqrtpi * spread )**2
       ELSE IF ( dim .EQ. 2 ) THEN
-        length = at(axis,axis) * alat
+        length = ABS( at(axis,axis) * alat )
         scale = charge * length / omega / ( sqrtpi * spread )
-      ELSE 
+      ELSE
         WRITE(stdout,*)'WARNING: wrong dim in generate_gaussian'
       ENDIF
       spr2 = ( spread / alat )**2
@@ -175,11 +165,8 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         i = idx0 + ir - 1
-         k = i / (dfftp%nr1x*dfftp%nr2x)
-         i = i - (dfftp%nr1x*dfftp%nr2x)*k
-         j = i / dfftp%nr1x
-         i = i - dfftp%nr1x*j
+         CALL fft_index_to_3d (ir, dfftp, i,j,k, offrange)
+         IF ( offrange ) CYCLE
          !
          DO ip = 1, 3
             r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -187,7 +174,7 @@ CONTAINS
                     DBLE( k )*inv_nr3*at(ip,3)
          END DO
          !
-         r(:) = pos(:) - r(:) 
+         r(:) = pos(:) - r(:)
          !
          !  ... possibly 2D or 1D gaussians
          !
@@ -205,10 +192,10 @@ CONTAINS
          s(:) = s(:) - ANINT(s(:))
          r(:) = MATMUL( at(:,:), s(:) )
          !
-         dist = SUM( r * r ) 
+         dist = SUM( r * r )
+         !
+         rholocal( ir ) = scale * EXP(-dist/spr2)
          !
-         rholocal( ir ) = scale * EXP(-dist/spr2) 
-         !      
       END DO
       !
       rho = rho + rholocal
@@ -228,6 +215,7 @@ CONTAINS
       USE io_global,        ONLY : stdout
       USE cell_base,        ONLY : at, bg, alat, omega
       USE fft_base,         ONLY : dfftp
+      USE fft_types,        ONLY : fft_index_to_3d
       USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
       IMPLICIT NONE
@@ -242,7 +230,7 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: idx0
+      LOGICAL                   :: offrange
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
       REAL( DP )                :: scale, spr2, dist, length
@@ -253,31 +241,23 @@ CONTAINS
       inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
       inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
       !
-      idx0 = 0
-      !
 #if defined (__MPI)
-      DO i = 1, me_bgrp
-        idx0 = idx0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
-      END DO
-#endif
-      !
-#if defined (__MPI)
-      ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
+      ir_end = MIN(nnr,dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p)
 #else
       ir_end = nnr
-#endif  
+#endif
       !
       IF (axis.LT.1.OR.axis.GT.3) &
            WRITE(stdout,*)'WARNING: wrong axis in generate_gaussian'
       IF ( dim .EQ. 0 ) THEN
         scale = charge / ( sqrtpi * spread )**3
       ELSE IF ( dim .EQ. 1 ) THEN
-        length = at(axis,axis) * alat
+        length = ABS( at(axis,axis) * alat )
         scale = charge / length / ( sqrtpi * spread )**2
       ELSE IF ( dim .EQ. 2 ) THEN
-        length = at(axis,axis) * alat
+        length = ABS( at(axis,axis) * alat )
         scale = charge * length / omega / ( sqrtpi * spread )
-      ELSE 
+      ELSE
         WRITE(stdout,*)'WARNING: wrong dim in generate_gaussian'
       ENDIF
       spr2 = ( spread / alat )**2
@@ -288,11 +268,8 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         i = idx0 + ir - 1
-         k = i / (dfftp%nr1x*dfftp%nr2x)
-         i = i - (dfftp%nr1x*dfftp%nr2x)*k
-         j = i / dfftp%nr1x
-         i = i - dfftp%nr1x*j
+         CALL fft_index_to_3d (ir, dfftp, i,j,k, offrange)
+         IF ( offrange ) CYCLE
          !
          DO ip = 1, 3
             r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -300,7 +277,7 @@ CONTAINS
                     DBLE( k )*inv_nr3*at(ip,3)
          END DO
          !
-         r(:) = pos(:) - r(:) 
+         r(:) = pos(:) - r(:)
          !
          !  ... possibly 2D or 1D gaussians
          !
@@ -318,10 +295,10 @@ CONTAINS
          s(:) = s(:) - ANINT(s(:))
          r(:) = MATMUL( at(:,:), s(:) )
          !
-         dist = SUM( r * r ) 
+         dist = SUM( r * r )
          !
          gradrholocal( :, ir ) = scale * EXP(-dist/spr2) * r(:) * alat
-         !      
+         !
       END DO
       !
       gradrho = gradrho + gradrholocal
@@ -339,6 +316,7 @@ CONTAINS
       USE kinds,            ONLY : DP
       USE cell_base,        ONLY : at, bg, alat
       USE fft_base,         ONLY : dfftp
+      USE fft_types,        ONLY : fft_index_to_3d
       USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
       IMPLICIT NONE
@@ -353,7 +331,7 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: idx0
+      LOGICAL                   :: offrange
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
       REAL( DP )                :: dist, arg
@@ -365,16 +343,8 @@ CONTAINS
       inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
       inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
       !
-      idx0 = 0
-      !
 #if defined (__MPI)
-      DO i = 1, me_bgrp
-        idx0 = idx0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
-      END DO
-#endif
-      !
-#if defined (__MPI)
-      ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
+      ir_end = MIN(nnr,dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p)
 #else
       ir_end = nnr
 #endif
@@ -386,12 +356,8 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         i = idx0 + ir - 1
-         k = i / (dfftp%nr1x*dfftp%nr2x)
-         i = i - (dfftp%nr1x*dfftp%nr2x)*k
-         j = i / dfftp%nr1x
-         i = i - dfftp%nr1x*j
-         r = 0.D0
+         CALL fft_index_to_3d (ir, dfftp, i,j,k, offrange)
+         IF ( offrange ) CYCLE
          !
          DO ip = 1, 3
             r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -399,7 +365,7 @@ CONTAINS
                     DBLE( k )*inv_nr3*at(ip,3)
          END DO
          !
-         r(:) = pos(:) - r(:) 
+         r(:) = pos(:) - r(:)
          !
          ! ... minimum image convention
          !
@@ -411,11 +377,11 @@ CONTAINS
          arg = dist - spread
          !
          IF( ABS( arg ) .LT. exp_arg_limit ) THEN
-           rholocal( ir ) = EXP( - arg ) 
-         ELSE 
+           rholocal( ir ) = EXP( - arg )
+         ELSE
            rholocal( ir ) = 0.D0
          END IF
-         !      
+         !
       END DO
       !
       rho = rho + rholocal
@@ -433,6 +399,7 @@ CONTAINS
       USE kinds,            ONLY : DP
       USE cell_base,        ONLY : at, bg, alat
       USE fft_base,         ONLY : dfftp
+      USE fft_types,        ONLY : fft_index_to_3d
       USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
       IMPLICIT NONE
@@ -447,7 +414,7 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: idx0
+      LOGICAL                   :: offrange
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
       REAL( DP )                :: dist, arg
@@ -459,16 +426,8 @@ CONTAINS
       inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
       inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
       !
-      idx0 = 0
-      !
 #if defined (__MPI)
-      DO i = 1, me_bgrp
-        idx0 = idx0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
-      END DO
-#endif
-      !
-#if defined (__MPI)
-      ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
+      ir_end = MIN(nnr,dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p)
 #else
       ir_end = nnr
 #endif
@@ -480,11 +439,8 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         i = idx0 + ir - 1
-         k = i / (dfftp%nr1x*dfftp%nr2x)
-         i = i - (dfftp%nr1x*dfftp%nr2x)*k
-         j = i / dfftp%nr1x
-         i = i - dfftp%nr1x*j
+         CALL fft_index_to_3d (ir, dfftp, i,j,k, offrange)
+         IF ( offrange ) CYCLE
          !
          DO ip = 1, 3
             r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -492,7 +448,7 @@ CONTAINS
                     DBLE( k )*inv_nr3*at(ip,3)
          END DO
          !
-         r(:) = pos(:) - r(:) 
+         r(:) = pos(:) - r(:)
          !
          ! ... minimum image convention
          !
@@ -500,14 +456,14 @@ CONTAINS
          s(:) = s(:) - ANINT(s(:))
          r(:) = MATMUL( at(:,:), s(:) )
          !
-         dist = SQRT(SUM( r * r )) * alat 
+         dist = SQRT(SUM( r * r )) * alat
          arg = dist - spread
          IF ( dist .GT. 1.D-6 .AND. ABS( arg ) .LT. exp_arg_limit ) THEN
-           gradrholocal( :, ir ) = r(:) * alat / dist * EXP( - arg ) 
+           gradrholocal( :, ir ) = r(:) * alat / dist * EXP( - arg )
          ELSE
            gradrholocal( :, ir ) = 0.D0
          ENDIF
-         !      
+         !
       END DO
       !
       gradrho = gradrho + gradrholocal
@@ -526,6 +482,7 @@ CONTAINS
       USE io_global,        ONLY : stdout
       USE cell_base,        ONLY : at, bg, alat, omega
       USE fft_base,         ONLY : dfftp
+      USE fft_types,        ONLY : fft_index_to_3d
       USE mp,               ONLY : mp_sum
       USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
@@ -541,11 +498,11 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: idx0, ntot
+      INTEGER                   :: ntot
+      LOGICAL                   :: offrange
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
-      REAL( DP )                :: scale, dist, arg, length, chargeanalytic, chargelocal
-      REAL( DP )                :: f1, f2 
+      REAL( DP )                :: scale, dist, arg, chargeanalytic, chargelocal
       REAL( DP )                :: r( 3 ), s( 3 )
       REAL( DP ), ALLOCATABLE   :: rholocal ( : )
       REAL( DP ), EXTERNAL      :: qe_erfc
@@ -554,22 +511,18 @@ CONTAINS
       inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
       inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
       !
-      idx0 = 0
-      ir_end = nnr
-      !
 #if defined (__MPI)
-      DO i = 1, me_bgrp
-        idx0 = idx0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
-      END DO
-      ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
-#endif  
+      ir_end = MIN(nnr,dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p)
+#else
+      ir_end = nnr
+#endif
       !
       ntot = dfftp%nr1 * dfftp%nr2 * dfftp%nr3
       !
       IF (axis.LT.1.OR.axis.GT.3) &
            WRITE(stdout,*)'WARNING: wrong axis in generate_erfc'
       chargeanalytic = erfcvolume(dim,axis,width,spread,alat,omega,at)
-      scale = charge / chargeanalytic * 0.5D0 
+      scale = charge / chargeanalytic * 0.5D0
       !
       ALLOCATE( rholocal( nnr ) )
       rholocal = 0.D0
@@ -578,11 +531,8 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         i = idx0 + ir - 1
-         k = i / (dfftp%nr1x*dfftp%nr2x)
-         i = i - (dfftp%nr1x*dfftp%nr2x)*k
-         j = i / dfftp%nr1x
-         i = i - dfftp%nr1x*j
+         CALL fft_index_to_3d (ir, dfftp, i,j,k, offrange)
+         IF ( offrange ) CYCLE
          !
          DO ip = 1, 3
             r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -590,7 +540,7 @@ CONTAINS
                     DBLE( k )*inv_nr3*at(ip,3)
          END DO
          !
-         r(:) = pos(:) - r(:) 
+         r(:) = pos(:) - r(:)
          !
          !  ... possibly 2D or 1D gaussians
          !
@@ -608,14 +558,14 @@ CONTAINS
          s(:) = s(:) - ANINT(s(:))
          r(:) = MATMUL( at(:,:), s(:) )
          !
-         dist = SQRT(SUM( r * r )) 
+         dist = SQRT(SUM( r * r ))
          arg = ( dist * alat - width ) / spread
          !
-         rholocal( ir ) = qe_erfc(arg) 
-         !      
+         rholocal( ir ) = qe_erfc(arg)
+         !
       END DO
       !
-      ! ... double check that the integral of the generated charge corresponds to 
+      ! ... double check that the integral of the generated charge corresponds to
       !     what is expected
       !
       chargelocal = SUM(rholocal)*omega/DBLE(ntot)*0.5D0
@@ -640,10 +590,11 @@ CONTAINS
 !----------------------------------------------------------------------
       !
       USE kinds,            ONLY : DP
-      USE constants,        ONLY : sqrtpi, fpi, pi
+      USE constants,        ONLY : sqrtpi
       USE io_global,        ONLY : stdout
       USE cell_base,        ONLY : at, bg, alat, omega
       USE fft_base,         ONLY : dfftp
+      USE fft_types,        ONLY : fft_index_to_3d
       USE mp,               ONLY : mp_sum
       USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
       !
@@ -659,10 +610,11 @@ CONTAINS
       ! ... Local variables
       !
       INTEGER                   :: i, j, k, ir, ir_end, ip
-      INTEGER                   :: idx0, ntot
+      INTEGER                   :: ntot
+      LOGICAL                   :: offrange
       !
       REAL( DP )                :: inv_nr1, inv_nr2, inv_nr3
-      REAL( DP )                :: scale, dist, arg, length, chargeanalytic, chargelocal
+      REAL( DP )                :: scale, dist, arg, chargeanalytic, chargelocal
       REAL( DP )                :: r( 3 ), s( 3 )
       REAL( DP ), ALLOCATABLE   :: gradrholocal ( :, : )
       REAL( DP ), EXTERNAL      :: qe_erfc
@@ -671,15 +623,11 @@ CONTAINS
       inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
       inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
       !
-      idx0 = 0
-      ir_end = nnr
-      !
 #if defined (__MPI)
-      DO i = 1, me_bgrp
-        idx0 = idx0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
-      END DO
-      ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
-#endif  
+      ir_end = MIN(nnr,dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p)
+#else
+      ir_end = nnr
+#endif
       !
       ntot = dfftp%nr1 * dfftp%nr2 * dfftp%nr3
       !
@@ -688,9 +636,9 @@ CONTAINS
       chargeanalytic = erfcvolume(dim,axis,width,spread,alat,omega,at)
       !
       ! ... scaling factor, take into account rescaling of generated density
-      !     to obtain the correct integrated total charge  
+      !     to obtain the correct integrated total charge
       !
-      scale = charge / chargeanalytic / sqrtpi / spread 
+      scale = charge / chargeanalytic / sqrtpi / spread
       !
       ALLOCATE( gradrholocal( 3, nnr ) )
       gradrholocal = 0.D0
@@ -700,11 +648,8 @@ CONTAINS
          !
          ! ... three dimensional indexes
          !
-         i = idx0 + ir - 1
-         k = i / (dfftp%nr1x*dfftp%nr2x)
-         i = i - (dfftp%nr1x*dfftp%nr2x)*k
-         j = i / dfftp%nr1x
-         i = i - dfftp%nr1x*j
+         CALL fft_index_to_3d (ir, dfftp, i,j,k, offrange)
+         IF ( offrange ) CYCLE
          !
          DO ip = 1, 3
             r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -712,7 +657,7 @@ CONTAINS
                     DBLE( k )*inv_nr3*at(ip,3)
          END DO
          !
-         r(:) = pos(:) - r(:) 
+         r(:) = pos(:) - r(:)
          !
          !  ... possibly 2D or 1D erfc
          !
@@ -730,25 +675,25 @@ CONTAINS
          s(:) = s(:) - ANINT(s(:))
          r(:) = MATMUL( at(:,:), s(:) )
          !
-         dist = SQRT(SUM( r * r )) 
+         dist = SQRT(SUM( r * r ))
          arg = ( dist * alat - width ) / spread
          !
          gradrholocal( :, ir ) = EXP( - arg**2 ) * r(:) / dist
-         chargelocal = chargelocal + qe_erfc(arg) 
-         !      
+         chargelocal = chargelocal + qe_erfc(arg)
+         !
       END DO
       !
-      ! ... double check that the integral of the generated charge corresponds to 
+      ! ... double check that the integral of the generated charge corresponds to
       !     what is expected
       !
-      CALL mp_sum( chargelocal, intra_bgrp_comm ) 
+      CALL mp_sum( chargelocal, intra_bgrp_comm )
       chargelocal = chargelocal*omega/DBLE(ntot)*0.5D0
       IF ( ABS(chargelocal-chargeanalytic)/chargeanalytic .GT. 1.D-4 ) &
         WRITE(stdout,*)'WARNING: significant discrepancy between the numerical and the expected erfc charge'
       !
       gradrholocal = gradrholocal * scale
       !
-      gradrho = gradrho + gradrholocal 
+      gradrho = gradrho + gradrholocal
       DEALLOCATE( gradrholocal )
       !
       RETURN
@@ -762,6 +707,7 @@ CONTAINS
    USE kinds,            ONLY : DP
    USE cell_base,        ONLY : at, bg, alat
    USE fft_base,         ONLY : dfftp
+   USE fft_types,        ONLY : fft_index_to_3d
    USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
   !
   INTEGER, INTENT(IN) :: nnr
@@ -769,24 +715,17 @@ CONTAINS
   REAL(DP), INTENT(IN) :: pos(3)
   REAL(DP), INTENT(OUT) :: axis( dfftp%nnr )
   !
-  INTEGER  :: i, j, k, ir, ir_end, ip, idx0
+  INTEGER  :: i, j, k, ir, ir_end, ip
+  LOGICAL  :: offrange
   REAL(DP) :: inv_nr1, inv_nr2, inv_nr3
-  REAL(DP) :: r(3), s(3)
+  REAL(DP) :: r(3)
   !
   inv_nr1 = 1.D0 / DBLE( dfftp%nr1 )
   inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
   inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
   !
-  idx0 = 0
-  !
 #if defined (__MPI)
-  DO i = 1, me_bgrp
-    idx0 = idx0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
-  END DO
-#endif
-  !
-#if defined (__MPI)
-  ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
+  ir_end = MIN(nnr,dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p)
 #else
   ir_end = nnr
 #endif
@@ -795,11 +734,8 @@ CONTAINS
      !
      ! ... three dimensional indexes
      !
-     i = idx0 + ir - 1
-     k = i / (dfftp%nr1x*dfftp%nr2x)
-     i = i - (dfftp%nr1x*dfftp%nr2x)*k
-     j = i / dfftp%nr1x
-     i = i - dfftp%nr1x*j
+     CALL fft_index_to_3d (ir, dfftp, i,j,k, offrange)
+     IF ( offrange ) CYCLE
      !
      DO ip = 1, 3
         r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -807,7 +743,7 @@ CONTAINS
                 DBLE( k )*inv_nr3*at(ip,3)
      END DO
      !
-     r(:) = r(:) - pos(:)  
+     r(:) = r(:) - pos(:)
      !
      ! ... minimum image convention
      !
@@ -826,7 +762,7 @@ CONTAINS
   RETURN
   !
 !----------------------------------------------------------------------
-  END SUBROUTINE generate_axis 
+  END SUBROUTINE generate_axis
 !----------------------------------------------------------------------
 !----------------------------------------------------------------------
    SUBROUTINE generate_distance( nnr, pos, distance )
@@ -834,13 +770,15 @@ CONTAINS
    USE kinds,            ONLY : DP
    USE cell_base,        ONLY : at, bg, alat
    USE fft_base,         ONLY : dfftp
+   USE fft_types,        ONLY : fft_index_to_3d
    USE mp_bands,         ONLY : me_bgrp, intra_bgrp_comm
   !
   INTEGER, INTENT(IN) :: nnr
   REAL(DP), INTENT(IN) :: pos(3)
   REAL(DP), INTENT(OUT) :: distance( 3, dfftp%nnr )
   !
-  INTEGER  :: i, j, k, ir, ir_end, ip, idx0
+  INTEGER  :: i, j, k, ir, ir_end, ip
+  LOGICAL  :: offrange
   REAL(DP) :: inv_nr1, inv_nr2, inv_nr3
   REAL(DP) :: r(3), s(3)
   !
@@ -848,16 +786,8 @@ CONTAINS
   inv_nr2 = 1.D0 / DBLE( dfftp%nr2 )
   inv_nr3 = 1.D0 / DBLE( dfftp%nr3 )
   !
-  idx0 = 0
-  !
 #if defined (__MPI)
-  DO i = 1, me_bgrp
-    idx0 = idx0 + dfftp%nr1x*dfftp%nr2x*dfftp%npp(i)
-  END DO
-#endif
-  !
-#if defined (__MPI)
-  ir_end = MIN(nnr,dfftp%nr1x*dfftp%nr2x*dfftp%npp(me_bgrp+1))
+  ir_end = MIN(nnr,dfftp%nr1x*dfftp%my_nr2p*dfftp%my_nr3p)
 #else
   ir_end = nnr
 #endif
@@ -866,11 +796,8 @@ CONTAINS
      !
      ! ... three dimensional indexes
      !
-     i = idx0 + ir - 1
-     k = i / (dfftp%nr1x*dfftp%nr2x)
-     i = i - (dfftp%nr1x*dfftp%nr2x)*k
-     j = i / dfftp%nr1x
-     i = i - dfftp%nr1x*j
+     CALL fft_index_to_3d (ir, dfftp, i,j,k, offrange)
+     IF ( offrange ) CYCLE
      !
      DO ip = 1, 3
         r(ip) = DBLE( i )*inv_nr1*at(ip,1) + &
@@ -878,7 +805,7 @@ CONTAINS
                 DBLE( k )*inv_nr3*at(ip,3)
      END DO
      !
-     r(:) = r(:) - pos(:)  
+     r(:) = r(:) - pos(:)
      !
      ! ... minimum image convention
      !
@@ -902,10 +829,10 @@ CONTAINS
 !----------------------------------------------------------------------
   FUNCTION erfcvolume(dim,axis,width,spread,alat,omega,at)
 !----------------------------------------------------------------------
-    
+
     USE constants,        ONLY : sqrtpi, fpi, pi
     USE io_global,        ONLY : stdout
-    
+
     REAL(DP), PARAMETER :: tol = 1.D-6
 
     REAL(DP) :: erfcvolume
@@ -928,26 +855,24 @@ CONTAINS
     f2 = exp(-(invt)**2) / 2.D0 / sqrtpi ! f2 is close to zero for t-->0
     SELECT CASE ( dim )
     CASE ( 0 )
-       ! zero-dimensional erfc, volume is approx the one of the 
-       ! sphere of radius=width 
+       ! zero-dimensional erfc, volume is approx the one of the
+       ! sphere of radius=width
        erfcvolume = fpi / 3.D0 * width**3 * &
          ( ( 1.D0 + 1.5D0 * t**2 ) * f1 + ( 1.D0 + t**2 ) * t * f2 )
     CASE ( 1 )
-       ! one-dimensional erfc, volume is approx the one of the 
-       ! cylinder of radius=width and lenght=alat*at(axis,axis) 
+       ! one-dimensional erfc, volume is approx the one of the
+       ! cylinder of radius=width and length=alat*at(axis,axis)
        erfcvolume = pi * width**2 * at(axis,axis) * alat * &
-         ( ( 1.D0 + 0.5D0 * t**2 ) * f1  + t * f2 ) 
-    CASE ( 2 ) 
-       ! two-dimensional erfc, volume is exactly the one of the 
-       ! box, does not depend on spread 
+         ( ( 1.D0 + 0.5D0 * t**2 ) * f1  + t * f2 )
+    CASE ( 2 )
+       ! two-dimensional erfc, volume is exactly the one of the
+       ! box, does not depend on spread
        erfcvolume = 2.D0 * width * omega / at(axis,axis) / alat
     END SELECT
-    
+
 !----------------------------------------------------------------------
-  END FUNCTION erfcvolume  
+  END FUNCTION erfcvolume
 !----------------------------------------------------------------------
 !=----------------------------------------------------------------------=!
 END MODULE generate_function
 !=----------------------------------------------------------------------=!
-
-
