@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2002-2008 Quantum ESPRESSO group
+! Copyright (C) 2002-2020 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -30,7 +30,7 @@ MODULE input_parameters
 !=----------------------------------------------------------------------------=!
   !
   USE kinds,      ONLY : DP
-  USE parameters, ONLY : nsx, lqmax
+  USE parameters, ONLY : nsx, natx, sc_size
   USE wannier_new,ONLY : wannier_data
   !
   IMPLICIT NONE
@@ -100,10 +100,10 @@ MODULE input_parameters
         CHARACTER(len=80) :: calculation = 'none'
           ! Specify the type of the simulation
           ! See below for allowed values
-        CHARACTER(len=80) :: calculation_allowed(14)
+        CHARACTER(len=80) :: calculation_allowed(15)
         DATA calculation_allowed / 'scf', 'nscf', 'relax', 'md', 'cp', &
           'vc-relax', 'vc-md', 'vc-cp', 'bands', 'neb', 'smd', 'cp-wf', &
-          'vc-cp-wf', 'cp-wf-nscf'/
+          'vc-cp-wf', 'cp-wf-nscf', 'ensemble'/
         CHARACTER(len=80) :: verbosity = 'default'
           ! define the verbosity of the code output
         CHARACTER(len=80) :: verbosity_allowed(6)
@@ -121,7 +121,9 @@ MODULE input_parameters
         INTEGER :: iprint = 10
           ! number of steps/scf iterations between successive writings
           ! of relevant physical quantities to standard output
-
+        INTEGER :: max_xml_steps = 0 
+          ! max number of steps between successive appending of an xml step 
+          ! in the xml data file, default 0 means all steps are printed. 
         INTEGER :: isave = 100
           ! number of steps between successive savings of
           ! information needed to restart the run (see "ndr", "ndw")
@@ -231,13 +233,12 @@ MODULE input_parameters
         INTEGER  :: nberrycyc = 1
           !number of covergence cycles on electric field
 
-        LOGICAL :: wf_collect = .false.
+        LOGICAL :: wf_collect = .true.
           ! This flag controls the way wavefunctions are stored to disk:
           !  .TRUE.  collect wavefunctions from all processors, store them
           !          into a single restart file on a single processors
           !  .FALSE. do not collect wavefunctions, store them into distributed
-          !          files
-          ! Only for PW and only in the parallel case
+          !          files - NO LONGER IMPLEMENTED SINCE v.6.3
 
         LOGICAL :: saverho = .true.
           ! This flag controls the saving of charge density in CP codes:
@@ -247,7 +248,7 @@ MODULE input_parameters
         LOGICAL :: tabps = .false. ! for ab-initio pressure and/or surface
                                    ! calculations
 
-        LOGICAL :: lkpoint_dir = .true. ! opens a directory for each k point
+        LOGICAL :: lkpoint_dir = .false. ! obsolete, for compatibility
 
         LOGICAL :: use_wannier = .false. ! use or not Wannier functions
 
@@ -270,8 +271,6 @@ MODULE input_parameters
           ! if memory = 'large' then QE tries to use (when implemented) algorithms using more memory
           !                     to enhance performance.
 
-          ! if .TRUE., perform exact exchange calculation using Wannier functions (X. Wu et al., Phys. Rev. B. 79, 085102 (2009))
-
         LOGICAL  :: lfcpopt = .FALSE. ! FCP optimisation switch
         LOGICAL  :: lfcpdyn = .FALSE. ! FCP thermostat enabled if .true.
         !
@@ -396,16 +395,26 @@ MODULE input_parameters
           ! ONLY PW
 
         LOGICAL :: lda_plus_u = .false.
-          ! Use DFT+U method - following are the needed parameters
+          ! Use DFT+U(+V) method - following are the needed parameters
         INTEGER :: lda_plus_u_kind = 0
-        INTEGER, PARAMETER :: nspinx=2
+        INTEGER :: lback(nsx) = -1
+        INTEGER :: l1back(nsx) = -1
+        INTEGER, PARAMETER :: nspinx=2, lqmax=7
         REAL(DP) :: starting_ns_eigenvalue(lqmax,nspinx,nsx) = -1.0_DP
         REAL(DP) :: hubbard_u(nsx) = 0.0_DP
+        REAL(DP) :: hubbard_u_back(nsx) = 0.0_DP
+        REAL(DP) :: hubbard_v(natx,natx*(2*sc_size+1)**3,4) = 0.0_DP 
         REAL(DP) :: hubbard_j0(nsx) = 0.0_DP
         REAL(DP) :: hubbard_j(3,nsx) = 0.0_DP
         REAL(DP) :: hubbard_alpha(nsx) = 0.0_DP
+        REAL(DP) :: hubbard_alpha_back(nsx) = 0.0_DP
         REAL(DP) :: hubbard_beta(nsx) = 0.0_DP
         CHARACTER(len=80) :: U_projection_type = 'atomic'
+        CHARACTER(len=80) :: Hubbard_parameters = 'input'
+        LOGICAL :: reserv(nsx) = .FALSE.
+        LOGICAL :: reserv_back(nsx) = .FALSE.
+        LOGICAL :: hub_pot_fix = .FALSE.
+        LOGICAL :: backall(nsx) = .FALSE.
 
         LOGICAL :: la2F = .false.
           ! For electron-phonon calculations
@@ -442,6 +451,7 @@ MODULE input_parameters
         REAL(DP) :: ecutfock = -1.d0
           ! variables used in Lin Lin's ACE and SCDM
         REAL(DP) :: localization_thr = 0.0_dp, scdmden=1.0d0, scdmgrd=1.0d0
+        INTEGER  :: nscdm = 1        
         INTEGER  :: n_proj  = 0
         LOGICAL  :: scdm=.FALSE.
         LOGICAL  :: ace=.TRUE.
@@ -471,7 +481,7 @@ MODULE input_parameters
         REAL(DP) :: fixed_magnetization(3) = 0.0_DP
         REAL(DP) :: angle1(nsx) = 0.0_DP
         REAL(DP) :: angle2(nsx) = 0.0_DP
-        INTEGER  :: report = 1
+        INTEGER  :: report =-1
         LOGICAL  :: no_t_rev = .FALSE.
 
         CHARACTER(len=80) :: constrained_magnetization = 'none'
@@ -511,7 +521,7 @@ MODULE input_parameters
           ! london_s6 = default global scaling parameter for PBE
           ! london_c6 = user specified atomic C6 coefficients
           ! london_rvdw = user specified atomic vdw radii
-        REAL ( DP ) :: london_s6   =   0.75_DP , &
+        REAL ( DP ) :: london_s6   =   0.75_DP ,&
                        london_rcut = 200.00_DP , &
                        london_c6( nsx ) = -1.0_DP, &
                        london_rvdw( nsx ) = -1.0_DP
@@ -619,12 +629,14 @@ MODULE input_parameters
              force_symmorphic, starting_charge, starting_magnetization,       &
              occupations, degauss, nspin, ecfixed,                            &
              qcutz, q2sigma, lda_plus_U, lda_plus_u_kind,                     &
-             Hubbard_U, Hubbard_J, Hubbard_alpha,                             &
-             Hubbard_J0, Hubbard_beta,                                        &
+             Hubbard_U, Hubbard_U_back, Hubbard_J, Hubbard_alpha,             &
+             Hubbard_alpha_back, Hubbard_J0, Hubbard_beta,                    &
+             hub_pot_fix, Hubbard_V, Hubbard_parameters,                      &
+             backall, lback, l1back, reserv, reserv_back,                     &
              edir, emaxpos, eopreg, eamp, smearing, starting_ns_eigenvalue,   &
              U_projection_type, input_dft, la2F, assume_isolated,             &
              nqx1, nqx2, nqx3, ecutfock, localization_thr, scdm, ace,         &
-             scdmden, scdmgrd, n_proj,                                        &
+             scdmden, scdmgrd, nscdm, n_proj,                                 &
              exxdiv_treatment, x_gamma_extrapolation, yukawa, ecutvcut,       &
              exx_fraction, screening_parameter, ref_alat,                     &
              noncolin, lspinorb, starting_spin_angle, lambda, angle1, angle2, &
@@ -846,7 +858,7 @@ MODULE input_parameters
           ! used only in PWscf
 
         CHARACTER(len=80) :: diagonalization = 'david'
-          ! diagonalization = 'david' or 'cg'
+          ! diagonalization = 'david', 'cg' or 'ppcg'
           ! algorithm used by PWscf for iterative diagonalization
 
         REAL(DP) :: diago_thr_init = 0.0_DP
@@ -857,6 +869,10 @@ MODULE input_parameters
           ! max number of iterations for the first iterative diagonalization
           ! using conjugate-gradient algorithm - used only in PWscf
 
+        INTEGER :: diago_ppcg_maxiter = 100
+          ! max number of iterations for the first iterative diagonalization
+          ! using projected preconditioned conjugate-gradient algorithm - used only in PWscf
+
         INTEGER :: diago_david_ndim = 4
           ! dimension of the subspace used in Davidson diagonalization
           ! used only in PWscf
@@ -1009,14 +1025,14 @@ MODULE input_parameters
 !=----------------------------------------------------------------------------=!
 !  IONS Namelist Input Parameters
 !=----------------------------------------------------------------------------=!
-!
+
 
         CHARACTER(len=80) :: ion_dynamics = 'none'
           ! set how ions should be moved
-        CHARACTER(len=80) :: ion_dynamics_allowed(9)
+        CHARACTER(len=80) :: ion_dynamics_allowed(10)
         DATA ion_dynamics_allowed / 'none', 'sd', 'cg', 'langevin', &
                                     'damp', 'verlet', 'bfgs', 'beeman',& 
-                                    'langevin-smc' /
+                                    'langevin-smc', 'ipi' /
 
         REAL(DP) :: ion_radius(nsx) = 0.5_DP
           ! pseudo-atomic radius of the i-th atomic species (CP only)
@@ -1191,9 +1207,9 @@ MODULE input_parameters
 
         CHARACTER(len=80) :: cell_dynamics  = 'none'
           ! set how the cell should be moved
-        CHARACTER(len=80) :: cell_dynamics_allowed(7)
+        CHARACTER(len=80) :: cell_dynamics_allowed(8)
         DATA cell_dynamics_allowed / 'sd', 'pr', 'none', 'w', 'damp-pr', &
-                                     'damp-w', 'bfgs'  /
+                                     'damp-w', 'bfgs', 'ipi'  /
 
         CHARACTER(len=80) :: cell_velocities = 'default'
           ! cell_velocities = 'zero' | 'default'*
@@ -1258,11 +1274,15 @@ MODULE input_parameters
           ! in a ionic steepest descent simulation
 
         REAL(DP) :: press_conv_thr = 0.5_DP
+        
+        LOGICAL :: treinit_gvecs  = .FALSE. 
+          ! if true all the quantities related to fft g vectors are updated at 
+          ! step of variable cell structural optimization 
 
         NAMELIST / cell / cell_parameters, cell_dynamics, cell_velocities, &
                           press, wmass, cell_temperature, temph, fnoseh,   &
                           cell_dofree, greash, cell_factor, cell_nstepe,   &
-                          cell_damping, press_conv_thr
+                          cell_damping, press_conv_thr, treinit_gvecs 
 
 !
 !=----------------------------------------------------------------------------=!!
@@ -1325,6 +1345,7 @@ MODULE input_parameters
           REAL(DP) :: exx_ps_rcut_pair
           REAL(DP) :: exx_me_rcut_self
           REAL(DP) :: exx_me_rcut_pair
+          LOGICAL  :: exx_use_cube_domain
 !=======================================================================
 
           INTEGER :: nit
@@ -1345,6 +1366,7 @@ MODULE input_parameters
           NAMELIST / wannier / wf_efield, wf_switch, sw_len, efx0, efy0, efz0,&
                                efx1, efy1, efz1, wfsd, wfdt,exx_neigh,exx_poisson_eps,&
                                exx_dis_cutoff,exx_ps_rcut_self, exx_me_rcut_self,   &
+                               exx_use_cube_domain,                      &
                                exx_ps_rcut_pair, exx_me_rcut_pair, vnbsp,&
                                maxwfdt, wf_q, wf_friction, nit, nsd, nsteps,  & 
                                tolw, adapt, calwf, nwf, wffort, writev
@@ -1622,6 +1644,7 @@ SUBROUTINE reset_input_checks()
     IF ( allocated( sp_vel ) ) DEALLOCATE( sp_vel )
     IF ( allocated( rd_for ) ) DEALLOCATE( rd_for )
     !
+    IF ( allocated( f_inp ) ) DEALLOCATE( f_inp )
     !
     IF ( allocated( constr_type_inp ) )   DEALLOCATE( constr_type_inp )
     IF ( allocated( constr_inp ) )        DEALLOCATE( constr_inp )
