@@ -402,12 +402,12 @@ MODULE io_base
     END SUBROUTINE read_wfc
     !
     !------------------------------------------------------------------------
-    SUBROUTINE write_rhog ( dirname, root_in_group, intra_group_comm, &
+    SUBROUTINE write_rhog ( filename, root_in_group, intra_group_comm, &
          b1, b2, b3, gamma_only, mill, ig_l2g, rho, ecutrho )
       !------------------------------------------------------------------------
       !! Collects rho(G), distributed on "intra_group_comm", writes it
-      !! together with related information to file 'charge-density.*'
-      !! (* = dat if fortran binary, * = hdf5 if HDF5) in directory "dirname"
+      !! together with related information to file "filename".*
+      !! (* = dat if fortran binary, * = hdf5 if HDF5)
       !! Processor "root_in_group" collects data and writes to file
       !
       USE mp,                   ONLY : mp_sum, mp_bcast, mp_size, mp_rank
@@ -418,8 +418,8 @@ MODULE io_base
       !
       IMPLICIT NONE
       !
-      CHARACTER(LEN=*), INTENT(IN) :: dirname
-      !! directory name where file is written - must end by '/'
+      CHARACTER(LEN=*), INTENT(IN) :: filename
+      !! name of file written (to which a suffix is added)
       INTEGER,            INTENT(IN) :: root_in_group
       !! root processor that collects and writes
       INTEGER,            INTENT(IN) :: intra_group_comm
@@ -450,7 +450,6 @@ MODULE io_base
       LOGICAL                  :: ionode_in_group
       INTEGER                  :: ngm, nspin, ngm_g, igwx
       INTEGER                  :: iun, ns, ig, ierr
-      CHARACTER(LEN=320)       :: filename
       !
 #if defined __HDF5
       TYPE (qeh5_file)          ::  h5file
@@ -482,13 +481,12 @@ MODULE io_base
       ngm_g = ngm
       CALL mp_sum( ngm_g, intra_group_comm )
       !
-      filename = TRIM( dirname ) // 'charge-density.dat'
       ierr = 0
 #if defined (__HDF5)
       IF ( ionode_in_group ) CALL qeh5_openfile(h5file, FILE = &
-           TRIM(dirname)//'charge-density.hdf5', ACTION = 'write', ERROR = ierr) 
+           TRIM(filename)//'.hdf5', ACTION = 'write', ERROR = ierr) 
 #else
-      IF ( ionode_in_group ) OPEN ( UNIT = iun, FILE = TRIM( filename ), &
+      IF ( ionode_in_group ) OPEN ( UNIT = iun, FILE = TRIM(filename)//'.dat', &
                 FORM = 'unformatted', STATUS = 'unknown', iostat = ierr )
 #endif
       CALL mp_bcast( ierr, root_in_group, intra_group_comm )
@@ -563,22 +561,9 @@ MODULE io_base
       !
       DO ns = 1, nspin
          !
-         ! Workaround for LSDA, while waiting for much-needed harmonization:
-         ! we have rhoup and rhodw, we write rhotot=up+dw and rhodif=up-dw
-         ! 
-         IF ( ns == 1 .AND. nspin == 2 ) THEN
-            DO ig = 1, ngm
-               rhoaux(ig) = rho(ig,ns) + rho(ig,ns+1)
-            END DO
-         ELSE IF ( ns == 2 .AND. nspin == 2 ) THEN
-            DO ig = 1, ngm
-               rhoaux(ig) = rho(ig,ns-1) - rho(ig,ns)
-            END DO
-        ELSE
-            DO ig = 1, ngm
+         DO ig = 1, ngm
                rhoaux(ig) = rho(ig,ns)
-            END DO
-         END IF
+         END DO
          !
          rho_g = 0
          CALL mergewf( rhoaux, rho_g, ngm, ig_l2g, me_in_group, &
@@ -615,10 +600,10 @@ MODULE io_base
     END SUBROUTINE write_rhog
     !
     !------------------------------------------------------------------------
-    SUBROUTINE read_rhog ( dirname, root_in_group, intra_group_comm, &
-         ig_l2g, nspin, rho, gamma_only )
+    SUBROUTINE read_rhog ( filename, root_in_group, intra_group_comm, &
+         ig_l2g, nspin, rho, gamma_only, ier_ )
       !------------------------------------------------------------------------
-      !! Read and distribute rho(G) from file  'charge-density.*' 
+      !! Read and distribute rho(G) from file  "filename".* 
       !! (* = dat if fortran binary, * = hdf5 if HDF5)
       !! Processor "root_in_group" reads from file, distributes to
       !! all processors in the intra_group_comm communicator 
@@ -632,10 +617,10 @@ MODULE io_base
 #endif
       IMPLICIT NONE
       !
-      CHARACTER(LEN=*), INTENT(IN) :: dirname
-      !! directory name where file is read - must end by '/'
+      CHARACTER(LEN=*), INTENT(IN) :: filename
+      !! name of file read (to which a suffix is added)
       INTEGER,          INTENT(IN) :: root_in_group
-      !! root processor that reads and sirtibutes
+      !! root processor that reads and distributes
       INTEGER,          INTENT(IN) :: intra_group_comm
       !! rho(G) is distributed over this group of processors
       INTEGER,          INTENT(IN) :: ig_l2g(:)
@@ -646,16 +631,16 @@ MODULE io_base
       COMPLEX(dp),  INTENT(INOUT) :: rho(:,:)
       !! temporary check while waiting for more definitive solutions
       LOGICAL, OPTIONAL, INTENT(IN) :: gamma_only
+      !! if present, don't stop in case of open error, return a nonzero value
+      INTEGER, OPTIONAL, INTENT(OUT):: ier_
       !
       COMPLEX(dp), ALLOCATABLE :: rho_g(:)
       COMPLEX(dp), ALLOCATABLE :: rhoaux(:)
-      COMPLEX(dp)              :: rhoup, rhodw
       REAL(dp)                 :: b1(3), b2(3), b3(3)
       INTEGER                  :: ngm, nspin_, isup, isdw
       INTEGER                  :: iun, ns, ig, ierr
       INTEGER                  :: me_in_group, nproc_in_group
       LOGICAL                  :: ionode_in_group, gamma_only_, readmill
-      CHARACTER(LEN=320)       :: filename
       INTEGER                  :: ngm_g_
       INTEGER, ALLOCATABLE     :: mill_g(:,:)
       !
@@ -672,9 +657,6 @@ MODULE io_base
          datasets(3)  = "m_y"
          datasets(4)  = "m_z"
       END IF
-      filename = TRIM( dirname ) // 'charge-density.hdf5'
-#else 
-      filename = TRIM( dirname ) // 'charge-density.dat'
 #endif 
       !
       ngm  = SIZE (rho, 1)
@@ -683,6 +665,7 @@ MODULE io_base
       !
       iun  = 4
       ierr = 0
+      IF ( PRESENT(ier_) ) ier_ = 0
       !
       me_in_group     = mp_rank( intra_group_comm )
       nproc_in_group  = mp_size( intra_group_comm )
@@ -690,7 +673,12 @@ MODULE io_base
       !
       IF ( ionode_in_group ) THEN
 #if defined (__HDF5) 
-         CALL qeh5_openfile(h5file, TRIM(filename), ACTION = 'read', error = ierr)
+         CALL qeh5_openfile(h5file, TRIM(filename)//'.hdf5', ACTION = 'read', &
+              error = ierr)
+         IF ( ierr /= 0 .AND. PRESENT(ier_) ) THEN
+            ier_ = ierr
+            RETURN
+         END IF
          CALL qeh5_read_attribute (h5file%id, "gamma_only", tempchar, MAXLEN = len(tempchar)  )
          CALL qeh5_read_attribute (h5file%id, "ngm_g", ngm_g_ ) 
          CALL qeh5_read_attribute (h5file%id, "nspin", nspin_)  
@@ -701,9 +689,13 @@ MODULE io_base
             gamma_only_ = .FALSE.
          END SELECT
 #else
-         OPEN ( UNIT = iun, FILE = TRIM( filename ), &
+         OPEN ( UNIT = iun, FILE = TRIM( filename ) // '.dat', &
               FORM = 'unformatted', STATUS = 'old', iostat = ierr )
          IF ( ierr /= 0 ) THEN
+            IF ( PRESENT(ier_) ) THEN
+               ier_ = ierr
+               RETURN
+            END IF
             ierr = 1
             GO TO 10
          END IF
@@ -728,7 +720,7 @@ MODULE io_base
       IF ( nspin > nspin_ ) &
          CALL infomsg('read_rhog', 'some spin components not found')
       IF ( ngm_g < MAXVAL (ig_l2g(:)) ) &
-           CALL infomsg('read_rhog', 'some G-vectors are missing' )
+           CALL infomsg('read_rhog', 'some G-vectors are missing, zero-padding' )
       !
       ! ... if required and if there is a mismatch between input gamma tricks
       ! ... and gamma tricks read from file: allocate and read Miller indices
@@ -762,7 +754,7 @@ MODULE io_base
       ! ... of the charge density (one spin at the time to save memory)
       !
       IF ( ionode_in_group ) THEN
-         ALLOCATE( rho_g( ngm_g_ ) )
+         ALLOCATE( rho_g(MAX(ngm_g_,ngm_g)) )
       ELSE
          ALLOCATE( rho_g( 1 ) )
       END IF
@@ -778,6 +770,7 @@ MODULE io_base
 #else 
             READ (iun, iostat=ierr) rho_g(1:ngm_g_)
 #endif
+            IF ( ngm_g > ngm_g_) rho_g(ngm_g_+1:ngm_g) = cmplx(0.d0,0.d0, KIND = DP) 
          END IF
          CALL mp_bcast( ierr, root_in_group, intra_group_comm )
          IF ( ierr > 0 ) CALL errore ( 'read_rhog','error reading file ' &
@@ -786,26 +779,15 @@ MODULE io_base
          ! ... Convert charge from full G-vector to half G-vector format
          !
          IF ( readmill ) CALL charge_k_to_g (ngm_g_, rho_g, mill_g, &
-              root_in_group,intra_group_comm)
+              root_in_group,intra_group_comm, gamma_only)
          !
          CALL splitwf( rhoaux, rho_g, ngm, ig_l2g, me_in_group, &
               nproc_in_group, root_in_group, intra_group_comm )
          DO ig = 1, ngm
             rho(ig,ns) = rhoaux(ig)
          END DO
-         !
-         ! Workaround for LSDA, while waiting for much-needed harmonization:
-         ! if file contains rhotot=up+dw and rhodif=up-dw (nspin_=2), and
-         ! if we want rhoup and rho down (nspin=2), convert 
          ! 
-         IF ( nspin_ == 2 .AND. nspin == 2 .AND. ns == 2 ) THEN
-            DO ig = 1, ngm
-               rhoup = (rho(ig,ns-1) + rhoaux(ig)) / 2.0_dp
-               rhodw = (rho(ig,ns-1) - rhoaux(ig)) / 2.0_dp
-               rho(ig,ns-1)= rhoup
-               rho(ig,ns  )= rhodw
-            END DO
-         ELSE IF ( nspin_ == 2 .AND. nspin == 4 .AND. ns == 2) THEN 
+         IF ( nspin_ == 2 .AND. nspin == 4 .AND. ns == 2) THEN 
             DO ig = 1, ngm 
                rho(ig, 4 ) = rho(ig, 2) 
                rho(ig, 2 ) = cmplx(0.d0,0.d0, KIND = DP) 
@@ -828,40 +810,46 @@ MODULE io_base
       !
     END SUBROUTINE read_rhog
     !
-    SUBROUTINE charge_k_to_g ( ngm_g_k, rho_g, mill_g_k, root_in_group, &
-         intra_group_comm )
+    SUBROUTINE charge_k_to_g ( ngm_g_file, rho_g, mill_g_file, root_in_group, &
+         intra_group_comm , this_run_is_gamma_only)
    !
    ! this routine reorders G-vectors for the charge density on global mesh
    ! from the k case to the gamma-only one
    !
       USE io_global,     ONLY : stdout
-      USE gvect,         ONLY : ngm, ngm_g, ig_l2g, mill
+      USE gvect,         ONLY : ngm, ngm_g, ig_l2g, mill, igtongl, ngl, gl
       USE mp,            ONLY : mp_size,mp_rank
       USE mp_wave,       ONLY : mergewf, mergekg
      
-      implicit none
+      IMPLICIT NONE
       INTEGER, INTENT(in) :: intra_group_comm,root_in_group
-      INTEGER, INTENT(in) :: ngm_g_k!relative to k case
-      INTEGER, INTENT(in) :: mill_g_k(:,:)
+      INTEGER, INTENT(in) :: ngm_g_file  
+      !! number of g vectors found in file 
+      INTEGER, INTENT(in) :: mill_g_file(:,:)
       COMPLEX(kind=DP), INTENT(inout) :: rho_g(:)!relative to k case  in input, gamma case in output
-       INTEGER                  :: me_in_group, npr
+      LOGICAL, OPTIONAL, INTENT(in) :: this_run_is_gamma_only 
+      INTEGER                  :: me_in_group, npr
       COMPLEX(kind=DP), ALLOCATABLE :: rho_aux(:)
       LOGICAL                  :: ionode_in_group
       INTEGER :: nproc_in_group
       INTEGER, ALLOCATABLE :: mill_g(:,:)
-      
-      INTEGER :: ig, jg
-
-      write(stdout,*) 'Conversion: K charge Gamma charge'
+      CHARACTER(len=256)   :: mesg
+      INTEGER :: ig, jg, minus_g_file(3), startjg, old_ig, new_startjg  
+      IF ( .NOT. PRESENT (this_run_is_gamma_only) ) RETURN 
+      IF ( this_run_is_gamma_only) THEN 
+         call infomsg('read_rhog','Conversion: K charge Gamma charge') 
+      ELSE 
+         call infomsg ('read_rhog', 'Conversion: Gamma charge to K charge') 
+      ENDIF 
 
       me_in_group     = mp_rank( intra_group_comm )
       nproc_in_group  = mp_size( intra_group_comm )
       ionode_in_group = ( me_in_group == root_in_group )
 
       IF(ionode_in_group) THEN
-         allocate(rho_aux(ngm_g_k))
+         allocate(rho_aux(MAX(ngm_g_file, ngm_g) ))
          allocate(mill_g(3,ngm_g))
-         rho_aux(1:ngm_g_k)=rho_g(1:ngm_g_k)
+         rho_aux(1:ngm_g_file)=rho_g(1:ngm_g_file)
       ELSE
          allocate(rho_aux(1))
          allocate(mill_g(1,1))
@@ -871,21 +859,73 @@ MODULE io_base
            nproc_in_group, root_in_group, intra_group_comm )
 
       IF(ionode_in_group) THEN
-         rho_g(1:ngm_g_k)=0.d0
-         DO ig=1,ngm_g
-            DO jg=1,ngm_g_k
-               if(  mill_g(1,ig)==mill_g_k(1,jg) .and. &
-                    mill_g(2,ig)==mill_g_k(2,jg) .and. &
-                    mill_g(3,ig)==mill_g_k(3,jg) ) then
+         rho_g(:)= cmplx(0.d0, 0.d0,KIND = DP) 
+         IF ( this_run_is_gamma_only ) THEN 
+            ig = 1 
+            DO jg=1,ngm_g_file
+               if(  mill_g(1,ig)==mill_g_file(1,jg) .and. &
+                  mill_g(2,ig)==mill_g_file(2,jg) .and. &
+                  mill_g(3,ig)==mill_g_file(3,jg) ) then
                   rho_g(ig)=rho_aux(jg)
+                  ig = ig + 1 
                endif
-            END DO
-         ENDDO
+               IF ( ig .GE. ngm_g ) EXIT  
+               END DO
+         ELSE ! this run uses full fft mesh 
+            ig = 1 
+            DO jg = 1, ngm_g
+               if (  mill_g (1,jg) == mill_g_file(1,ig) .and. &
+                     mill_g (2,jg) == mill_g_file(2,ig) .and. &
+                     mill_g (3,jg) == mill_g_file(3,ig) )  then 
+                  !
+                  rho_g(jg) = rho_aux(ig) 
+                  ig = ig + 1 
+               end if 
+               if ( ig .GE. ngm_g_file ) EXIT 
+            END DO 
+            WRITE (stdout, *) " Plus vectors done"
+            WRITE (stdout, *) " Search of minus vectors is going to take a while" 
+            ig = 1 
+            minus_g_file = minus_g(ig) 
+            startjg = 1 
+            new_startjg = 1 
+            igloop: DO 
+               old_ig = ig 
+               DO jg = startjg, ngm_g  
+                  if ( mill_g (1,jg) == minus_g_file(1) .and. &
+                       mill_g (2,jg) == minus_g_file(2) .and. &
+                       mill_g (3,jg) == minus_g_file(3)  )  then 
+                       !
+                       rho_g(jg) = dconjg(rho_aux(ig))
+                       ig = ig + 1 
+                       if ( ig .le. ngm_g_file) minus_g_file = minus_g(ig) 
+                       if (jg == new_startjg) new_startjg = new_startjg+1 
+                  end if 
+                  if ( ig .GT. ngm_g_file ) EXIT igloop
+               END DO 
+               startjg =  new_startjg    
+               IF ( ig == old_ig) THEN 
+                  WRITE(mesg,*)  "minus_g for ig = ", old_ig,":" , minus_g_file,  "-->", mill_g_file(:,ig), " not found" 
+                  CALL infomsg("read_rhog:" , TRIM(mesg)) 
+                  ig = ig +1 
+                  minus_g_file = minus_g(ig) 
+                  startjg = 1 
+               END IF
+            END DO igloop 
+         END IF
       ENDIF
         
       deallocate(rho_aux,mill_g)
       
       return
+      CONTAINS 
+         function minus_g  (imill) result ( minus_mill) 
+            implicit none
+            integer   :: minus_mill(3)  
+            integer   :: ipol, imill 
+            minus_mill = - mill_g_file(:,imill) 
+         end function minus_g 
+ 
     END SUBROUTINE charge_k_to_g
     !
   END MODULE io_base
