@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2002-2013 Quantum ESPRESSO group
+! Copyright (C) 2002-2020 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -12,6 +12,8 @@ MODULE io_files
   USE parameters, ONLY: ntypx
   USE kinds,      ONLY: dp
   USE io_global,  ONLY: ionode, ionode_id, stdout
+  USE mp,         ONLY: mp_barrier, mp_bcast, mp_sum
+  USE mp_images,  ONLY: me_image, intra_image_comm, nproc_image
   !
   ! ... I/O related variables: file names, units, utilities
   ! ... IMPORTANT: when directory names are set, they must always end with "/"
@@ -20,7 +22,7 @@ MODULE io_files
   !
   SAVE
   PUBLIC :: create_directory, check_tempdir, clean_tempdir, check_file_exist, &
-       delete_if_present, check_writable
+       delete_if_present, check_writable, restart_dir, xmlfile, check_restartfile
   !
   ! ... directory for all temporary files
   CHARACTER(len=256) :: tmp_dir = './'
@@ -30,7 +32,11 @@ MODULE io_files
   CHARACTER(len=256) :: prefix  = 'os'
   ! ... postfix is appended to directory names
 #if defined (_WIN32)
+#if defined (__PGI)
+  CHARACTER(len=6) :: postfix  = '.save/'
+#else
   CHARACTER(len=6) :: postfix  = '.save\'
+#endif
 #else
   CHARACTER(len=6) :: postfix  = '.save/'
 #endif
@@ -42,21 +48,17 @@ MODULE io_files
   CHARACTER(len=256) :: pseudo_dir_cur = ' '
   CHARACTER(len=256) :: psfile( ntypx ) = 'UPF'
   !
-  CHARACTER(len=256) :: qexml_version = ' '       ! the format of the current qexml datafile 
-  LOGICAL            :: qexml_version_init = .FALSE.  ! whether the fmt has been read or not
-  !
-!
   CHARACTER(LEN=256) :: qexsd_fmt = ' ', qexsd_version = ' '
   LOGICAL            :: qexsd_init = .FALSE. 
-!
-  CHARACTER(LEN=256) :: input_drho = ' '          ! name of the file with the input drho
-  CHARACTER(LEN=256) :: output_drho = ' '         ! name of the file with the output drho
+  ! ... next two variables are no longer read from input but can be set
+  ! ... by external codes using QE routines to perform an interpolation
+  ! ... of valence electrons only, without the atomic-like part
+  CHARACTER(LEN=256) :: input_drho = ' '
+  CHARACTER(LEN=256) :: output_drho= ' '
   !
   CHARACTER(LEN=5 ), PARAMETER :: crash_file  = 'CRASH'
   CHARACTER (LEN=261) :: exit_file = 'os.EXIT' ! file required for a soft exit  
   !
-  CHARACTER (LEN=13), PARAMETER :: xmlpun      = 'data-file.xml'
-  !
   CHARACTER (LEN=20), PARAMETER :: xmlpun_schema = 'data-file-schema.xml'
   !
   ! ... The units where various variables are saved
@@ -67,7 +69,7 @@ MODULE io_files
   INTEGER :: iunwfc      = 10 ! unit with wavefunctions
   INTEGER :: iunoldwfc   = 11 ! unit with old wavefunctions
   INTEGER :: iunoldwfc2  = 12 ! as above at step -2
-  INTEGER :: iunhub      = 13 ! unit for saving Hubbard-U atomic wfcs 
+  INTEGER :: iunhub      = 13 ! unit for saving Hubbard-U atomic wfcs * S 
   INTEGER :: iunsat      = 14 ! unit for saving (orthogonal) atomic wfcs * S
   INTEGER :: iunmix      = 15 ! unit for saving mixing information
   INTEGER :: iunwfc_exx  = 16 ! unit with exx wavefunctions
@@ -110,8 +112,6 @@ CONTAINS
     !------------------------------------------------------------------------
     !
     USE wrappers,  ONLY : f_mkdir_safe
-    USE mp,        ONLY : mp_barrier, mp_bcast
-    USE mp_images, ONLY : me_image, intra_image_comm
     !
     CHARACTER(LEN=*), INTENT(IN) :: dirname
     !
@@ -122,7 +122,11 @@ CONTAINS
     length = LEN_TRIM(dirname)
 #if defined (_WIN32)
     ! Windows returns error if tmp_dir ends with a backslash
+#if defined (__PGI)
+    IF ( dirname(length:length) == '\\' ) length=length-1
+#else
     IF ( dirname(length:length) == '\' ) length=length-1
+#endif
 #endif
     IF ( ionode ) ierr = f_mkdir_safe( dirname(1:length ) )
     CALL mp_bcast ( ierr, ionode_id, intra_image_comm )
@@ -156,8 +160,6 @@ CONTAINS
     ! ...    pfs = .t. if tmp_dir visible from all procs of an image
     !
     USE wrappers,      ONLY : f_mkdir_safe
-    USE mp_images,     ONLY : intra_image_comm, nproc_image, me_image
-    USE mp,            ONLY : mp_barrier, mp_bcast, mp_sum
     !
     IMPLICIT NONE
     !
@@ -176,12 +178,17 @@ CONTAINS
     length = LEN_TRIM(tmp_dir)
 #if defined (_WIN32)
     ! Windows returns error if tmp_dir ends with a backslash
+#if defined (__PGI)
+    IF ( tmp_dir(length:length) == '\\' ) length=length-1
+#else
     IF ( tmp_dir(length:length) == '\' ) length=length-1
+#endif
 #endif
     IF ( ionode ) ios = f_mkdir_safe( tmp_dir(1:length) )
     CALL mp_bcast ( ios, ionode_id, intra_image_comm )
     exst = ( ios == -1 )
-    IF ( ios > 0 ) CALL errore ('check_tempdir','tmp_dir cannot be opened',1)
+    IF ( ios > 0 ) CALL errore ('check_tempdir', 'temporary directory ' &
+            & // tmp_dir(1:length) // ' cannot be created or accessed',1)
     !
     ! ... let us check now if tmp_dir is visible on all nodes
     ! ... if not, a local tmp_dir is created on each node
@@ -221,9 +228,6 @@ CONTAINS
   FUNCTION check_file_exist( filename )
     !------------------------------------------------------------------------
     !
-    USE mp,        ONLY : mp_bcast
-    USE mp_images, ONLY : intra_image_comm
-    !
     IMPLICIT NONE
     !
     LOGICAL          :: check_file_exist
@@ -245,41 +249,54 @@ CONTAINS
   END FUNCTION check_file_exist
   !
   !--------------------------------------------------------------------------
-  SUBROUTINE delete_if_present( filename, in_warning )
-    !--------------------------------------------------------------------------
+  SUBROUTINE delete_if_present(filename, para)
+  !--------------------------------------------------------------------------
+  !!
+  !! Same as the delete_if_present subroutine but allows for other cores to 
+  !! enters (SP - Jan 2020). Ideally, both could be merged. 
+  !!  
+  !
+  IMPLICIT NONE
+  !
+  CHARACTER(len = *), INTENT(in) :: filename
+  !! Name of the file 
+  LOGICAL, OPTIONAL, INTENT(in) :: para
+  !! Optionally, the remove can be done by all the cores. 
+  ! 
+  ! Local variables
+  LOGICAL :: exst
+  !! Check if the file exist
+  INTEGER :: iunit
+  !! UNit of the file 
+  INTEGER, EXTERNAL :: find_free_unit
+  !! Find a unallocated unit
+  ! 
+  IF (PRESENT(para)) THEN
+    IF (.NOT. para) THEN
+      IF (.NOT. ionode) RETURN
+    ENDIF
+  ELSE ! Default if not present
+    IF (.NOT. ionode) RETURN
+  ENDIF
+  !
+  INQUIRE(FILE = filename, EXIST = exst)
+  !
+  IF (exst) THEN
     !
-    IMPLICIT NONE
+    iunit = find_free_unit()
     !
-    CHARACTER(LEN=*),  INTENT(IN) :: filename
-    LOGICAL, OPTIONAL, INTENT(IN) :: in_warning
-    LOGICAL                       :: exst, warning
-    INTEGER                       :: iunit
-    INTEGER, EXTERNAL :: find_free_unit
+    OPEN(UNIT = iunit, FILE = filename, STATUS = 'OLD')
+    CLOSE(UNIT = iunit, STATUS = 'DELETE')
     !
-    IF ( .NOT. ionode ) RETURN
-    !
-    INQUIRE( FILE = filename, EXIST = exst )
-    !
-    IF ( exst ) THEN
-       !
-       iunit = find_free_unit()
-       !
-       warning = .FALSE.
-       !
-       IF ( PRESENT( in_warning ) ) warning = in_warning
-       !
-       OPEN(  UNIT = iunit, FILE = filename , STATUS = 'OLD' )
-       CLOSE( UNIT = iunit, STATUS = 'DELETE' )
-       !
-       IF ( warning ) &
-          WRITE( UNIT = stdout, FMT = '(/,5X,"WARNING: ",A, &
-               & " file was present; old file deleted")' ) filename
-       !
-    END IF
-    !
-    RETURN
+    WRITE(UNIT = stdout, FMT = '(/,5X,"File ", A, " deleted, as requested")') TRIM(filename)
     !
+  ENDIF
+  !
+  RETURN
+  ! 
+  !--------------------------------------------------------------------------
   END SUBROUTINE delete_if_present
+  !--------------------------------------------------------------------------
   !
   !--------------------------------------------------------------------------
   FUNCTION check_writable ( file_path, process_id ) RESULT ( ios )
@@ -314,7 +331,72 @@ CONTAINS
     !
     !-----------------------------------------------------------------------
   END FUNCTION check_writable 
-!-----------------------------------------------------------------------
+  !-----------------------------------------------------------------------
+  !
+  !------------------------------------------------------------------------
+  FUNCTION restart_dir( runit )
+    !------------------------------------------------------------------------
+    !
+    CHARACTER(LEN=256)  :: restart_dir
+    INTEGER, INTENT(IN), OPTIONAL :: runit
+    !
+    CHARACTER(LEN=6), EXTERNAL :: int_to_char
+    !
+    ! ... main restart directory (contains final / or Windows equivalent)
+    !
+    IF ( PRESENT (runit) ) THEN
+       restart_dir = TRIM(tmp_dir) // TRIM(prefix) // '_' // &
+               TRIM(int_to_char(runit)) // postfix
+    ELSE
+       restart_dir = TRIM(tmp_dir) // TRIM(prefix) // postfix
+    END IF
+    !
+    RETURN
+    !
+  END FUNCTION restart_dir
+  !
+  !------------------------------------------------------------------------
+  FUNCTION xmlfile ( runit )
+    !------------------------------------------------------------------------
+    !
+    CHARACTER(LEN=320)  :: xmlfile
+    INTEGER, INTENT(IN), OPTIONAL :: runit
+    !
+    ! ... xml file in main restart directory 
+    !
+    xmlfile = TRIM( restart_dir(runit) ) // xmlpun_schema
+    !
+    RETURN
+    !
+    END FUNCTION xmlfile
+    !
+    !------------------------------------------------------------------------
+    FUNCTION check_restartfile( ndr )
+      !------------------------------------------------------------------------
+      !
+      IMPLICIT NONE
+      !
+      LOGICAL                      :: check_restartfile
+      INTEGER, INTENT(IN), OPTIONAL:: ndr
+      CHARACTER(LEN=320)           :: filename
+      LOGICAL                      :: lval
+      !
+      !
+      filename = xmlfile( ndr )
+      !
+      IF ( ionode ) THEN
+         !
+         INQUIRE( FILE = TRIM( filename ), EXIST = lval )
+         !
+      END IF
+      !
+      CALL mp_bcast( lval, ionode_id, intra_image_comm )
+      !
+      check_restartfile = lval
+      !
+      RETURN
+      !
+    END FUNCTION check_restartfile
 !
 !-----------------------------------------------------------------------
 subroutine diropn (unit, extension, recl, exst, tmp_dir_)
@@ -512,13 +594,13 @@ SUBROUTINE davcio( vect, nword, unit, nrec, io )
      !
      READ( UNIT = unit, REC = nrec, IOSTAT = ios ) vect
      IF ( ios /= 0 ) CALL errore( 'davcio', &
-         & 'error while reading from file "' // TRIM(name) // '"', unit )
+         & 'error reading file "' // TRIM(name) // '"', unit )
      !
   ELSE IF ( io > 0 ) THEN
      !
      WRITE( UNIT = unit, REC = nrec, IOSTAT = ios ) vect
      IF ( ios /= 0 ) CALL errore( 'davcio', &
-         & 'error while writing from file "' // TRIM(name) // '"', unit )
+         & 'error writing file "' // TRIM(name) // '"', unit )
      !
   END IF
   !
