@@ -178,6 +178,10 @@ CONTAINS
             CALL card_kpoints( input_line )
          ENDIF
          !
+      ELSEIF ( trim(card) == 'ADDITIONAL_K_POINTS' ) THEN
+         !
+         CALL card_add_kpoints( input_line )
+ 
       ELSEIF ( trim(card) == 'OCCUPATIONS' ) THEN
          !
          CALL card_occupations( input_line )
@@ -870,6 +874,68 @@ CONTAINS
             & // trim(k_points) // ' k points', 1)
       !
    END SUBROUTINE card_kpoints
+
+   SUBROUTINE card_add_kpoints( input_line )
+     USE additional_kpoints, ONLY : nkstot_add, xk_add
+     IMPLICIT NONE
+     CHARACTER(len=*),INTENT(in) :: input_line
+     CHARACTER(len=256) :: input_line_aux
+     REAL(DP),ALLOCATABLE :: xk_old(:,:), wk_old(:)
+     INTEGER :: nk1_old, nk2_old, nk3_old, nkstot_old
+     INTEGER :: k1_old,  k2_old,  k3_old
+     LOGICAL, EXTERNAL  :: matches
+     !
+     IF(.not.allocated(xk) .or. .not.allocated(wk))&
+       CALL errore("add_kpoints", "ADDITIONAL_K_POINTS must appear after K_POINTS",1)
+     IF(.not.tkpoints) &
+       CALL errore("add_kpoints", "ADDITIONAL_K_POINTS must appear after K_POINTS",2)
+     IF(matches( "AUTOMATIC", input_line )) &
+       CALL errore("add_kpoints", "ADDITIONAL_K_POINTS cannot be 'automatic'", 3)
+
+     ! Back-up existing points
+     nkstot_old = nkstot
+     ALLOCATE(xk_old(3,nkstot_old))
+     ALLOCATE(wk_old(nkstot_old))
+     xk_old  = xk
+     wk_old  = wk
+     nk1_old = nk1
+     nk2_old = nk2
+     nk3_old = nk3
+     k1_old  = k1
+     k2_old  = k2
+     k3_old  = k3
+     DEALLOCATE(xk,wk)
+
+     ! Prepare to read k-points again
+     nkstot = 0
+     input_line_aux = TRIM(ADJUSTL(input_line))
+     input_line_aux = input_line_aux(12:)
+     tkpoints = .false.
+     CALL card_kpoints(input_line_aux)
+     !
+     ! Backup new points to module
+     nkstot_add = nkstot
+     IF(nkstot_add==0) CALL errore("add_kpoints", "No new k_points?",1)
+     ALLOCATE(xk_add(3,nkstot_add))
+     xk_add = xk
+
+     ! Put back previous stuff
+     DEALLOCATE(xk, wk)
+     nkstot = nkstot_old
+     ALLOCATE(xk(3,nkstot))
+     ALLOCATE(wk(nkstot))
+     xk  = xk_old
+     wk  = wk_old
+     nk1 = nk1_old
+     nk2 = nk2_old
+     nk3 = nk3_old
+     k1  = k1_old
+     k2  = k2_old
+     k3  = k3_old
+     DEALLOCATE(xk_old,wk_old)
+
+     RETURN 
+   END SUBROUTINE card_add_kpoints
    !
    !------------------------------------------------------------------------
    !    BEGIN manual
@@ -1170,7 +1236,7 @@ CONTAINS
                READ(input_line,*) lb_vel, ( rd_vel(k,ia), k = 1, 3 )
             ELSE
                CALL errore( ' iosys ', &
-                           & ' wrong entries in ION_VELOCITIES ', ia )
+                           & ' wrong entries in ATOMIC_VELOCITIES ', ia )
             ENDIF
             !
             match_label: DO is = 1, ntyp
@@ -1805,7 +1871,7 @@ CONTAINS
                IF ( nfield == 4 ) THEN
                   READ(input_line,*) text, iwan, b_from, b_to
                   ning = 1
-               ELSEIF ( nfield == 4 ) THEN
+               ELSEIF ( nfield == 5 ) THEN
                   READ(input_line,*) text, iwan, b_from, b_to, ning
                ELSE
                   CALL errore( 'read_cards', &
