@@ -318,58 +318,3 @@ CONTAINS
   !=----------------------------------------------------------------------=
 END MODULE recvec_subs
 !=----------------------------------------------------------------------=
-!
-!-----------------------------------------------------------------------
-SUBROUTINE gshells ( vc )
-   !----------------------------------------------------------------------
-   !
-   ! calculate number of G shells: ngl, and the index ng = igtongl(ig)
-   ! that gives the shell index ng for (local) G-vector of index ig
-   !
-   USE kinds,              ONLY : DP
-   USE gvect,              ONLY : gg, ngm, gl, ngl, igtongl
-   USE constants,          ONLY : eps8
-   !
-   IMPLICIT NONE
-   !
-   LOGICAL, INTENT(IN) :: vc
-   !
-   INTEGER :: ng, igl
-   !
-   IF ( vc ) THEN
-      !
-      ! in case of a variable cell run each G vector has its shell
-      !
-      ngl = ngm
-      gl => gg
-      DO ng = 1, ngm
-         igtongl (ng) = ng
-      ENDDO
-   ELSE
-      !
-      ! G vectors are grouped in shells with the same norm
-      !
-      ngl = 1
-      igtongl (1) = 1
-      DO ng = 2, ngm
-         IF (gg (ng) > gg (ng - 1) + eps8) THEN
-            ngl = ngl + 1
-         ENDIF
-         igtongl (ng) = ngl
-      ENDDO
-
-      ALLOCATE (gl( ngl))
-      gl (1) = gg (1)
-      igl = 1
-      DO ng = 2, ngm
-         IF (gg (ng) > gg (ng - 1) + eps8) THEN
-            igl = igl + 1
-            gl (igl) = gg (ng)
-         ENDIF
-      ENDDO
-
-      IF (igl /= ngl) CALL errore ('gshells', 'igl <> ngl', ngl)
-
-   ENDIF
-
-   END SUBROUTINE gshells
