@@ -1,15 +1,15 @@
-# LaTeX2HTML 2008 (1.71)
+# LaTeX2HTML 2019.2 (Released June 5, 2019)
 # Associate labels original text with physical files.
 
 
 $key = q/Sec:para/;
-$external_labels{$key} = "$URL/" . q|node10.html|; 
+$external_labels{$key} = "$URL/" . q|node16.html|; 
 $noresave{$key} = "$nosave";
 
 1;
 
 
-# LaTeX2HTML 2008 (1.71)
+# LaTeX2HTML 2019.2 (Released June 5, 2019)
 # labels from external_latex_labels array.
 
 
@@ -17,5 +17,9 @@ $key = q/Sec:para/;
 $external_latex_labels{$key} = q|5|; 
 $noresave{$key} = "$nosave";
 
+$key = q/_/;
+$external_latex_labels{$key} = q|<|; 
+$noresave{$key} = "$nosave";
+
 1;
 
