@@ -16,10 +16,9 @@ SUBROUTINE cg_setup
   USE scf,        ONLY: rho, rho_core, v, vltot, vrs, kedtau
   USE uspp,       ONLY: vkb, nlcc_any
   USE uspp_param, ONLY: upf
-  USE mp_global,  ONLY: kunit
-  USE wavefunctions_module,  ONLY: evc
+  USE wavefunctions,  ONLY: evc
   USE io_files,   ONLY: prefix, iunpun, iunres, diropn
-  USE funct,      ONLY: dft_is_gradient, dmxc
+  USE funct,      ONLY: dft_is_gradient
   USE dfunct,     ONLY: newd
   USE fft_base,   ONLY: dfftp
   USE gvect,      ONLY: g, ngm, eigts1, eigts2, eigts3
@@ -29,16 +28,17 @@ SUBROUTINE cg_setup
   USE vlocal,     ONLY: strf
   USE wvfct,      ONLY: nbnd, npwx
   USE gvecw,      ONLY: gcutw
-  USE cgcom
+  USE gc_lr, ONLY:  grho, dvxc_rr, dvxc_sr, dvxc_ss, dvxc_s
+  USE cgcom, ONLY: dmuxc, dvpsi, dpsi, auxr, aux2, aux3, lrwfc
   !
   IMPLICIT NONE
   !
   INTEGER :: i, l, nt, ik
   LOGICAL :: exst
   CHARACTER (len=256) :: filint
-  REAL(DP) :: rhotot
-  INTEGER       :: ndr, kunittmp, ierr
+  INTEGER  :: ndr, ierr
   REAL(DP) :: edum(1,1), wdum(1,1)
+  REAL(DP), DIMENSION(dfftp%nnr) :: rhotot
   !
   CALL start_clock('cg_setup')
   !
@@ -65,37 +65,18 @@ SUBROUTINE cg_setup
      ALLOCATE  ( grho   (3, dfftp%nnr, nspin))
   ENDIF
   !
-  !
-  !  initialize structure factor array
-  !
-  CALL struc_fact ( nat, tau, ntyp, ityp, ngm, g, bg,               &
-       &                  dfftp%nr1, dfftp%nr2, dfftp%nr3, strf, eigts1, eigts2, eigts3 )
-  !
   !  compute drhocore/dtau for each atom type (if needed)
   !
   nlcc_any = any  ( upf(1:ntyp)%nlcc )
   !!! if (nlcc_any) call set_drhoc(xq, drc)
   !
-  !  local potential
+  rhotot(:) = rho%of_r(:,1) + rho_core(:)
   !
-  CALL init_vloc
-  !
-  CALL init_us_1
-  !
-  CALL newd
-  !
-  !  derivative of the xc potential
-  !
-  dmuxc(:) = 0.d0
-  DO i = 1,dfftp%nnr
-     rhotot = rho%of_r(i,current_spin)+rho_core(i)
-     IF ( rhotot> 1.d-30 ) dmuxc(i)= dmxc( rhotot)
-     IF ( rhotot<-1.d-30 ) dmuxc(i)=-dmxc(-rhotot)
-  ENDDO
+  CALL dmxc_lda( dfftp%nnr, rhotot, dmuxc )
   !
   !  initialize data needed for gradient corrections
   !
-  CALL cg_setupdgc
+  CALL setup_dgc( ) 
   !
   iunres=88
   !
