@@ -413,37 +413,34 @@ END SUBROUTINE cg_eps0dyn
 SUBROUTINE cg_neweps
   !-----------------------------------------------------------------------
   !
+  !!  Recalculate self-consistent potential etc
+  !
   USE constants, ONLY : bohr_radius_angs, fpi
   USE io_global, ONLY : stdout
   USE cell_base, ONLY : omega
   USE ions_base, ONLY : nat, tau
   USE fft_base,  ONLY : dfftp
   USE scf,       ONLY : rho, rho_core
-  USE lsda_mod,  ONLY : current_spin
-  USE funct,     ONLY : dmxc
   USE cgcom
   !
   IMPLICIT NONE
-
-  INTEGER :: i, j
-  REAL(DP) :: rhotot, chi(3,3)
   !
-  !  recalculate self-consistent potential etc
+  INTEGER :: i, j
+  REAL(DP), DIMENSION(3,3) :: chi(3,3)
+  REAL(DP), DIMENSION(dfftp%nnr) ::  rhotot, sign_r
   !
   CALL newscf
   !
-  !  new derivative of the xc potential
+  !  new derivative of the xc potential - NOT IMPLEMENTED FOR LSDA
+  !
+  rhotot(:) = rho%of_r(:,1) + rho_core(:)
+  !
+  CALL dmxc_lda( dfftp%nnr, rhotot, dmuxc )
   !
-  dmuxc(:) = 0.d0
-  DO i = 1,dfftp%nnr
-     rhotot = rho%of_r(i,current_spin)+rho_core(i)
-     IF ( rhotot> 1.d-30 ) dmuxc(i)= dmxc( rhotot)
-     IF ( rhotot<-1.d-30 ) dmuxc(i)=-dmxc(-rhotot)
-  ENDDO
   !
   !  re-initialize data needed for gradient corrections
   !
-  CALL cg_setupdgc
+  CALL setup_dgc( )
   !
   !   calculate linear response to macroscopic fields
   !
@@ -485,7 +482,7 @@ SUBROUTINE newscf
   USE wvfct, ONLY: nbnd, nbndx
   USE noncollin_module, ONLY: report
   USE symm_base,     ONLY : nsym
-  USE io_files,      ONLY : iunwfc, input_drho, output_drho, prefix, tmp_dir, postfix
+  USE io_files,      ONLY : iunwfc, prefix, tmp_dir, postfix
   USE ldaU,          ONLY : lda_plus_u
   USE control_flags, ONLY : restart, io_level, lscf, iprint, &
                             david, max_cg_iter, &
@@ -508,8 +505,6 @@ SUBROUTINE newscf
   doublegrid=.false.
   lmovecell=.false.
   iprint=10000
-  input_drho=' '
-  output_drho=' '
   starting_wfc='file'
   report=1
   if ( .not. allocated (btype) ) then
