@@ -24,15 +24,14 @@ subroutine compute_drhous_nc (drhous, dbecsum, wgg, becq, alpq)
   USE fft_interfaces, ONLY : invfft
   USE wvfct,      ONLY : npwx, nbnd
   USE noncollin_module, ONLY : npol, nspin_mag
-  USE wavefunctions_module,  ONLY: evc
+  USE wavefunctions,  ONLY: evc
   USE uspp,       ONLY: okvan, nkb, vkb
   USE uspp_param, ONLY: nhm
 
   USE qpoint,     ONLY : nksq, ikks, ikqs
   USE eqv,        ONLY : evq
   USE control_lr, ONLY : lgamma
-
-  USE units_ph,   ONLY : lrwfc, iuwfc
+  USE units_lr,   ONLY : lrwfc, iuwfc
   USE becmod,     ONLY : bec_type
 
   implicit none
