@@ -31,7 +31,6 @@ SUBROUTINE ep_matrix_element_wannier()
   USE paw_variables, ONLY : okpaw
   USE uspp_param, ONLY : nhm
   USE lsda_mod, ONLY : nspin
-
   USE lrus,   ONLY : int3, int3_nc, int3_paw
   USE qpoint, ONLY : xq, nksq, ikks
   !
@@ -163,7 +162,7 @@ SUBROUTINE elphsum_wannier(q_index)
   USE kinds, ONLY : DP
   USE ions_base, ONLY : nat, ityp, tau,amass,tau, ntyp => nsp, atm
   USE cell_base, ONLY : at, bg, ibrav, celldm 
-  USE symm_base, ONLY : s, sr, irt, nsym, time_reversal, invs, ftau, copy_sym, inverse_s
+  USE symm_base, ONLY : s, sr, irt, nsym, time_reversal, invs, copy_sym, inverse_s
   USE klist, ONLY : xk, nelec
   USE wvfct, ONLY : nbnd, et
   USE el_phon
@@ -264,7 +263,7 @@ SUBROUTINE elphsum_wannier(q_index)
      sym = .false.
      sym(1:nsym) = .true.
      
-     call smallg_q (xq, 0, at, bg, nsym, s, ftau, sym, minus_qloc)
+     call smallg_q (xq, 0, at, bg, nsym, s, sym, minus_qloc)
      nsymq = copy_sym(nsym, sym)
      ! recompute the inverses as the order of sym.ops. has changed
      CALL inverse_s ( )
@@ -329,7 +328,7 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
   !
   USE kinds, ONLY : DP
   USE fft_base, ONLY : dffts
-  USE wavefunctions_module,  ONLY: evc
+  USE wavefunctions,  ONLY: evc
   USE io_files, ONLY: prefix, diropn
   USE klist, ONLY: xk, ngk, igk_k
   USE lsda_mod, ONLY: lsda, current_spin, isk
@@ -351,6 +350,8 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
   USE eqv,        ONLY : dvpsi!, evq
   USE qpoint,     ONLY : nksq, ikks, ikqs
   USE control_lr, ONLY : lgamma
+  USE lrus,       ONLY : becp1
+  USE phus,       ONLY : alphap
 
   IMPLICIT NONE
   !
@@ -438,7 +439,7 @@ SUBROUTINE elphel_refolded (npe, imode0, dvscfins)
         ELSE
            mode = imode0 + ipert
            ! FIXME : .false. or .true. ???
-           CALL dvqpsi_us (ik, u (1, mode), .FALSE. )
+           CALL dvqpsi_us (ik, u (1, mode), .FALSE., becp1, alphap)
         ENDIF
         !
         ! calculate dvscf_q*psi_k
@@ -623,7 +624,7 @@ subroutine calculate_and_apply_phase(ik, ikqg, igqg, npwq_refolded, g_kpq, xk_ga
   USE gvecw, ONLY : gcutw
   USE cell_base, ONLY : bg
   USE qpoint, ONLY : nksq
-  USE wavefunctions_module, ONLY : evc
+  USE wavefunctions, ONLY : evc
   USE noncollin_module,     ONLY : npol, noncolin
   USE el_phon, ONLY:iunwfcwann, lrwfcr
 
