@@ -35,7 +35,7 @@ PROGRAM phonon
   !
   ! Not implemented in ph.x:
   ! [6] [5] + constraints on the magnetization
-  ! [7] Hubbard U
+  ! [7] Tkatchenko-Scheffler, DFT-D3
   ! [8] Hybrid and meta-GGA functionals
   ! [9] External Electric field
   ! [10] nonperiodic boundary conditions.
@@ -43,14 +43,20 @@ PROGRAM phonon
   USE control_ph,      ONLY : bands_computed, qplot
   USE check_stop,      ONLY : check_stop_init
   USE ph_restart,      ONLY : ph_writefile
-  USE mp_global,       ONLY : mp_startup
   USE environment,     ONLY : environment_start
+  USE mp_global,       ONLY : mp_startup
+  USE mp_world,        ONLY : world_comm
+  USE mp_pools,        ONLY : intra_pool_comm
+  USE mp_bands,        ONLY : intra_bgrp_comm, inter_bgrp_comm
+  USE command_line_options,  ONLY : input_file_, ndiag_
   ! YAMBO >
   USE YAMBO,           ONLY : elph_yambo,dvscf_yambo
   ! YAMBO <
   !
   IMPLICIT NONE
   !
+  include 'laxlib.fh'
+  !
   INTEGER :: iq, ierr
   LOGICAL :: do_band, do_iq, setup_pw
   CHARACTER (LEN=9)   :: code = 'PHONON'
@@ -59,6 +65,10 @@ PROGRAM phonon
   ! Initialize MPI, clocks, print initial messages
   !
   CALL mp_startup ( start_images=.true. )
+  CALL laxlib_start ( ndiag_, world_comm, intra_bgrp_comm, &
+       do_distr_diag_inside_bgrp_ = .true. )
+  CALL set_mpi_comm_4_solvers( intra_pool_comm, intra_bgrp_comm, &
+       inter_bgrp_comm )
   CALL environment_start ( code )
   !
   ! ... and begin with the initialization part
@@ -92,6 +102,7 @@ PROGRAM phonon
   ENDIF
   ! YAMBO <
   !
+  CALL laxlib_end()
   CALL stop_smoothly_ph( .TRUE. )
   !
   STOP
