@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2020 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -11,8 +11,7 @@ SUBROUTINE phq_readin()
   !----------------------------------------------------------------------------
   !
   !    This routine reads the control variables for the program phononq.
-  !    from standard input (unit 5).
-  !    A second routine readfile reads the variables saved on a file
+  !    A second routine, read_file, reads the variables saved to file
   !    by the self-consistent program.
   !
   !
@@ -22,11 +21,10 @@ SUBROUTINE phq_readin()
   USE mp_world,      ONLY : world_comm
   USE ions_base,     ONLY : amass, atm
   USE check_stop,    ONLY : max_seconds
-  USE input_parameters, ONLY : nk1, nk2, nk3, k1, k2, k3
   USE start_k,       ONLY : reset_grid
   USE klist,         ONLY : xk, nks, nkstot, lgauss, two_fermi_energies, ltetra
-  USE control_flags, ONLY : gamma_only, tqr, restart, lkpoint_dir, io_level, &
-                            ts_vdw
+  USE control_flags, ONLY : gamma_only, tqr, restart, io_level, &
+                            ts_vdw, ldftd3, lxdm, isolve
   USE funct,         ONLY : dft_is_meta, dft_is_hybrid
   USE uspp,          ONLY : okvan
   USE fixed_occ,     ONLY : tfixed_occ
@@ -34,7 +32,7 @@ SUBROUTINE phq_readin()
   USE fft_base,      ONLY : dffts
   USE spin_orb,      ONLY : domag
   USE cellmd,        ONLY : lmovecell
-  USE run_info, ONLY : title
+  USE run_info,      ONLY : title
   USE control_ph,    ONLY : maxter, alpha_mix, lgamma_gamma, epsil, &
                             zue, zeu, xmldyn, newgrid,                      &
                             trans, reduce_io, tr2_ph, niter_ph,       &
@@ -43,24 +41,21 @@ SUBROUTINE phq_readin()
                             ext_recover, ext_restart, u_from_file, ldiag, &
                             search_sym, lqdir, electron_phonon, tmp_dir_phq, &
                             rec_code_read, qplot, only_init, only_wfc, &
-                            low_directory_check
-
+                            low_directory_check, nk1, nk2, nk3, k1, k2, k3
   USE save_ph,       ONLY : tmp_dir_save, save_ph_input_variables
   USE gamma_gamma,   ONLY : asr
   USE partial,       ONLY : atomo, nat_todo, nat_todo_input
   USE output,        ONLY : fildyn, fildvscf, fildrho
   USE disp,          ONLY : nq1, nq2, nq3, x_q, wq, nqs, lgamma_iq
-  USE io_files,      ONLY : tmp_dir, prefix, postfix, create_directory, check_tempdir
+  USE io_files,      ONLY : tmp_dir, prefix, postfix, create_directory, &
+                            check_tempdir, xmlpun_schema
   USE noncollin_module, ONLY : i_cons, noncolin
-  USE ldaU,          ONLY : lda_plus_u
-  USE control_flags, ONLY : iverbosity, modenum, twfcollect
-  USE io_global,     ONLY : meta_ionode, meta_ionode_id, ionode, ionode_id, stdout
+  USE control_flags, ONLY : iverbosity, modenum
+  USE io_global,     ONLY : meta_ionode, meta_ionode_id, ionode, ionode_id, &
+                            qestdin, stdout
   USE mp_images,     ONLY : nimage, my_image_id, intra_image_comm,   &
                             me_image, nproc_image
-  USE mp_global,     ONLY : nproc_pool_file, &
-                            nproc_bgrp_file, nproc_image_file
-  USE mp_pools,      ONLY : nproc_pool, npool 
-  USE mp_bands,      ONLY : nproc_bgrp, ntask_groups
+  USE mp_pools,      ONLY : npool
   USE paw_variables, ONLY : okpaw
   USE ramanm,        ONLY : eth_rps, eth_ns, lraman, elop, dek
   USE freq_ph,       ONLY : fpol, fiu, nfs
@@ -72,12 +67,19 @@ SUBROUTINE phq_readin()
 
   USE qpoint,        ONLY : nksq, xq
   USE control_lr,    ONLY : lgamma, lrpa
-
   ! YAMBO >
   USE YAMBO,         ONLY : elph_yambo,dvscf_yambo
   ! YAMBO <
   USE elph_tetra_mod,ONLY : elph_tetra, lshift_q, in_alpha2f
   USE ktetra,        ONLY : tetra_type
+  USE ldaU,          ONLY : lda_plus_u, U_projection, lda_plus_u_kind
+  USE ldaU_ph,       ONLY : read_dns_bare, d2ns_type
+  USE dvscf_interpolate, ONLY : ldvscf_interpolate, do_long_range, &
+      do_charge_neutral, wpot_dir
+  USE ahc,           ONLY : elph_ahc, ahc_dir, ahc_nbnd, ahc_nbndskip, &
+      skip_upperfan
+  USE read_namelists_module, ONLY : check_namelist_read
+  USE open_close_input_file, ONLY : open_input_file, close_input_file
   !
   IMPLICIT NONE
   !
@@ -94,7 +96,6 @@ SUBROUTINE phq_readin()
   CHARACTER (LEN=256) :: outdir, filename
   CHARACTER (LEN=8)   :: verbosity
   CHARACTER(LEN=80)          :: card
-  CHARACTER(LEN=1), EXTERNAL :: capital
   CHARACTER(LEN=6) :: int_to_char
   INTEGER                    :: i
   LOGICAL                    :: nogg
@@ -107,6 +108,7 @@ SUBROUTINE phq_readin()
   REAL(DP), ALLOCATABLE :: xqaux(:,:)
   INTEGER, ALLOCATABLE :: wqaux(:)
   INTEGER :: nqaux, iq
+  CHARACTER(len=80) :: diagonalization='david'
   !
   NAMELIST / INPUTPH / tr2_ph, amass, alpha_mix, niter_ph, nmix_ph,  &
                        nat_todo, verbosity, iverbosity, outdir, epsil,  &
@@ -121,7 +123,10 @@ SUBROUTINE phq_readin()
                        elph_nbnd_min, elph_nbnd_max, el_ph_ngauss, &
                        el_ph_nsigma, el_ph_sigma,  electron_phonon, &
                        q_in_band_form, q2d, qplot, low_directory_check, &
-                       lshift_q
+                       lshift_q, read_dns_bare, d2ns_type, diagonalization, &
+                       ldvscf_interpolate, do_long_range, do_charge_neutral, &
+                       wpot_dir, ahc_dir, ahc_nbnd, ahc_nbndskip, &
+                       skip_upperfan
 
   ! tr2_ph       : convergence threshold
   ! amass        : atomic masses
@@ -161,9 +166,9 @@ SUBROUTINE phq_readin()
   ! ldiag        : if .true. force diagonalization of the dyn mat
   ! lqdir        : if .true. each q writes in its own directory
   ! search_sym   : if .true. analyze symmetry if possible
-  ! nk1,nk2,nk3,
-  ! ik1, ik2, ik3: when specified in input it uses for the phonon run
-  !                a different mesh than that used for the charge density.
+  ! nk1,nk2,nk3, k1,k2,k3 : 
+  !              when specified in input, the phonon run uses a different
+  !              k-point mesh from that used for the charge density.
   !
   ! dvscf_star%open : if .true. write in dvscf_star%dir the dvscf_q
   !                   'for all q' in the star of q with suffix dvscf_star%ext. 
@@ -185,7 +190,26 @@ SUBROUTINE phq_readin()
   ! q2d, : if .true. the q list define a mesh in a square.
   ! low_directory_check : if .true. only the requested representations
   !                       are searched on file
-  ! 
+  !
+  ! read_dns_bare : If .true. the code tries to read three files in DFPT+U calculations:
+  !                 dnsorth, dnsbare, d2nsbare 
+  ! d2ns_type     : DFPT+U - the 2nd bare derivative of occupation matrices ns 
+  !                 (d2ns_bare matrix). Experimental! This is why it is not documented in Doc.
+  !                 d2ns_type='full': matrix calculated with no approximation. 
+  !                 d2ns_type='fmmp': assume a m <=> m' symmetry. 
+  !                 d2ns_type='diag': if okvan=.true. the matrix is calculated retaining only
+  !                                     for <\beta_J|\phi_I> products where for J==I.   
+  !                 d2ns_type='dmmp': same as 'diag', but also assuming a m <=> m'.
+  ! diagonalization : diagonalization method used in the nscf calc
+  ! ldvscf_interpolate: if .true., use Fourier interpolation of phonon potential
+  ! do_long_range: if .true., add the long-range part of the potential to dvscf
+  ! do_charge_neutral: if .true., impose the neutrality condition on Born effective charges
+  ! wpot_dir: folder where w_pot binary files are located
+  ! ahc_dir: Directory where the output binary files for AHC e-ph coupling are written
+  ! ahc_nbnd: Number of bands where the electron self-energy is to be computed.
+  ! ahc_nbndskip: Number of bands to exclude when computing the self-energy.
+  ! skip_upperfan: If .true., skip the calculation of upper Fan self-energy.
+  !
   ! Note: meta_ionode is a single processor that reads the input
   !       (ionode is also a single processor but per image)
   !       Data read from input is subsequently broadcast to all processors
@@ -193,26 +217,26 @@ SUBROUTINE phq_readin()
   !
   IF (meta_ionode) THEN
   !
-  ! ... Input from file ?
+  ! ... Input from file (ios=0) or standard input (ios=-1) on unit "qestdin"
   !
-     CALL input_from_file ( )
+     ios = open_input_file (  )
   !
   ! ... Read the first line of the input file
   !
-     READ( 5, '(A)', IOSTAT = ios ) title
+     IF ( ios <= 0 ) READ( qestdin, '(A)', IOSTAT = ios ) title
   !
   ENDIF
   !
   CALL mp_bcast(ios, meta_ionode_id, world_comm )
-  CALL errore( 'phq_readin', 'reading title ', ABS( ios ) )
+  CALL errore( 'phq_readin', 'reading input file ', ABS( ios ) )
   CALL mp_bcast(title, meta_ionode_id, world_comm  )
   !
   ! Rewind the input if the title is actually the beginning of inputph namelist
   !
   IF( imatches("&inputph", title) ) THEN
-    WRITE(*, '(6x,a)') "Title line not specified: using 'default'."
+    WRITE(stdout,'(6x,a)') "Title line not specified: using 'default'."
     title='default'
-    IF (meta_ionode) REWIND(5, iostat=ios)
+    IF (meta_ionode) REWIND(qestdin, iostat=ios)
     CALL mp_bcast(ios, meta_ionode_id, world_comm  )
     CALL errore('phq_readin', 'Title line missing from input.', abs(ios))
   ENDIF
@@ -241,8 +265,8 @@ SUBROUTINE phq_readin()
   electron_phonon=' '
   elph_nbnd_min = 1
   elph_nbnd_max = 0
-  el_ph_sigma = 0.02
-  el_ph_nsigma = 30
+  el_ph_sigma  = 0.02
+  el_ph_nsigma = 10
   el_ph_ngauss = 1
   lraman       = .FALSE.
   elop         = .FALSE.
@@ -282,6 +306,19 @@ SUBROUTINE phq_readin()
   k2       = 0
   k3       = 0
   !
+  ! dvscf_interpolate
+  ldvscf_interpolate = .FALSE.
+  do_charge_neutral = .FALSE.
+  do_long_range = .FALSE.
+  wpot_dir = ' '
+  !
+  ! electron_phonon == 'ahc'
+  ahc_dir = ' '
+  ahc_nbnd = 0
+  ahc_nbndskip = 0
+  skip_upperfan = .FALSE.
+  elph_ahc = .FALSE.
+  !
   drho_star%open = .FALSE.
   drho_star%basis = 'modes'
   drho_star%pat  = .TRUE.
@@ -299,11 +336,13 @@ SUBROUTINE phq_readin()
       dvscf_star%dir = TRIM(outdir)//"/Rotated_DVSCF/"
   !
   lshift_q = .false.
+  read_dns_bare =.false.
+  d2ns_type = 'full'
   !
   ! ...  reading the namelist inputph
   !
   IF (meta_ionode) THEN
-     READ( 5, INPUTPH, ERR=30, IOSTAT = ios )
+     READ( qestdin, INPUTPH, IOSTAT = ios )
      !
      ! ...  iverbosity/verbosity hack
      !
@@ -320,6 +359,7 @@ SUBROUTINE phq_readin()
         ios = 1234567
      END IF
   END IF
+  CALL check_namelist_read(ios, qestdin, "inputph")
 30 CONTINUE
   CALL mp_bcast(ios, meta_ionode_id, world_comm )
   IF ( ios == 1234567 ) THEN
@@ -328,6 +368,16 @@ SUBROUTINE phq_readin()
   ELSE IF ( ABS(ios) /= 0 ) THEN
      CALL errore( 'phq_readin', 'reading inputph namelist', ABS( ios ) )
   END IF
+  ! diagonalization option
+  SELECT CASE(TRIM(diagonalization))
+  CASE ('david','davidson')
+     isolve = 0
+  CASE ('cg')
+     isolve = 1
+  CASE DEFAULT
+     CALL errore('phq_readin','diagonalization '//trim(diagonalization)//' not implemented',1)
+  END SELECT
+
   !
   ! ...  broadcast all input variables
   !
@@ -429,18 +479,20 @@ SUBROUTINE phq_readin()
      elph_simple=.false.
      trans = .false.
      elph_tetra = 3
+  CASE( 'ahc' )
+     elph = .true.
+     elph_ahc = .true.
+     elph_mat = .false.
+     elph_simple = .false.
+     elph_epa = .false.
+     trans = .false.
+     nogg = .true.
   CASE DEFAULT
      elph=.false.
      elph_mat=.false.
      elph_simple=.false.
      elph_epa=.false.
   END SELECT
-  ! YAMBO >
-  IF (.not.elph_yambo) then
-    ! YAMBO <
-    IF (elph.AND.qplot) &
-       CALL errore('phq_readin', 'qplot and elph not implemented',1)
-  ENDIF
 
   ! YAMBO >
   IF (.not.elph_yambo.and..not.dvscf_yambo) then
@@ -456,9 +508,9 @@ SUBROUTINE phq_readin()
   epsil = epsil .OR. lraman .OR. elop
 
   IF (modenum /= 0) search_sym=.FALSE.
-  
-  IF (elph_mat) THEN
-     trans=.false.
+
+  IF (elph_mat .OR. elph_ahc) THEN
+     trans = .FALSE.
   ELSEIF (.NOT. elph) THEN
      trans = trans .OR. ldisp
   ENDIF
@@ -478,9 +530,9 @@ SUBROUTINE phq_readin()
   IF (meta_ionode) THEN
      ios = 0
      IF (qplot) THEN
-        READ (5, *, iostat = ios) nqaux
+        READ (qestdin, *, iostat = ios) nqaux
      ELSE
-        IF (.NOT. ldisp) READ (5, *, iostat = ios) (xq (ipol), ipol = 1, 3)
+        IF (.NOT. ldisp) READ (qestdin, *, iostat = ios) (xq (ipol), ipol=1,3)
      ENDIF
   END IF
   CALL mp_bcast(ios, meta_ionode_id, world_comm )
@@ -491,7 +543,7 @@ SUBROUTINE phq_readin()
      ALLOCATE(wqaux(nqaux))
      IF (meta_ionode) THEN
         DO iq=1, nqaux
-           READ (5, *, iostat = ios) (xqaux (ipol,iq), ipol = 1, 3), wqaux(iq)
+           READ (qestdin, *, iostat = ios) (xqaux (ipol,iq), ipol=1,3), wqaux(iq)
         ENDDO
      ENDIF
      CALL mp_bcast(ios, meta_ionode_id, world_comm )
@@ -518,6 +570,46 @@ SUBROUTINE phq_readin()
      elop = .FALSE.
   ENDIF
   !
+  ! dvscf_interpolate
+  !
+  IF (ldvscf_interpolate) THEN
+    !
+    IF (wpot_dir == ' ') wpot_dir = TRIM(tmp_dir) // "w_pot/"
+    wpot_dir = trimcheck(wpot_dir)
+    !
+    IF (do_charge_neutral .AND. (.NOT. do_long_range)) THEN
+      WRITE(stdout, '(5x,a)') 'charge neutrality for dvscf_interpolate is &
+        & meaningful only if do_long_range is true.'
+      WRITE(stdout, '(5x,a)') 'Set do_charge_neutral = .false.'
+      do_charge_neutral = .FALSE.
+    ENDIF
+    !
+  ENDIF
+  !
+  IF (trans .AND. ldvscf_interpolate) CALL errore ('phq_readin', &
+    'ldvscf_interpolate should be used only when trans = .false.', 1)
+  IF (domag .AND. ldvscf_interpolate) CALL errore ('phq_readin', &
+    'ldvscf_interpolate and magnetism not implemented', 1)
+  IF (okpaw .AND. ldvscf_interpolate) CALL errore ('phq_readin', &
+    'PAW and ldvscf_interpolate not tested.', 1)
+  !
+  ! AHC e-ph coupling
+  !
+  IF (elph_ahc) THEN
+    !
+    IF (ahc_nbnd <= 0) CALL errore('phq_readin', &
+      'ahc_nbnd must be specified as a positive integer', 1)
+    IF (ahc_nbndskip < 0) CALL errore('phq_readin', &
+      'ahc_nbndskip cannot be negative', 1)
+    !
+    IF (ahc_dir == ' ') ahc_dir = TRIM(tmp_dir) // "ahc_dir/"
+    ahc_dir = trimcheck(ahc_dir)
+    !
+  ENDIF ! elph_ahc
+  !
+  IF (elph_ahc .AND. trans) CALL errore ('phq_readin', &
+    'elph_ahc can be used only when trans = .false.', 1)
+  !
   ! reads the frequencies ( just if fpol = .true. )
   !
   IF ( fpol ) THEN
@@ -525,11 +617,11 @@ SUBROUTINE phq_readin()
                                     'fpol=.TRUE. needs epsil=.TRUE.', 1 )
      nfs=0
      IF (meta_ionode) THEN
-        READ (5, *, iostat = ios) card
+        READ (qestdin, *, iostat = ios) card
         IF ( TRIM(card)=='FREQUENCIES'.OR. &
              TRIM(card)=='frequencies'.OR. &
              TRIM(card)=='Frequencies') THEN
-           READ (5, *, iostat = ios) nfs
+           READ (qestdin, *, iostat = ios) nfs
         ENDIF
      ENDIF
      CALL mp_bcast(ios, meta_ionode_id, world_comm  )
@@ -542,7 +634,7 @@ SUBROUTINE phq_readin()
              TRIM(card) == 'frequencies' .OR. &
              TRIM(card) == 'Frequencies' ) THEN
            DO i = 1, nfs
-              READ (5, *, iostat = ios) fiu(i)
+              READ (qestdin, *, iostat = ios) fiu(i)
            END DO
         END IF
      END IF
@@ -600,11 +692,7 @@ SUBROUTINE phq_readin()
         IF (.NOT.ext_recover.AND..NOT.ext_restart) tmp_dir_phq=tmp_dir_ph
      ENDIF
      !
-#if defined (__OLDXML)
-     filename=TRIM(tmp_dir_phq)//TRIM(prefix)//postfix//'data-file.xml'
-#else
-     filename=TRIM(tmp_dir_phq)//TRIM(prefix)//postfix//'data-file-schema.xml'
-#endif
+     filename=TRIM(tmp_dir_phq)//TRIM(prefix)//postfix//xmlpun_schema
      IF (ionode) inquire (file =TRIM(filename), exist = exst)
      !
      CALL mp_bcast( exst, ionode_id, intra_image_comm )
@@ -652,14 +740,16 @@ SUBROUTINE phq_readin()
         WRITE(stdout, '(5x,i3, 3f14.9)') iq, x_q(1,iq), x_q(2,iq), x_q(3,iq)
      END DO
   ENDIF
-
+  !
+  ! DFPT+U: the occupation matrix ns is read via read_file
+  !
   CALL read_file ( )
-
+  !
   magnetic_sym=noncolin .AND. domag
   !
   ! init_start_grid returns .true. if a new k-point grid is set from values
-  ! read from input (this happens if nk1*nk2*nk3, else it returns .false.,
-  ! leaves the current values, as read in read_file, unchanged)
+  ! read from input (this happens if nk1*nk2*nk3 > 0; otherwise reset_grid
+  ! returns .false., leaves the current values, read in read_file, unchanged)
   !
   newgrid = reset_grid (nk1, nk2, nk3, k1, k2, k3) 
   !
@@ -670,11 +760,38 @@ SUBROUTINE phq_readin()
   IF (gamma_only) CALL errore('phq_readin',&
      'cannot start from pw.x data file using Gamma-point tricks',1)
 
-  IF (lda_plus_u) CALL errore('phq_readin',&
-     'The phonon code with LDA+U is not yet available',1)
+  IF (lda_plus_u) THEN
+     ! 
+     WRITE(stdout,'(/5x,a)') "Phonon calculation with DFPT+U; please cite"
+     WRITE(stdout,'(5x,a)')  "A. Floris et al., Phys. Rev. B 84, 161102(R) (2011)"
+     WRITE(stdout,'(5x,a)')  "A. Floris et al., Phys. Rev. B 101, 064305 (2020)"
+     WRITE(stdout,'(5x,a)')  "in publications or presentations arising from this work."
+     ! 
+     IF (U_projection.NE."atomic") CALL errore("phq_readin", &
+          " The phonon code for this U_projection_type is not implemented",1)
+     IF (lda_plus_u_kind.NE.0) CALL errore("phq_readin", &
+          " The phonon code for this lda_plus_u_kind is not implemented",1)
+     IF (elph) CALL errore("phq_readin", &
+          " Electron-phonon with Hubbard U is not supported",1)
+     IF (lraman) CALL errore("phq_readin", &
+          " The phonon code with Raman and Hubbard U is not implemented",1)
+     !
+  ENDIF
+  ! checks
+  IF (elph_ahc .AND. domag) CALL errore ('phq_readin', &
+    'elph_ahc and magnetism not implemented', 1)
+  IF (elph_ahc .AND. okpaw) CALL errore ('phq_readin', &
+    'elph_ahc and PAW not tested.', 1)
+  IF (elph_ahc .AND. okvan) CALL errore ('phq_readin', &
+    'elph_ahc and PAW not tested.', 1)
+  IF (elph_ahc .AND. lda_plus_u) CALL errore ('phq_readin', &
+    'elph_ahc and lda_plus_u not tested.', 1)
 
   IF (ts_vdw) CALL errore('phq_readin',&
      'The phonon code with TS-VdW is not yet available',1)
+  
+  IF (ldftd3) CALL errore('phq_readin',&
+     'The phonon code with Grimme''s DFT-D3 is not yet available',1)
 
   IF ( dft_is_meta() ) CALL errore('phq_readin',&
      'The phonon code with meta-GGA functionals is not yet available',1)
@@ -685,8 +802,15 @@ SUBROUTINE phq_readin()
   IF (okpaw.and.(lraman.or.elop)) CALL errore('phq_readin',&
      'The phonon code with paw and raman or elop is not yet available',1)
 
-  IF (okpaw.and.noncolin.and.domag) CALL errore('phq_readin',&
-     'The phonon code with paw and domag is not available yet',1)
+  IF (magnetic_sym) THEN 
+     
+     WRITE(stdout,'(/5x,a)') "Phonon calculation in the non-collinear magnetic case;"
+     WRITE(stdout,'(5x,a)')  "please cite A. Urru and A. Dal Corso, Phys. Rev. B 100," 
+     WRITE(stdout,'(5x,a)')  "045115 (2019) for the theoretical background."
+
+     IF (okpaw) CALL errore('phq_readin',&
+          'The phonon code with paw and domag is not available yet',1)
+  ENDIF
 
   IF (okvan.and.(lraman.or.elop)) CALL errore('phq_readin',&
      'The phonon code with US-PP and raman or elop not yet available',1)
@@ -697,19 +821,8 @@ SUBROUTINE phq_readin()
   IF (lmovecell) CALL errore('phq_readin', &
       'The phonon code is not working after vc-relax',1)
 
-  IF (reduce_io) io_level=0
+  IF (reduce_io) io_level=1
 
-  IF (nproc_image /= nproc_image_file .and. .not. twfcollect  .AND. .NOT. in_alpha2f)  &
-     CALL errore('phq_readin',&
-     'pw.x run with a different number of processors. Use wf_collect=.true.',1)
-
-  IF (nproc_pool /= nproc_pool_file .and. .not. twfcollect .AND. .NOT. in_alpha2f)  &
-     CALL errore('phq_readin',&
-     'pw.x run with a different number of pools. Use wf_collect=.true.',1)
-  !
-  IF (nproc_bgrp_file /= nproc_bgrp .AND. .NOT. twfcollect .AND. .NOT. in_alpha2f) &
-     CALL errore('phq_readin','pw.x run with different band parallelization',1)
-  
   if(elph_mat.and.fildvscf.eq.' ') call errore('phq_readin',&
        'el-ph with wannier requires fildvscf',1)
 
@@ -719,9 +832,6 @@ SUBROUTINE phq_readin()
   IF(elph.and.nimage>1) call errore('phq_readin',&
        'el-ph with images not implemented',1)
   
-  IF( fildvscf /= ' ' .and. nimage > 1 ) call errore('phq_readin',&
-       'saving dvscf to file images not implemented',1)
-
   IF (elph.OR.fildvscf /= ' ') lqdir=.TRUE.
 
   IF(dvscf_star%open.and.nimage>1) CALL errore('phq_readin',&
@@ -729,6 +839,8 @@ SUBROUTINE phq_readin()
   IF(drho_star%open.and.nimage>1) CALL errore('phq_readin',&
        'drho_star with image parallelization is not yet available',1)
 
+  IF (lda_plus_u .AND. read_dns_bare .AND. ldisp) lqdir=.TRUE.
+
   IF (.NOT.ldisp) lqdir=.FALSE.
 
   IF (i_cons /= 0) &
@@ -755,7 +867,6 @@ SUBROUTINE phq_readin()
   ! If a band structure calculation needs to be done do not open a file
   ! for k point
   !
-  lkpoint_dir=.FALSE.
   restart = recover
   !
   !  set masses to values read from input, if available;
@@ -789,16 +900,18 @@ SUBROUTINE phq_readin()
      ENDIF
   ENDIF
   IF (lgamma_gamma.AND.ldiag) CALL errore('phq_readin','incompatible flags',1)
+  IF (lgamma_gamma.AND.elph ) CALL errore('phq_readin','lgamma_gamma and electron-phonon are incompatible',1)
   !
   IF (tfixed_occ) &
      CALL errore('phq_readin','phonon with arbitrary occupations not tested',1)
   !
   !YAMBO >
-  IF (elph.AND..NOT.(lgauss .or. ltetra).and..NOT.elph_yambo) CALL errore ('phq_readin', 'Electron-&
-       &phonon only for metals', 1)
+  IF (elph .AND. .NOT.(lgauss .OR. ltetra) &
+      .AND. .NOT. (elph_yambo .OR. elph_ahc)) &
+          CALL errore ('phq_readin', 'Electron-phonon only for metals', 1)
   !YAMBO <
-  IF (elph.AND.fildvscf.EQ.' ') CALL errore ('phq_readin', 'El-ph needs &
-       &a DeltaVscf file', 1)
+  IF (elph .AND. fildvscf.EQ.' ' .AND. .NOT. ldvscf_interpolate) &
+      CALL errore ('phq_readin', 'El-ph needs a DeltaVscf file', 1)
   !   There might be other variables in the input file which describe
   !   partial computation of the dynamical matrix. Read them here
   !
@@ -807,12 +920,16 @@ SUBROUTINE phq_readin()
   IF ( nat_todo < 0 .OR. nat_todo > nat ) &
      CALL errore ('phq_readin', 'nat_todo is wrong', 1)
   IF (nat_todo.NE.0) THEN
-     IF (meta_ionode) READ (5, *, iostat = ios) (atomo (na), na = 1, nat_todo)
+     IF (meta_ionode) READ (qestdin, *, iostat = ios) (atomo (na), na=1,nat_todo)
      CALL mp_bcast(ios, meta_ionode_id, world_comm  )
      CALL errore ('phq_readin', 'reading atoms', ABS (ios) )
      CALL mp_bcast(atomo, meta_ionode_id, world_comm  )
   ENDIF
   nat_todo_input=nat_todo
+  !
+  ! end of reading, close unit qestdin, remove tenporary input file if existing
+  !
+  IF (meta_ionode) ios = close_input_file () 
 
   IF (epsil.AND.(lgauss .OR. ltetra)) &
         CALL errore ('phq_readin', 'no elec. field with metals', 1)
