@@ -13,13 +13,35 @@ SUBROUTINE set_int12_nc(iflag)
 !  by the Pauli matrices the integrals.
 !
 USE ions_base, ONLY : nat, ntyp => nsp, ityp
-USE spin_orb, ONLY : lspinorb
+USE spin_orb, ONLY : lspinorb, domag
+USE noncollin_module, ONLY : noncolin
 USE uspp_param, only: upf
 USE phus, ONLY : int1, int2, int1_nc, int2_so
+USE nc_mag_aux, ONLY : int1_nc_save
 IMPLICIT NONE
 INTEGER :: iflag
 INTEGER :: np, na
 
+IF (noncolin.AND.domag) THEN
+   int1_nc=(0.d0,0.d0)
+   int1 ( :, :, :, :, 2:4)=-int1 ( :, :, :, :, 2:4)
+   DO np = 1, ntyp
+      IF ( upf(np)%tvanp ) THEN
+         DO na = 1, nat
+            IF (ityp(na)==np) THEN
+               IF (upf(np)%has_so) THEN
+                  CALL transform_int1_so(int1,na,iflag)
+               ELSE
+                  CALL transform_int1_nc(int1,na,iflag)
+               END IF
+            END IF
+         END DO
+      END IF
+   END DO
+   int1 ( :, :, :, :, 2:4)=-int1 ( :, :, :, :, 2:4)
+   int1_nc_save(:,:,:,:,:,2) = int1_nc
+END IF
+
 int1_nc=(0.d0,0.d0)
 IF (lspinorb) int2_so=(0.d0,0.d0)
 DO np = 1, ntyp
@@ -37,5 +59,7 @@ DO np = 1, ntyp
       END DO
    END IF
 END DO
+IF (noncolin.AND.domag) int1_nc_save(:,:,:,:,:,1) = int1_nc
+
 END SUBROUTINE set_int12_nc
 
