@@ -25,7 +25,7 @@ subroutine set_irr_new (xq, u, npert, nirr, eigen)
   USE kinds, only : DP
   USE ions_base, ONLY : nat, tau, ntyp => nsp, ityp, amass
   USE cell_base, ONLY : at, bg
-  USE symm_base, ONLY : s, sr, ftau, invs, nsym, irt, t_rev
+  USE symm_base, ONLY : s, sr, invs, nsym, irt, t_rev
   USE modes,     ONLY : num_rap_mode, name_rap_mode
   USE noncollin_module, ONLY : noncolin, nspin_mag
   USE spin_orb,  ONLY : domag
@@ -139,7 +139,7 @@ subroutine set_irr_new (xq, u, npert, nirr, eigen)
         end do
      end if
 
-     IF (search_sym) THEN
+     IF (search_sym.AND.nspin_mag/=4) THEN
         CALL find_mode_sym_new (u, eigen, tau, nat, nsymq, s, sr, irt, xq,    &
              rtau, amass, ntyp, ityp, 0, .FALSE., .TRUE., num_rap_mode, ierr)
 
@@ -246,7 +246,7 @@ subroutine set_irr_new (xq, u, npert, nirr, eigen)
         endif
      enddo
 
-     IF (search_sym) THEN
+     IF (search_sym.AND.nspin_mag/=4) THEN
 !
 !  Here we set the name of the representation for each mode
 !
