@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2018 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -15,9 +15,13 @@ subroutine solve_e
   !    It performs the following tasks:
   !     a) computes the bare potential term  x | psi >
   !     b) adds to it the screening term Delta V_{SCF} | psi >
+  !        If lda_plus_u=.true. compute also the SCF part
+  !        of the response Hubbard potential.
   !     c) applies P_c^+ (orthogonalization to valence states)
   !     d) calls cgsolve_all to solve the linear system
   !     e) computes Delta rho, Delta V_{SCF} and symmetrizes them
+  !     f) If lda_plus_u=.true. compute also the response occupation
+  !        matrices dnsscf
   !
   USE kinds,                 ONLY : DP
   USE ions_base,             ONLY : nat
@@ -33,7 +37,7 @@ subroutine solve_e
   USE wvfct,                 ONLY : nbnd, npwx, g2kin, et
   USE check_stop,            ONLY : check_stop_now
   USE buffers,               ONLY : get_buffer, save_buffer
-  USE wavefunctions_module,  ONLY : evc
+  USE wavefunctions,         ONLY : evc
   USE uspp,                  ONLY : okvan, vkb
   USE uspp_param,            ONLY : upf, nhm
   USE noncollin_module,      ONLY : noncolin, npol, nspin_mag
@@ -42,26 +46,25 @@ subroutine solve_e
   USE paw_onecenter,         ONLY : paw_dpotential
   USE paw_symmetry,          ONLY : paw_desymmetrize
 
-  USE units_ph,              ONLY : lrdwf, iudwf, lrwfc, iuwfc, lrdrho, &
-                                    iudrho
+  USE units_ph,              ONLY : lrdwf, iudwf, lrdrho, iudrho
+  USE units_lr,              ONLY : iuwfc, lrwfc
   USE output,                ONLY : fildrho
   USE control_ph,            ONLY : ext_recover, rec_code, &
                                     lnoloc, convt, tr2_ph, nmix_ph, &
                                     alpha_mix, lgamma_gamma, niter_ph, &
                                     flmixdpot, rec_code_read
   USE recover_mod,           ONLY : read_rec, write_rec
-
   USE mp_pools,              ONLY : inter_pool_comm
   USE mp_bands,              ONLY : intra_bgrp_comm, ntask_groups
   USE mp,                    ONLY : mp_sum
-
   USE lrus,                  ONLY : int3_paw
-  USE qpoint,                ONLY : nksq
+  USE qpoint,                ONLY : nksq, ikks, ikqs
   USE eqv,                   ONLY : dpsi, dvpsi
   USE control_lr,            ONLY : nbnd_occ, lgamma
   USE dv_of_drho_lr
   USE fft_helper_subroutines
   USE fft_interfaces,        ONLY : fft_interpolate
+  USE ldaU,                  ONLY : lda_plus_u
 
   implicit none
 
@@ -86,8 +89,6 @@ subroutine solve_e
                    tg_dv(:,:), &
                    tg_psic(:,:), aux2(:,:)
 
-  complex(DP), EXTERNAL :: zdotc      ! the scalar product function
-
   logical :: conv_root, exst
   ! conv_root: true if linear system is converged
 
@@ -95,6 +96,7 @@ subroutine solve_e
   integer :: kter, iter0, ipol, ibnd, iter, lter, ik, ig, is, nrec, ndim, ios
   ! counters
   integer :: ltaver, lintercall, incr, jpol, v_siz
+  integer :: ikk, ikq
 
   real(DP) :: tcpu, get_clock
   ! timing variables
@@ -162,36 +164,46 @@ subroutine solve_e
   !
   if ( (lgauss .or. ltetra) .or..not.lgamma) call errore ('solve_e', &
        'called in the wrong case', 1)
-
   !
   !   The outside loop is over the iterations
   !
   do kter = 1, niter_ph
-
-!     write(6,*) 'kter', kter
+     !
      FLUSH( stdout )
      iter = kter + iter0
      ltaver = 0
      lintercall = 0
-
+     !
      dvscfout(:,:,:)=(0.d0,0.d0)
      dbecsum(:,:,:,:)=(0.d0,0.d0)
      IF (noncolin) dbecsum_nc=(0.d0,0.d0)
-
+     !
+     ! DFPT+U: at each iteration calculate dnsscf,
+     ! i.e. the scf variation of the occupation matrix ns.
+     !
+     IF (lda_plus_u .AND. (iter.NE.1)) &
+        CALL dnsq_scf (3, .false., 0, 1, .false.)
+     !
      do ik = 1, nksq
-        npw = ngk(ik)
+        ikk=ikks(ik)
+        ikq=ikqs(ik)
+        !
+        npw = ngk(ikk)
         npwq= npw     ! q=0 always in this routine
-        if (lsda) current_spin = isk (ik)
+        !
+        if (lsda) current_spin = isk (ikk)
         !
         ! reads unperturbed wavefunctions psi_k in G_space, for all bands
         !
-        if (nksq.gt.1) call get_buffer (evc, lrwfc, iuwfc, ik)
+        if (nksq.ge.1) THEN 
+           call get_buffer (evc, lrwfc, iuwfc, ikk)
+        end if
         !
         ! compute beta functions and kinetic energy for k-point ik
         ! needed by h_psi, called by ch_psi_all, called by cgsolve_all
         !
-        CALL init_us_2 (npw, igk_k(1,ik), xk (1, ik), vkb)
-        CALL g2_kin(ik)
+        CALL init_us_2 (npw, igk_k(1,ikk), xk (1, ikk), vkb)
+        CALL g2_kin(ikk)
         !
         ! compute preconditioning matrix h_diag used by cgsolve_all
         !
@@ -221,11 +233,11 @@ subroutine solve_e
                  ENDIF
               ENDIF
               aux2=(0.0_DP,0.0_DP)
-              do ibnd = 1, nbnd_occ (ik), incr
+              do ibnd = 1, nbnd_occ (ikk), incr
                  IF ( dffts%has_task_groups ) THEN
-                    call cft_wave_tg (ik, evc, tg_psic, 1, v_siz, ibnd, nbnd_occ (ik) )
+                    call cft_wave_tg (ik, evc, tg_psic, 1, v_siz, ibnd, nbnd_occ (ikk) )
                     call apply_dpot(v_siz, tg_psic, tg_dv, 1)
-                    call cft_wave_tg (ik, aux2, tg_psic, -1, v_siz, ibnd, nbnd_occ (ik))
+                    call cft_wave_tg (ik, aux2, tg_psic, -1, v_siz, ibnd, nbnd_occ (ikk))
                  ELSE
                     call cft_wave (ik, evc (1, ibnd), aux1, +1)
                     call apply_dpot(dffts%nnr, aux1, dvscfins(1,1,ipol), current_spin)
@@ -234,13 +246,22 @@ subroutine solve_e
               enddo
               dvpsi=dvpsi+aux2
               !
+              !  In the case of US pseudopotentials there is an additional
+              !  selfconsist term which comes from the dependence of D on
+              !  V_{eff} on the bare change of the potential
+              !
               call adddvscf(ipol,ik)
               !
+              ! DFPT+U: add to dvpsi the scf part of the response
+              ! Hubbard potential dV_hub
+              !
+              if (lda_plus_u) call adddvhubscf (ipol, ik)
+              !
            endif
            !
            ! Orthogonalize dvpsi to valence states: ps = <evc|dvpsi>
            !
-           CALL orthogonalize(dvpsi, evc, ik, ik, dpsi, npwq, .false.)
+           CALL orthogonalize(dvpsi, evc, ikk, ikk, dpsi, npwq, .false.)
            !
            if (iter == 1) then
               !
@@ -271,8 +292,8 @@ subroutine solve_e
 
            conv_root = .true.
 
-           call cgsolve_all (ch_psi_all,cg_psi,et(1,ik),dvpsi,dpsi, &
-              h_diag,npwx,npw,thresh,ik,lter,conv_root,anorm,nbnd_occ(ik),npol)
+           call cgsolve_all (ch_psi_all,cg_psi,et(1,ikk),dvpsi,dpsi, &
+              h_diag,npwx,npw,thresh,ik,lter,conv_root,anorm,nbnd_occ(ikk),npol)
 
            ltaver = ltaver + lter
            lintercall = lintercall + 1
@@ -288,10 +309,10 @@ subroutine solve_e
            ! calculates dvscf, sum over k => dvscf_q_ipert
            !
            IF (noncolin) THEN
-              call incdrhoscf_nc(dvscfout(1,1,ipol),wk(ik),ik, &
-                                 dbecsum_nc(1,1,1,1,ipol), dpsi)
+              call incdrhoscf_nc(dvscfout(1,1,ipol),wk(ikk),ik, &
+                                 dbecsum_nc(1,1,1,1,ipol), dpsi, 1.0d0)
            ELSE
-              call incdrhoscf (dvscfout(1,current_spin,ipol), wk(ik), &
+              call incdrhoscf (dvscfout(1,current_spin,ipol), wk(ikk), &
                             ik, dbecsum(1,1,current_spin,ipol), dpsi)
            ENDIF
         enddo   ! on polarizations
@@ -410,6 +431,7 @@ subroutine solve_e
 
   enddo
 155 continue
+  !
   deallocate (h_diag)
   deallocate (aux1)
   deallocate (dbecsum)
