@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2018 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -14,32 +14,40 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   !    defines the change of the wavefunction due to a lattice distorsion
   !    It performs the following tasks:
   !     a) computes the bare potential term Delta V | psi >
-  !        and an additional term in the case of US pseudopotentials
-  !     b) adds to it the screening term Delta V_{SCF} | psi >
+  !        and an additional term in the case of US pseudopotentials.
+  !        If lda_plus_u=.true. compute also the bare potential
+  !        term Delta V_hub | psi >.
+  !     b) adds to it the screening term Delta V_{SCF} | psi >.
+  !        If lda_plus_u=.true. compute also the SCF part
+  !        of the response Hubbard potential.
   !     c) applies P_c^+ (orthogonalization to valence states)
   !     d) calls cgsolve_all to solve the linear system
   !     e) computes Delta rho, Delta V_{SCF} and symmetrizes them
+  !     f) If lda_plus_u=.true. compute also the response occupation
+  !        matrices dnsscf
+  !     g) (Introduced in February 2020) If noncolin=.true. and domag=.true. 
+  !        the linear system is solved twice (nsolv = 2, the case 
+  !        isolv = 2 needs the time-reversed wave functions). For the 
+  !        theoretical background, please refer to Phys. Rev. B 100, 
+  !        045115 (2019)
   !
-
   USE kinds,                ONLY : DP
   USE ions_base,            ONLY : nat, ntyp => nsp, ityp
   USE io_global,            ONLY : stdout, ionode
   USE io_files,             ONLY : prefix, diropn
   USE check_stop,           ONLY : check_stop_now
-  USE wavefunctions_module, ONLY : evc
-  USE constants,            ONLY : degspin
-  USE cell_base,            ONLY : at, tpiba2
-  USE klist,                ONLY : ltetra, lgauss, degauss, ngauss, &
-                                   xk, wk, ngk, igk_k
+  USE wavefunctions,        ONLY : evc
+  USE cell_base,            ONLY : at
+  USE klist,                ONLY : ltetra, lgauss, xk, wk, ngk, igk_k
   USE gvect,                ONLY : g
   USE gvecs,                ONLY : doublegrid
   USE fft_base,             ONLY : dfftp, dffts
   USE lsda_mod,             ONLY : lsda, nspin, current_spin, isk
   USE spin_orb,             ONLY : domag
-  USE wvfct,                ONLY : nbnd, npwx, g2kin,  et
-  USE scf,                  ONLY : rho
-  USE uspp,                 ONLY : okvan, vkb
-  USE uspp_param,           ONLY : upf, nhm, nh
+  USE wvfct,                ONLY : nbnd, npwx, et
+  USE scf,                  ONLY : rho, vrs
+  USE uspp,                 ONLY : okvan, vkb, deeq_nc
+  USE uspp_param,           ONLY : upf, nhm
   USE noncollin_module,     ONLY : noncolin, npol, nspin_mag
   USE paw_variables,        ONLY : okpaw
   USE paw_onecenter,        ONLY : paw_dpotential
@@ -52,29 +60,31 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   USE el_phon,              ONLY : elph
   USE uspp,                 ONLY : nlcc_any
   USE units_ph,             ONLY : iudrho, lrdrho, iudwf, lrdwf, iubar, lrbar, &
-                                   iuwfc, lrwfc, iudvscf, iuint3paw, lint3paw
+                                   iudvscf, iuint3paw, lint3paw
+  USE units_lr,             ONLY : iuwfc, lrwfc
   USE output,               ONLY : fildrho, fildvscf
-  USE phus,                 ONLY : becsumort
+  USE phus,                 ONLY : becsumort, alphap, int1_nc
   USE modes,                ONLY : npertx, npert, u, t, tmq
-
   USE recover_mod,          ONLY : read_rec, write_rec
   ! used to write fildrho:
   USE dfile_autoname,       ONLY : dfile_name
   USE save_ph,              ONLY : tmp_dir_save
   ! used oly to write the restart file
   USE mp_pools,             ONLY : inter_pool_comm
-  USE mp_bands,             ONLY : intra_bgrp_comm, ntask_groups, me_bgrp
+  USE mp_bands,             ONLY : intra_bgrp_comm, me_bgrp
   USE mp,                   ONLY : mp_sum
   USE efermi_shift,         ONLY : ef_shift, ef_shift_paw,  def
-
-  USE lrus,         ONLY : int3_paw
-  USE lr_symm_base, ONLY : irotmq, minus_q, nsymq, rtau
-  USE eqv,          ONLY : dvpsi, dpsi, evq
-  USE qpoint,       ONLY : xq, nksq, ikks, ikqs
-  USE control_lr,   ONLY : alpha_pv, nbnd_occ, lgamma
-  USE dv_of_drho_lr
+  USE lrus,                 ONLY : int3_paw, becp1, int3_nc
+  USE lr_symm_base,         ONLY : irotmq, minus_q, nsymq, rtau
+  USE eqv,                  ONLY : dvpsi, dpsi, evq
+  USE qpoint,               ONLY : xq, nksq, ikks, ikqs
+  USE qpoint_aux,           ONLY : ikmks, ikmkmqs, becpt, alphapt
+  USE control_lr,           ONLY : nbnd_occ, lgamma
+  USE dv_of_drho_lr,        ONLY : dv_of_drho
   USE fft_helper_subroutines
-  USE fft_interfaces, ONLY : fft_interpolate
+  USE fft_interfaces,       ONLY : fft_interpolate
+  USE ldaU,                 ONLY : lda_plus_u
+  USE nc_mag_aux,           ONLY : int1_nc_save, deeq_nc_save, int3_save
 
   implicit none
 
@@ -88,11 +98,12 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
 
   real(DP) , allocatable :: h_diag (:,:)
   ! h_diag: diagonal part of the Hamiltonian
-  real(DP) :: thresh, anorm, averlt, dr2
+  real(DP) :: thresh, anorm, averlt, dr2, rsign
   ! thresh: convergence threshold
   ! anorm : the norm of the error
   ! averlt: average number of iterations
   ! dr2   : self-consistency error
+  ! rsign : sign or the term in the magnetization
   real(DP) :: dos_ef, weight, aux_avg (2)
   ! Misc variables for metals
   ! dos_ef: density of states at Ef
@@ -105,8 +116,8 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   ! change of rho / scf potential (output)
   ! change of scf potential (output)
   complex(DP), allocatable :: ldos (:,:), ldoss (:,:), mixin(:), mixout(:), &
-       dbecsum (:,:,:,:), dbecsum_nc(:,:,:,:,:), aux1 (:,:), tg_dv(:,:), &
-       tg_psic(:,:), aux2(:,:), drhoc(:)
+       dbecsum (:,:,:,:), dbecsum_nc(:,:,:,:,:,:), aux1 (:,:), tg_dv(:,:), &
+       tg_psic(:,:), aux2(:,:), drhoc(:), dbecsum_aux (:,:,:,:)
   ! Misc work space
   ! ldos : local density of states af Ef
   ! ldoss: as above, without augmentation charges
@@ -136,7 +147,11 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
              incr,       & ! used for tg
              v_siz,      & ! size of the potential
              ipol,       & ! counter on polarization
-             mode          ! mode index
+             mode,       &  ! mode index
+             isolv,      & ! counter on linear systems
+             nsolv,      & ! number of linear systems
+             ikmk,       & ! index of mk
+             ikmkmq        ! index of mk-mq
 
   integer  :: npw, npwq
   integer  :: iq_dummy
@@ -151,6 +166,9 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
 !
 !  This routine is task group aware
 !
+  nsolv=1
+  IF (noncolin.AND.domag) nsolv=2
+
   allocate (dvscfin ( dfftp%nnr , nspin_mag , npe))
   if (doublegrid) then
      allocate (dvscfins (dffts%nnr , nspin_mag , npe))
@@ -164,12 +182,18 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      allocate (mixin(dfftp%nnr*nspin_mag*npe+(nhm*(nhm+1)*nat*nspin_mag*npe)/2) )
      allocate (mixout(dfftp%nnr*nspin_mag*npe+(nhm*(nhm+1)*nat*nspin_mag*npe)/2) )
      mixin=(0.0_DP,0.0_DP)
+  ELSE
+     ALLOCATE(mixin(1))
   ENDIF
-  IF (noncolin) allocate (dbecsum_nc (nhm,nhm, nat , nspin , npe))
+  IF (noncolin) allocate (dbecsum_nc (nhm,nhm, nat , nspin , npe, nsolv))
   allocate (aux1 ( dffts%nnr, npol))
   allocate (h_diag ( npwx*npol, nbnd))
   allocate (aux2(npwx*npol, nbnd))
   allocate (drhoc(dfftp%nnr))
+  IF (noncolin.AND.domag.AND.okvan) THEN
+     ALLOCATE (int3_save( nhm, nhm, nat, nspin_mag, npe, 2))
+     ALLOCATE (dbecsum_aux ( (nhm * (nhm + 1))/2 , nat , nspin_mag , npe))
+  ENDIF
   incr=1
   IF ( dffts%has_task_groups ) THEN
      !
@@ -217,7 +241,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      allocate ( ldos ( dfftp%nnr  , nspin_mag) )
      allocate ( ldoss( dffts%nnr , nspin_mag) )
      allocate (becsum1 ( (nhm * (nhm + 1))/2 , nat , nspin_mag))
-     call localdos_paw ( ldos , ldoss , becsum1, dos_ef )
+     call localdos ( ldos , ldoss , becsum1, dos_ef )
      IF (.NOT.okpaw) deallocate(becsum1)
   endif
   !
@@ -231,15 +255,21 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
   !   The outside loop is over the iterations
   !
   do kter = 1, niter_ph
+     !
      iter = kter + iter0
-
      ltaver = 0
-
      lintercall = 0
+     !
      drhoscf(:,:,:) = (0.d0, 0.d0)
      dbecsum(:,:,:,:) = (0.d0, 0.d0)
      IF (noncolin) dbecsum_nc = (0.d0, 0.d0)
      !
+     ! DFPT+U: at each ph iteration calculate dnsscf,
+     ! i.e. the scf variation of the occupation matrix ns.
+     !
+     IF (lda_plus_u .AND. (iter.NE.1)) &
+        CALL dnsq_scf (npe, lmetq0, imode0, irr, .true.)
+     !
      do ik = 1, nksq
         !
         ikk = ikks(ik)
@@ -249,144 +279,215 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
         !
         if (lsda) current_spin = isk (ikk)
         !
-        ! read unperturbed wavefunctions psi(k) and psi(k+q)
-        !
-        if (nksq.gt.1) then
-           if (lgamma) then
-              call get_buffer (evc, lrwfc, iuwfc, ikk)
-           else
-              call get_buffer (evc, lrwfc, iuwfc, ikk)
-              call get_buffer (evq, lrwfc, iuwfc, ikq)
-           endif
-
-        endif
-        !
         ! compute beta functions and kinetic energy for k-point ikq
         ! needed by h_psi, called by ch_psi_all, called by cgsolve_all
         !
         CALL init_us_2 (npwq, igk_k(1,ikq), xk (1, ikq), vkb)
         CALL g2_kin (ikq) 
         !
-        ! compute preconditioning matrix h_diag used by cgsolve_all
+        ! Start the loop on the two linear systems, one at B and one at
+        ! -B
         !
-        CALL h_prec (ik, evq, h_diag)
-        !
-        do ipert = 1, npe
-           mode = imode0 + ipert
-           nrec = (ipert - 1) * nksq + ik
-           !
-           !  and now adds the contribution of the self consistent term
-           !
-           if (where_rec =='solve_lint'.or.iter>1) then
-              !
-              ! After the first iteration dvbare_q*psi_kpoint is read from file
-              !
-              call get_buffer (dvpsi, lrbar, iubar, nrec)
-              !
-              ! calculates dvscf_q*psi_k in G_space, for all bands, k=kpoint
-              ! dvscf_q from previous iteration (mix_potential)
-              !
-              call start_clock ('vpsifft')
-              IF( dffts%has_task_groups ) THEN
-                 IF (noncolin) THEN
-                    CALL tg_cgather( dffts, dvscfins(:,1,ipert), tg_dv(:,1))
-                    IF (domag) THEN
-                       DO ipol=2,4
-                          CALL tg_cgather( dffts, dvscfins(:,ipol,ipert), tg_dv(:,ipol))
-                       ENDDO
-                    ENDIF
-                 ELSE
-                    CALL tg_cgather( dffts, dvscfins(:,current_spin,ipert), tg_dv(:,1))
-                 ENDIF
-              ENDIF
-              aux2=(0.0_DP,0.0_DP)
-              do ibnd = 1, nbnd_occ (ikk), incr
-                 IF( dffts%has_task_groups ) THEN
-                    call cft_wave_tg (ik, evc, tg_psic, 1, v_siz, ibnd, nbnd_occ (ikk) )
-                    call apply_dpot(v_siz, tg_psic, tg_dv, 1)
-                    call cft_wave_tg (ik, aux2, tg_psic, -1, v_siz, ibnd, nbnd_occ (ikk))
-                 ELSE
-                    call cft_wave (ik, evc (1, ibnd), aux1, +1)
-                    call apply_dpot(dffts%nnr,aux1, dvscfins(1,1,ipert), current_spin)
-                    call cft_wave (ik, aux2 (1, ibnd), aux1, -1)
-                 ENDIF
-              enddo
-              dvpsi=dvpsi+aux2
-              call stop_clock ('vpsifft')
-              !
-              !  In the case of US pseudopotentials there is an additional
-              !  selfconsist term which comes from the dependence of D on
-              !  V_{eff} on the bare change of the potential
-              !
-              call adddvscf (ipert, ik)
-           else
-              !
-              !  At the first iteration dvbare_q*psi_kpoint is calculated
-              !  and written to file
-              !
-              call dvqpsi_us (ik, u (1, mode),.false. )
-              call save_buffer (dvpsi, lrbar, iubar, nrec)
-           endif
-           !
-           ! Ortogonalize dvpsi to valence states: ps = <evq|dvpsi>
-           ! Apply -P_c^+.
-           !
-           CALL orthogonalize(dvpsi, evq, ikk, ikq, dpsi, npwq, .false.)
-           !
-           if (where_rec=='solve_lint'.or.iter > 1) then
-              !
-              ! starting value for delta_psi is read from iudwf
-              !
-              call get_buffer( dpsi, lrdwf, iudwf, nrec)
-              !
-              ! threshold for iterative solution of the linear system
-              !
-              thresh = min (1.d-1 * sqrt (dr2), 1.d-2)
-           else
-              !
-              !  At the first iteration dpsi and dvscfin are set to zero
-              !
-              dpsi(:,:) = (0.d0, 0.d0)
-              dvscfin (:, :, ipert) = (0.d0, 0.d0)
-              !
-              ! starting threshold for iterative solution of the linear system
-              !
-              thresh = 1.0d-2
-           endif
-
-           !
-           ! iterative solution of the linear system (H-eS)*dpsi=dvpsi,
-           ! dvpsi=-P_c^+ (dvbare+dvscf)*psi , dvscf fixed.
-           !
-           conv_root = .true.
-
-           call cgsolve_all (ch_psi_all, cg_psi, et(1,ikk), dvpsi, dpsi, &
-                             h_diag, npwx, npwq, thresh, ik, lter, conv_root, &
-                             anorm, nbnd_occ(ikk), npol )
-
-           ltaver = ltaver + lter
-           lintercall = lintercall + 1
-           if (.not.conv_root) WRITE( stdout, '(5x,"kpoint",i4," ibnd",i4,  &
-                &              " solve_linter: root not converged ",es10.3)') &
-                &              ik , ibnd, anorm
-           !
-           ! writes delta_psi on iunit iudwf, k=kpoint,
-           !
-           !               if (nksq.gt.1 .or. npert(irr).gt.1)
-           call save_buffer (dpsi, lrdwf, iudwf, nrec)
-           !
-           ! calculates dvscf, sum over k => dvscf_q_ipert
-           !
-           weight = wk (ikk)
-           IF (noncolin) THEN
-              call incdrhoscf_nc(drhoscf(1,1,ipert),weight,ik, &
-                                 dbecsum_nc(1,1,1,1,ipert), dpsi)
+        DO isolv=1, nsolv
+           IF (isolv==2) THEN
+              ikmk = ikmks(ik)
+              ikmkmq = ikmkmqs(ik)
+              rsign=-1.0_DP
            ELSE
-              call incdrhoscf (drhoscf(1,current_spin,ipert), weight, ik, &
-                            dbecsum(1,1,current_spin,ipert), dpsi)
-           END IF
-           ! on perturbations
-        enddo
+              ikmk=ikk
+              ikmkmq=ikq
+              rsign=1.0_DP
+           ENDIF
+           !
+           ! read unperturbed wavefunctions psi(k) and psi(k+q)
+           !
+           if (nksq.gt.1.OR.nsolv==2) then
+              if (lgamma) then
+                 call get_buffer (evc, lrwfc, iuwfc, ikmk)
+              else
+                 call get_buffer (evc, lrwfc, iuwfc, ikmk)
+                 call get_buffer (evq, lrwfc, iuwfc, ikmkmq)
+              end if
+           endif
+           !
+           ! compute preconditioning matrix h_diag used by cgsolve_all
+           !
+           CALL h_prec (ik, evq, h_diag)
+           !
+           do ipert = 1, npe
+              mode = imode0 + ipert
+              nrec = (ipert - 1) * nksq + ik + (isolv-1) * npe * nksq
+              !
+              !  and now adds the contribution of the self consistent term
+              !
+              if (where_rec =='solve_lint'.or.iter>1) then
+                 !
+                 ! After the first iteration dvbare_q*psi_kpoint is read from file
+                 !
+                 call get_buffer (dvpsi, lrbar, iubar, nrec)
+                 !
+                 ! calculates dvscf_q*psi_k in G_space, for all bands, k=kpoint
+                 ! dvscf_q from previous iteration (mix_potential)
+                 !
+                 call start_clock ('vpsifft')
+                 !
+                 !  change the sign of the magnetic field if required
+                 !
+                 IF (isolv==2) THEN
+                    dvscfins(:,2:4,ipert)=-dvscfins(:,2:4,ipert)
+                    IF (okvan) int3_nc(:,:,:,:,ipert)=int3_save(:,:,:,:,ipert,2)
+                 ENDIF
+                 !
+                 !  Set the potential for task groups
+                 !
+                 IF( dffts%has_task_groups ) THEN
+                    IF (noncolin) THEN
+                       CALL tg_cgather( dffts, dvscfins(:,1,ipert), tg_dv(:,1))
+                       IF (domag) THEN
+                          DO ipol=2,4
+                             CALL tg_cgather( dffts, dvscfins(:,ipol,ipert), tg_dv(:,ipol))
+                          ENDDO
+                       ENDIF
+                    ELSE
+                       CALL tg_cgather( dffts, dvscfins(:,current_spin,ipert), tg_dv(:,1))
+                    ENDIF
+                 ENDIF
+                 aux2=(0.0_DP,0.0_DP)
+                 do ibnd = 1, nbnd_occ (ikk), incr
+                    IF( dffts%has_task_groups ) THEN
+                       call cft_wave_tg (ik, evc, tg_psic, 1, v_siz, ibnd, nbnd_occ (ikk) )
+                       call apply_dpot(v_siz, tg_psic, tg_dv, 1)
+                       call cft_wave_tg (ik, aux2, tg_psic, -1, v_siz, ibnd, nbnd_occ (ikk))
+                    ELSE
+                       call cft_wave (ik, evc (1, ibnd), aux1, +1)
+                       call apply_dpot(dffts%nnr,aux1, dvscfins(1,1,ipert), current_spin)
+                       call cft_wave (ik, aux2 (1, ibnd), aux1, -1)
+                    ENDIF
+                 enddo
+                 dvpsi=dvpsi+aux2
+                 call stop_clock ('vpsifft')
+                 !
+                 !  In the case of US pseudopotentials there is an additional
+                 !  selfconsist term which comes from the dependence of D on
+                 !  V_{eff} on the bare change of the potential
+                 !
+                 IF (isolv==1) THEN
+                    call adddvscf_ph_mag (ipert, ik, becp1)
+                    !
+                    ! DFPT+U: add to dvpsi the scf part of the response
+                    ! Hubbard potential dV_hub
+                    !
+                    if (lda_plus_u) call adddvhubscf (ipert, ik)
+                 ELSE
+                    call adddvscf_ph_mag (ipert, ik, becpt)
+                 END IF
+                 !
+                 !  reset the original magnetic field if it was changed
+                 !
+                 IF (isolv==2) THEN
+                    dvscfins(:,2:4,ipert)=-dvscfins(:,2:4,ipert)
+                    IF (okvan) int3_nc(:,:,:,:,ipert)=int3_save(:,:,:,:,ipert,1)
+                 ENDIF
+                 !
+              else
+                 !
+                 ! At the first iteration dvbare_q*psi_kpoint is calculated
+                 ! and written to file.
+                 !
+                 IF (isolv==1) THEN
+                    call dvqpsi_us (ik, u (1, mode),.false., becp1, alphap )
+                    !
+                    ! DFPT+U: At the first ph iteration the bare perturbed 
+                    ! Hubbard potential dvbare_hub_q * psi_kpoint 
+                    ! is calculated and added to dvpsi.
+                    !
+                    if (lda_plus_u) call dvqhub_barepsi_us (ik, u(1,mode))
+                    !
+                 ELSE
+                    IF (okvan) THEN
+                       deeq_nc(:,:,:,:)=deeq_nc_save(:,:,:,:,2)
+                       int1_nc(:,:,:,:,:)=int1_nc_save(:,:,:,:,:,2)
+                    ENDIF
+                    call dvqpsi_us (ik, u (1, mode),.false., becpt, alphapt)
+                    IF (okvan) THEN
+                       deeq_nc(:,:,:,:)=deeq_nc_save(:,:,:,:,1)
+                       int1_nc(:,:,:,:,:)=int1_nc_save(:,:,:,:,:,1)
+                    ENDIF
+                 ENDIF
+                 call save_buffer (dvpsi, lrbar, iubar, nrec)
+              !
+              endif
+              !
+              ! Ortogonalize dvpsi to valence states: ps = <evq|dvpsi>
+              ! Apply -P_c^+.
+              !
+              CALL orthogonalize(dvpsi, evq, ikmk, ikmkmq, dpsi, npwq, .false.)
+              !
+              if (where_rec=='solve_lint'.or.iter > 1) then
+                 !
+                 ! starting value for delta_psi is read from iudwf
+                 !
+                 call get_buffer( dpsi, lrdwf, iudwf, nrec)
+                 !
+                 ! threshold for iterative solution of the linear system
+                 !
+                 thresh = min (1.d-1 * sqrt (dr2), 1.d-2)
+              else
+                 !
+                 !  At the first iteration dpsi and dvscfin are set to zero
+                 !
+                 dpsi(:,:) = (0.d0, 0.d0)
+                 dvscfin (:, :, ipert) = (0.d0, 0.d0)
+                 !
+                 ! starting threshold for iterative solution of the linear system
+                 !
+                 thresh = 1.0d-2
+              endif
+              !
+              ! iterative solution of the linear system (H-eS)*dpsi=dvpsi,
+              ! dvpsi=-P_c^+ (dvbare+dvscf)*psi , dvscf fixed.
+              !
+              IF (isolv==2) THEN
+                 vrs(:,2:4)=-vrs(:,2:4)
+                 IF (okvan) deeq_nc(:,:,:,:)=deeq_nc_save(:,:,:,:,2)
+              ENDIF
+              conv_root = .true.
+              
+              call cgsolve_all (ch_psi_all, cg_psi, et(1,ikmk), dvpsi, dpsi, &
+                   h_diag, npwx, npwq, thresh, ik, lter, conv_root, &
+                   anorm, nbnd_occ(ikk), npol )
+
+              IF (isolv==2) THEN
+                 vrs(:,2:4)=-vrs(:,2:4)
+                 IF (okvan) deeq_nc(:,:,:,:)=deeq_nc_save(:,:,:,:,1)
+              ENDIF
+              
+              ltaver = ltaver + lter
+              lintercall = lintercall + 1
+              if (.not.conv_root) WRITE( stdout, '(5x,"kpoint",i4," ibnd",i4,  &
+                   &              " solve_linter: root not converged ",es10.3)') &
+                   &              ik , ibnd, anorm
+              !
+              ! writes delta_psi on iunit iudwf, k=kpoint,
+              !
+              !               if (nksq.gt.1 .or. npert(irr).gt.1)
+              call save_buffer (dpsi, lrdwf, iudwf, nrec)
+              !
+              ! calculates dvscf, sum over k => dvscf_q_ipert
+              !
+              weight = wk (ikk)
+              IF (nsolv==2) weight=weight/2.0_DP
+              IF (noncolin) THEN
+                 call incdrhoscf_nc(drhoscf(1,1,ipert),weight,ik, &
+                      dbecsum_nc(1,1,1,1,ipert,isolv), dpsi, rsign)
+              ELSE
+                 call incdrhoscf (drhoscf(1,current_spin,ipert), weight, ik, &
+                      dbecsum(1,1,current_spin,ipert), dpsi)
+              END IF
+              ! on perturbations
+           enddo
+           ! on isolv
+        END DO
         ! on k-points
      enddo
      !
@@ -411,7 +512,15 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      !
      !  In the noncolinear, spin-orbit case rotate dbecsum
      !
-     IF (noncolin.and.okvan) CALL set_dbecsum_nc(dbecsum_nc, dbecsum, npe)
+     IF (noncolin.and.okvan) THEN
+        CALL set_dbecsum_nc(dbecsum_nc, dbecsum, npe)
+        IF (nsolv==2) THEN
+           dbecsum_aux=(0.0_DP,0.0_DP)
+           CALL set_dbecsum_nc(dbecsum_nc(1,1,1,1,1,2), dbecsum_aux, npe)
+           dbecsum(:,:,1,:)=dbecsum(:,:,1,:)+dbecsum_aux(:,:,1,:)
+           dbecsum(:,:,2:4,:)=dbecsum(:,:,2:4,:)-dbecsum_aux(:,:,2:4,:)
+        ENDIF
+     ENDIF
      !
      !    Now we compute for all perturbations the total charge and potential
      !
@@ -485,8 +594,7 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      !
         call setmixout(npe*dfftp%nnr*nspin_mag,(nhm*(nhm+1)*nat*nspin_mag*npe)/2, &
                     mixout, dvscfout, dbecsum, ndim, -1 )
-        call mix_potential (2*npe*dfftp%nnr*nspin_mag+2*ndim, &
-                         mixout, mixin, &
+        call mix_potential (2*npe*dfftp%nnr*nspin_mag+2*ndim, mixout, mixin, &
                          alpha_mix(kter), dr2, npe*tr2_ph/npol, iter, &
                          nmix_ph, flmixdpot, convt)
         call setmixout(npe*dfftp%nnr*nspin_mag,(nhm*(nhm+1)*nat*nspin_mag*npe)/2, &
@@ -515,12 +623,49 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
 !   calculate here the change of the D1-~D1 coefficients due to the phonon
 !   perturbation
 !
-     IF (okpaw) CALL PAW_dpotential(dbecsum,rho%bec,int3_paw,npe)
-     !
-     !     with the new change of the potential we compute the integrals
-     !     of the change of potential and Q
-     !
-     call newdq (dvscfin, npe)
+     IF (okvan) THEN
+        IF (okpaw) CALL PAW_dpotential(dbecsum,rho%bec,int3_paw,npe)
+        !
+        !     with the new change of the potential we compute the integrals
+        !     of the change of potential and Q
+        !
+        call newdq (dvscfin, npe)
+        !
+        !  In the noncollinear magnetic case computes the int3 coefficients with
+        !  the opposite sign of the magnetic field. They are saved in int3_save,
+        !  that must have been allocated by the calling routine 
+        !
+        IF (noncolin.AND.domag) THEN
+           int3_save(:,:,:,:,:,1)=int3_nc(:,:,:,:,:)
+           IF (okpaw) rho%bec(:,:,2:4)=-rho%bec(:,:,2:4)
+           DO ipert=1,npe
+              dvscfin(:,2:4,ipert)=-dvscfin(:,2:4,ipert)
+              IF (okpaw) dbecsum(:,:,2:4,ipert)=-dbecsum(:,:,2:4,ipert)
+           ENDDO
+           !
+           !   if needed recompute the paw coeffients with the opposite sign of
+           !   the magnetic field
+           !
+           IF (okpaw) CALL PAW_dpotential(dbecsum,rho%bec,int3_paw,npe)
+           !
+           !   here compute the int3 integrals
+           !
+           CALL newdq (dvscfin, npe)
+           int3_save(:,:,:,:,:,2)=int3_nc(:,:,:,:,:)
+           !
+           !  restore the correct sign of the magnetic field.
+           !
+           DO ipert=1,npe
+              dvscfin(:,2:4,ipert)=-dvscfin(:,2:4,ipert)
+              IF (okpaw) dbecsum(:,:,2:4,ipert)=-dbecsum(:,:,2:4,ipert)
+           ENDDO
+           IF (okpaw) rho%bec(:,:,2:4)=-rho%bec(:,:,2:4)
+           !
+           !  put into int3_nc the coefficient with +B
+           !
+           int3_nc(:,:,:,:,:)=int3_save(:,:,:,:,:,1)
+        ENDIF
+     END IF
 #if defined(__MPI)
      aux_avg (1) = DBLE (ltaver)
      aux_avg (2) = DBLE (lintercall)
@@ -597,58 +742,38 @@ SUBROUTINE solve_linter (irr, imode0, npe, drhoscf)
      DEALLOCATE( tg_dv )
      DEALLOCATE( tg_psic )
   ENDIF
+  IF (noncolin.AND.domag.AND.okvan) THEN
+     DEALLOCATE (int3_save)
+     DEALLOCATE (dbecsum_aux)
+  ENDIF
 
   call stop_clock ('solve_linter')
+
+  RETURN  
+
 END SUBROUTINE solve_linter
 
-
-SUBROUTINE setmixout(in1, in2, mix, dvscfout, dbecsum, ndim, flag )
-USE kinds, ONLY : DP
-USE mp_bands, ONLY : intra_bgrp_comm
-USE mp, ONLY : mp_sum
-IMPLICIT NONE
-INTEGER :: in1, in2, flag, ndim, startb, lastb
-COMPLEX(DP) :: mix(in1+in2), dvscfout(in1), dbecsum(in2)
-
-CALL divide (intra_bgrp_comm, in2, startb, lastb)
-ndim=lastb-startb+1
-
-IF (flag==-1) THEN
-   mix(1:in1)=dvscfout(1:in1)
-   mix(in1+1:in1+ndim)=dbecsum(startb:lastb)
-ELSE
-   dvscfout(1:in1)=mix(1:in1)
-   dbecsum=(0.0_DP,0.0_DP)
-   dbecsum(startb:lastb)=mix(in1+1:in1+ndim)
-   CALL mp_sum(dbecsum, intra_bgrp_comm)
-ENDIF
-END SUBROUTINE setmixout
-
 SUBROUTINE check_all_convt(convt)
   USE mp,        ONLY : mp_sum
   USE mp_images, ONLY : nproc_image, me_image, intra_image_comm
   IMPLICIT NONE
   LOGICAL,INTENT(in) :: convt
-  INTEGER,ALLOCATABLE :: convt_check(:)
+  INTEGER            :: tot_conv
   !
   IF(nproc_image==1) RETURN
   !
-  ALLOCATE(convt_check(nproc_image+1))
+  ! Work out how many processes have converged
   !
-  convt_check = 1
-  IF(convt) convt_check(me_image+1) = 0
+  tot_conv = 0
+  IF(convt) tot_conv = 1
+  CALL mp_sum(tot_conv, intra_image_comm)
   !
-  CALL mp_sum(convt_check, intra_image_comm)
-  !CALL mp_sum(ios, inter_pool_comm)
-  !CALL mp_sum(ios, intra_bgrp_comm)
-  !
-!  convt = ALL(convt_check==0)
-  IF(ANY(convt_check==0).and..not.ALL(convt_check==0) ) THEN
+  IF ((tot_conv > 0) .and. (tot_conv < nproc_image)) THEN
     CALL errore('check_all_convt', 'Only some processors converged: '&
-               &' something is wrong with solve_linter', 1)
+               &' either something is wrong with solve_linter, or a different'&
+               &' parallelism scheme should be used.', 1)
   ENDIF
   !
-  DEALLOCATE(convt_check)
   RETURN
   !
 END SUBROUTINE
