@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2018 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -7,46 +7,46 @@
 !
 !--------------------------------------------------------------
 subroutine zstar_eu_us
-!----------===========-----------------------------------------
+  !--------------------------------------------------------------
   !
   ! Calculates the additional part of the Born effective charges
-  ! in the case of USPP
+  ! in the case of USPP.
   !
-  !
-  USE kinds,     ONLY : DP
-  USE mp,        ONLY : mp_sum
-  USE mp_pools,  ONLY : inter_pool_comm
-  USE mp_bands,  ONLY : intra_bgrp_comm
-  USE cell_base, ONLY : omega
-  USE ions_base, ONLY : nat, ntyp => nsp, ityp
-  USE buffers,   ONLY : get_buffer
-  USE klist,     ONLY : xk, wk, ngk, igk_k
-  USE gvecs,     ONLY : doublegrid
-  USE fft_base,  ONLY : dfftp, dffts
-  USE fft_interfaces, ONLY : fft_interpolate
-  USE lsda_mod,  ONLY : nspin, current_spin, isk, lsda
-  USE uspp,      ONLY : okvan, nkb, vkb, nlcc_any
-  USE wvfct,     ONLY : nbnd, npwx
-  USE paw_variables, ONLY : okpaw
-  USE wavefunctions_module,    ONLY : evc
+  USE kinds,            ONLY : DP
+  USE mp,               ONLY : mp_sum
+  USE mp_pools,         ONLY : inter_pool_comm
+  USE mp_bands,         ONLY : intra_bgrp_comm
+  USE cell_base,        ONLY : omega
+  USE ions_base,        ONLY : nat, ntyp => nsp, ityp
+  USE buffers,          ONLY : get_buffer
+  USE klist,            ONLY : xk, wk, ngk, igk_k
+  USE gvecs,            ONLY : doublegrid
+  USE fft_base,         ONLY : dfftp, dffts
+  USE fft_interfaces,   ONLY : fft_interpolate
+  USE lsda_mod,         ONLY : nspin, current_spin, isk, lsda
+  USE uspp,             ONLY : okvan, nkb, vkb, nlcc_any
+  USE wvfct,            ONLY : nbnd, npwx
+  USE paw_variables,    ONLY : okpaw
+  USE wavefunctions,    ONLY : evc
   USE uspp_param,       ONLY : upf, nhm, nh
   USE noncollin_module, ONLY : noncolin, npol, nspin_mag
-  USE efield_mod, ONLY : zstareu0
-  USE phus,       ONLY : becsumort
-  USE modes,      ONLY : u, npert, nirr
-  USE units_ph,   ONLY : lrdwf, iucom, lrcom, lrebar, iuebar, lrdrhous, &
-                         iudrhous, iudwf, lrwfc, iuwfc
-  USE mp_pools, ONLY : nproc_pool, npool
-
-  USE control_lr, ONLY : nbnd_occ
-  USE lrus,       ONLY : int3, int3_paw
-  USE eqv,        ONLY : dvpsi, dpsi
-  USE qpoint,     ONLY : nksq
+  USE efield_mod,       ONLY : zstareu0
+  USE phus,             ONLY : becsumort
+  USE modes,            ONLY : u, npert, nirr
+  USE units_ph,         ONLY : lrdwf, iucom, lrcom, lrebar, iuebar, &
+                               lrdrhous, iudrhous, iudwf
+  USE units_lr,         ONLY : iuwfc, lrwfc
+  USE mp_pools,         ONLY : nproc_pool, npool
+  USE control_lr,       ONLY : nbnd_occ
+  USE lrus,             ONLY : int3, int3_paw
+  USE eqv,              ONLY : dvpsi, dpsi
+  USE qpoint,           ONLY : nksq, ikks
+  USE ldaU,             ONLY : lda_plus_u
   USE dv_of_drho_lr
   !
   implicit none
   integer :: npw, ibnd, jbnd, ipol, jpol, imode0, irr, imode, nrec, mode
-  integer :: ik, ig, ir, is, i, j, mu, ipert
+  integer :: ik, ikk, ig, ir, is, i, j, mu, ipert
   integer :: ih, jh, ijh
   integer :: iuhxc, lrhxc
   !
@@ -88,17 +88,18 @@ subroutine zstar_eu_us
   ! for the calculation of the Hartree and xc part
   !
   do ik = 1, nksq
-     npw = ngk(ik)
-     if (nksq.gt.1) call get_buffer (evc, lrwfc, iuwfc, ik)
-     if (lsda) current_spin = isk (ik)
-     call init_us_2 (npw, igk_k(1,ik), xk(1,ik), vkb)
-     weight = wk (ik)
+     ikk = ikks(ik)
+     npw = ngk(ikk)
+     if (nksq.gt.1) call get_buffer (evc, lrwfc, iuwfc, ikk)
+     if (lsda) current_spin = isk (ikk)
+     call init_us_2 (npw, igk_k(1,ikk), xk(1,ikk), vkb)
+     weight = wk (ikk)
      do jpol = 1, 3
         nrec = (jpol - 1) * nksq + ik
         call get_buffer(dpsi, lrdwf, iudwf, nrec)
         if (noncolin) then
            call incdrhoscf_nc (dvscf(1,1,jpol),weight,ik, &
-                              dbecsum_nc(1,1,1,1,jpol), dpsi)
+                              dbecsum_nc(1,1,1,1,jpol), dpsi, 1.0d0)
         else
            call incdrhoscf (dvscf(1,current_spin,jpol),weight,ik, &
                             dbecsum(1,1,current_spin,jpol), dpsi)
@@ -184,10 +185,11 @@ subroutine zstar_eu_us
   !
   allocate (dvkb(npwx,nkb,3))
   do ik = 1, nksq
-     npw = ngk(ik)
-     weight = wk (ik)
-     if (nksq.gt.1) call get_buffer (evc, lrwfc, iuwfc, ik)
-     call init_us_2 (npw, igk_k(1,ik), xk (1, ik), vkb)
+     ikk = ikks(ik)
+     npw = ngk(ikk)
+     weight = wk (ikk)
+     if (nksq.gt.1) call get_buffer (evc, lrwfc, iuwfc, ikk)
+     call init_us_2 (npw, igk_k(1,ikk), xk (1, ikk), vkb)
      call dvkb3(ik, dvkb)
      imode0 = 0
      do irr = 1, nirr
@@ -206,11 +208,17 @@ subroutine zstar_eu_us
 #if defined(__MPI)
               call mp_sum( pdsp, intra_bgrp_comm )
 #endif
+              !
+              ! DFPT+U: add to dvpsi the scf term 
+              ! dV_Hub(is)/dE_jpol|psi(ik)> contributing to 
+              ! the derivative of the eigenvalue w.r.t. E_jpol
+              !
+              if (lda_plus_u) call adddvhubscf (jpol, ik)
               !
               ! add the term of the double summation
               !
-              do ibnd = 1, nbnd_occ(ik)
-                 do jbnd = 1, nbnd_occ(ik)
+              do ibnd = 1, nbnd_occ(ikk)
+                 do jbnd = 1, nbnd_occ(ikk)
                     zstareu0(jpol,mode)=zstareu0(jpol, mode) +           &
                          weight *                                        &
                          dot_product(evc(1:npwx*npol,ibnd),              &
@@ -239,7 +247,7 @@ subroutine zstar_eu_us
               !
               ! And calculate finally the scalar product
               !
-              do ibnd = 1, nbnd_occ(ik)
+              do ibnd = 1, nbnd_occ(ikk)
                  zstareu0(jpol,mode)=zstareu0(jpol, mode) - weight *  &
                       dot_product(evc(1:npwx*npol,ibnd),dvpsi(1:npwx*npol,ibnd))
               enddo
