@@ -25,7 +25,6 @@ PROGRAM do_dos
                          two_fermi_energies
   USE wvfct,      ONLY : nbnd, et
   USE lsda_mod,   ONLY : lsda, nspin
-  USE noncollin_module, ONLY: noncolin
   USE mp,         ONLY : mp_bcast
   USE mp_world,   ONLY : world_comm
   USE mp_global,     ONLY : mp_startup
@@ -43,12 +42,14 @@ PROGRAM do_dos
   !
   CHARACTER(len=256) :: fildos, outdir
   CHARACTER(LEN=33) :: fermi_str
-  REAL(DP) :: E, DOSofE (2), DOSint, DeltaE, Emin, Emax, &
+  CHARACTER(LEN=20) :: bz_sum
+  REAL(DP) :: E, DOSofE (2), DOSint (2), DeltaE, Emin, Emax, &
               degauss1, E_unset=1000000.d0
   INTEGER :: nks2, n, ndos, ngauss1, ios
+  LOGICAL :: needwf = .FALSE.
 
   NAMELIST /dos/ outdir, prefix, fildos, degauss, ngauss, &
-       Emin, Emax, DeltaE
+       Emin, Emax, DeltaE, bz_sum
   !
   ! initialise environment
   !
@@ -72,6 +73,7 @@ PROGRAM do_dos
      DeltaE = 0.01d0
      ngauss = 0
      degauss= 0.d0
+     bz_sum = "" 
      !
      CALL input_from_file ( )
      !
@@ -92,26 +94,38 @@ PROGRAM do_dos
   CALL mp_bcast( tmp_dir, ionode_id, world_comm )
   CALL mp_bcast( prefix, ionode_id, world_comm )
   !
-  CALL read_xml_file( )
+  CALL read_file_new ( needwf )
   !
   IF ( ionode ) THEN
      !
      IF (nks /= nkstot) &
         CALL errore ('dos', 'pools not implemented, or incorrect file read', 1)
      !
-     IF (degauss1/=0.d0) THEN
-        degauss=degauss1
-        ngauss =ngauss1
-        WRITE( stdout,'(/5x,"Gaussian broadening (read from input): ",&
-             &        "ngauss,degauss=",i4,f12.6/)') ngauss,degauss
-        ltetra=.false.
-        lgauss=.true.
-     ELSEIF (ltetra) THEN
+     SELECT CASE (TRIM(bz_sum)) 
+        CASE ('tetrahedra', 'TETRAHEDRA') 
+           ltetra = .TRUE. 
+           lgauss = .FALSE.
+           tetra_type = 0 
+        CASE ('tetrahedra_lin', 'TETRAHEDRA_LIN' ) 
+           ltetra = .TRUE. 
+           lgauss = .FALSE.
+           tetra_type = 1 
+        CASE ('tetrahedra_opt' , 'TETRAHEDRA_OPT') 
+           ltetra = .TRUE. 
+           lgauss = .FALSE.
+           tetra_type = 2 
+        CASE ('smearing')
+           ltetra = .FALSE.
+           lgauss = .TRUE. 
+     END SELECT 
+     !
+     IF ( ltetra .AND. degauss1==0.d0 ) THEN
+        !
+        IF ( nk1*nk2*nk3 .eq. 0 ) CALL errore ('dos:', 'tetrahedra integration only with automatic ' // &
+                            & 'uniform k_point meshes.', tetra_type + 1) 
         !
         ! info on tetrahedra is no longer saved to file and must be rebuilt
         !
-        ! workaround for old xml file, to be removed
-        IF ( ALLOCATED ( tetra ) ) DEALLOCATE (tetra)
         ! in the lsda case, only the first half of the k points
         ! are needed in the input of "tetrahedra"
         !
@@ -135,17 +149,26 @@ PROGRAM do_dos
                 &                k1, k2, k3, nk1, nk2, nk3, nks2, xk, 1)
            !
         END IF
+        lgauss = .FALSE.
         !
-     ELSEIF (lgauss) THEN
+     ELSE IF ( degauss1/=0.d0 ) THEN
+        degauss=degauss1
+        ngauss =ngauss1
+        WRITE( stdout,'(/5x,"Gaussian broadening (read from input): ",&
+             &        "ngauss,degauss=",i4,f12.6/)') ngauss,degauss
+        lgauss=.true.
+        ltetra=.false.
+     ELSEIF ( lgauss ) THEN
         WRITE( stdout,'(/5x,"Gaussian broadening (read from file): ",&
              &        "ngauss,degauss=",i4,f12.6/)') ngauss,degauss
+        ltetra=.false.
      ELSE
         degauss=DeltaE/rytoev
         ngauss =0
         WRITE( stdout,'(/5x,"Gaussian broadening (default values): ",&
              &        "ngauss,degauss=",i4,f12.6/)') ngauss,degauss
-        ltetra=.false.
         lgauss=.true.
+        ltetra=.false.
      ENDIF
      !
      ! find min and max energy for plot (band extrema if not set)
@@ -170,9 +193,9 @@ PROGRAM do_dos
      IF ( fildos == ' ' ) fildos = trim(prefix)//'.dos'
      OPEN (unit = 4, file = fildos, status = 'unknown', form = 'formatted')
      IF ( two_fermi_energies ) THEN
-        WRITE(fermi_str,'(" EFermi = ",2f7.3," eV")') ef_up*rytoev, ef_dw*rytoev
+        WRITE(fermi_str,'(" EFermi = ",2f8.3," eV")') ef_up*rytoev, ef_dw*rytoev
      ELSE
-        WRITE(fermi_str,'(" EFermi = ",f7.3," eV")') ef*rytoev
+        WRITE(fermi_str,'(" EFermi = ",f8.3," eV")') ef*rytoev
      ENDIF
 
      IF (nspin==1.or.nspin==4) THEN
@@ -186,19 +209,23 @@ PROGRAM do_dos
         E = Emin + (n - 1) * DeltaE
         IF (ltetra) THEN
            IF (tetra_type == 0) THEN
-              CALL tetra_dos_t( et, nspin, nbnd, nks, E, DOSofE)
+              CALL tetra_dos_t( et, nspin, nbnd, nks, E, DOSofE, dosint)
            ELSE
-              CALL opt_tetra_dos_t( et, nspin, nbnd, nks, E, DOSofE)
+              CALL opt_tetra_dos_t( et, nspin, nbnd, nks, E, DOSofE, dosint)
            END IF
         ELSE
            CALL dos_g(et,nspin,nbnd, nks,wk,degauss,ngauss, E, DOSofE)
         ENDIF
         IF (nspin==1.or.nspin==4) THEN
-           DOSint = DOSint + DOSofE (1) * DeltaE
-           WRITE (4, '(f8.3,2e12.4)') E * rytoev, DOSofE(1)/rytoev, DOSint
+           IF ( .not. ltetra ) DOSint(1)  = DOSint(1) + DOSofE (1) * DeltaE
+           WRITE (4, '(f8.3,2e12.4)') E * rytoev, DOSofE(1)/rytoev, DOSint(1) 
         ELSE
-           DOSint = DOSint + (DOSofE (1) + DOSofE (2) ) * DeltaE
-           WRITE (4, '(f8.3,3e12.4)') E * rytoev, DOSofE/rytoev, DOSint
+           IF ( .not. ltetra )  THEN 
+               DOSint(1) = DOSint(1) + (DOSofE (1) + DOSofE (2) ) * DeltaE
+               WRITE (4, '(f8.3,3e12.4)') E * rytoev, DOSofE/rytoev, DOSint(1)
+           ELSE
+               WRITE (4, '(f8.3,3e12.4)') E * rytoev, DOSofE/rytoev, DOSint(1)+DOSint(2) 
+           END IF
         ENDIF
      ENDDO
 
