@@ -30,16 +30,17 @@ SUBROUTINE do_elf (elf)
   USE fft_interfaces, ONLY : fwfft, invfft, fft_interpolate
   USE gvect, ONLY: gcutm, g, ngm
   USE gvecs, ONLY : ngms, doublegrid, dual
-  USE io_files, ONLY: iunwfc, nwordwfc
+  USE io_files, ONLY: restart_dir
   USE klist, ONLY: nks, xk, ngk, igk_k
   USE lsda_mod, ONLY: nspin
   USE scf, ONLY: rho
   USE symme, ONLY: sym_rho, sym_rho_init
   USE wvfct, ONLY: nbnd, wg
   USE control_flags, ONLY: gamma_only
-  USE wavefunctions_module,  ONLY: evc
+  USE wavefunctions,  ONLY: evc
   USE mp_pools, ONLY: inter_pool_comm, intra_pool_comm
   USE mp, ONLY: mp_sum
+  USE pw_restart_new, ONLY : read_collected_wfc
   !
   ! I/O variables
   !
@@ -66,7 +67,7 @@ SUBROUTINE do_elf (elf)
      !
      !   reads the eigenfunctions
      !
-     CALL davcio (evc, 2*nwordwfc, iunwfc, ik, - 1)
+     CALL read_collected_wfc ( restart_dir(), ik, evc )
      !
      DO ibnd = 1, nbnd
         DO j = 1, 3
@@ -201,13 +202,6 @@ SUBROUTINE do_rdg (rdg)
   ! gradient of rho
   ALLOCATE( grho(3,dfftp%nnr) )
 
-  ! put the total (up+down) charge density in rho%of_r(*,1)
-  DO is = 2, nspin
-     rho%of_g(:,1) =  rho%of_g(:,1) + rho%of_g(:,is)
-     rho%of_r(:,1) =  rho%of_r(:,1) + rho%of_r(:,is)
-  ENDDO
-
-  ! gradient of rho
   CALL fft_gradient_g2r(dfftp, rho%of_g(1,1), g, grho)
 
   ! calculate rdg
