@@ -21,7 +21,7 @@ SUBROUTINE PAW_make_ae_charge(rho,withcore)
    USE uspp_param,        ONLY : nh, nhm, upf
    USE scf,               ONLY : scf_type
    USE fft_base,          ONLY : dfftp
-   USE mp_global,         ONLY : me_pool
+   USE fft_types,         ONLY : fft_index_to_3d
    USE splinelib,         ONLY : spline, splint
    USE cell_base,         ONLY : at, bg, alat
 
@@ -32,11 +32,12 @@ SUBROUTINE PAW_make_ae_charge(rho,withcore)
    INTEGER                 :: ir                    ! counter on grid point
    INTEGER                 :: is                    ! spin index
    INTEGER                 :: lm                    ! counters on angmom and radial grid
-   INTEGER                 :: j,k,l, idx, idx0
+   INTEGER                 :: j,k,l
    INTEGER                 :: ia
+   LOGICAL                 :: offrange
    REAL(DP),ALLOCATABLE    :: wsp_lm(:,:,:), ylm_posi(:,:), d1y(:), d2y(:)
    REAL(DP),ALLOCATABLE    :: rho_lm(:,:,:), rho_lm_ae(:,:,:), rho_lm_ps(:,:,:)
-   REAL(DP)                :: posi(3), first, second
+   REAL(DP)                :: posi(3), first, second, rhoup, rhodw
    REAL(DP)                :: inv_nr1, inv_nr2, inv_nr3, distsq
 
    ! Some initialization
@@ -110,18 +111,10 @@ SUBROUTINE PAW_make_ae_charge(rho,withcore)
          !
          rsp_point : DO ir = 1, dfftp%nr1x * dfftp%my_nr2p * dfftp%my_nr3p
             !
-            ! three dimensional indices (i,j,k)
-            idx   = ir - 1
-            k     = idx / (dfftp%nr1x*dfftp%my_nr2p)
-            idx   = idx - (dfftp%nr1x*dfftp%my_nr2p) * k
-            k     = k + dfftp%my_i0r3p
-            j     = idx /  dfftp%nr1x
-            idx   = idx -  dfftp%nr1x*j
-            j     = j + dfftp%my_i0r2p
-            l     = idx
+            ! three dimensional indices (l,j,k)
             !
-            ! ... do not include points outside the physical range!
-            IF ( l >=  dfftp%nr1 .or. j >=  dfftp%nr2 .or. k >=  dfftp%nr3 ) CYCLE rsp_point
+            CALL fft_index_to_3d (ir, dfftp, l,j,k, offrange)
+            IF ( offrange ) CYCLE rsp_point
             !
             DO ipol = 1, 3
                posi(ipol) = dble( l )*inv_nr1*at(ipol,1) + &
@@ -148,14 +141,30 @@ SUBROUTINE PAW_make_ae_charge(rho,withcore)
             !
             ! prepare spherical harmonics
             CALL ylmr2( i%l**2, 1, posi, distsq, ylm_posi )
-            DO is = 1,nspin
+            IF ( nspin/=2 ) THEN
+               DO is = 1,nspin
+                  DO lm = 1, i%l**2
+                     ! do interpolation - distsq depends upon ir
+                     rho%of_r(ir,is)= rho%of_r(ir,is) + ylm_posi(1,lm) &
+                          * splint(g(i%t)%r(:) , rho_lm(:,lm,is), &
+                          wsp_lm(:,lm,is), sqrt(distsq) )
+                  ENDDO
+               ENDDO
+            ELSE
                DO lm = 1, i%l**2
                   ! do interpolation
-                  rho%of_r(ir,is)= rho%of_r(ir,is) + ylm_posi(1,lm) &
-                       * splint(g(i%t)%r(:) , rho_lm(:,lm,is), &
+                  is = 1
+                  rhoup = splint(g(i%t)%r(:) , rho_lm(:,lm,is), &
                        wsp_lm(:,lm,is), sqrt(distsq) )
+                  is = 2
+                  rhodw = splint(g(i%t)%r(:) , rho_lm(:,lm,is), &
+                       wsp_lm(:,lm,is), sqrt(distsq) )
+                  rho%of_r(ir,1)= rho%of_r(ir,1) + ylm_posi(1,lm) * &
+                       (rhoup + rhodw)
+                  rho%of_r(ir,2)= rho%of_r(ir,2) + ylm_posi(1,lm) * &
+                       (rhoup - rhodw)
                ENDDO
-            ENDDO
+            ENDIF
          ENDDO rsp_point
          !
          DEALLOCATE(rho_lm, ylm_posi, wsp_lm)
