@@ -34,7 +34,6 @@ SUBROUTINE PAW_make_ae_charge_ ( rho, flag, nx, r, rhopaw )
    USE uspp_param,        ONLY : nh, nhm, upf
    USE scf,               ONLY : scf_type
    USE fft_base,          ONLY : dfftp
-   USE mp_global,         ONLY : me_pool
    USE splinelib,         ONLY : spline, splint
    USE cell_base,         ONLY : at, bg, alat
    !
@@ -274,12 +273,13 @@ PROGRAM PAWplot
   ELSE
      IF (spin_component == 0) THEN
         rhog (:) = rho%of_g(:,1)
-        DO is = 2, nspin
-           rhog(:) = rhog (:) + rho%of_g(:,is)
-        ENDDO
      ELSE
-        IF (nspin == 2) current_spin = spin_component
-        rhog (:) = rho%of_g(:,current_spin)
+        IF (nspin == 2) THEN
+           rhog (:) = ( rho%of_g(:,1) + REAL(1-2*(spin_component/2)) * &
+                                                   rho%of_g(:,2) )*0.5d0
+        ELSE
+           rhog (:) = rho%of_g(:,current_spin)
+        ENDIF
      ENDIF
   ENDIF
   !
@@ -350,9 +350,9 @@ SUBROUTINE plot_1d_ (nx, x0, e, rhog, rhoplot, flag, filplot )
   USE kinds, ONLY : DP
   USE constants, ONLY:  pi
   USE io_global, ONLY : stdout, ionode
-  USE mp_global,  ONLY : intra_pool_comm
-  USE mp,         ONLY : mp_sum
-  USE gvect,      ONLY : g, gstart, ngm
+  USE mp_pools,  ONLY : intra_pool_comm
+  USE mp,        ONLY : mp_sum
+  USE gvect,     ONLY : g, gstart, ngm
   USE control_flags, ONLY : gamma_only
 
   IMPLICIT NONE
