@@ -33,16 +33,16 @@ SUBROUTINE projwave_boxes( filpdos, filproj, n_proj_boxes, irmin, irmax, plotbox
   USE control_flags, ONLY: gamma_only
   USE uspp,      ONLY: okvan
   USE noncollin_module, ONLY: noncolin, npol
-  USE wavefunctions_module, ONLY: evc,    psic
-  USE wavefunctions_module, ONLY: psic_nc
-  USE io_files,             ONLY : iunwfc, nwordwfc
+  USE wavefunctions, ONLY: evc,    psic, psic_nc
+  USE io_files,             ONLY : restart_dir
   USE scf,                  ONLY : rho
   USE projections_ldos,     ONLY : proj
   USE fft_base,             ONLY : dfftp
   USE scatter_mod,          ONLY : scatter_grid
   USE fft_interfaces,       ONLY : invfft
-  USE mp_global,            ONLY : intra_pool_comm
+  USE mp_pools,             ONLY : intra_pool_comm
   USE mp,                   ONLY : mp_sum
+  USE pw_restart_new,       ONLY : read_collected_wfc
 !
   !
   IMPLICIT NONE
@@ -187,9 +187,6 @@ SUBROUTINE projwave_boxes( filpdos, filproj, n_proj_boxes, irmin, irmax, plotbox
      CALL DCOPY (dfftp%nnr, rho%of_r, 1, raux, 1)
   ELSE
      CALL DCOPY (dfftp%nnr, rho%of_r (1, 1), 1, raux, 1)
-     DO is = 2, nspin
-        CALL DAXPY (dfftp%nnr, 1.d0, rho%of_r (1, is), 1, raux, 1)
-     ENDDO
   ENDIF
   !
   ! B2. Integrate the charge
@@ -226,7 +223,7 @@ SUBROUTINE projwave_boxes( filpdos, filproj, n_proj_boxes, irmin, irmax, plotbox
      !
      IF ( lsda ) current_spin = isk(ik)
      npw = ngk(ik)
-     CALL davcio (evc, 2*nwordwfc, iunwfc, ik, - 1)
+     CALL read_collected_wfc ( restart_dir(), ik, evc )
      !
      bnd_loop: DO ibnd = 1, nbnd
         !
@@ -453,7 +450,7 @@ SUBROUTINE partialdos_boxes(Emin, Emax, DeltaE, kresolveddos, filpdos, n_proj_bo
            WRITE (4,'(i5," ")', advance="NO") ik
         ENDIF
         etev = Emin + ie * DeltaE
-        WRITE (4,'(f7.3,4(2e11.3),999(2e11.3))') etev*rytoev,  &
+        WRITE (4,'(f8.3,4(2e11.3),999(2e11.3))') etev*rytoev,  &
              dostot(ie,1:nspin0,ik), dosboxtot(ie,1:nspin0,ik), &
              ( dosbox(ie,ibox,1:nspin0,ik), ibox = 1, n_proj_boxes )
      ENDDO
