@@ -20,18 +20,18 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
   USE wvfct,                ONLY : et, nbnd, npwx
   USE klist,                ONLY : xk, nks, nkstot, ngk, igk_k
   USE io_files,             ONLY : nwordwfc, iunwfc
-  USE symm_base,            ONLY : s, ftau, nsym, t_rev, invs, sname
+  USE symm_base,            ONLY : s, nsym, ft, t_rev, invs, sname
   USE rap_point_group,      ONLY : code_group, nclass, nelem, elem, which_irr, &
        char_mat, name_rap, name_class, gname, ir_ram
   USE rap_point_group_so,   ONLY : nrap, nelem_so, elem_so, has_e, &
        which_irr_so, char_mat_so, name_rap_so, &
        name_class_so, d_spin, name_class_so1
-  USE rap_point_group_is,   ONLY : nsym_is, sr_is, ftau_is, gname_is, &
+  USE rap_point_group_is,   ONLY : nsym_is, sr_is, ft_is, gname_is, &
        sname_is, code_group_is
   USE uspp,                 ONLY : nkb, vkb
   USE spin_orb,             ONLY : domag
   USE noncollin_module,     ONLY : noncolin
-  USE wavefunctions_module, ONLY : evc
+  USE wavefunctions, ONLY : evc
   USE io_global,            ONLY : ionode, ionode_id, stdout
   USE mp,                   ONLY : mp_bcast
   USE mp_images,            ONLY : intra_image_comm
@@ -42,12 +42,13 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
   INTEGER :: npw, spin_component, nks1, nks2, firstk, lastk
   INTEGER :: nks1tot, nks2tot
   INTEGER :: iunout, igroup, irap, dim_rap, ios
-  INTEGER :: sk(3,3,48), ftauk(3,48), gk(3,48), sk_is(3,3,48), &
+  INTEGER :: sk(3,3,48), gk(3,48), sk_is(3,3,48), &
        gk_is(3,48), invs_is(48), t_revk(48), invsk(48), nsymk, isym, ipol, jpol
+  REAL(dp) :: ftk(3,48)
   LOGICAL :: is_complex, is_complex_so, is_symmorphic, search_sym
   LOGICAL, ALLOCATABLE :: high_symmetry(:)
   REAL(DP), PARAMETER :: accuracy=1.d-4
-  COMPLEX(DP) :: d_spink(2,2,48), d_spin_is(2,2,48), zdotc
+  COMPLEX(DP) :: d_spink(2,2,48), d_spin_is(2,2,48)
   COMPLEX(DP),ALLOCATABLE :: times(:,:,:)
   REAL(DP) :: dxk(3), dkmod, dkmod_save, modk1, modk2, k1(3), k2(3), ps
   INTEGER, ALLOCATABLE :: rap_et(:,:), code_group_k(:)
@@ -96,12 +97,12 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
      !
      ! Find the small group of k
      !
-     CALL smallgk (xk(1,ik), at, bg, s, ftau, t_rev, sname, nsym, sk, &
-          ftauk, gk, t_revk, invsk, snamek, nsymk)
+     CALL smallgk (xk(1,ik), at, bg, s, ft, t_rev, sname, nsym, sk, &
+          ftk, gk, t_revk, invsk, snamek, nsymk)
      !
      !  character of the irreducible representations
      !
-     CALL find_info_group(nsymk,sk,t_revk,ftauk,d_spink,gk,snamek,&
+     CALL find_info_group(nsymk,sk,t_revk,ftk,d_spink,gk,snamek,&
           sk_is,d_spin_is,gk_is,invs_is,is_symmorphic,search_sym)
      code_group_k(ik)=code_group
      !
@@ -115,16 +116,16 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
      IF (noncolin) THEN
         IF (domag) THEN
            CALL find_band_sym_so(ik,evc,et(1,ik),nsym_is, &
-                sk_is,ftau_is,d_spin_is,gk_is,invs_is,&
+                sk_is,ft_is,d_spin_is,gk_is,invs_is,&
                 rap_et(1,ik),times(1,1,ik), &
                 ngroup(ik),istart(1,ik),accuracy)
         ELSE
-           CALL find_band_sym_so(ik,evc,et(1,ik),nsymk,sk,ftauk,d_spink,&
+           CALL find_band_sym_so(ik,evc,et(1,ik),nsymk,sk,ftk,d_spink,&
                 gk,invsk,rap_et(1,ik),times(1,1,ik),ngroup(ik),&
                 istart(1,ik),accuracy)
         ENDIF
      ELSE
-        CALL find_band_sym (ik,evc, et(1,ik), nsymk, sk, ftauk, gk, invsk, &
+        CALL find_band_sym (ik,evc, et(1,ik), nsymk, sk, ftk, gk, invsk, &
              rap_et(1,ik), times(1,1,ik), ngroup(ik),&
              istart(1,ik),accuracy)
      ENDIF
@@ -166,9 +167,9 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
      END DO
 !
      DO ik=nks1tot, nks2tot
-        CALL smallgk (xk(1,ik), at, bg, s, ftau, t_rev, sname, &
-             nsym, sk, ftauk, gk, t_revk, invsk, snamek, nsymk)
-        CALL find_info_group(nsymk,sk,t_revk,ftauk,d_spink,gk,snamek,&
+        CALL smallgk (xk(1,ik), at, bg, s, ft, t_rev, sname, &
+             nsym, sk, ftk, gk, t_revk, invsk, snamek, nsymk)
+        CALL find_info_group(nsymk,sk,t_revk,ftk,d_spink,gk,snamek,&
              sk_is,d_spin_is,gk_is,invs_is,is_symmorphic,search_sym)
         IF (code_group_k(ik) /= code_group) &
              CALL errore('sym_band','problem with code_group',1)
@@ -309,7 +310,7 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
   RETURN
 END SUBROUTINE sym_band
 !
-SUBROUTINE find_band_sym (ik,evc,et,nsym,s,ftau,gk,invs,rap_et,times,ngroup,&
+SUBROUTINE find_band_sym (ik,evc,et,nsym,s,ft,gk,invs,rap_et,times,ngroup,&
                           istart,accuracy)
   !
   !   This subroutine finds the irreducible representations which give
@@ -342,14 +343,14 @@ SUBROUTINE find_band_sym (ik,evc,et,nsym,s,ftau,gk,invs,rap_et,times,ngroup,&
   INTEGER ::                  &
        nsym,             &
        rap_et(nbnd),     &
-       ftau(3,48),       &
        gk(3,48),         &
        s(3,3,48),        &
        invs(48),         &
        ngroup,           &  ! number of different frequencies groups
        istart(nbnd+1)
 
-  REAL(DP) ::                 &
+  REAL(DP) ::             &
+       ft(3,48),          &
        et(nbnd)
 
   COMPLEX(DP) ::  &
@@ -367,8 +368,7 @@ SUBROUTINE find_band_sym (ik,evc,et,nsym,s,ftau,gk,invs,rap_et,times,ngroup,&
        iclass,    &
        shift,     &
        na, i, j, ig, dimen, nrxx, npw
-
-  COMPLEX(DP) :: zdotc
+  INTEGER :: s_scaled(3,3,nsym), ftau(3,nsym)
 
   REAL(DP), ALLOCATABLE ::  w1(:)
   COMPLEX(DP), ALLOCATABLE ::  evcr(:,:), trace(:,:), psic(:,:)
@@ -404,6 +404,10 @@ SUBROUTINE find_band_sym (ik,evc,et,nsym,s,ftau,gk,invs,rap_et,times,ngroup,&
      CALL invfft ('Rho', psic(:,ibnd), dfftp)
   ENDDO
   !
+  !  scale sym.ops. and fractional translations with FFT grids
+  !
+  CALL scale_sym_ops (nsym, s, ft, dfftp%nr1, dfftp%nr2, dfftp%nr3, s_scaled, ftau)
+  !
   !  Find the character of one symmetry operation per class
   !
   DO iclass=1,nclass
@@ -417,8 +421,8 @@ SUBROUTINE find_band_sym (ik,evc,et,nsym,s,ftau,gk,invs,rap_et,times,ngroup,&
      IF (irot==1) THEN
         evcr=evc
      ELSE
-        CALL rotate_all_psi(ik,psic,evcr,s(1,1,invs(irot)), &
-                               ftau(1,invs(irot)),gk(1,invs(irot)))
+        CALL rotate_all_psi(ik, psic, evcr, s_scaled(1,1,invs(irot)), &
+                               ftau(1,invs(irot)), gk(1,invs(irot)))
      ENDIF
      !
      !   and apply S if necessary
@@ -436,7 +440,7 @@ SUBROUTINE find_band_sym (ik,evc,et,nsym,s,ftau,gk,invs,rap_et,times,ngroup,&
         DO i=1,dim_rap
            ibnd=istart(igroup)+i-1
            trace(iclass,igroup)=trace(iclass,igroup) + &
-                zdotc(npw,evc(1,ibnd),1,evcr(1,ibnd),1)
+                DOT_PRODUCT (evc(1:npw,ibnd),evcr(1:npw,ibnd))
         ENDDO
         !      write(6,*) igroup, iclass, trace(iclass,igroup)
      ENDDO
@@ -589,7 +593,7 @@ SUBROUTINE rotate_all_psi(ik,psic,evcr,s,ftau,gk)
         DO k = 1, nr3
           DO j = 1, nr2
              DO i = 1, nr1
-              CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+              CALL rotate_grid_point (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
               ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
               rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
               arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
@@ -603,7 +607,7 @@ SUBROUTINE rotate_all_psi(ik,psic,evcr,s,ftau,gk)
         DO k = 1, nr3
            DO j = 1, nr2
               DO i = 1, nr1
-                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                 CALL rotate_grid_point (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
                  ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
                  rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
                  psir_collect(ir)=psic_collect(rir, ibnd)
@@ -632,7 +636,7 @@ SUBROUTINE rotate_all_psi(ik,psic,evcr,s,ftau,gk)
         DO k = 1, nr3
            DO j = 1, nr2
               DO i = 1, nr1
-                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                 CALL rotate_grid_point (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
                  ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
                  rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
                  arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
@@ -646,7 +650,7 @@ SUBROUTINE rotate_all_psi(ik,psic,evcr,s,ftau,gk)
         DO k = 1, nr3
            DO j = 1, nr2
               DO i = 1, nr1
-                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                 CALL rotate_grid_point (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
                  ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
                  rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
                  psir(ir)=psic(rir,ibnd)
@@ -666,7 +670,7 @@ SUBROUTINE rotate_all_psi(ik,psic,evcr,s,ftau,gk)
   RETURN
 END SUBROUTINE rotate_all_psi
 
-SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ftau,d_spin,gk, &
+SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ft,d_spin,gk, &
      invs,rap_et,times,ngroup,istart,accuracy)
 
   !
@@ -680,6 +684,7 @@ SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ftau,d_spin,gk, &
   USE io_global,          ONLY : stdout
   USE kinds,              ONLY : DP
   USE constants,          ONLY : rytoev
+  USE fft_base,           ONLY : dfftp
   USE rap_point_group,    ONLY : code_group, nclass, gname
   USE rap_point_group_so, ONLY : nrap, nelem_so, elem_so, has_e, which_irr_so, &
        char_mat_so, name_rap_so, name_class_so,      &
@@ -688,7 +693,6 @@ SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ftau,d_spin,gk, &
   USE gvect,              ONLY : ngm
   USE wvfct,              ONLY : nbnd, npwx
   USE klist,              ONLY : ngk
-  USE spin_orb,           ONLY : domag
   USE uspp,               ONLY : vkb, nkb, okvan
   USE noncollin_module,   ONLY : npol
   USE becmod,             ONLY : bec_type, becp, calbec, allocate_bec_type, deallocate_bec_type
@@ -705,12 +709,12 @@ SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ftau,d_spin,gk, &
        ngroup,           &
        istart(nbnd+1),   &
        rap_et(nbnd),     &
-       ftau(3,48),       &
        gk(3,48),         &
        invs(48),         &
        s(3,3,48)
 
-  REAL(DP) ::                 &
+  REAL(DP) ::            &
+       ft(3,48),         &
        et(nbnd)
 
   COMPLEX(DP) ::  &
@@ -720,6 +724,7 @@ SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ftau,d_spin,gk, &
 
   REAL(DP), PARAMETER :: eps=1.d-5
 
+  INTEGER :: s_scaled(3,3,nsym), ftau(3,nsym)
   INTEGER ::         &
        ibnd,      &
        igroup,    &
@@ -730,8 +735,6 @@ SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ftau,d_spin,gk, &
        iclass,    &
        na, i, j, ig, ipol, jpol, jrap, dimen, npw
 
-  COMPLEX(DP) :: zdotc          ! moltiplication factors
-
   REAL(DP), ALLOCATABLE ::  w1(:)      ! list of energy eigenvalues in eV
   COMPLEX(DP), ALLOCATABLE ::  evcr(:,:), & ! the rotated of each wave function
        trace(:,:)   ! the trace of the symmetry matrix
@@ -759,6 +762,10 @@ SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ftau,d_spin,gk, &
      ENDIF
   ENDDO
   istart(ngroup+1)=nbnd+1
+  !
+  !  scale sym.ops. and fractional translations with FFT grid
+  !
+  CALL scale_sym_ops (nsym, s, ft, dfftp%nr1, dfftp%nr2, dfftp%nr3, s_scaled, ftau)
 
   trace=(0.d0,0.d0)
   DO iclass=1,nclass
@@ -768,7 +775,7 @@ SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ftau,d_spin,gk, &
      !   NB: rotate_psi assumes that s is in the small group of k. It does not
      !       rotate the k point.
      !
-      CALL rotate_all_psi_so(ik,evc,evcr,s(1,1,invs(irot)),        &
+     CALL rotate_all_psi_so(ik, evc, evcr, s_scaled(1,1,invs(irot)),        &
            ftau(1,invs(irot)),d_spin(1,1,irot),has_e(1,iclass),gk(1,invs(irot)))
      !
      !   and apply S in the US case.
@@ -786,7 +793,7 @@ SUBROUTINE find_band_sym_so (ik,evc,et,nsym,s,ftau,d_spin,gk, &
         DO i=1,dim_rap
            ibnd=istart(igroup)+i-1
            trace(iclass,igroup)=trace(iclass,igroup) +            &
-                zdotc(2*npwx,evc(1,ibnd),1,evcr(1,ibnd),1)
+                DOT_PRODUCT (evc(:,ibnd),evcr(:,ibnd))
         ENDDO
         !      write(6,*) igroup, iclass, dim_rap, trace(iclass,igroup)
      ENDDO
@@ -962,7 +969,7 @@ SUBROUTINE rotate_all_psi_so(ik,evc_nc,evcr,s,ftau,d_spin,has_e,gk)
            DO k = 1, nr3
               DO j = 1, nr2
                  DO i = 1, nr1
-                    CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                    CALL rotate_grid_point (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
                     ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
                     rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
                     arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
@@ -976,7 +983,7 @@ SUBROUTINE rotate_all_psi_so(ik,evc_nc,evcr,s,ftau,d_spin,has_e,gk)
            DO k = 1, nr3
               DO j = 1, nr2
                  DO i = 1, nr1
-                    CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                    CALL rotate_grid_point (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
                     ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
                     rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
                     psir_collect(ir)=psic_collect(rir,ibnd)
@@ -1001,7 +1008,7 @@ SUBROUTINE rotate_all_psi_so(ik,evc_nc,evcr,s,ftau,d_spin,has_e,gk)
            DO k = 1, nr3
               DO j = 1, nr2
                  DO i = 1, nr1
-                    CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                    CALL rotate_grid_point (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
                     ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
                     rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
                     arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
@@ -1015,7 +1022,7 @@ SUBROUTINE rotate_all_psi_so(ik,evc_nc,evcr,s,ftau,d_spin,has_e,gk)
            DO k = 1, nr3
               DO j = 1, nr2
                  DO i = 1, nr1
-                    CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                    CALL rotate_grid_point (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
                     ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
                     rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
                     psir(ir)=psic(rir,ibnd)
@@ -1060,8 +1067,8 @@ SUBROUTINE find_nks1nks2(firstk,lastk,nks1tot,nks1,nks2tot,nks2,spin_component)
   !  by the  current pool.
   !
   USE lsda_mod, ONLY : nspin
-  USE klist,  ONLY : nks, nkstot
-  USE mp_global, ONLY : my_pool_id, npool, kunit
+  USE klist,    ONLY : nks, nkstot
+  USE mp_pools, ONLY : my_pool_id, npool, kunit
 
   IMPLICIT NONE
   INTEGER, INTENT(out) :: nks1tot,nks1,nks2tot,nks2
@@ -1104,7 +1111,7 @@ SUBROUTINE find_nks1nks2(firstk,lastk,nks1tot,nks1,nks2tot,nks2,spin_component)
 
 END SUBROUTINE find_nks1nks2
 
-SUBROUTINE find_info_group(nsym,s,t_rev,ftau,d_spink,gk,sname,  &
+SUBROUTINE find_info_group(nsym,s,t_rev,ft,d_spink,gk,sname,  &
      s_is,d_spin_is,gk_is, invs_is,is_symmorphic,search_sym)
   !
   ! This routine receives as input a point group and sets the corresponding
@@ -1122,7 +1129,7 @@ SUBROUTINE find_info_group(nsym,s,t_rev,ftau,d_spink,gk,sname,  &
   USE rap_point_group_so,   ONLY : nrap, nelem_so, elem_so, has_e, &
        which_irr_so, char_mat_so, name_rap_so, &
        name_class_so, d_spin, name_class_so1
-  USE rap_point_group_is,   ONLY : nsym_is, sr_is, ftau_is, gname_is, &
+  USE rap_point_group_is,   ONLY : nsym_is, sr_is, ft_is, gname_is, &
        sname_is, code_group_is
 
   IMPLICIT NONE
@@ -1130,8 +1137,8 @@ SUBROUTINE find_info_group(nsym,s,t_rev,ftau,d_spink,gk,sname,  &
   INTEGER, INTENT(in) :: nsym,        & ! dimension of the group
        s(3,3,48),   & ! rotation matrices
        t_rev(48),   & ! if time reversal is need
-       ftau(3,48),  & ! fractionary translation
        gk(3,48)
+  REAL(dp), INTENT(IN) :: ft(3,48)! fractionary translation
 
   INTEGER, INTENT(out) :: s_is(3,3,48),   & ! rotation matrices
        gk_is(3,48), invs_is(48)
@@ -1144,7 +1151,7 @@ SUBROUTINE find_info_group(nsym,s,t_rev,ftau,d_spink,gk,sname,  &
 
   CHARACTER(len=45), INTENT(in) :: sname(48)
 
-  REAL(DP) :: sr(3,3,48), ft(3,48)
+  REAL(DP) :: sr(3,3,48)
   INTEGER :: isym, jsym, ss(3,3)
   LOGICAL :: found
 
@@ -1153,17 +1160,12 @@ SUBROUTINE find_info_group(nsym,s,t_rev,ftau,d_spink,gk,sname,  &
   search_sym=.true.
 
   DO isym=1,nsym
-     is_symmorphic=( is_symmorphic.and.(ftau(1,isym)==0).and.  &
-          (ftau(2,isym)==0).and.  &
-          (ftau(3,isym)==0) )
+     is_symmorphic=( is_symmorphic.and.(ft(1,isym)==0.d0).and.  &
+          (ft(2,isym)==0.d0).and.  &
+          (ft(3,isym)==0.d0) )
   ENDDO
 
   IF (.NOT.is_symmorphic) THEN
-     DO isym = 1, nsym
-        ft(1,isym) = DBLE(ftau(1,isym)) / DBLE(dfftp%nr1)
-        ft(2,isym) = DBLE(ftau(2,isym)) / DBLE(dfftp%nr2)
-        ft(3,isym) = DBLE(ftau(3,isym)) / DBLE(dfftp%nr3)
-     END DO
 
      DO isym=1,nsym
         DO jsym=1,nsym
@@ -1191,7 +1193,7 @@ SUBROUTINE find_info_group(nsym,s,t_rev,ftau,d_spink,gk,sname,  &
               CALL find_u(sr_is(1,1,nsym_is),d_spin_is(1,1,nsym_is))
               s_is(:,:,nsym_is)=s(:,:,isym)
               gk_is(:,nsym_is)=gk(:,isym)
-              ftau_is(:,nsym_is)=ftau(:,isym)
+              ft_is(:,nsym_is)=ft(:,isym)
               sname_is(nsym_is)=sname(isym)
            ENDIF
         ELSE
@@ -1224,21 +1226,16 @@ SUBROUTINE find_info_group(nsym,s,t_rev,ftau,d_spink,gk,sname,  &
      search_sym=.TRUE.
 
      DO isym=1,nsym_is
-        is_symmorphic=( is_symmorphic.AND.(ftau_is(1,isym)==0).AND.  &
-                                          (ftau_is(2,isym)==0).AND.  &
-                                          (ftau_is(3,isym)==0) )
+        is_symmorphic=( is_symmorphic.AND.(ft_is(1,isym)==0.d0).AND.  &
+                                          (ft_is(2,isym)==0.d0).AND.  &
+                                          (ft_is(3,isym)==0.d0) )
      ENDDO
      IF (.NOT.is_symmorphic) THEN
-        DO isym = 1, nsym_is
-           ft(1,isym) = DBLE(ftau_is(1,isym)) / DBLE(dfftp%nr1)
-           ft(2,isym) = DBLE(ftau_is(2,isym)) / DBLE(dfftp%nr2)
-           ft(3,isym) = DBLE(ftau_is(3,isym)) / DBLE(dfftp%nr3)
-        END DO
         DO isym=1,nsym_is
            DO jsym=1,nsym_is
-              search_sym=search_sym.AND.(ABS(gk_is(1,isym)*ft(1,jsym)+ &
-                                             gk_is(2,isym)*ft(2,jsym)+     &
-                                             gk_is(3,isym)*ft(3,jsym))<1.D-8) 
+              search_sym=search_sym.AND.(ABS(gk_is(1,isym)*ft_is(1,jsym)+ &
+                                             gk_is(2,isym)*ft_is(2,jsym)+     &
+                                             gk_is(3,isym)*ft_is(3,jsym))<1.D-8) 
            ENDDO
         ENDDO
      ENDIF
