@@ -1,4 +1,4 @@
-# LaTeX2HTML 2008 (1.71)
+# LaTeX2HTML 2019.2 (Released June 5, 2019)
 # Associate labels original text with physical files.
 
 
@@ -13,17 +13,21 @@ $noresave{$key} = "$nosave";
 1;
 
 
-# LaTeX2HTML 2008 (1.71)
+# LaTeX2HTML 2019.2 (Released June 5, 2019)
 # labels from external_latex_labels array.
 
 
-$key = q/pw2casino_info/;
-$external_latex_labels{$key} = q|3.5|; 
-$noresave{$key} = "$nosave";
-
 $key = q/SubSec:badpara/;
 $external_latex_labels{$key} = q|4.4|; 
 $noresave{$key} = "$nosave";
 
+$key = q/_/;
+$external_latex_labels{$key} = q|<|; 
+$noresave{$key} = "$nosave";
+
+$key = q/pw2casino_info/;
+$external_latex_labels{$key} = q|3.5|; 
+$noresave{$key} = "$nosave";
+
 1;
 
