@@ -5,12 +5,10 @@
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
-!
 !----------------------------------------------------------------------
-SUBROUTINE addusdens(rho)
+SUBROUTINE addusdens( rho )
   !----------------------------------------------------------------------
-  !
-  ! ... Add US contribution to the charge density to rho(G)
+  !! Add US contribution to the charge density to \(\text{rho}(G)\).
   !
   USE realus,               ONLY : addusdens_r
   USE control_flags,        ONLY : tqr
@@ -20,28 +18,29 @@ SUBROUTINE addusdens(rho)
   !
   IMPLICIT NONE
   !
-  !
-  COMPLEX(kind=dp), INTENT(inout) :: rho(dfftp%ngm,nspin_mag)
+  COMPLEX(DP), INTENT(INOUT) :: rho(dfftp%ngm,nspin_mag)
+  !! Charge density in G space
   !
   IF ( tqr ) THEN
-     CALL addusdens_r(rho)
+     CALL addusdens_r( rho )
   ELSE
-     CALL addusdens_g(rho)
+     CALL addusdens_g( rho )
   ENDIF
   !
   RETURN
   !
 END SUBROUTINE addusdens
 !
+!
 !----------------------------------------------------------------------
-SUBROUTINE addusdens_g(rho)
+SUBROUTINE addusdens_g( rho )
   !----------------------------------------------------------------------
-  !
-  !  This routine adds to the charge density rho(G) in reciprocal space
-  !  the part which is due to the US augmentation.
+  !! This routine adds to the charge density \(\text{rho}(G)\) in reciprocal space
+  !! the part which is due to the US augmentation.
   !
   USE kinds,                ONLY : DP
   USE ions_base,            ONLY : nat, ntyp => nsp, ityp
+  USE cell_base,            ONLY : tpiba
   USE fft_base,             ONLY : dfftp
   USE fft_interfaces,       ONLY : invfft
   USE gvect,                ONLY : ngm, gg, g, &
@@ -51,19 +50,19 @@ SUBROUTINE addusdens_g(rho)
   USE uspp_param,           ONLY : upf, lmaxq, nh
   USE control_flags,        ONLY : gamma_only
   USE mp_pools,             ONLY : inter_pool_comm
+  USE mp_bands,             ONLY : inter_bgrp_comm
   USE mp,                   ONLY : mp_sum
   !
   IMPLICIT NONE
   !
-  COMPLEX(kind=dp), INTENT(inout) :: rho(dfftp%ngm,nspin_mag)
+  COMPLEX(DP), INTENT(INOUT) :: rho(dfftp%ngm,nspin_mag)
   !
-  !     here the local variables
+  ! ... local variables
   !
-  INTEGER :: ngm_s, ngm_e, ngm_l
+  INTEGER :: ngm_s, ngm_e, ngm_l, ngm_s_tmp, ngm_e_tmp, ngm_l_tmp
   ! starting/ending indices, local number of G-vectors
   INTEGER :: ig, na, nt, ih, jh, ijh, is, nab, nb, nij
   ! counters
-
   REAL(DP), ALLOCATABLE :: tbecsum(:,:,:)
   ! \sum_kv <\psi_kv|\beta_l><beta_m|\psi_kv> for each species of atoms
   REAL(DP), ALLOCATABLE :: qmod (:), ylmk0 (:,:)
@@ -72,30 +71,31 @@ SUBROUTINE addusdens_g(rho)
   ! structure factors, US contribution to rho
   COMPLEX(DP), ALLOCATABLE ::  aux (:,:), qgm(:)
   ! work space for rho(G,nspin), Fourier transform of q
-
-  IF (.not.okvan) RETURN
-
-  CALL start_clock ('addusdens')
   !
-  ALLOCATE (aux (ngm, nspin_mag) )
-  aux (:,:) = (0.d0, 0.d0)
+  IF (.NOT.okvan) RETURN
   !
-  ! With k-point parallelization, distribute G-vectors across processors
-  ! ngm_s = index of first G-vector for this processor
-  ! ngm_e = index of last  G-vector for this processor
+  CALL start_clock( 'addusdens' )
+  !
+  ALLOCATE( aux(ngm,nspin_mag) )
+  aux(:,:) = (0.d0, 0.d0)
+  !
+  ! With k-point/bgrp parallelization, distribute G-vectors across all processors
+  ! ngm_s = index of first G-vector for this processor (in the k-point x bgrp pool)
+  ! ngm_e = index of last  G-vector for this processor (in the k-point x bgrp pool)
   ! ngm_l = local number of G-vectors 
   !
-  CALL divide (inter_pool_comm, ngm, ngm_s, ngm_e)
-  ngm_l = ngm_e-ngm_s+1
+  CALL divide( inter_pool_comm, ngm, ngm_s_tmp, ngm_e_tmp ) ; ngm_l_tmp = ngm_e_tmp - ngm_s_tmp + 1
+  CALL divide( inter_bgrp_comm, ngm_l_tmp, ngm_s, ngm_e ) ; ngm_l = ngm_e - ngm_s + 1 
+  ngm_s = ngm_s + ngm_s_tmp - 1 ; ngm_e = ngm_e + ngm_s_tmp -1
   ! for the extraordinary unlikely case of more processors than G-vectors
-  IF ( ngm_l <= 0 ) GO TO 10
+  IF ( ngm_l <= 0 ) GOTO 10
   !
-  ALLOCATE (qmod(ngm_l), qgm(ngm_l) )
-  ALLOCATE (ylmk0(ngm_l, lmaxq * lmaxq) )
-
-  CALL ylmr2 (lmaxq * lmaxq, ngm_l, g(1,ngm_s), gg(ngm_s), ylmk0)
+  ALLOCATE( qmod(ngm_l), qgm(ngm_l)   )
+  ALLOCATE( ylmk0(ngm_l, lmaxq*lmaxq) )
+  !
+  CALL ylmr2( lmaxq*lmaxq, ngm_l, g(1,ngm_s), gg(ngm_s), ylmk0 )
   DO ig = 1, ngm_l
-     qmod (ig) = sqrt (gg (ngm_s+ig-1) )
+     qmod(ig) = SQRT(gg(ngm_s+ig-1))*tpiba
   ENDDO
   !
   DO nt = 1, ntyp
@@ -112,8 +112,9 @@ SUBROUTINE addusdens_g(rho)
            IF ( ityp(na) == nt ) nab = nab + 1
         ENDDO
         !
-        ALLOCATE ( skk(ngm_l,nab), tbecsum(nij,nab,nspin_mag), aux2(ngm_l,nij) )
+        ALLOCATE( skk(ngm_l,nab), tbecsum(nij,nab,nspin_mag), aux2(ngm_l,nij) )
         !
+        call start_clock( 'addusd:skk')
         nb = 0
         DO na = 1, nat
            IF ( ityp(na) == nt ) THEN
@@ -121,49 +122,56 @@ SUBROUTINE addusdens_g(rho)
               tbecsum(:,nb,:) = becsum(1:nij,na,1:nspin_mag)
 !$omp parallel do default(shared) private(ig)
               DO ig = 1, ngm_l
-                 skk(ig,nb) = eigts1 (mill (1,ngm_s+ig-1), na) * &
-                              eigts2 (mill (2,ngm_s+ig-1), na) * &
-                              eigts3 (mill (3,ngm_s+ig-1), na)
+                 skk(ig,nb) = eigts1(mill(1,ngm_s+ig-1),na) * &
+                              eigts2(mill(2,ngm_s+ig-1),na) * &
+                              eigts3(mill(3,ngm_s+ig-1),na)
               ENDDO
 !$omp end parallel do
            ENDIF
         ENDDO
+        call stop_clock( 'addusd:skk')
 
         DO is = 1, nspin_mag
            ! sum over atoms
-           CALL dgemm( 'N', 'T', 2*ngm_l, nij, nab, 1.0_dp, skk, 2*ngm_l,&
-                tbecsum(1,1,is), nij, 0.0_dp, aux2, 2*ngm_l )
+           call start_clock( 'addusd:dgemm')
+           CALL dgemm( 'N', 'T', 2*ngm_l, nij, nab, 1.0_dp, skk, 2*ngm_l, &
+                       tbecsum(1,1,is), nij, 0.0_dp, aux2, 2*ngm_l )
+           call stop_clock( 'addusd:dgemm')
            ! sum over lm indices of Q_{lm}
+           call start_clock( 'addusd:qvan2')
            ijh = 0
-           DO ih = 1, nh (nt)
-              DO jh = ih, nh (nt)
+           DO ih = 1, nh(nt)
+              DO jh = ih, nh(nt)
                  ijh = ijh + 1
-                 CALL qvan2 (ngm_l, ih, jh, nt, qmod, qgm, ylmk0)
+                 CALL qvan2( ngm_l, ih, jh, nt, qmod, qgm, ylmk0 )
 !$omp parallel do default(shared) private(ig)
                  DO ig = 1, ngm_l
-                    aux(ngm_s+ig-1,is) = aux(ngm_s+ig-1,is)+aux2(ig,ijh)*qgm(ig)
+                    aux(ngm_s+ig-1,is) = aux(ngm_s+ig-1,is) + aux2(ig,ijh)*qgm(ig)
                  ENDDO
 !$omp end parallel do
              ENDDO
            ENDDO
+           call stop_clock( 'addusd:qvan2')
         ENDDO
-        DEALLOCATE (aux2, tbecsum, skk )
+        DEALLOCATE( aux2, tbecsum, skk )
      ENDIF
   ENDDO
   !
-  DEALLOCATE (ylmk0)
-  DEALLOCATE (qgm, qmod)
+  DEALLOCATE( ylmk0 )
+  DEALLOCATE( qgm, qmod )
   !
   10 CONTINUE
+  CALL mp_sum( aux, inter_bgrp_comm )
   CALL mp_sum( aux, inter_pool_comm )
   !
   !     add aux to the charge density in reciprocal space
   !
-  rho(:,:) = rho(:,:) + aux (:,:)
+  rho(:,:) = rho(:,:) + aux(:,:)
   !
-  DEALLOCATE (aux)
+  DEALLOCATE( aux )
+  !
+  CALL stop_clock( 'addusdens' )
   !
-  CALL stop_clock ('addusdens')
   RETURN
+  !
 END SUBROUTINE addusdens_g
-
