@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2013 Quantum ESPRESSO group
+! Copyright (C) 2001-2020 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -9,11 +9,10 @@
 !----------------------------------------------------------------------------
 SUBROUTINE c_bands( iter )
   !----------------------------------------------------------------------------
-  !
-  ! ... Driver routine for Hamiltonian diagonalization routines
-  ! ... It reads the Hamiltonian and an initial guess of the wavefunctions
-  ! ... from a file and computes initialization quantities for the
-  ! ... diagonalization routines.
+  !! Driver routine for the Hamiltonian diagonalization ones.
+  !! It reads the Hamiltonian and an initial guess of the wavefunctions
+  !! from a file and computes initialization quantities for the
+  !! diagonalization routines.
   !
   USE kinds,                ONLY : DP
   USE io_global,            ONLY : stdout
@@ -24,9 +23,9 @@ SUBROUTINE c_bands( iter )
   USE gvect,                ONLY : g
   USE wvfct,                ONLY : et, nbnd, npwx, current_k
   USE control_flags,        ONLY : ethr, isolve, restart
-  USE ldaU,                 ONLY : lda_plus_u, U_projection, wfcU
+  USE ldaU,                 ONLY : lda_plus_u, lda_plus_u_kind, U_projection, wfcU
   USE lsda_mod,             ONLY : current_spin, lsda, isk
-  USE wavefunctions_module, ONLY : evc
+  USE wavefunctions,        ONLY : evc
   USE bp,                   ONLY : lelfield
   USE mp_pools,             ONLY : npool, kunit, inter_pool_comm
   USE mp,                   ONLY : mp_sum
@@ -34,7 +33,8 @@ SUBROUTINE c_bands( iter )
   !
   IMPLICIT NONE
   !
-  INTEGER, INTENT (in) :: iter
+  INTEGER, INTENT(IN) :: iter
+  !! iteration index
   !
   ! ... local variables
   !
@@ -44,14 +44,13 @@ SUBROUTINE c_bands( iter )
   ! ik : counter on k points
   ! ik_: k-point already done in a previous run
   LOGICAL :: exst
-!------------------------------------------------------------------------
-
+  !
   !
   CALL start_clock( 'c_bands' ); !write (*,*) 'start c_bands' ; FLUSH(6)
   !
   ik_ = 0
   avg_iter = 0.D0
-  IF ( restart ) CALL restart_in_cbands(ik_, ethr, avg_iter, et )
+  IF ( restart ) CALL restart_in_cbands( ik_, ethr, avg_iter, et )
   !
   ! ... If restarting, calculated wavefunctions have to be read from file
   ! ... (not needed for a single k-point: this is done in wfcinit, 
@@ -60,15 +59,19 @@ SUBROUTINE c_bands( iter )
   DO ik = 1, ik_
      IF ( nks > 1 .OR. lelfield ) &
         CALL get_buffer ( evc, nwordwfc, iunwfc, ik )
-  END DO
+  ENDDO
   !
   IF ( isolve == 0 ) THEN
      WRITE( stdout, '(5X,"Davidson diagonalization with overlap")' )
-  ELSE IF ( isolve == 1 ) THEN
+  ELSEIF ( isolve == 1 ) THEN
      WRITE( stdout, '(5X,"CG style diagonalization")')
+  ELSEIF ( isolve == 2 ) THEN
+     WRITE( stdout, '(5X,"PPCG style diagonalization")')
+  ELSEIF ( isolve == 3 ) THEN
+     WRITE( stdout, '(5X,"ParO style diagonalization")')
   ELSE
      CALL errore ( 'c_bands', 'invalid type of diagonalization', isolve)
-  END IF
+  ENDIF
   !
   ! ... For each k point diagonalizes the hamiltonian
   !
@@ -77,8 +80,12 @@ SUBROUTINE c_bands( iter )
      ! ... Set k-point, spin, kinetic energy, needed by Hpsi
      !
      current_k = ik
+     !
+     IF (lda_plus_u .AND. lda_plus_u_kind.EQ.2) CALL phase_factor(ik)
+     !
      IF ( lsda ) current_spin = isk(ik)
-     call g2_kin( ik )
+     !
+     CALL g2_kin( ik )
      !
      ! ... More stuff needed by the hamiltonian: nonlocal projectors
      !
@@ -111,14 +118,14 @@ SUBROUTINE c_bands( iter )
      !
      nkdum  = kunit * ( nkstot / kunit / npool )
      !
-     IF (ik .le. nkdum) THEN
+     IF (ik <= nkdum) THEN
         IF (check_stop_now()) THEN
-           CALL save_in_cbands(ik, ethr, avg_iter, et )
+           CALL save_in_cbands( ik, ethr, avg_iter, et )
            RETURN
-        END IF
+        ENDIF
      ENDIF
      !
-  END DO k_loop
+  ENDDO k_loop
   !
   CALL mp_sum( avg_iter, inter_pool_comm )
   avg_iter = avg_iter / nkstot
@@ -136,20 +143,21 @@ END SUBROUTINE c_bands
 !----------------------------------------------------------------------------
 SUBROUTINE diag_bands( iter, ik, avg_iter )
   !----------------------------------------------------------------------------
+  !! Driver routine for diagonalization at each k-point. Types of iterative
+  !! diagonalizations currently in use:
   !
-  ! ... Driver routine for diagonalization at each k-point
-  ! ... Two types of iterative diagonalizations are currently used:
-  ! ... a) Davidson algorithm (all-band)
-  ! ... b) Conjugate Gradient (band-by-band)
-  ! ...
-  ! ... internal procedures :
+  !! * Davidson algorithm (all-band);
+  !! * Conjugate Gradient (band-by-band);
+  !! * Projected Preconditioned Conjugate Gradient (block);
+  !! * Parallel Orbital update (all-band).
   !
-  ! ... diag_bands_gamma(): optimized algorithms for gamma sampling of the BZ
-  ! ...                    (real Hamiltonian)
-  ! ... diag_bands_k()    : general algorithm for arbitrary BZ sampling
-  ! ...                     (complex Hamiltonian)
-  ! ... test_exit_cond()  : the test on the iterative diagonalization
+  !! Internal procedures:
   !
+  !! * \(\textrm{diag_bands_gamma}\)(): optimized algorithms for gamma sampling
+  !!                                    of the BZ (real Hamiltonian);
+  !! * \(\textrm{diag_bands_k}\)(): general algorithm for arbitrary BZ sampling
+  !!                                (complex Hamiltonian);
+  !! * \(\textrm{test_exit_cond}\)(): the test on the iterative diagonalization.
   !
   USE kinds,                ONLY : DP
   USE buffers,              ONLY : get_buffer
@@ -158,13 +166,13 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   USE uspp,                 ONLY : vkb, nkb, okvan
   USE gvect,                ONLY : gstart
   USE wvfct,                ONLY : g2kin, nbndx, et, nbnd, npwx, btype
-  USE control_flags,        ONLY : ethr, lscf, max_cg_iter, isolve, &
+  USE control_flags,        ONLY : ethr, lscf, max_cg_iter, max_ppcg_iter, isolve, &
                                    gamma_only, use_para_diag
-  USE noncollin_module,     ONLY : noncolin, npol
-  USE wavefunctions_module, ONLY : evc
+  USE noncollin_module,     ONLY : npol
+  USE wavefunctions,        ONLY : evc
   USE g_psi_mod,            ONLY : h_diag, s_diag
   USE scf,                  ONLY : v_of_0
-  USE bp,                   ONLY : lelfield, evcel, evcelp, evcelm, bec_evcel,&
+  USE bp,                   ONLY : lelfield, evcel, evcelp, evcelm, bec_evcel, &
                                    gdir, l3dstring, efield, efield_cry
   USE becmod,               ONLY : bec_type, becp, calbec, &
                                    allocate_bec_type, deallocate_bec_type
@@ -175,13 +183,18 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   !
   IMPLICIT NONE
   !
-  INTEGER, INTENT(IN)  :: iter, ik
+  INTEGER, INTENT(IN) :: iter
+  !! iteration index
+  INTEGER, INTENT(IN) :: ik
+  !! k-point index
+  REAL(KIND=DP), INTENT(INOUT) :: avg_iter
+  !! average number of H*psi products
   !
-  REAL (KIND=DP), INTENT(INOUT) :: avg_iter
+  ! ... local variables
   !
-  REAL (KIND=DP) :: cg_iter
+  REAL(KIND=DP) :: cg_iter, ppcg_iter
   ! (weighted) number of iterations in Conjugate-Gradient
-  INTEGER :: npw, ig, dav_iter, ntry, notconv
+  INTEGER :: npw, ig, dav_iter, ntry, notconv, nhpsi
   ! number of iterations in Davidson
   ! number or repeated call to diagonalization in case of non convergence
   ! number of notconverged elements
@@ -190,17 +203,30 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   LOGICAL :: lrot
   ! .TRUE. if the wfc have already be rotated
   !
-! Davidson diagonalization uses these external routines on groups of nvec bands
-  external h_psi, s_psi, g_psi
-! subroutine h_psi(npwx,npw,nvec,psi,hpsi)  computes H*psi
-! subroutine s_psi(npwx,npw,nvec,psi,spsi)  computes S*psi (if needed)
-! subroutine g_psi(npwx,npw,nvec,psi,eig)   computes G*psi -> psi
-!------------------------------------------------------------------------
-! CG diagonalization uses these external routines on a single band
-   external h_1psi, s_1psi
-!  subroutine h_1psi(npwx,npw,psi,hpsi,spsi)  computes H*psi and S*psi
-!  subroutine s_1psi(npwx,npw,psi,spsi)  computes S*psi (if needed)
-! In addition to the above ithe initial wfc rotation uses h_psi, and s_psi
+  INTEGER, PARAMETER :: sbsize = 5, rrstep = 7
+  ! block dimensions used in PPCG 
+  !
+  ! Davidson diagonalization uses these external routines on groups of nvec bands
+  EXTERNAL h_psi, s_psi, g_psi
+  ! subroutine h_psi(npwx,npw,nvec,psi,hpsi)  computes H*psi
+  ! subroutine s_psi(npwx,npw,nvec,psi,spsi)  computes S*psi (if needed)
+  ! subroutine g_psi(npwx,npw,nvec,psi,eig)   computes G*psi -> psi
+  !------------------------------------------------------------------------
+  ! CG diagonalization uses these external routines on a single band
+  EXTERNAL hs_1psi, s_1psi, hs_psi
+  ! subroutine hs_1psi(npwx,npw,psi,hpsi,spsi)  computes H*psi and S*psi
+  ! subroutine s_1psi(npwx,npw,psi,spsi)        computes S*psi (if needed)
+  ! In addition to the above the initial wfc rotation uses h_psi, and s_psi
+  !------------------------------------------------------------------------
+  ! PPCG diagonalization uses these external routines on groups of bands
+  ! subroutine h_psi(npwx,npw,nvec,psi,hpsi)  computes H*psi
+  ! subroutine s_psi(npwx,npw,nvec,psi,spsi)  computes S*psi (if needed)
+  !------------------------------------------------------------------------
+  ! ParO diagonalization uses these external routines on a single band
+  ! subroutine hs_1psi(npwx,npw,psi,hpsi,spsi)  computes H*psi and S*psi
+  ! subroutine g_1psi(npwx,npw,psi,eig)         computes G*psi -> psi
+  ! In addition to the above the initial wfc rotation uses h_psi, and s_psi
+  external g_1psi
 
   ALLOCATE( h_diag( npwx, npol ), STAT=ierr )
   IF( ierr /= 0 ) &
@@ -217,7 +243,7 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
   !
   ! ... allocate space for <beta_i|psi_j> - used in h_psi and s_psi
   !
-  CALL allocate_bec_type ( nkb, nbnd, becp, intra_bgrp_comm )
+  CALL allocate_bec_type( nkb, nbnd, becp, intra_bgrp_comm )
   !
   npw = ngk(ik)
   IF ( gamma_only ) THEN
@@ -228,29 +254,27 @@ SUBROUTINE diag_bands( iter, ik, avg_iter )
      !
      CALL diag_bands_k()
      !
-  END IF
+  ENDIF
   !
   ! ... deallocate work space
   !
-  CALL deallocate_bec_type ( becp )
+  CALL deallocate_bec_type( becp )
   DEALLOCATE( s_diag )
   DEALLOCATE( h_diag )
   !
   IF ( notconv > MAX( 5, nbnd / 4 ) ) THEN
      !
-     CALL errore( 'c_bands', &
-          & 'too many bands are not converged', 1 )
+     CALL errore( 'c_bands', 'too many bands are not converged', 1 )
      !
-  ELSE IF ( notconv > 0 ) THEN
+  ELSEIF ( notconv > 0 ) THEN
      !
-     WRITE( stdout, '(5X,"c_bands: ",I2, &
-               &   " eigenvalues not converged")' ) notconv
+     WRITE( stdout, '(5X,"c_bands: ",I2, " eigenvalues not converged")' ) notconv
      !
-  END IF
+  ENDIF
   !
   RETURN
   !
-CONTAINS
+ CONTAINS
   !
   ! ... internal procedures
   !
@@ -262,37 +286,63 @@ CONTAINS
     !
     IMPLICIT NONE
     !
-    IF ( isolve == 1 ) THEN
+    IF ( isolve == 1 .OR. isolve == 2 .OR. isolve == 3) THEN
        !
-       ! ... Conjugate-Gradient diagonalization
+       ! ... (Projected Preconditioned) Conjugate-Gradient diagonalization
        !
        ! ... h_diag is the precondition matrix
        !
-       FORALL( ig = 1 : npw )
-          !
-          h_diag(ig,1) = 1.D0 + g2kin(ig) + SQRT( 1.D0 + ( g2kin(ig) - 1.D0 )**2 )
-          !
-       END FORALL
+       IF ( isolve == 1 .OR. isolve == 2 ) THEN
+          FORALL( ig = 1 : npw )
+             h_diag(ig,1) = 1.D0 + g2kin(ig) + SQRT( 1.D0 + ( g2kin(ig) - 1.D0 )**2 )
+          END FORALL
+       ELSE
+          FORALL( ig = 1 : npw )
+             h_diag(ig, 1) = g2kin(ig) + v_of_0
+          END FORALL
+          CALL usnldiag( npw, h_diag, s_diag )
+       END IF
        !
        ntry = 0
        !
        CG_loop : DO
           !
-          lrot = ( iter == 1 .AND. ntry == 0 )
+          IF ( isolve == 1 .OR. isolve == 2 ) THEN
+             lrot = ( iter == 1 .AND. ntry == 0 )
+             !
+             IF ( .NOT. lrot ) THEN
+                !
+                CALL rotate_wfc( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
+                !
+                avg_iter = avg_iter + 1.D0
+                !
+             ENDIF
+          ENDIF
           !
-          IF ( .NOT. lrot ) THEN
-             !
-             CALL rotate_wfc ( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
-             !
-             avg_iter = avg_iter + 1.D0
-             !
-          END IF
-          !
-          CALL rcgdiagg( h_1psi, s_1psi, h_diag, &
+          IF ( isolve == 1 ) THEN
+             CALL rcgdiagg( hs_1psi, s_1psi, h_diag, &
                          npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
+             !
+             avg_iter = avg_iter + cg_iter
+             !
+          ELSE IF ( isolve == 2 ) THEN
+             CALL ppcg_gamma( h_psi, s_psi, okvan, h_diag, &
+                         npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), &
+                         0.1d0*ethr, max_ppcg_iter, notconv, ppcg_iter, sbsize , rrstep, iter )
+             !
+             avg_iter = avg_iter + ppcg_iter
+             !
+          ELSE
+             !
+             CALL paro_gamma_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
+                        npwx, npw, nbnd, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+             !
+             avg_iter = avg_iter + nhpsi/float(nbnd) 
+             ! write (6,*) ntry, avg_iter, nhpsi
+             !
+          ENDIF
           !
-          avg_iter = avg_iter + cg_iter
           !
           ntry = ntry + 1
           !
@@ -300,7 +350,7 @@ CONTAINS
           !
           IF ( test_exit_cond() ) EXIT  CG_loop
           !
-       END DO CG_loop
+       ENDDO CG_loop
        !
     ELSE
        !
@@ -325,14 +375,16 @@ CONTAINS
 !             ! make sure that all processors have the same wfc
              CALL pregterg( h_psi, s_psi, okvan, g_psi, &
                          npw, npwx, nbnd, nbndx, evc, ethr, &
-                         et(1,ik), btype(1,ik), notconv, lrot, dav_iter ) !    BEWARE gstart has been removed from call 
+                         et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
+                                   !    BEWARE gstart has been removed from call 
              !
           ELSE
              !
              CALL regterg (  h_psi, s_psi, okvan, g_psi, &
                          npw, npwx, nbnd, nbndx, evc, ethr, &
-                         et(1,ik), btype(1,ik), notconv, lrot, dav_iter ) !    BEWARE gstart has been removed from call
-          END IF
+                         et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
+                                   !    BEWARE gstart has been removed from call
+          ENDIF
           !
           avg_iter = avg_iter + dav_iter
           !
@@ -342,9 +394,10 @@ CONTAINS
           !
           IF ( test_exit_cond() ) EXIT  david_loop
           !
-       END DO david_loop
+       ENDDO david_loop
        !
-    END IF
+    ENDIF
+    !
     !
     RETURN
     !
@@ -353,16 +406,15 @@ CONTAINS
   !-----------------------------------------------------------------------
   SUBROUTINE diag_bands_k()
     !-----------------------------------------------------------------------
-    !
-    ! ... Complex Hamiltonian diagonalization
+    !! Complex Hamiltonian diagonalization.
     !
     IMPLICIT NONE
     !
-    ! ... here the local variables
+    ! ... local variables
     !
     INTEGER :: ipol
-    REAL(dp) :: eps=0.000001d0
-    !  --- Define a small number ---
+    REAL(DP) :: eps=0.000001d0
+    ! --- Define a small number ---
     !
     !write (*,*) ' enter diag_bands_k'; FLUSH(6)
     IF ( lelfield ) THEN
@@ -373,72 +425,96 @@ CONTAINS
        !
        !... read projectors from disk
        !
-       if(.not.l3dstring .and. ABS(efield)>eps ) then
+       IF (.NOT.l3dstring .AND. ABS(efield)>eps ) THEN
           CALL get_buffer (evcelm(:,:,gdir), nwordwfc, iunefieldm, ik+(gdir-1)*nks)
           CALL get_buffer (evcelp(:,:,gdir), nwordwfc, iunefieldp, ik+(gdir-1)*nks)
-       else
-          do ipol=1,3
-             if(ABS(efield_cry(ipol))>eps) then
-                CALL get_buffer (evcelm(:,:,ipol), nwordwfc, iunefieldm, ik+(ipol-1)*nks)
-                CALL get_buffer (evcelp(:,:,ipol), nwordwfc, iunefieldp, ik+(ipol-1)*nks)
-             endif
-          enddo
-       endif
+       ELSE
+          DO ipol = 1, 3
+             IF ( ABS(efield_cry(ipol))>eps ) THEN
+                CALL get_buffer( evcelm(:,:,ipol), nwordwfc, iunefieldm, ik+(ipol-1)*nks )
+                CALL get_buffer( evcelp(:,:,ipol), nwordwfc, iunefieldp, ik+(ipol-1)*nks )
+             ENDIF
+          ENDDO
+       ENDIF
        !
        IF ( okvan ) THEN
           !
-          call allocate_bec_type(nkb,nbnd,bec_evcel)
-          
+          CALL allocate_bec_type( nkb, nbnd, bec_evcel )
           !
-          CALL calbec(npw, vkb, evcel, bec_evcel)
+          CALL calbec( npw, vkb, evcel, bec_evcel )
           !
        ENDIF
        !
-    END IF
+    ENDIF
     !
-    !write (*,*) ' current isolve value ( 1 CG, 2 Davidson)', isolve; FLUSH(6)
-    IF ( isolve == 1 ) THEN
+    !write (*,*) ' current isolve value ( 0 Davidson, 1 CG, 2 PPCG)', isolve; FLUSH(6)
+    IF ( isolve == 1 .OR. isolve == 2 .OR. isolve == 3 ) THEN
        !
-       ! ... Conjugate-Gradient diagonalization
+       ! ... (Projected Preconditioned) Conjugate-Gradient diagonalization
        !
        ! ... h_diag is the precondition matrix
        !
        !write (*,*) ' inside CG solver branch '
-       h_diag = 1.D0
        !
-       FORALL( ig = 1 : npwx )
-          !
-          h_diag(ig,:) = 1.D0 + g2kin(ig) + SQRT( 1.D0 + ( g2kin(ig) - 1.D0 )**2 )
-          !
-       END FORALL
+       h_diag = 1.D0
+       IF ( isolve == 1 .OR. isolve == 2) THEN
+          FORALL( ig = 1 : npwx )
+             h_diag(ig,:) = 1.D0 + g2kin(ig) + SQRT( 1.D0 + ( g2kin(ig) - 1.D0 )**2 )
+          END FORALL
+       ELSE
+          FORALL( ig = 1 : npwx )
+             h_diag(ig, :) = g2kin(ig) + v_of_0
+          END FORALL
+          CALL usnldiag( npw, h_diag, s_diag )
+       ENDIF
        !
        ntry = 0
        !
        CG_loop : DO
           !
-          lrot = ( iter == 1 .AND. ntry == 0 )
+          IF ( isolve == 1 .OR. isolve == 2 ) THEN
+             lrot = ( iter == 1 .AND. ntry == 0 )
+             !
+             IF ( .NOT. lrot ) THEN
+                !
+                CALL rotate_wfc( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
+                !
+                avg_iter = avg_iter + 1.D0
+                !
+             ENDIF
+          ENDIF
           !
-          IF ( .NOT. lrot ) THEN
-             !
-             CALL rotate_wfc ( npwx, npw, nbnd, gstart, nbnd, evc, npol, okvan, evc, et(1,ik) )
-             !
-             avg_iter = avg_iter + 1.D0
-             !
-          END IF
-          !
-          CALL ccgdiagg( h_1psi, s_1psi, h_diag, &
+          IF ( isolve == 1) then
+             CALL ccgdiagg( hs_1psi, s_1psi, h_diag, &
                          npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
                          ethr, max_cg_iter, .NOT. lscf, notconv, cg_iter )
-          !
-          avg_iter = avg_iter + cg_iter
-          !
+             !
+             avg_iter = avg_iter + cg_iter
+             !
+          ELSE IF ( isolve == 2) then
+             ! BEWARE npol should be added to the arguments
+             CALL ppcg_k( h_psi, s_psi, okvan, h_diag, &
+                         npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), &
+                         0.1d0*ethr, max_ppcg_iter, notconv, ppcg_iter, sbsize , rrstep, iter )
+             !
+             avg_iter = avg_iter + ppcg_iter
+             !
+          ELSE 
+             !
+             CALL paro_k_new( h_psi, s_psi, hs_psi, g_1psi, okvan, &
+                        npwx, npw, nbnd, npol, evc, et(1,ik), btype(1,ik), ethr, notconv, nhpsi )
+             !
+             avg_iter = avg_iter + nhpsi/float(nbnd) 
+             ! write (6,*) ntry, avg_iter, nhpsi
+             !
+          ENDIF
           ntry = ntry + 1
           !
           ! ... exit condition
           !
           IF ( test_exit_cond() ) EXIT  CG_loop
           !
-       END DO CG_loop
+       ENDDO CG_loop
        !
     ELSE
        !
@@ -452,7 +528,7 @@ CONTAINS
           !
           h_diag(1:npw, ipol) = g2kin(1:npw) + v_of_0
           !
-       END DO
+       ENDDO
        !
        CALL usnldiag( npw, h_diag, s_diag )
        !
@@ -462,18 +538,18 @@ CONTAINS
           !
           lrot = ( iter == 1 )
           !
-          IF ( use_para_diag ) then
+          IF ( use_para_diag ) THEN
              !
              CALL pcegterg( h_psi, s_psi, okvan, g_psi, &
                             npw, npwx, nbnd, nbndx, npol, evc, ethr, &
-                            et(1,ik), btype(1,ik), notconv, lrot, dav_iter )
+                            et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
              !
           ELSE
              !
-             CALL cegterg ( h_psi, s_psi, okvan, g_psi, &
-                            npw, npwx, nbnd, nbndx, npol, evc, ethr, &
-                            et(1,ik), btype(1,ik), notconv, lrot, dav_iter )
-          END IF
+             CALL cegterg( h_psi, s_psi, okvan, g_psi, &
+                           npw, npwx, nbnd, nbndx, npol, evc, ethr, &
+                           et(1,ik), btype(1,ik), notconv, lrot, dav_iter, nhpsi )
+          ENDIF
           !
           avg_iter = avg_iter + dav_iter
           !
@@ -487,11 +563,11 @@ CONTAINS
           !
           IF ( test_exit_cond() ) EXIT david_loop
           !
-       END DO david_loop
+       ENDDO david_loop
        !
-    END IF
+    ENDIF
     !
-    IF ( lelfield .AND. okvan ) call deallocate_bec_type( bec_evcel) 
+    IF ( lelfield .AND. okvan ) CALL deallocate_bec_type( bec_evcel )
     !
     RETURN
     !
@@ -500,9 +576,8 @@ CONTAINS
   !-----------------------------------------------------------------------
   FUNCTION test_exit_cond()
     !-----------------------------------------------------------------------
-    !
-    ! ... this logical function is .TRUE. when iterative diagonalization
-    ! ... is converged
+    !! This logical function is .TRUE. when iterative diagonalization
+    !! is converged.
     !
     IMPLICIT NONE
     !
@@ -518,12 +593,11 @@ CONTAINS
 END SUBROUTINE diag_bands
 !
 !----------------------------------------------------------------------------
-SUBROUTINE c_bands_efield ( iter )
+SUBROUTINE c_bands_efield( iter )
   !----------------------------------------------------------------------------
+  !! Driver routine for Hamiltonian diagonalization under an electric field.
   !
-  ! ... Driver routine for Hamiltonian diagonalization under an electric field
-  !
-  USE noncollin_module,     ONLY : noncolin, npol
+  USE noncollin_module,     ONLY : npol
   USE kinds,                ONLY : DP
   USE bp,                   ONLY : nberrycyc, fact_hepsi, &
                                    evcel, evcelp, evcelm, gdir, l3dstring,&
@@ -534,7 +608,10 @@ SUBROUTINE c_bands_efield ( iter )
   !
   IMPLICIT NONE
   !
-  INTEGER, INTENT (in) :: iter
+  INTEGER, INTENT(IN) :: iter
+  !! iteration index
+  !
+  ! ... local variables
   !
   INTEGER :: inberry, ipol, ierr
   !
@@ -557,18 +634,18 @@ SUBROUTINE c_bands_efield ( iter )
      !...set up electric field hermitean operator
      !
      FLUSH(stdout)
-     if(.not.l3dstring) then
+     IF (.NOT.l3dstring) THEN
         CALL h_epsi_her_set (gdir, efield)
-     else
-        do ipol=1,3
+     ELSE
+        DO ipol=1,3
            CALL h_epsi_her_set(ipol, efield_cry(ipol))
-        enddo
-     endif
+        ENDDO
+     ENDIF
      FLUSH(stdout)
      !
      CALL c_bands( iter )
      !
-  END DO
+  ENDDO
   !
   DEALLOCATE( fact_hepsi )
   DEALLOCATE( evcelp )
@@ -579,11 +656,11 @@ SUBROUTINE c_bands_efield ( iter )
   !
 END SUBROUTINE c_bands_efield
 !
+!------------------------------------------------------------------------------
 SUBROUTINE c_bands_nscf( )
   !----------------------------------------------------------------------------
-  !
-  ! ... Driver routine for Hamiltonian diagonalization routines
-  ! ... specialized to non-self-consistent calculations (no electric field)
+  !! Driver routine for Hamiltonian diagonalization routines
+  !! specialized to non-self-consistent calculations (no electric field).
   !
   USE kinds,                ONLY : DP
   USE io_global,            ONLY : stdout
@@ -595,15 +672,17 @@ SUBROUTINE c_bands_nscf( )
   USE gvect,                ONLY : g
   USE wvfct,                ONLY : et, nbnd, npwx, current_k
   USE control_flags,        ONLY : ethr, restart, isolve, io_level, iverbosity
-  USE ldaU,                 ONLY : lda_plus_u, U_projection, wfcU
+  USE ldaU,                 ONLY : lda_plus_u, lda_plus_u_kind, U_projection, wfcU
   USE lsda_mod,             ONLY : current_spin, lsda, isk
-  USE wavefunctions_module, ONLY : evc
+  USE wavefunctions,        ONLY : evc
   USE mp_pools,             ONLY : npool, kunit, inter_pool_comm
   USE mp,                   ONLY : mp_sum
   USE check_stop,           ONLY : check_stop_now
   !
   IMPLICIT NONE
   !
+  ! ... local variables
+  !
   REAL(DP) :: avg_iter, ethr_
   ! average number of H*psi products
   INTEGER :: ik_, ik, nkdum, ios
@@ -613,25 +692,30 @@ SUBROUTINE c_bands_nscf( )
   !
   REAL(DP), EXTERNAL :: get_clock
   !
+  !
   CALL start_clock( 'c_bands' )
   !
   ik_ = 0
   avg_iter = 0.D0
-  IF ( restart ) CALL restart_in_cbands(ik_, ethr, avg_iter, et )
+  IF ( restart ) CALL restart_in_cbands( ik_, ethr, avg_iter, et )
   !
   ! ... If restarting, calculated wavefunctions have to be read from file
   !
   DO ik = 1, ik_
-     CALL get_buffer ( evc, nwordwfc, iunwfc, ik )
-  END DO
+     CALL get_buffer( evc, nwordwfc, iunwfc, ik )
+  ENDDO
   !
   IF ( isolve == 0 ) THEN
      WRITE( stdout, '(5X,"Davidson diagonalization with overlap")' )
-  ELSE IF ( isolve == 1 ) THEN
-     WRITE( stdout, '(5X,"CG style diagonalization")')
+  ELSEIF ( isolve == 1 ) THEN
+     WRITE( stdout, '(5X,"CG style diagonalization")' )
+  ELSEIF ( isolve == 2 ) THEN
+     WRITE( stdout, '(5X,"PPCG style diagonalization")' )
+  ELSEIF ( isolve == 3 ) THEN
+     WRITE( stdout, '(5X,"ParO style diagonalization")')
   ELSE
-     CALL errore ( 'c_bands', 'invalid type of diagonalization', isolve)
-  END IF
+     CALL errore ( 'c_bands', 'invalid type of diagonalization', isolve )
+  ENDIF
   !
   ! ... For each k point (except those already calculated if restarting)
   ! ... diagonalizes the hamiltonian
@@ -641,8 +725,12 @@ SUBROUTINE c_bands_nscf( )
      ! ... Set k-point, spin, kinetic energy, needed by Hpsi
      !
      current_k = ik
+     !
+     IF (lda_plus_u .AND. lda_plus_u_kind.EQ.2) CALL phase_factor(ik)
+     !
      IF ( lsda ) current_spin = isk(ik)
-     call g2_kin( ik )
+     !
+     CALL g2_kin( ik )
      ! 
      ! ... More stuff needed by the hamiltonian: nonlocal projectors
      !
@@ -651,44 +739,49 @@ SUBROUTINE c_bands_nscf( )
      ! ... Needed for LDA+U
      !
      IF ( nks > 1 .AND. lda_plus_u .AND. (U_projection .NE. 'pseudo') ) &
-          CALL get_buffer ( wfcU, nwordwfcU, iunhub, ik )
+          CALL get_buffer( wfcU, nwordwfcU, iunhub, ik )
      !
      ! ... calculate starting  wavefunctions
      !
-     IF ( iverbosity > 0 ) WRITE( stdout, 9001 ) ik
+     IF ( iverbosity > 0 .AND. npool == 1 ) THEN
+        WRITE( stdout, 9001 ) ik, nks
+     ELSEIF ( iverbosity > 0 .AND. npool > 1 ) THEN
+        WRITE( stdout, 9002 ) ik, nks
+     ENDIF
      !
      IF ( TRIM(starting_wfc) == 'file' ) THEN
         !
-        CALL get_buffer ( evc, nwordwfc, iunwfc, ik )
+        CALL get_buffer( evc, nwordwfc, iunwfc, ik )
         !
      ELSE
         !
-        CALL init_wfc ( ik )
+        CALL init_wfc( ik )
         !
-     END IF
+     ENDIF
      !
      ! ... diagonalization of bands for k-point ik
      !
-     call diag_bands ( 1, ik, avg_iter )
+     CALL diag_bands( 1, ik, avg_iter )
      !
      ! ... save wave-functions (unless disabled in input)
      !
-     IF ( io_level > -1 ) CALL save_buffer ( evc, nwordwfc, iunwfc, ik )
+     IF ( io_level > -1 ) CALL save_buffer( evc, nwordwfc, iunwfc, ik )
      !
      ! ... beware: with pools, if the number of k-points on different
      ! ... pools differs, make sure that all processors are still in
      ! ... the loop on k-points before checking for stop condition
      !
      nkdum  = kunit * ( nkstot / kunit / npool )
-     IF (ik .le. nkdum) THEN
+     IF (ik <= nkdum) THEN
         !
         ! ... stop requested by user: save restart information,
         ! ... save wavefunctions to file
         !
-        IF (check_stop_now()) THEN
-           CALL save_in_cbands(ik, ethr, avg_iter, et )
+        IF ( check_stop_now() ) THEN
+           CALL save_in_cbands( ik, ethr, avg_iter, et )
            RETURN
-        END IF
+        ENDIF
+        !
      ENDIF
      !
      ! report about timing
@@ -698,7 +791,7 @@ SUBROUTINE c_bands_nscf( )
         FLUSH( stdout )
      ENDIF
      !
-  END DO k_loop
+  ENDDO k_loop
   !
   CALL mp_sum( avg_iter, inter_pool_comm )
   avg_iter = avg_iter / nkstot
@@ -712,7 +805,8 @@ SUBROUTINE c_bands_nscf( )
   !
   ! formats
   !
-9001 FORMAT(/'     Computing kpt #: ',I5 )
+9002 FORMAT(/'     Computing kpt #: ',I5, '  of ',I5,' on this pool' )
+9001 FORMAT(/'     Computing kpt #: ',I5, '  of ',I5 )
 9000 FORMAT( '     total cpu time spent up to now is ',F10.1,' secs' )
   !
 END SUBROUTINE c_bands_nscf
