@@ -7,12 +7,11 @@
 !
 !
 !----------------------------------------------------------------------------
-SUBROUTINE compute_becsum ( iflag )
+SUBROUTINE compute_becsum( iflag )
   !----------------------------------------------------------------------------
-  !
-  ! ... Compute "becsum" = \sum_i w_i <psi_i|beta_l><beta_m|\psi_i> term
-  ! ... Output in module uspp and (PAW only) in rho%bec (symmetrized)
-  ! ... if iflag = 1, weights w_k are re-computed
+  !! Compute "becsum" = \sum_i w_i <psi_i|beta_l><beta_m|\psi_i> term.
+  !! Output in module uspp and (PAW only) in rho%bec (symmetrized)
+  !! if iflag = 1, weights w_k are re-computed.
   !
   USE kinds,                ONLY : DP
   USE control_flags,        ONLY : gamma_only
@@ -22,8 +21,7 @@ SUBROUTINE compute_becsum ( iflag )
   USE buffers,              ONLY : get_buffer
   USE scf,                  ONLY : rho
   USE uspp,                 ONLY : nkb, vkb, becsum, okvan
-  USE wavefunctions_module, ONLY : evc
-  USE noncollin_module,     ONLY : noncolin
+  USE wavefunctions,        ONLY : evc
   USE wvfct,                ONLY : nbnd, npwx, wg
   USE mp_pools,             ONLY : inter_pool_comm
   USE mp_bands,             ONLY : intra_bgrp_comm, inter_bgrp_comm 
@@ -37,7 +35,7 @@ SUBROUTINE compute_becsum ( iflag )
   !
   INTEGER, INTENT(IN) :: iflag
   !
-  INTEGER :: ik,& ! counter on k points
+  INTEGER :: ik, & ! counter on k points
              ibnd_start, ibnd_end, this_bgrp_nbnd ! first, last and number of band in this bgrp
   !
   !
@@ -47,26 +45,26 @@ SUBROUTINE compute_becsum ( iflag )
   !
   ! ... calculates weights of Kohn-Sham orbitals
   !
-  IF ( iflag == 1) CALL weights ( )
+  IF ( iflag == 1) CALL weights( )
   !
   becsum(:,:,:) = 0.D0
-  CALL allocate_bec_type (nkb,nbnd, becp,intra_bgrp_comm)
-  call divide (inter_bgrp_comm, nbnd, ibnd_start, ibnd_end )
+  CALL allocate_bec_type( nkb,nbnd, becp,intra_bgrp_comm )
+  CALL divide( inter_bgrp_comm, nbnd, ibnd_start, ibnd_end )
   this_bgrp_nbnd = ibnd_end - ibnd_start + 1
   !
   k_loop: DO ik = 1, nks
      !
      IF ( lsda ) current_spin = isk(ik)
      IF ( nks > 1 ) &
-        CALL get_buffer ( evc, nwordwfc, iunwfc, ik )
+        CALL get_buffer( evc, nwordwfc, iunwfc, ik )
      IF ( nkb > 0 ) &
           CALL init_us_2( ngk(ik), igk_k(1,ik), xk(1,ik), vkb )
      !
      ! ... actual calculation is performed inside routine "sum_bec"
      !
-     CALL sum_bec ( ik, current_spin, ibnd_start,ibnd_end,this_bgrp_nbnd  )
+     CALL sum_bec( ik, current_spin, ibnd_start, ibnd_end, this_bgrp_nbnd )
      !
-  END DO k_loop
+  ENDDO k_loop
   !
   ! ... with distributed <beta|psi>, sum over bands
   !
@@ -78,14 +76,14 @@ SUBROUTINE compute_becsum ( iflag )
   ! ... For USPP there is no need to do this as becsums are only used
   ! ... to compute the density, which is symmetrized later.
   !
-  IF( okpaw )  THEN
+  IF ( okpaw )  THEN
      rho%bec(:,:,:) = becsum(:,:,:) ! becsum is filled in sum_band_{k|gamma}
-     CALL mp_sum(rho%bec, inter_pool_comm )
-     call mp_sum(rho%bec, inter_bgrp_comm )
-     CALL PAW_symmetrize(rho%bec)
+     CALL mp_sum( rho%bec, inter_pool_comm )
+     call mp_sum( rho%bec, inter_bgrp_comm )
+     CALL PAW_symmetrize( rho%bec )
   ENDIF
   !
-  CALL deallocate_bec_type ( becp )
+  CALL deallocate_bec_type( becp )
   !
   CALL stop_clock( 'compute_becsum' )
   !
