@@ -9,8 +9,7 @@
 !----------------------------------------------------------------------------
 SUBROUTINE force_us( forcenl )
   !----------------------------------------------------------------------------
-  !
-  ! ... nonlocal potential contribution to forces
+  !! The nonlocal potential contribution to forces.
   !
   USE kinds,                ONLY : DP
   USE control_flags,        ONLY : gamma_only
@@ -19,11 +18,11 @@ SUBROUTINE force_us( forcenl )
   USE klist,                ONLY : nks, xk, ngk, igk_k
   USE gvect,                ONLY : g
   USE uspp,                 ONLY : nkb, vkb, qq_at, deeq, qq_so, deeq_nc, indv_ijkb0
-  USE uspp_param,           ONLY : upf, nh, newpseudo, nhm
+  USE uspp_param,           ONLY : upf, nh, nhm
   USE wvfct,                ONLY : nbnd, npwx, wg, et
   USE lsda_mod,             ONLY : lsda, current_spin, isk, nspin
   USE symme,                ONLY : symvector
-  USE wavefunctions_module, ONLY : evc
+  USE wavefunctions,        ONLY : evc
   USE noncollin_module,     ONLY : npol, noncolin
   USE spin_orb,             ONLY : lspinorb
   USE io_files,             ONLY : iunwfc, nwordwfc
@@ -36,7 +35,10 @@ SUBROUTINE force_us( forcenl )
   !
   IMPLICIT NONE
   !
-  REAL(DP), INTENT(OUT) :: forcenl(3,nat) ! the nonlocal contribution
+  REAL(DP), INTENT(OUT) :: forcenl(3,nat)
+  !! the nonlocal contribution
+  !
+  ! ... local variables
   !
   COMPLEX(DP), ALLOCATABLE :: vkb1(:,:)   ! contains g*|beta>
   COMPLEX(DP), ALLOCATABLE :: deff_nc(:,:,:,:)
@@ -44,14 +46,17 @@ SUBROUTINE force_us( forcenl )
   TYPE(bec_type) :: dbecp                 ! contains <dbeta|psi>
   INTEGER    :: npw, ik, ipol, ig, jkb
   !
+  !
   forcenl(:,:) = 0.D0
   !
-  CALL allocate_bec_type ( nkb, nbnd, becp, intra_bgrp_comm )   
-  CALL allocate_bec_type ( nkb, nbnd, dbecp, intra_bgrp_comm )   
+  CALL allocate_bec_type( nkb, nbnd, becp, intra_bgrp_comm )   
+  CALL allocate_bec_type( nkb, nbnd, dbecp, intra_bgrp_comm )   
+  !
   ALLOCATE( vkb1( npwx, nkb ) )   
-  IF (noncolin) then
+  !
+  IF (noncolin) THEN
      ALLOCATE( deff_nc(nhm,nhm,nat,nspin) )
-  ELSE IF (.NOT. gamma_only ) THEN
+  ELSEIF (.NOT. gamma_only ) THEN
      ALLOCATE( deff(nhm,nhm,nat) )
   ENDIF
   !
@@ -63,23 +68,22 @@ SUBROUTINE force_us( forcenl )
      npw = ngk (ik)
 
      IF ( nks > 1 ) THEN
-        CALL get_buffer ( evc, nwordwfc, iunwfc, ik )
-        IF ( nkb > 0 ) &
-             CALL init_us_2( npw, igk_k(1,ik), xk(1,ik), vkb )
-     END IF
+        CALL get_buffer( evc, nwordwfc, iunwfc, ik )
+        IF ( nkb > 0 ) CALL init_us_2( npw, igk_k(1,ik), xk(1,ik), vkb )
+     ENDIF
      !
-     CALL calbec ( npw, vkb, evc, becp )
+     CALL calbec( npw, vkb, evc, becp )
      !
      DO ipol = 1, 3
+!$omp parallel do collapse(2) private(ig)
         DO jkb = 1, nkb
-!$omp parallel do default(shared) private(ig)
-           do ig = 1, npw
+           DO ig = 1, npw
               vkb1(ig,jkb) = vkb(ig,jkb) * (0.D0,-1.D0) * g(ipol,igk_k(ig,ik))
-           END DO
+           ENDDO
+        ENDDO
 !$omp end parallel do
-        END DO
         !
-        CALL calbec ( npw, vkb1, evc, dbecp )
+        CALL calbec( npw, vkb1, evc, dbecp )
         !
         IF ( gamma_only ) THEN
            !
@@ -89,22 +93,24 @@ SUBROUTINE force_us( forcenl )
            !
            CALL force_us_k( forcenl )
            !
-        END IF
-     END DO
-  END DO
+        ENDIF
+     ENDDO
+  ENDDO
   !
   ! ... if sums over bands are parallelized over the band group
   !
-  IF( becp%comm /= mp_get_comm_null() ) CALL mp_sum( forcenl, becp%comm )
+  IF ( becp%comm /= mp_get_comm_null() ) CALL mp_sum( forcenl, becp%comm )
   !
   IF (noncolin) THEN
      DEALLOCATE( deff_nc )
-  ELSE IF ( .NOT. GAMMA_ONLY) THEN
+  ELSEIF ( .NOT. GAMMA_ONLY) THEN
      DEALLOCATE( deff )
   ENDIF
+  !
   DEALLOCATE( vkb1 )
-  CALL deallocate_bec_type ( dbecp )   
-  CALL deallocate_bec_type ( becp )   
+  !
+  CALL deallocate_bec_type( dbecp )
+  CALL deallocate_bec_type( becp )
   !
   ! ... collect contributions across pools from all k-points
   !
@@ -119,7 +125,7 @@ SUBROUTINE force_us( forcenl )
   ! ... Since our summation over k points was only on the irreducible 
   ! ... BZ we have to symmetrize the forces.
   !
-  CALL symvector ( nat, forcenl )
+  CALL symvector( nat, forcenl )
   !
   RETURN
   !
@@ -128,12 +134,15 @@ SUBROUTINE force_us( forcenl )
      !-----------------------------------------------------------------------
      SUBROUTINE force_us_gamma( forcenl )
        !-----------------------------------------------------------------------
-       !
-       ! ... calculation at gamma
+       !! Nonlocal contributiuon. Calculation at gamma.
        !
        IMPLICIT NONE
        !
        REAL(DP) :: forcenl(3,nat)
+       !! the nonlocal contribution
+       !
+       ! ... local variables
+       !
        REAL(DP), ALLOCATABLE :: aux(:,:)
        INTEGER ::  nt, na, ibnd, ibnd_loc, ih, jh, ijkb0 ! counters
        !
@@ -146,63 +155,70 @@ SUBROUTINE force_us( forcenl )
        !
        DO nt = 1, ntyp
           IF ( nh(nt) == 0 ) CYCLE
-          ALLOCATE ( aux(nh(nt),becp%nbnd_loc) )
+          ALLOCATE( aux(nh(nt),becp%nbnd_loc) )
           DO na = 1, nat
              IF ( ityp(na) == nt ) THEN
                 ijkb0 = indv_ijkb0(na)
                 ! this is \sum_j q_{ij} <beta_j|psi>
-                CALL DGEMM ('N','N', nh(nt), becp%nbnd_loc, nh(nt), &
-                     1.0_dp, qq_at(1,1,na), nhm, becp%r(ijkb0+1,1),&
-                     nkb, 0.0_dp, aux, nh(nt) )
+                CALL DGEMM( 'N','N', nh(nt), becp%nbnd_loc, nh(nt),        &
+                            1.0_dp, qq_at(1,1,na), nhm, becp%r(ijkb0+1,1), &
+                            nkb, 0.0_dp, aux, nh(nt) )
                 ! multiply by -\epsilon_n
 !$omp parallel do default(shared) private(ibnd_loc,ibnd,ih)
                 DO ih = 1, nh(nt)
                    DO ibnd_loc = 1, becp%nbnd_loc
                       ibnd = ibnd_loc + becp%ibnd_begin - 1
                       aux(ih,ibnd_loc) = - et(ibnd,ik) * aux(ih,ibnd_loc)
-                   END DO
-                END DO
+                   ENDDO
+                ENDDO
 !$omp end parallel do
                 ! add  \sum_j d_{ij} <beta_j|psi>
-                CALL DGEMM ('N','N', nh(nt), becp%nbnd_loc, nh(nt), &
-                     1.0_dp, deeq(1,1,na,current_spin), nhm, &
-                     becp%r(ijkb0+1,1), nkb, 1.0_dp, aux, nh(nt) )
+                CALL DGEMM( 'N','N', nh(nt), becp%nbnd_loc, nh(nt), &
+                            1.0_dp, deeq(1,1,na,current_spin), nhm, &
+                            becp%r(ijkb0+1,1), nkb, 1.0_dp, aux, nh(nt) )
 !$omp parallel do default(shared) private(ibnd_loc,ibnd,ih) reduction(-:forcenl)
                 DO ih = 1, nh(nt)
                    DO ibnd_loc = 1, becp%nbnd_loc
                       ibnd = ibnd_loc + becp%ibnd_begin - 1
-                      forcenl(ipol,na) = forcenl(ipol,na) - &
+                      forcenl(ipol,na) = forcenl(ipol,na) -    &
                            2.0_dp * tpiba * aux(ih,ibnd_loc) * &
                            dbecp%r(ijkb0+ih,ibnd_loc) * wg(ibnd,ik)
-                   END DO
-                END DO
+                   ENDDO
+                ENDDO
 !$omp end parallel do
                 !
-             END IF
-          END DO
-          DEALLOCATE (aux)
-       END DO
+             ENDIF
+          ENDDO
+          DEALLOCATE( aux )
+       ENDDO
        !
      END SUBROUTINE force_us_gamma
      !     
      !-----------------------------------------------------------------------
      SUBROUTINE force_us_k( forcenl )
        !-----------------------------------------------------------------------
-       !  
+       !! Nonlocal contributiuon. Calculation for k-points.
+       !
        IMPLICIT NONE
        !
        REAL(DP) :: forcenl(3,nat)
+       !! the nonlocal contribution
+       !
+       ! ... local variables
        !
        REAL(DP) :: fac
        INTEGER  :: ibnd, ih, jh, na, nt, ikb, jkb, ijkb0, is, js, ijs !counters
        !
        DO ibnd = 1, nbnd
+          !
           IF (noncolin) THEN
-             CALL compute_deff_nc(deff_nc,et(ibnd,ik))
+             CALL compute_deff_nc( deff_nc, et(ibnd,ik) )
           ELSE
-             CALL compute_deff(deff,et(ibnd,ik))
+             CALL compute_deff( deff, et(ibnd,ik) )
           ENDIF
-          fac=wg(ibnd,ik)*tpiba
+          !
+          fac = wg(ibnd,ik)*tpiba
+          !
           DO nt = 1, ntyp
              DO na = 1, nat
                 ijkb0 = indv_ijkb0(na)
@@ -211,26 +227,26 @@ SUBROUTINE force_us( forcenl )
                       ikb = ijkb0 + ih
                       IF (noncolin) THEN
                          ijs=0
-                         DO is=1,npol
-                            DO js=1,npol
+                         DO is = 1, npol
+                            DO js = 1, npol
                                ijs=ijs+1
                                forcenl(ipol,na) = forcenl(ipol,na)- &
-                                    deff_nc(ih,ih,na,ijs)*fac*( &
-                                    CONJG(dbecp%nc(ikb,is,ibnd))* &
-                                    becp%nc(ikb,js,ibnd)+ &
-                                    CONJG(becp%nc(ikb,is,ibnd))* &
+                                    deff_nc(ih,ih,na,ijs)*fac*(     &
+                                    CONJG(dbecp%nc(ikb,is,ibnd))*   &
+                                    becp%nc(ikb,js,ibnd)+           &
+                                    CONJG(becp%nc(ikb,is,ibnd))*    &
                                     dbecp%nc(ikb,js,ibnd) )
-                            END DO
-                         END DO
+                            ENDDO
+                         ENDDO
                       ELSE
-                         forcenl(ipol,na) = forcenl(ipol,na) - &
-                              2.D0 * fac * deff(ih,ih,na)*&
+                         forcenl(ipol,na) = forcenl(ipol,na) -   &
+                              2.D0 * fac * deff(ih,ih,na)*       &
                               DBLE( CONJG( dbecp%k(ikb,ibnd) ) * &
                               becp%k(ikb,ibnd) )
-                      END IF
-                   END DO
+                      ENDIF
+                   ENDDO
                    !
-                   IF ( upf(nt)%tvanp .OR. newpseudo(nt) ) THEN
+                   IF ( upf(nt)%tvanp .OR. upf(nt)%is_multiproj ) THEN
                       DO ih = 1, nh(nt)
                          ikb = ijkb0 + ih
                          !
@@ -242,39 +258,40 @@ SUBROUTINE force_us( forcenl )
                             jkb = ijkb0 + jh
                             IF (noncolin) THEN
                                ijs=0
-                               DO is=1,npol
-                                  DO js=1,npol
-                                     ijs=ijs+1
-                                     forcenl(ipol,na)=forcenl(ipol,na)- &
-                                          deff_nc(ih,jh,na,ijs)*fac*( &
-                                          CONJG(dbecp%nc(ikb,is,ibnd))* &
-                                          becp%nc(jkb,js,ibnd)+ &
-                                          CONJG(becp%nc(ikb,is,ibnd))* &
-                                          dbecp%nc(jkb,js,ibnd))- &
-                                          deff_nc(jh,ih,na,ijs)*fac*( &
-                                          CONJG(dbecp%nc(jkb,is,ibnd))* &
-                                          becp%nc(ikb,js,ibnd)+ &
-                                          CONJG(becp%nc(jkb,is,ibnd))* &
+                               DO is = 1, npol
+                                  DO js = 1, npol
+                                     ijs = ijs + 1
+                                     forcenl(ipol,na) = forcenl(ipol,na)- &
+                                          deff_nc(ih,jh,na,ijs)*fac*(     &
+                                          CONJG(dbecp%nc(ikb,is,ibnd))*   &
+                                          becp%nc(jkb,js,ibnd)+           &
+                                          CONJG(becp%nc(ikb,is,ibnd))*    &
+                                          dbecp%nc(jkb,js,ibnd))-         &
+                                          deff_nc(jh,ih,na,ijs)*fac*(     &
+                                          CONJG(dbecp%nc(jkb,is,ibnd))*   &
+                                          becp%nc(ikb,js,ibnd)+           &
+                                          CONJG(becp%nc(jkb,is,ibnd))*    &
                                           dbecp%nc(ikb,js,ibnd) )
-                                  END DO
-                               END DO
+                                  ENDDO
+                               ENDDO
                             ELSE
-                               forcenl(ipol,na) = forcenl (ipol,na) - &
-                                    2.D0 * fac * deff(ih,jh,na)* &
-                                    DBLE( CONJG( dbecp%k(ikb,ibnd) ) * &
-                                    becp%k(jkb,ibnd) +       &
-                                    dbecp%k(jkb,ibnd) * &
-                                    CONJG( becp%k(ikb,ibnd) ) )
-                            END IF
-                         END DO !jh
-                      END DO !ih
-                   END IF ! tvanp
-                END IF ! ityp(na) == nt
-             END DO ! nat
-          END DO ! ntyp
-       END DO ! nbnd
-
+                               forcenl(ipol,na) = forcenl(ipol,na) -     &
+                                    2.D0 * fac * deff(ih,jh,na) *        &
+                                    DBLE( CONJG( dbecp%k(ikb,ibnd) ) *   &
+                                    becp%k(jkb,ibnd) + dbecp%k(jkb,ibnd) &
+                                    * CONJG( becp%k(ikb,ibnd) ) )
+                            ENDIF
+                         ENDDO !jh
+                      ENDDO !ih
+                   ENDIF ! tvanp
+                   !
+                ENDIF ! ityp(na) == nt
+             ENDDO ! nat
+          ENDDO ! ntyp
+       ENDDO ! nbnd
+       !
        !
      END SUBROUTINE force_us_k
-     !     
+     !
+     !
 END SUBROUTINE force_us
