@@ -21,25 +21,23 @@ subroutine init_us_1
   !      nh <-> combined (l,m) index for the beta function.
   !   e) It computes the coefficients c_{LM}^{nm} which relates the
   !      spherical harmonics in the Q expansion
-  !   f) It computes the radial fourier transform of the Q function on
-  !      all the g vectors
-  !   g) It computes the q terms which define the S matrix.
-  !   h) It fills the interpolation table for the beta functions
+  !   f) It computes the interpolation table "qrad" for Q(G)
+  !   g) It computes the qq terms which define the S matrix.
+  !   h) It fills the interpolation table "tab" for the beta functions
   !
   USE kinds,        ONLY : DP
-  USE parameters,   ONLY : lmaxx
   USE constants,    ONLY : fpi, sqrt2
   USE atom,         ONLY : rgrid
   USE ions_base,    ONLY : ntyp => nsp, ityp, nat
-  USE cell_base,    ONLY : omega, tpiba
+  USE cell_base,    ONLY : omega
   USE gvect,        ONLY : g, gg
   USE lsda_mod,     ONLY : nspin
   USE us,           ONLY : nqxq, dq, nqx, tab, tab_d2y, qrad, spline_ps
   USE splinelib
   USE uspp,         ONLY : nhtol, nhtoj, nhtolm, ijtoh, dvan, qq_at, qq_nt, indv,&
                            ap, aainit, qq_so, dvan_so, okvan, indv_ijkb0
-  USE uspp_param,   ONLY : upf, lmaxq, nbetam, nh, nhm, lmaxkb
-  USE spin_orb,     ONLY : lspinorb, rot_ylm, fcoef
+  USE uspp_param,   ONLY : upf, lmaxq, nh, nhm, lmaxkb
+  USE spin_orb,     ONLY : lspinorb, rot_ylm, fcoef, lmaxx
   USE paw_variables,ONLY : okpaw
   USE mp_bands,     ONLY : intra_bgrp_comm
   USE mp,           ONLY : mp_sum
@@ -51,12 +49,10 @@ subroutine init_us_1
   integer :: nt, ih, jh, nb, mb, ijv, l, m, ir, iq, is, startq, &
        lastq, ilast, ndm, ia
   ! various counters
-  real(DP), allocatable :: aux (:), aux1 (:), besr (:), qtot (:,:)
+  real(DP), allocatable :: aux (:), besr (:)
   ! various work space
-  real(DP) :: prefr, pref, q, qi
-  ! the prefactor of the q functions
+  real(DP) :: pref, qi
   ! the prefactor of the beta functions
-  ! the modulus of g for each shell
   ! q-point grid for interpolation
   real(DP), allocatable :: ylmk0 (:)
   ! the spherical harmonics
@@ -67,22 +63,15 @@ subroutine init_us_1
              lk, mk, vk, kh, lh, ijkb0, na
   integer, external :: sph_ind
   complex(DP) :: coeff, qgm(1)
-  real(DP) :: spinor, ji, jk
-  !
+  real(DP) :: ji, jk, d1
   real(DP), allocatable :: xdata(:)
-  real(DP) :: d1
+  real(DP), EXTERNAL :: spinor
   !
   call start_clock ('init_us_1')
   !
   !    Initialization of the variables
   !
-  ndm = MAXVAL ( upf(:)%kkbeta )
-  allocate (aux ( ndm))    
-  allocate (aux1( ndm))    
-  allocate (ylmk0( lmaxq * lmaxq))    
-  allocate (qtot( ndm , nbetam*(nbetam+1)/2 ))    
   ap (:,:,:)   = 0.d0
-  if (lmaxq > 0) qrad(:,:,:,:)= 0.d0
   !
   ! the following prevents an out-of-bound error: upf(nt)%nqlc=2*lmax+1
   ! but in some versions of the PP files lmax is not set to the maximum
@@ -93,7 +82,6 @@ subroutine init_us_1
      IF ( upf(nt)%nqlc < 0 )  upf(nt)%nqlc = 0
   end do
 
-  prefr = fpi / omega
   if (lspinorb) then
 !
 !  In the spin-orbit case we need the unitary matrix u which rotates the
@@ -162,9 +150,9 @@ subroutine init_us_1
          enddo
      enddo
      !
-     ! ijkb0 is just before the first beta "in the solid" for atom ia
+     ! ijkb0 points to the last beta "in the solid" for atom ia-1
      ! i.e. ijkb0+1,.. ijkb0+nh(ityp(ia)) are the nh beta functions of
-     !      atom ia in the global list of beta functions
+     !      atom ia in the global list of beta functions (ijkb0=0 for ia=1)
      do ia = 1,nat
        IF ( ityp(ia) == nt ) THEN
           indv_ijkb0(ia) = ijkb0
@@ -173,7 +161,7 @@ subroutine init_us_1
      enddo
      !
      !    From now on the only difference between KB and US pseudopotentials
-     !    is in the presence of the q and Q functions.
+     !    is in the presence of the qq and Q functions.
      !
      !    Here we initialize the D of the solid
      !
@@ -250,73 +238,16 @@ subroutine init_us_1
   !
   !   here for the US types we compute the Fourier transform of the
   !   Q functions.
-  !   
-  call divide (intra_bgrp_comm, nqxq, startq, lastq)
   !
-  do nt = 1, ntyp
-     if ( upf(nt)%tvanp ) then
-        do l = 0, upf(nt)%nqlc -1
-           !
-           !     first we build for each nb,mb,l the total Q(|r|) function
-           !     note that l is the true (combined) angular momentum
-           !     and that the arrays have dimensions 0..l (no more 1..l+1)
-           !
-           do nb = 1, upf(nt)%nbeta
-              do mb = nb, upf(nt)%nbeta
-               respect_sum_rule : &
-               if ( ( l >= abs(upf(nt)%lll(nb) - upf(nt)%lll(mb)) ) .and. &
-                    ( l <=     upf(nt)%lll(nb) + upf(nt)%lll(mb)  ) .and. &
-                    (mod (l+upf(nt)%lll(nb)+upf(nt)%lll(mb), 2) == 0) ) then
-                 ijv = mb * (mb-1) / 2 + nb
-                 ! in PAW and now in US as well q(r) is stored in an l-dependent array
-                 qtot(1:upf(nt)%kkbeta,ijv) = upf(nt)%qfuncl(1:upf(nt)%kkbeta,ijv,l)
-               endif respect_sum_rule
-              enddo ! mb
-           enddo ! nb
-           !
-           !     here we compute the spherical bessel function for each |g|
-           !
-           do iq = startq, lastq
-              q = (iq - 1) * dq * tpiba
-              call sph_bes ( upf(nt)%kkbeta, rgrid(nt)%r, q, l, aux)
-              !
-              !   and then we integrate with all the Q functions
-              !
-              do nb = 1, upf(nt)%nbeta
-                 !
-                 !    the Q are symmetric with respect to indices
-                 !
-                 do mb = nb, upf(nt)%nbeta
-                    ijv = mb * (mb - 1) / 2 + nb
-                    if ( ( l >= abs(upf(nt)%lll(nb) - upf(nt)%lll(mb)) ) .and. &
-                         ( l <=     upf(nt)%lll(nb) + upf(nt)%lll(mb)  ) .and. &
-                         (mod(l+upf(nt)%lll(nb)+upf(nt)%lll(mb),2)==0) ) then
-                       do ir = 1, upf(nt)%kkbeta
-                          aux1 (ir) = aux (ir) * qtot (ir, ijv)
-                       enddo
-                       call simpson ( upf(nt)%kkbeta, aux1, rgrid(nt)%rab, &
-                                     qrad(iq,ijv,l + 1, nt) )
-                    endif
-                 enddo
-              enddo
-              ! igl
-           enddo
-           ! l
-        enddo
-        qrad (:, :, :, nt) = qrad (:, :, :, nt)*prefr
-        call mp_sum ( qrad (:, :, :, nt), intra_bgrp_comm )
-     endif
-     ! ntyp
-  enddo
-  deallocate (aux1)
-  deallocate (qtot)
+  IF ( lmaxq > 0 ) CALL compute_qrad( )
   !
   !   and finally we compute the qq coefficients by integrating the Q.
-  !   q are the g=0 components of Q.
+  !   The qq are the g=0 components of Q
   !
 #if defined(__MPI)
   if (gg (1) > 1.0d-8) goto 100
 #endif
+  allocate (ylmk0( lmaxq * lmaxq))
   call ylmr2 (lmaxq * lmaxq, 1, g, gg, ylmk0)
   do nt = 1, ntyp
     if ( upf(nt)%tvanp ) then
@@ -359,6 +290,7 @@ subroutine init_us_1
       endif
     endif
   enddo
+  deallocate (ylmk0)
 #if defined(__MPI)
 100 continue
   if (lspinorb) then
@@ -372,14 +304,12 @@ subroutine init_us_1
   do na=1, nat
      qq_at(:,:, na) = qq_nt(:,:,ityp(na))
   end do
-
-  deallocate (ylmk0)
-  deallocate (aux)
   !
   !     fill the interpolation table tab
   !
+  ndm = MAXVAL ( upf(:)%kkbeta )
   allocate( aux (ndm) )
-  allocate (besr( ndm))    
+  allocate (besr( ndm))
   pref = fpi / sqrt (omega)
   call divide (intra_bgrp_comm, nqx, startq, lastq)
   tab (:,:,:) = 0.d0
@@ -398,6 +328,8 @@ subroutine init_us_1
         enddo
      enddo
   enddo
+  deallocate (besr)
+  deallocate (aux)
 
   call mp_sum(  tab, intra_bgrp_comm )
 
@@ -416,10 +348,92 @@ subroutine init_us_1
      deallocate(xdata)
   endif
 
-  deallocate (besr)
-  deallocate (aux)
-
   call stop_clock ('init_us_1')
   return
 end subroutine init_us_1
 
+!----------------------------------------------------------------------
+SUBROUTINE compute_qrad ( )
+  !----------------------------------------------------------------------
+  !
+  ! Compute interpolation table qrad(i,nm,l+1,nt) = Q^{(L)}_{nm,nt}(q_i)
+  ! of angular momentum L, for atom of type nt, on grid q_i, where
+  ! nm = combined index for n,m=1,nh(nt)
+  !
+  USE kinds,        ONLY : dp
+  USE constants,    ONLY : fpi
+  USE ions_base,    ONLY : ntyp => nsp
+  USE cell_base,    ONLY : omega
+  USE atom,         ONLY : rgrid
+  USE uspp_param,   ONLY : upf, lmaxq, nbetam, nh, nhm, lmaxkb
+  USE us,           ONLY : nqxq, dq, qrad
+  USE mp_bands,     ONLY : intra_bgrp_comm
+  USE mp,           ONLY : mp_sum
+  !
+  IMPLICIT NONE
+  !
+  INTEGER :: ndm, startq, lastq, nt, l, nb, mb, ijv, iq, ir
+  ! various indices
+  REAL(dp) :: prefr
+  ! the prefactor of the Q functions
+  REAL(dp) :: q
+  REAL(dp), ALLOCATABLE :: aux (:), besr (:)
+  ! various work space
+  !
+  prefr = fpi / omega
+  ndm = MAXVAL ( upf(:)%kkbeta )
+  ALLOCATE (aux ( ndm))
+  ALLOCATE (besr( ndm))
+
+  CALL divide (intra_bgrp_comm, nqxq, startq, lastq)
+  !
+  qrad(:,:,:,:)= 0.d0
+  DO nt = 1, ntyp
+     if ( upf(nt)%tvanp ) then
+        DO l = 0, upf(nt)%nqlc -1
+           !
+           !     note that l is the true (combined) angular momentum
+           !     and that the arrays have dimensions 0..l (no more 1..l+1)
+           !
+           DO iq = startq, lastq
+              !
+              q = (iq - 1) * dq
+              !
+              !     here we compute the spherical bessel function for each q_i
+              !
+              CALL sph_bes ( upf(nt)%kkbeta, rgrid(nt)%r, q, l, besr)
+              !
+              DO nb = 1, upf(nt)%nbeta
+                 !
+                 !    the Q are symmetric with respect to indices
+                 !
+                 DO mb = nb, upf(nt)%nbeta
+                    ijv = mb * (mb - 1) / 2 + nb
+                    IF ( ( l >= abs(upf(nt)%lll(nb) - upf(nt)%lll(mb)) ) .AND. &
+                         ( l <=     upf(nt)%lll(nb) + upf(nt)%lll(mb)  ) .AND. &
+                         (mod(l+upf(nt)%lll(nb)+upf(nt)%lll(mb),2)==0) ) THEN
+                       DO ir = 1, upf(nt)%kkbeta
+                          aux  (ir) = besr (ir) * upf(nt)%qfuncl(ir,ijv,l)
+                       ENDDO
+                       !
+                       !   and then we integrate with all the Q functions
+                       !
+                       CALL simpson ( upf(nt)%kkbeta, aux, rgrid(nt)%rab, &
+                                     qrad(iq,ijv,l+1, nt) )
+                    ENDIF
+                 ENDDO
+              ENDDO
+              ! igl
+           ENDDO
+           ! l
+        ENDDO
+        qrad (:, :, :, nt) = qrad (:, :, :, nt)*prefr
+        CALL mp_sum ( qrad (:, :, :, nt), intra_bgrp_comm )
+     ENDIF
+     ! ntyp
+  ENDDO
+  !
+  DEALLOCATE (besr)
+  DEALLOCATE (aux)
+  !
+END SUBROUTINE compute_qrad
