@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2020 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -8,10 +8,9 @@
 !----------------------------------------------------------------------------
 SUBROUTINE openfil()
   !----------------------------------------------------------------------------
-  !
-  ! ... This routine opens some files needed to the self consistent run,
-  ! ... sets various file names, units, record lengths
-  ! ... All units are set in Modules/io_files.f90
+  !! This routine opens some files needed to the self consistent run,
+  !! sets various file names, units, record lengths. 
+  !! All units are set in Modules/io_files.f90
   !
   USE kinds,            ONLY : DP
   USE buffers,          ONLY : open_buffer
@@ -21,22 +20,22 @@ SUBROUTINE openfil()
   USE fixed_occ,        ONLY : one_atom_occupations
   USE ldaU,             ONLY : lda_plus_U, U_projection, nwfcU
   USE io_files,         ONLY : prefix, iunpun, iunsat, &
-                               iunhub, nwordwfcU, nwordwfc, nwordatwfc,&
+                               iunhub, nwordwfcU, nwordwfc, nwordatwfc, &
                                iunefield, iunefieldm, iunefieldp, seqopn
   USE noncollin_module, ONLY : npol
   USE bp,               ONLY : lelfield
   USE wannier_new,      ONLY : use_wannier
-#if defined(__HDF5) && defined(__OLDXML)
+#if defined(__HDF5) && defined(__MPI) 
   USE hdf5_qe,          ONLY : initialize_hdf5
-#endif
+#endif 
   !
   IMPLICIT NONE
   !
-  LOGICAL            :: exst
+  LOGICAL :: exst
   !
-  ! ... Files needed for LDA+U
+  ! ... Files needed for DFT+U(+V)
   ! ... iunsat contains the (orthogonalized) atomic wfcs * S
-  ! ... iunhub as above, only wfcs with a U correction
+  ! ... iunhub  as above, only wfcs * S with a U correction
   !
   ! ... nwordwfc is the record length (IN COMPLEX WORDS)
   ! ... for the direct-access file containing wavefunctions
@@ -46,10 +45,11 @@ SUBROUTINE openfil()
   nwordatwfc= npwx*natomwfc*npol
   nwordwfcU = npwx*nwfcU*npol
   !
-  IF ( lda_plus_u .AND. (U_projection.NE.'pseudo') ) &
-     CALL open_buffer ( iunhub, 'hub',    nwordwfcU, io_level, exst )
+  IF ( lda_plus_u .AND. (U_projection.NE.'pseudo') ) THEN
+     CALL open_buffer( iunhub,  'hub',  nwordwfcU, io_level, exst )
+  ENDIF
   IF ( use_wannier .OR. one_atom_occupations ) &
-     CALL open_buffer ( iunsat, 'satwfc', nwordatwfc, io_level, exst )
+     CALL open_buffer( iunsat, 'satwfc', nwordatwfc, io_level, exst )
   !
   ! ... open units for electric field calculations
   !
@@ -57,9 +57,9 @@ SUBROUTINE openfil()
       CALL open_buffer( iunefield , 'ewfc' , nwordwfc, io_level, exst )
       CALL open_buffer( iunefieldm, 'ewfcm', nwordwfc, io_level, exst )
       CALL open_buffer( iunefieldp, 'ewfcp', nwordwfc, io_level, exst )
-  END IF
+  ENDIF
   !
-#if defined(__HDF5) && defined(__OLDXML)
+#if defined(__HDF5) && defined(__MPI) 
   ! calls h5open_f mandatory in any application using hdf5
   CALL initialize_hdf5()
 #endif 
