@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2020 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -31,30 +31,25 @@ SUBROUTINE potinit()
   USE klist,                ONLY : nelec
   USE lsda_mod,             ONLY : lsda, nspin
   USE fft_base,             ONLY : dfftp
-  USE fft_interfaces,       ONLY : fwfft
   USE gvect,                ONLY : ngm, gstart, g, gg, ig_l2g
   USE gvecs,                ONLY : doublegrid
   USE control_flags,        ONLY : lscf, gamma_only
   USE scf,                  ONLY : rho, rho_core, rhog_core, &
                                    vltot, v, vrs, kedtau
   USE funct,                ONLY : dft_is_meta
-  USE wavefunctions_module, ONLY : psic
   USE ener,                 ONLY : ehart, etxc, vtxc, epaw
   USE ldaU,                 ONLY : lda_plus_u, Hubbard_lmax, eth, &
-                                   niter_with_fixed_ns
+                                   niter_with_fixed_ns, lda_plus_u_kind, &
+                                   nsg, nsgnew
   USE noncollin_module,     ONLY : noncolin, report
-  USE io_files,             ONLY : tmp_dir, prefix, postfix, input_drho, check_file_exist
+  USE io_files,             ONLY : restart_dir, input_drho, check_file_exist
   USE spin_orb,             ONLY : domag, lforcet
   USE mp,                   ONLY : mp_sum
   USE mp_bands ,            ONLY : intra_bgrp_comm, root_bgrp
   USE io_global,            ONLY : ionode, ionode_id
   USE io_rho_xml,           ONLY : read_scf
-#if defined(__OLDXML)
-  USE xml_io_base,          ONLY : read_rho
-#else
   USE io_base,              ONLY : read_rhog
-  USE fft_rho,              ONLY : rho_g2r
-#endif
+  USE fft_rho,              ONLY : rho_g2r, rho_r2g
   !
   USE uspp,                 ONLY : becsum
   USE paw_variables,        ONLY : okpaw, ddd_PAW
@@ -63,46 +58,39 @@ SUBROUTINE potinit()
   !
   IMPLICIT NONE
   !
-  REAL(DP)              :: charge           ! the starting charge
-  REAL(DP)              :: etotefield       !
-  REAL(DP)              :: fact
-  INTEGER               :: is
-  LOGICAL               :: exst 
-  CHARACTER(LEN=256)    :: dirname, filename
+  REAL(DP)                  :: charge           ! the starting charge
+  REAL(DP)                  :: etotefield       !
+  REAL(DP)                  :: fact
+  INTEGER                   :: is
+  LOGICAL                   :: exst 
+  CHARACTER(LEN=320)        :: filename
+  COMPLEX (DP), ALLOCATABLE :: work(:,:)
   !
   CALL start_clock('potinit')
   !
-  dirname = TRIM(tmp_dir) // TRIM (prefix) // postfix
+  filename = TRIM (restart_dir( )) // 'charge-density'
 #if defined __HDF5
-  filename = TRIM(dirname) // 'charge-density.hdf5'
+  exst     =  check_file_exist( TRIM(filename) // '.hdf5' )
 #else 
-  filename = TRIM(dirname) // 'charge-density.dat'
+  exst     =  check_file_exist( TRIM(filename) // '.dat' )
 #endif
-  exst     =  check_file_exist( TRIM(filename) )
   !
   IF ( starting_pot == 'file' .AND. exst ) THEN
      !
      ! ... Cases a) and b): the charge density is read from file
-     ! ... this also reads rho%ns if lda+U and rho%bec if PAW
+     ! ... this also reads rho%ns if DFT+U, rho%bec if PAW, rho%kin if metaGGA
      !
      IF ( .NOT.lforcet ) THEN
         CALL read_scf ( rho, nspin, gamma_only )
-#if !defined (__OLDXML)
-        CALL rho_g2r ( dfftp, rho%of_g, rho%of_r )
-#endif
      ELSE
         !
         ! ... 'force theorem' calculation of MAE: read rho only from previous
         ! ... lsda calculation, set noncolinear magnetization from angles
+        ! ... FIXME: why not calling read_scf also in this case?
         !
-#if defined (__OLDXML)
-        CALL read_rho ( dirname, rho%of_r, 2 )
-#else
-        CALL read_rhog ( dirname, root_bgrp, intra_bgrp_comm, &
+        CALL read_rhog ( filename, root_bgrp, intra_bgrp_comm, &
              ig_l2g, nspin, rho%of_g, gamma_only )
-        CALL rho_g2r ( dfftp, rho%of_g, rho%of_r )
-#endif
-        CALL nc_magnetization_from_lsda ( dfftp%nnr, nspin, rho%of_r )
+        CALL nc_magnetization_from_lsda ( dfftp%ngm, nspin, rho%of_g )
      END IF
      !
      IF ( lscf ) THEN
@@ -119,6 +107,23 @@ SUBROUTINE potinit()
         !
      END IF
      !
+     IF ( input_drho /= ' ' ) THEN
+        !
+        filename = TRIM( restart_dir( )) // input_drho
+        CALL read_rhog ( filename, root_bgrp, intra_bgrp_comm, &
+             ig_l2g, nspin, v%of_g, gamma_only )
+        ! 
+        WRITE( UNIT = stdout, &
+               FMT = '(/5X,"a scf correction to at. rho is read from",A)' ) &
+            TRIM( filename )
+        !
+        ALLOCATE( work( dfftp%ngm, nspin ) )
+        CALL atomic_rho_g( work, nspin )
+        rho%of_g(:,1) = work(:,1) + v%of_g(:,1)
+        DEALLOCATE(work)
+        !
+     END IF
+     !
   ELSE
      !
      ! ... Case c): the potential is built from a superposition 
@@ -130,15 +135,22 @@ SUBROUTINE potinit()
      WRITE( UNIT = stdout, &
             FMT = '(/5X,"Initial potential from superposition of free atoms")' )
      !
-     CALL atomic_rho( rho%of_r, nspin )
+     CALL atomic_rho_g( rho%of_g, nspin )
 
-     ! ... in the lda+U case set the initial value of ns
+     ! ... in the DFT+U(+V) case set the initial value of ns (or nsg)
+     !
      IF (lda_plus_u) THEN
         !
-        IF (noncolin) THEN
-           CALL init_ns_nc()
-        ELSE
+        IF (lda_plus_u_kind == 0) THEN
            CALL init_ns()
+        ELSEIF (lda_plus_u_kind == 1) THEN
+           IF (noncolin) THEN
+              CALL init_ns_nc()
+           ELSE
+              CALL init_ns()
+           ENDIF
+        ELSEIF (lda_plus_u_kind == 2) THEN
+           CALL init_nsg()
         ENDIF
         !
      ENDIF
@@ -148,39 +160,26 @@ SUBROUTINE potinit()
      !
      IF ( input_drho /= ' ' ) THEN
         !
-        IF ( nspin > 1 ) CALL errore &
-             ( 'potinit', 'spin polarization not allowed in drho', 1 )
-        !
-#if defined (__OLDXML)
-        CALL read_rho ( dirname, v%of_r, 1, input_drho )
-#else
-        CALL read_rhog ( dirname, root_bgrp, intra_bgrp_comm, &
+        filename = TRIM( restart_dir( )) // input_drho
+        CALL read_rhog ( filename, root_bgrp, intra_bgrp_comm, &
              ig_l2g, nspin, v%of_g, gamma_only )
-        CALL rho_g2r ( dfftp, v%of_g, v%of_r )
-#endif
         !
         WRITE( UNIT = stdout, &
                FMT = '(/5X,"a scf correction to at. rho is read from",A)' ) &
-            TRIM( input_drho )
+            TRIM( filename )
         !
-        rho%of_r = rho%of_r + v%of_r
+        rho%of_g = rho%of_g + v%of_g
         !
      END IF
      !
   END IF
   !
-  ! ... check the integral of the starting charge
+  ! ... check the integral of the starting charge, renormalize if needed
   !
-  IF ( nspin == 2 ) THEN
-     !
-     charge = SUM ( rho%of_r(:,1:nspin) )*omega / ( dfftp%nr1*dfftp%nr2*dfftp%nr3 )
-     !
-  ELSE
-     !
-     charge = SUM ( rho%of_r(:,1) )*omega / ( dfftp%nr1*dfftp%nr2*dfftp%nr3 )
-     !
+  charge = 0.D0
+  IF ( gstart == 2 ) THEN
+     charge = omega*REAL( rho%of_g(1,1) )
   END IF
-  !
   CALL mp_sum(  charge , intra_bgrp_comm )
   !
   IF ( lscf .AND. ABS( charge - nelec ) > ( 1.D-7 * charge ) ) THEN
@@ -188,14 +187,11 @@ SUBROUTINE potinit()
      IF ( charge > 1.D-8 .AND. nat > 0 ) THEN
         WRITE( stdout, '(/,5X,"starting charge ",F10.5, &
                          & ", renormalised to ",F10.5)') charge, nelec
-        rho%of_r = rho%of_r / charge * nelec
+        rho%of_g = rho%of_g / charge * nelec
      ELSE 
         WRITE( stdout, '(/,5X,"Starting from uniform charge")')
-        IF ( nspin == 2 ) THEN
-           rho%of_r(:,1:nspin) = nelec / omega / nspin
-        ELSE
-           rho%of_r(:,1) = nelec / omega
-        END IF
+        rho%of_g(:,1:nspin) = (0.0_dp,0.0_dp)
+        IF ( gstart == 2 ) rho%of_g(1,1) = nelec / omega
      ENDIF
      !
   ELSE IF ( .NOT. lscf .AND. ABS( charge - nelec ) > (1.D-3 * charge ) ) THEN
@@ -204,34 +200,30 @@ SUBROUTINE potinit()
      !
   END IF
   !
-  ! ... bring starting rho to G-space
+  ! ... bring starting rho from G- to R-space
   !
-  DO is = 1, nspin
-     !
-     psic(:) = rho%of_r(:,is)
-     !
-     CALL fwfft ('Rho', psic, dfftp)
-     !
-     rho%of_g(:,is) = psic(dfftp%nl(:))
-     !
-  END DO
+  CALL rho_g2r (dfftp, rho%of_g, rho%of_r)
   !
-  if ( dft_is_meta()) then
-     ! ... define a starting (TF) guess for rho%kin_r and rho%kin_g
-     fact = (3.d0/5.d0)*(3.d0*pi*pi)**(2.0/3.0)
+  IF  ( dft_is_meta() ) THEN
+     IF (starting_pot /= 'file') THEN
+        ! ... define a starting (TF) guess for rho%kin_r from rho%of_r
+        ! ... to be verified for LSDA: rho is (tot,magn), rho_kin is (up,down)
+        fact = (3.d0/5.d0)*(3.d0*pi*pi)**(2.0/3.0)
+        DO is = 1, nspin
+           rho%kin_r(:,is) = fact * abs(rho%of_r(:,is)*nspin)**(5.0/3.0)/nspin
+        END DO
+        !if (nspin==2) then
+        !     rho%kin_r(:,1) = fact * abs(rho%of_r(:,1)+rho%of_r(:,2))**(5.0/3.0)/2.0
+        !     rho%kin_r(:,2) = fact * abs(rho%of_r(:,1)-rho%of_r(:,2))**(5.0/3.0)/2.0
+        !endif
+        ! ... bring it to g-space
+        CALL rho_r2g (dfftp, rho%kin_r, rho%kin_g)
+     ELSE
+        ! ... rho%kin was read from file in G-space, bring it to R-space
+        CALL rho_g2r (dfftp, rho%kin_g, rho%kin_r)
+     ENDIF
      !
-     ! ... for obscure reasons this starting guess doesn't seem much better
-     ! ... (and sometimes it is much worse) than starting from zero
-     !
-     !!! fact = 0.0_dp
-     DO is = 1, nspin
-        if (starting_pot /= 'file') rho%kin_r(:,is) = fact * abs(rho%of_r(:,is)*nspin)**(5.0/3.0)/nspin
-        psic(:) = rho%kin_r(:,is)
-        CALL fwfft ('Rho', psic, dfftp)
-        rho%kin_g(:,is) = psic(dfftp%nl(:))
-     END DO
-     !
-  end if
+  END IF
   !
   ! ... plugin contribution to local potential
   !
@@ -247,7 +239,7 @@ SUBROUTINE potinit()
   !
   CALL set_vrs( vrs, vltot, v%of_r, kedtau, v%kin_r, dfftp%nnr, nspin, doublegrid )
   !
-  ! ... write on output the parameters used in the lda+U calculation
+  ! ... write on output the parameters used in the DFT+U(+V) calculation
   !
   IF ( lda_plus_u ) THEN
      !
@@ -255,10 +247,17 @@ SUBROUTINE potinit()
          niter_with_fixed_ns
      WRITE( stdout, '(5X,"Starting occupations:")')
      !
-     IF (noncolin) THEN
-       CALL write_ns_nc()
-     ELSE
-       CALL write_ns()
+     IF (lda_plus_u_kind == 0) THEN
+        CALL write_ns()
+     ELSEIF (lda_plus_u_kind == 1) THEN
+        IF (noncolin) THEN
+           CALL write_ns_nc()
+        ELSE
+           CALL write_ns()
+        ENDIF
+     ELSEIF (lda_plus_u_kind == 2) THEN
+        nsgnew = nsg
+        CALL write_nsg()
      ENDIF
      !
   END IF
@@ -273,7 +272,7 @@ SUBROUTINE potinit()
 END SUBROUTINE potinit
 !
 !-------------
-SUBROUTINE nc_magnetization_from_lsda ( nnr, nspin, rho )
+SUBROUTINE nc_magnetization_from_lsda ( ngm, nspin, rho )
   !-------------
   !
   USE kinds,     ONLY: dp
@@ -282,8 +281,8 @@ SUBROUTINE nc_magnetization_from_lsda ( nnr, nspin, rho )
   USE noncollin_module, ONLY: angle1, angle2
   !
   IMPLICIT NONE
-  INTEGER, INTENT (in):: nnr, nspin
-  REAL(dp), INTENT (inout):: rho(nnr,nspin)
+  INTEGER, INTENT (in):: ngm, nspin
+  COMPLEX(dp), INTENT (inout):: rho(ngm,nspin)
   !---  
   !  set up noncollinear m_x,y,z from collinear m_z (AlexS) 
   !
@@ -293,19 +292,10 @@ SUBROUTINE nc_magnetization_from_lsda ( nnr, nspin, rho )
        angle1(1)/PI*180.d0, angle2(1)/PI*180.d0 
   WRITE(stdout,*) '-----------'
   !
-#if defined(__OLDXML)
-  ! On input, rho(1)=rho_up, rho(2)=rho_down
-  ! Set rho(1)=rho_tot, rho(3)=rho_up-rho_down=magnetization
-  ! 
-  rho(:,4) = rho(:,1)-rho(:,2)
-  rho(:,1) = rho(:,1)+rho(:,2)
-#endif
-  !
   ! now set rho(2)=magn*sin(theta)*cos(phi)   x
   !         rho(3)=magn*sin(theta)*sin(phi)   y
   !         rho(4)=magn*cos(theta)            z
   !
-  !rho(:,4) = rho(:,3)*cos(angle1(1))
   rho(:,2) = rho(:,4)*sin(angle1(1))
   rho(:,3) = rho(:,2)*sin(angle2(1))
   rho(:,4) = rho(:,4)*cos(angle1(1))
