@@ -7,38 +7,34 @@
 !
 !
 !----------------------------------------------------------------------------
-subroutine stres_nonloc_dft( rho, rho_core, nspin, sigma_nonloc_dft )
+SUBROUTINE stres_nonloc_dft( rho, rho_core, nspin, sigma_nonloc_dft )
 
   !----------------------------------------------------------------------------
   !
   USE kinds,            ONLY : DP
-  use funct,            ONLY : get_igcc, get_inlc 
+  USE funct,            ONLY : get_igcc, get_inlc 
   USE mp,               ONLY : mp_sum
   USE fft_base,         ONLY : dfftp
-  USE vdW_DF,           ONLY : stress_vdW_DF
-  USE rVV10,            ONLY : stress_rVV10
+  USE vdW_DF,           ONLY : vdW_DF_stress
+  USE rVV10,            ONLY : rVV10_stress
   USE io_global,        ONLY : stdout
   !
   IMPLICIT NONE
   !
-  integer, intent(in) ::nspin
+  integer,  intent(in)     :: nspin
   real(DP), intent(in)     :: rho (dfftp%nnr, nspin), rho_core (dfftp%nnr)
   real(DP), intent(inout)  :: sigma_nonloc_dft (3, 3)
 
   integer :: l, m, inlc
 
 
-  sigma_nonloc_dft(:,:) = 0.d0
+  sigma_nonloc_dft(:,:) = 0.D0
   inlc = get_inlc()
 
-  if ( inlc==1 .or. inlc==2 .or. inlc==4 .or. inlc==5 .or. inlc==6 ) then
-     CALL stress_vdW_DF(rho, rho_core, nspin, sigma_nonloc_dft)
-  elseif ( inlc == 3 ) then
-     if ( nspin>2 ) call errore ('stress_rVV10', 'rVV10 stress not implemented for nspin > 2',1)
-     CALL stress_rVV10(rho, rho_core, nspin, sigma_nonloc_dft)
-  end if
-
-  return
-
-end subroutine stres_nonloc_dft
+  IF ( inlc > 0 .AND. inlc < 26 ) THEN
+     CALL vdW_DF_stress (rho, rho_core, nspin, sigma_nonloc_dft)
+  ELSEIF ( inlc == 26 ) THEN
+     CALL rVV10_stress  (rho(:,1), rho_core, nspin, sigma_nonloc_dft)
+  END IF
 
+END SUBROUTINE stres_nonloc_dft
