@@ -9,48 +9,50 @@
 !----------------------------------------------------------------------------
 SUBROUTINE vcsmd( conv_ions )
   !----------------------------------------------------------------------------
+  !! Main (interface) routine between PWSCF and the variable-cell shape
+  !! molecular dynamics code by R.M. Wentzcovitch, PRB 44, 2358 (1991).
   !
-  ! Main (interface) routine between PWSCF and the variable-cell shape
-  ! molecular dynamics code by R.M. Wentzcovitch, PRB 44, 2358 (1991).
+  !! Molecular and/or cell dynamics is performed according to the value of
+  !! the switch variable calc:
   !
-  ! Molecular and/or cell dynamics is performed according to the value of
-  ! the switch variable calc:
+  !! * calc  = 'md'   : standard molecular dynamics;
+  !! * calc  = 'mm'   : structural minimization by damped dynamics;
+  !! * calc  = 'cd'   : Parrinello-Rahman cell dynamics;
+  !! * calc  = 'cm'   : Parrinello-Rahman cell minimization by damped dynamics;
+  !! * calc  = 'nd'   : Wentzcovitch's new cell dynamics;
+  !! * calc  = 'nm'   : Wentzcovitch's new cell minimization by damped dynamics.
   !
-  !  calc  = 'md'   : standard molecular dynamics
-  !  calc  = 'mm'   : structural minimization by damped dynamics
-  !  calc  = 'cd'   : Parrinello-Rahman cell dynamics
-  !  calc  = 'cm'   : Parrinello-Rahman cell minimization by damped dynamics
-  !  calc  = 'nd'   : Wentzcovitch's new cell dynamics
-  !  calc  = 'nm'   : Wentzcovitch's new cell minimization by damped dynamics
+  !! Dynamics performed using Beeman algorithm, J. Comp. Phys. 20, 130 (1976).
   !
-  ! Dynamics performed using Beeman algorithm, J. Comp. Phys. 20, 130 (1976))
+  !! Contraints with vcsmd have been implemented by Vivek Ranjan in 2012
+  !! from the Department of Physics, North Carolina State University
+  !! Raleigh, North Carolina, USA.
   !
-  ! Contraints with vcsmd have been implemented by Vivek Ranjan in 2012
-  ! from the Department of Physics, North Carolina State University
-  ! Raleigh, North Carolina, USA
+  USE kinds,               ONLY : DP
+  USE io_global,           ONLY : stdout
+  USE constants,           ONLY : e2, ry_kbar, amu_ry, au_ps
+  USE cell_base,           ONLY : wmass, omega, alat, at, bg, iforceh, &
+       press, fix_volume, fix_area
+  USE ions_base,           ONLY : tau, nat, ntyp => nsp, ityp, atm, if_pos
+  USE cellmd,              ONLY : nzero, ntimes, calc, at_old, omega_old, &
+                                  ntcheck, lmovecell
+  USE dynamics_module,     ONLY : dt, temperature
+  USE ions_base,           ONLY : amass, if_pos 
+  USE relax,               ONLY : epse, epsf, epsp
+  USE force_mod,           ONLY : force, sigma
+  USE control_flags,       ONLY : nstep, istep, tolp, lconstrain
+  USE parameters,          ONLY : ntypx
+  USE ener,                ONLY : etot
+  USE io_files,            ONLY : prefix, delete_if_present, seqopn
   !
-  USE kinds,           ONLY : DP
-  USE io_global,       ONLY : stdout
-  USE constants,       ONLY : e2, ry_kbar, amu_ry
-  USE cell_base,       ONLY : omega, alat, at, bg, iforceh, fix_volume, fix_area
-  USE ions_base,       ONLY : tau, nat, ntyp => nsp, ityp, atm, if_pos
-  USE cellmd,          ONLY : nzero, ntimes, calc, press, at_old, omega_old, &
-                              cmass, ntcheck, lmovecell
-  USE dynamics_module, ONLY : dt, temperature
-  USE ions_base,       ONLY : amass, if_pos 
-  USE relax,           ONLY : epse, epsf, epsp
-  USE force_mod,       ONLY : force, sigma
-  USE control_flags,   ONLY : nstep, istep, tolp, lconstrain
-  USE parameters,      ONLY : ntypx
-  USE ener,            ONLY : etot
-  USE io_files,        ONLY : prefix, delete_if_present, seqopn
-
-  USE constraints_module, ONLY : nconstr
-  USE constraints_module, ONLY : remove_constr_force, check_constraint  
+  USE constraints_module,  ONLY : nconstr
+  USE constraints_module,  ONLY : remove_constr_force, check_constraint  
   !
   !
   IMPLICIT NONE
-  LOGICAL, INTENT (OUT) :: conv_ions
+  !
+  LOGICAL, INTENT(OUT) :: conv_ions
+  !! .TRUE. if convergence is achieved
   !
   ! ... I/O variable first
   !
@@ -62,7 +64,6 @@ SUBROUTINE vcsmd( conv_ions )
   !  at (icar,ivec) = direct Bravais lattice vectors
   !  bg (icar,ivec) = reciprocal lattice vectors
   !  amass_(nt) = mass (in atomic ryd units) for atom of nt-th type
-  !  cmass = cell mass in ryd units.
   !  press = target pressure in ryd/(a.u.)^3
   !
   ! ... local variables
@@ -73,6 +74,7 @@ SUBROUTINE vcsmd( conv_ions )
 #endif
   !
   REAL(DP) :: p,            & ! virial pressure
+                   cmass,   & !  cell mass in Ry units.
                    vcell,        & ! cell volume
                    avec(3,3),    & ! at(3,3) * alat
                    aveci(3,3),   & ! avec at t-dt
@@ -100,7 +102,7 @@ SUBROUTINE vcsmd( conv_ions )
            sigmamet(3,3),        & ! sigma = avec^-1 * vcell = bg/alat*omega
            vx2(ntypx), vy2(ntypx), vz2(ntypx),     & ! work vectors
            vmean(ntypx), rms(ntypx), ekin(ntypx),  & ! work vectors
-           tempo, time_au
+           tempo
   CHARACTER(LEN=3) :: ios          ! status (old or new) for I/O files
   CHARACTER(LEN=6) :: ipos         ! status ('append' or 'asis') for I/O files
   CHARACTER(LEN=80):: calc_long    ! Verbose description of type of calculation
@@ -234,9 +236,7 @@ SUBROUTINE vcsmd( conv_ions )
   ! 
   tauold(:,:,1) = tau(:,:)
   !
-  time_au = 0.0000242d0 * e2
-  !
-  tempo = ( istep - 1 ) * dt * time_au
+  tempo = ( istep - 1 ) * dt * e2*au_ps
   !
   IF ( istep == 1 .AND. ( calc(2:2) == 'm' ) ) THEN
         WRITE( stdout,'(/5X,A,/,5x,"convergence thresholds EPSE =",ES9.2, &
@@ -256,6 +256,8 @@ SUBROUTINE vcsmd( conv_ions )
      !
   END IF
   !
+  cmass  = wmass * amu_ry
+  !
   ! ... save cell shape of previous step
   !
   at_old = at
@@ -320,12 +322,12 @@ SUBROUTINE vcsmd( conv_ions )
      !
      IF ( istep == 1 ) THEN
         !
-        CALL delete_if_present( 'e' )
-        CALL delete_if_present( 'eal' )
-        CALL delete_if_present( 'ave' )
-        CALL delete_if_present( 'p' )
-        CALL delete_if_present( 'avec' )
-        CALL delete_if_present( 'tv' )
+        CALL delete_if_present( TRIM( prefix ) // '.e' )
+        CALL delete_if_present( TRIM( prefix ) // '.eal' )
+        CALL delete_if_present( TRIM( prefix ) // '.ave' )
+        CALL delete_if_present( TRIM( prefix ) // '.p' )
+        CALL delete_if_present( TRIM( prefix ) // '.avec' )
+        CALL delete_if_present( TRIM( prefix ) // '.tv' )
         !
         ios  = 'NEW'
         ipos = 'ASIS'
@@ -337,17 +339,17 @@ SUBROUTINE vcsmd( conv_ions )
         !
      END IF
      !
-     OPEN( UNIT = iun_e,    FILE = 'e',    STATUS = ios, &
+     OPEN( UNIT = iun_e,    FILE = TRIM( prefix ) // '.e',    STATUS = ios, &
            FORM = 'FORMATTED', POSITION = ipos )
-     OPEN( UNIT = iun_eal,  FILE = 'eal',  STATUS = ios, &
+     OPEN( UNIT = iun_eal,  FILE = TRIM( prefix ) // '.eal',  STATUS = ios, &
            FORM = 'FORMATTED', POSITION = ipos )
-     OPEN( UNIT = iun_ave,  FILE = 'ave',  STATUS = ios, &
+     OPEN( UNIT = iun_ave,  FILE = TRIM( prefix ) // '.ave',  STATUS = ios, &
            FORM = 'FORMATTED', POSITION = ipos )
-     OPEN( UNIT = iun_p,    FILE = 'p',    STATUS = ios, &
+     OPEN( UNIT = iun_p,    FILE = TRIM( prefix ) // '.p',    STATUS = ios, &
            FORM = 'FORMATTED', POSITION = ipos )
-     OPEN( UNIT = iun_avec, FILE = 'avec', STATUS = ios, &
+     OPEN( UNIT = iun_avec, FILE = TRIM( prefix ) // '.avec', STATUS = ios, &
            FORM = 'FORMATTED', POSITION = ipos )
-     OPEN( UNIT = iun_tv,   FILE = 'tv',   STATUS = ios, &
+     OPEN( UNIT = iun_tv,   FILE = TRIM( prefix ) // '.tv',   STATUS = ios, &
            FORM = 'FORMATTED', POSITION = ipos )
      !
      nst = istep - 1
@@ -402,7 +404,7 @@ SUBROUTINE vcsmd( conv_ions )
   !
   WRITE( stdout,'(A3,3X,3F14.9)') ( atm(ityp(na)), tau(:,na), na = 1, nat )
   WRITE( stdout, '(/5X,"Ekin = ",F14.8," Ry    T = ",F6.1," K ", &
-       &       " Etot = ",F14.8)') ekint, tnew, edyn + e_start
+       &       " Etot = ",0PF17.8)') ekint, tnew, edyn + e_start
   !
   CALL cryst_to_cart( nat, force, at, 1 )
   force = force*alat
