@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2003-2010 Quantum ESPRESSO group
+! Copyright (C) 2003-2018 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -58,17 +58,15 @@ PROGRAM ev
 !!
       USE kinds, ONLY: DP
       USE constants, ONLY: bohr_radius_angs, ry_kbar
-      USE ev_xml,    ONLY : write_evdata_xml
       USE mp_global, ONLY : mp_startup, mp_global_end
       USE mp_world,  ONLY : world_comm
       USE mp,        ONLY : mp_bcast
       USE io_global, ONLY : ionode, ionode_id
       IMPLICIT NONE
-      INTEGER, PARAMETER:: nmaxpar=4, nmaxpt=100, nseek=10000, nmin=4
+      INTEGER, PARAMETER:: nmaxpar=4, nmaxpt=100 
       INTEGER :: npar,npt,istat, ierr
       CHARACTER :: bravais*8, au_unit*3, filin*256
-      REAL(DP) :: par(nmaxpar), deltapar(nmaxpar), parmin(nmaxpar), &
-             parmax(nmaxpar), v0(nmaxpt), etot(nmaxpt), efit(nmaxpt), &
+      REAL(DP) :: par(nmaxpar), v0(nmaxpt), etot(nmaxpt), efit(nmaxpt), &
              fac, emin, chisq, a
       REAL(DP), PARAMETER :: gpa_kbar = 10.0_dp
       LOGICAL :: in_angstrom
@@ -148,24 +146,9 @@ PROGRAM ev
       par(2) = 500.0d0
       par(3) = 5.0d0
       par(4) = -0.01d0
-!
-      parmin(1) = 0.0d0
-      parmin(2) = 0.0d0
-      parmin(3) = 1.0d0
-      parmin(4) = -1.0d0
-!
-      parmax(1) = 100000.d0
-      parmax(2) = 100000.d0
-      parmax(3) = 15.0d0
-      parmax(4) = 0.0d0
-!
-      deltapar(1) = 1.0d0
-      deltapar(2) = 100.d0
-      deltapar(3) = 1.0d0
-      deltapar(4) = 0.01d0
-!
+
       CALL find_minimum &
-           (npar,par,deltapar,parmin,parmax,nseek,nmin,chisq)
+           (npar,par,chisq)
 !
       CALL write_results &
            (npt,in_angstrom,fac,v0,etot,efit,istat,par,npar,emin,chisq, &
@@ -381,52 +364,68 @@ PROGRAM ev
  99   RETURN
     END SUBROUTINE write_results
 !
+      
+      ! This subroutine is passed to LMDIF to be minimized
+      SUBROUTINE SCHISQ(m_, n_, par_, f_, i_)
+           IMPLICIT NONE
+           INTEGER,INTENT(in)  :: m_, n_
+           INTEGER,INTENT(inout)   :: i_
+           REAL(DP),INTENT(in)    :: par_(n_)
+           REAL(DP),INTENT(out)   :: f_(m_)
+           REAL(DP) :: chisq_
+           IF(m_/=n_) THEN
+              i_ = -999
+              RETURN
+           ENDIF
+           !
+           CALL eqstate(n_,par_,chisq_)
+           f_=0._dp
+           f_(1) = chisq_
+        END SUBROUTINE
+
 !-----------------------------------------------------------------------
-      SUBROUTINE find_minimum &
-         (npar,par,deltapar,parmin,parmax,nseek,nmin,chisq)
+      SUBROUTINE find_minimum(npar,par,chisq)
 !-----------------------------------------------------------------------
 !
-!     Very Stupid Minimization
-!
-      USE random_numbers, ONLY : randy
+      USE lmdif_module, ONLY : lmdif1
       IMPLICIT NONE
-      INTEGER maxpar, nseek, npar, nmin, n,j,i
-      PARAMETER (maxpar=4)
-      REAL(DP) par(npar), deltapar(npar), parmin(npar), parmax(npar), &
-             parnew(maxpar), chisq, chinew, bidon
-!
-!      various initializations
-!
-      chisq = 1.0d30
-      chinew= 1.0d30
+      INTEGER ,INTENT(in)  :: npar
+      REAL(DP),INTENT(out) :: par(nmaxpar)
+      REAL(DP),INTENT(out) :: chisq
+      !
+      REAL(DP) :: xi(npar,npar), vchisq(npar)
+      INTEGER :: i
+      INTEGER :: lwa, iwa(npar)
+      REAL(DP),ALLOCATABLE :: wa(:)
+      !
+      xi = 0._dp
+      FORALL(i=1:npar) xi(i,i) = 1._dp
+      par(1) = v0(npt/2)
+      par(2) = 500.0d0
+      par(3) = 5.0d0
+      par(4) = -0.01d0 ! unused for some eos
+      
+      lwa = npar**2 + 6*npar
+      ALLOCATE(wa(lwa))
+      !
+      CALL lmdif1(SCHISQ, npar, npar, par, vchisq, 1.d-12, i, iwa, wa, lwa)
+      DEALLOCATE(wa)
+      !
+      IF(i>0 .and. i<5) THEN
+         PRINT*, "Minimization succeeded"
+      ELSEIF(i>=5) THEN
+         PRINT*, "Minimization stopped before convergence"
+      ELSEIF(i<=0) THEN 
+        PRINT*, "Minimization error"
+        STOP
+      ENDIF
+      !chisq = vchisq(1)
+      !
       CALL eqstate(npar,par,chisq)
-      DO j = 1,nmin
-         DO i = 1,nseek
-            DO n = 1,npar
-  10           parnew(n) = par(n) + (0.5d0 - randy())*deltapar(n)
-               IF(parnew(n)>parmax(n) .or. parnew(n)<parmin(n)) &
-               GOTO 10
-            ENDDO
-!
-            CALL eqstate(npar,parnew,chinew)
-!
-            IF(chinew<chisq) THEN
-               DO n = 1,npar
-                  par(n) = parnew(n)
-               ENDDO
-               chisq = chinew
-            ENDIF
-         ENDDO
-         DO n = 1,npar
-            deltapar(n) = deltapar(n)/10.d0
-         ENDDO
-      ENDDO
-!
-      CALL eqstate(npar,par,chisq)
-!
-      RETURN
-    END SUBROUTINE find_minimum
-!
+
+      END SUBROUTINE
+!-----------------------------------------------------------------------
+
   END PROGRAM ev
 
       FUNCTION birch(x,k0,dk0,d2k0)
@@ -460,3 +459,119 @@ PROGRAM ev
 
       RETURN
     END FUNCTION keane
+!-----------------------------------------------------------------------
+    SUBROUTINE write_evdata_xml &
+        (npt,fac,v0,etot,efit,istat,par,npar,emin,chisq,filout, ierr)
+!-----------------------------------------------------------------------
+!
+  USE kinds,     ONLY : dp
+  USE constants, ONLY : ry_kbar, bohr_radius_angs
+  IMPLICIT NONE
+  INTEGER, INTENT(in) :: npt, istat, npar
+  REAL(DP), INTENT(in):: v0(npt), etot(npt), efit(npt), emin, chisq, fac
+  REAL(DP), INTENT(in):: par(npar)
+  CHARACTER(len=256), INTENT(IN) :: filout
+  INTEGER, INTENT(out) :: ierr
+  !
+  INTEGER :: iunout = 11
+  REAL(DP) :: p(npt), volume(2), a0(2), alldata(6,npt)
+  INTEGER :: i, iun
+  CHARACTER(len=256) :: filename
+  REAL(DP), EXTERNAL :: birch, keane
+
+  IF (filout/=' ') THEN
+     filename = TRIM(filout) // '.xml'
+  ELSE
+     filename = 'ev.xml'
+  ENDIF
+  !
+  ! ... open XML descriptor
+  !
+  OPEN ( UNIT=iunout, FILE = TRIM( filename ), FORM='formatted', IOSTAT = ierr )
+  IF ( ierr /= 0 ) THEN
+     WRITE (6,*) 'Failed opening file ' // TRIM(filename)
+     RETURN
+  END IF
+
+  WRITE (iunout,'("<xml>")')
+  WRITE (iunout,'("<EQUATIONS_OF_STATE>")')
+  WRITE (iunout,'("<EQUATION_TYPE>")')
+  IF (istat==1) THEN
+     WRITE (iunout,'("Birch 1st order")')
+  ELSEIF (istat==2) THEN
+     WRITE (iunout,'("Birch 2nd order")')
+  ELSEIF (istat==3) THEN
+     WRITE (iunout,'("Keane")')
+  ELSEIF (istat==4) THEN
+     WRITE (iunout,'("Murnaghan")')
+  ENDIF
+  WRITE (iunout,'("</EQUATION_TYPE>")')
+  WRITE (iunout,'("<CHI_SQUARE>")')
+  WRITE (iunout,'(1pe25.12)') chisq
+  WRITE (iunout,'("</CHI_SQUARE>")')
+  WRITE (iunout,'("</EQUATIONS_OF_STATE>")')
+
+  IF (istat==1 .or. istat==2) THEN
+     DO i=1,npt
+        p(i)=birch(v0(i)/par(1),par(2),par(3),par(4))
+     ENDDO
+  ELSE
+     DO i=1,npt
+        p(i)=keane(v0(i)/par(1),par(2),par(3),par(4))
+     ENDDO
+  ENDIF
+
+  DO i=1,npt
+     alldata (1,i) = v0(i)
+     alldata (2,i) = etot(i) 
+     alldata (3,i) = efit(i)
+     alldata (4,i) = etot(i) - efit(i)
+     alldata (5,i) = p(i) 
+     alldata (6,i) = etot(i) + p(i) * v0(i) / ry_kbar
+  ENDDO
+
+  WRITE (iunout,'("<EQUATIONS_PARAMETERS>")')
+
+  volume(1)=par(1)
+  volume(2)=par(1)*bohr_radius_angs**3
+  WRITE (iunout, '("<EQUILIBRIUM_VOLUME_AU_A>")')
+  WRITE (iunout, '(2(1pe25.15))') volume(:)
+  WRITE (iunout, '("</EQUILIBRIUM_VOLUME_AU_A>")')
+  WRITE (iunout, '("<BULK_MODULUS_KBAR>")')
+  WRITE (iunout, '(1pe25.15)') par(2)
+  WRITE (iunout, '("</BULK_MODULUS_KBAR>")')
+  WRITE (iunout, '("<DERIVATIVE_BULK_MODULUS>")')
+  WRITE (iunout, '(1pe25.15)') par(3)
+  WRITE (iunout, '("</DERIVATIVE_BULK_MODULUS>")')
+  WRITE (iunout, '("<SECOND_DERIVATIVE_BULK_MODULUS>")')
+  WRITE (iunout, '(1pe25.15)') par(4)
+  WRITE (iunout, '("</SECOND_DERIVATIVE_BULK_MODULUS>")')
+  WRITE (iunout, '("<MINIMUM_ENERGY_RY>")')
+  WRITE (iunout, '(1pe25.15)') emin
+  WRITE (iunout, '("</MINIMUM_ENERGY_RY>")')
+  WRITE (iunout, '("<CELL_FACTOR>")')
+  WRITE (iunout, '(1pe25.15)') fac
+  WRITE (iunout, '("</CELL_FACTOR>")')
+  IF (fac /= 0.0_DP) THEN
+     a0(1) = (par(1)/fac)**(1d0/3d0)
+     a0(2) = (par(1)/fac)**(1d0/3d0) * bohr_radius_angs
+     WRITE (iunout, '("<CELL_PARAMETER_AU_A>")')
+     WRITE (iunout, '(2(1pe25.15))') a0
+     WRITE (iunout, '("</CELL_PARAMETER_AU_A>")')
+  ENDIF
+  WRITE (iunout,'("</EQUATIONS_PARAMETERS>")')
+
+  WRITE (iunout,'("<FIT_CHECK>")')
+  WRITE (iunout,'("<NUMBER_OF_DATA>")')
+  WRITE (iunout,'(i8)') npt
+  WRITE (iunout,'("</NUMBER_OF_DATA>")')
+  WRITE (iunout,'("<VOL_ENE_EFIT_DELTA_P_GIBBS>")')
+  WRITE (iunout,'(6(1pe25.15))') alldata(:,:)
+  WRITE (iunout,'("</VOL_ENE_EFIT_DELTA_P_GIBBS>")')
+
+  WRITE (iunout,'("</FIT_CHECK>")')
+  CLOSE (unit=iunout, status='keep')
+  
+  RETURN
+END SUBROUTINE write_evdata_xml
+
