@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2016 Quantum ESPRESSO group
+! Copyright (C) 2001-2019 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -24,7 +24,7 @@ SUBROUTINE lr_apply_liouvillian_eels ( evc1, evc1_new, interaction )
   USE wvfct,                ONLY : nbnd, npwx, et, current_k
   USE uspp,                 ONLY : vkb
   USE io_files,             ONLY : iunwfc, nwordwfc
-  USE wavefunctions_module, ONLY : evc, psic, psic_nc
+  USE wavefunctions, ONLY : evc, psic, psic_nc
   USE noncollin_module,     ONLY : noncolin, npol, nspin_mag
   USE uspp,                 ONLY : okvan
   USE mp_bands,             ONLY : ntask_groups, me_bgrp
@@ -121,6 +121,12 @@ SUBROUTINE lr_apply_liouvillian_eels ( evc1, evc1_new, interaction )
      !
      CALL dv_of_drho (dvrsc, .false.)
      !
+     ! USPP: Compute the integral of the HXC response potential with the Q function.
+     ! Input : dvrsc = V_HXC(r), Output: int3 = \int V_HXC(r) * Q_nm(r) dr 
+     ! See Eq.(B22) in Ref. A. Dal Corso, PRB 64, 235118 (2001)
+     !
+     IF (okvan) CALL newdq(dvrsc, 1)
+     !
      ! Interpolation of the HXC potential from the thick mesh 
      ! to a smoother mesh (if doublegrid=.true.)
      ! dvrsc -> dvrssc
@@ -225,20 +231,9 @@ SUBROUTINE lr_apply_liouvillian_eels ( evc1, evc1_new, interaction )
         ENDDO
         !
         ! In the case of US pseudopotentials there is an additional term.
-        ! See second term in Eq.(11) in J. Chem. Phys. 127, 164106 (2007)
+        ! See the second term in Eq.(11) in J. Chem. Phys. 127, 164106 (2007).
         !
-        IF (okvan) THEN
-           !
-           ! Compute the integral of the HXC response potential with the Q function.
-           ! Input : dvrsc = V_HXC(r)
-           ! Output: int3 = \int V_HXC(r) * Q_nm(r) dr 
-           ! See Eq.(B22) in Ref. A. Dal Corso, PRB 64, 235118 (2001)
-           !
-           CALL newdq(dvrsc, 1)
-           !
-           CALL adddvscf(1, ik) 
-           !
-        ENDIF
+        IF (okvan) CALL adddvscf(1, ik) 
         !
         ! Ortogonalize dvpsi to valence states.
         ! Apply -P_c^+, and then change the sign, because we need P_c^+.
@@ -296,8 +291,8 @@ SUBROUTINE lr_apply_liouvillian_eels ( evc1, evc1_new, interaction )
      !    evc1_new = S^{-1} * sevc1_new
      !    If not ultrasoft: evc1_new = sevc1_new
      !
-     CALL lr_sm1_psiq (.FALSE., ik, npwx, npwq, nbnd_occ(ikk), &
-                               & sevc1_new(1,1,ik), evc1_new(1,1,ik))
+     CALL lr_sm1_psi(ik, npwx, npwq, nbnd_occ(ikk), &
+                          & sevc1_new(1,1,ik), evc1_new(1,1,ik))
      !
   ENDDO ! loop on ik
   !
