@@ -29,7 +29,7 @@ SUBROUTINE lr_calc_dens_eels_nc (drhoscf, dpsi)
   USE wvfct,                 ONLY : nbnd, npwx
   USE gvecw,                 ONLY : gcutw
   USE qpoint,                ONLY : nksq, ikks, ikqs
-  USE wavefunctions_module,  ONLY : evc
+  USE wavefunctions,  ONLY : evc
   USE noncollin_module,      ONLY : npol, nspin_mag
   USE uspp_param,            ONLY : nhm
   USE uspp,                  ONLY : okvan, vkb
@@ -89,7 +89,7 @@ SUBROUTINE lr_calc_dens_eels_nc (drhoscf, dpsi)
      !
      ! Calculation of the response charge density
      !
-     CALL incdrhoscf_nc(drhoscfh(:,:), weight, ik, dbecsum_nc(:,:,:,:), dpsi(:,:,ik))
+     CALL incdrhoscf_nc(drhoscfh(:,:), weight, ik, dbecsum_nc(:,:,:,:), dpsi(:,:,ik),1.0d0)
      !
   ENDDO 
   !
