@@ -31,15 +31,15 @@ MODULE lr_exx_kernel
   USE fft_interfaces,         ONLY : invfft, fwfft
   USE lsda_mod,               ONLY : nspin
   USE wvfct,                  ONLY : nbnd, npwx, wg
-  USE gvecs,                  ONLY : nls, nlsm
   USE gvect,                  ONLY : g, ngm
   USE klist,                  ONLY : xk, wk, nks
   USE lr_variables,           ONLY : gamma_only, lr_verbosity
   USE realus,                 ONLY : invfft_orbital_gamma, fwfft_orbital_gamma,&
                                    & invfft_orbital_k, fwfft_orbital_k
-  USE wavefunctions_module,   ONLY : psic
+  USE wavefunctions,          ONLY : psic
   USE cell_base,              ONLY : omega
-  USE exx,                    ONLY : exxalfa, g2_convolution, exx_fft
+  USE exx_base,               ONLY : g2_convolution
+  USE exx,                    ONLY : exxalfa, npwt, gt, dfftt 
 
 
   REAL(kind=dp),    PUBLIC, ALLOCATABLE :: revc_int(:,:)
@@ -61,16 +61,55 @@ MODULE lr_exx_kernel
   INTEGER, PRIVATE, ALLOCATABLE :: k2q(:)
 
 CONTAINS
-SUBROUTINE lr_exx_alloc()
+  !------------------------------------------------------------------------
+  SUBROUTINE lr_exx_restart( set_ace )
+     !------------------------------------------------------------------------
+     !This SUBROUTINE is called when restarting an exx calculation
+     USE funct,     ONLY : get_exx_fraction, start_exx, &
+                           exx_is_active, get_screening_parameter
+     USE cell_base, ONLY : at
+     USE exx_base,  ONLY : exxdiv, erfc_scrlen, exx_divergence, exx_grid_init,&
+                           exx_div_check
+     ! FIXME: are these variable useful?
+     USE exx,       ONLY : fock0, exxenergy2, local_thr, use_ace
+     USE exx,       ONLY : exxinit, aceinit, exx_gvec_reinit 
 
-  USE exx,         ONLY : nkqs
+     IMPLICIT NONE
+     LOGICAL, INTENT(in) :: set_ace
+     !
+     CALL exx_grid_init( reinit=.true. )
+     CALL exx_gvec_reinit( at )
+     CALL exx_div_check()
+     !
+     use_ace = set_ace
+     erfc_scrlen = get_screening_parameter()
+     
+     exxdiv = exx_divergence()
+     exxalfa = get_exx_fraction()
+     CALL start_exx()
+     CALL weights()
+     ! FIXME: is this useful ?
+     IF(local_thr.gt.0.0d0) CALL errore('exx_restart','SCDM with restart NYI',1)
+     CALL exxinit(.false.)
+     IF (use_ace) CALL aceinit ( DOLOC = .FALSE. )
+     ! FIXME: are these variable useful?
+     fock0 = exxenergy2()
+     !
+     RETURN
+     !------------------------------------------------------------------------
+   END SUBROUTINE lr_exx_restart
+  !------------------------------------------------------------------------
+
+  SUBROUTINE lr_exx_alloc()
+
+  USE exx_base,    ONLY : nkqs
   USE klist,       ONLY : nks
   
   IMPLICIT NONE
   INTEGER :: nrxxs
 
   IF (gamma_only) THEN
-     nrxxs= exx_fft%dfftt%nnr
+     nrxxs= dfftt%nnr
   ELSE
      nrxxs= dffts%nnr
   ENDIF
@@ -144,7 +183,7 @@ SUBROUTINE lr_exx_apply_revc_int(psi, ibnd, nbnd, ik)
 
   CALL start_clock('lr_exx_apply')
 
-  nrxxs= exx_fft%dfftt%nnr
+  nrxxs= dfftt%nnr
 
   IF (gamma_only) THEN
      !
@@ -165,22 +204,22 @@ SUBROUTINE lr_exx_apply_revc_int(psi, ibnd, nbnd, ik)
         !
         ! To g-space
         !
-        CALL fwfft ('CustomWave', tempphic(:,1), exx_fft%dfftt)
+        CALL fwfft ('Wave', tempphic(:,1), dfftt)
         !
         ! Now separate the two bands and apply the correct nl mapping
         !
         DO j = 1, npw
-           fp = (tempphic(exx_fft%nlt(j),1) + &
-                 tempphic(exx_fft%nltm(j),1))*0.5d0 
-           fm = (tempphic(exx_fft%nlt(j),1) - &
-                 tempphic(exx_fft%nltm(j),1))*0.5d0 
+           fp = (tempphic(dfftt%nl(j),1) + &
+                 tempphic(dfftt%nlm(j),1))*0.5d0 
+           fm = (tempphic(dfftt%nl(j),1) - &
+                 tempphic(dfftt%nlm(j),1))*0.5d0 
            temppsic( j, 1) = CMPLX( DBLE(fp), AIMAG(fm),kind=DP)
            temppsic( j, 2) = CMPLX(AIMAG(fp),- DBLE(fm),kind=DP)
         ENDDO
 
-        psi_t(nls(1:npw))= temppsic(1:npw,1)+ (0.0_dp,1.0_dp)&
+        psi_t(dffts%nl(1:npw))= temppsic(1:npw,1)+ (0.0_dp,1.0_dp)&
              &*temppsic(1:npw,2) 
-        psi_t(nlsm(1:npw))= CONJG(temppsic(1:npw,1)- (0.0_dp,1.0_dp)&
+        psi_t(dffts%nlm(1:npw))= CONJG(temppsic(1:npw,1)- (0.0_dp,1.0_dp)&
              &*temppsic(1:npw,2))
      ELSE
         tempphic(1:nrxxs,1) = 0.5d0 * CMPLX( revc_int(1:nrxxs,ibnd),&
@@ -188,13 +227,13 @@ SUBROUTINE lr_exx_apply_revc_int(psi, ibnd, nbnd, ik)
         !
         ! To g-space
         !
-        CALL fwfft ('CustomWave', tempphic(:,1), exx_fft%dfftt)
+        CALL fwfft ('Wave', tempphic(:,1), dfftt)
         !
         ! Correct the nl mapping for the two grids.
         !
-        temppsic(1:npw,1)=tempphic(exx_fft%nlt(1:npw),1)
-        psi_t(nls(1:npw))=temppsic(1:npw,1)
-        psi_t(nlsm(1:npw))=CONJG(temppsic(1:npw,1))
+        temppsic(1:npw,1)=tempphic(dfftt%nl(1:npw),1)
+        psi_t(dffts%nl(1:npw))=temppsic(1:npw,1)
+        psi_t(dffts%nlm(1:npw))=CONJG(temppsic(1:npw,1))
         !
      ENDIF
      !
@@ -231,8 +270,7 @@ SUBROUTINE lr_exx_revc0_init(orbital, ik)
   !
   
   USE mp_global,    ONLY : me_bgrp
-  USE exx,          ONLY : rir, nkqs, index_sym, index_xk
-  USE exx,          ONLY : exx_fft
+  USE exx_base,     ONLY : rir, nkqs, index_sym, index_xk
   USE scatter_mod,  ONLY : gather_grid, scatter_grid
   USE symm_base,    ONLY : sname
 
@@ -246,12 +284,12 @@ SUBROUTINE lr_exx_revc0_init(orbital, ik)
 
   IF (gamma_only) THEN
      !
-     nnr_= exx_fft%dfftt%nnr
+     nnr_= dfftt%nnr
      !
      DO ibnd=1,nbnd,2
         !
-        CALL invfft_orbital_custom_gamma(orbital(:,:,1), ibnd, nbnd,&
-             & exx_fft)
+        CALL invfft_orbital_custom_gamma(orbital(:,:,1), ibnd, nbnd, &
+             npwt, dfftt)
         red_revc0(1:nnr_,ibnd,1)=psic(1:nnr_)
         !
      ENDDO
@@ -317,8 +355,9 @@ SUBROUTINE lr_exx_kernel_noint ( evc, int_vect )
 
   USE kinds,                  ONLY : DP
   USE lr_variables,           ONLY : gamma_only, lr_verbosity
-  USE exx,                    ONLY : exxalfa, g2_convolution, nqs,&
-                                   & index_sym, index_xkq, index_xk, rir, nkqs
+  USE exx_base,               ONLY : g2_convolution, nqs, index_sym, &
+                                     index_xkq, index_xk, rir, nkqs
+  USE exx,                    ONLY : exxalfa
   USE symm_base,              ONLY : s
   USE cell_base,              ONLY : bg, at
   USE funct,                  ONLY : exx_is_active
@@ -352,6 +391,10 @@ SUBROUTINE lr_exx_kernel_noint ( evc, int_vect )
   COMPLEX(DP), ALLOCATABLE :: psic_all(:), temppsic_all(:), temppsic(:)
   !
   INTEGER   :: ibnd_start_gamma, ibnd_end_gamma
+
+  LOGICAL   :: exst
+
+
   ! Offset ibnd_start and ibnd_ed to avoid conflicts with gamma_tricks for
   ! even values
   !
@@ -361,8 +404,8 @@ SUBROUTINE lr_exx_kernel_noint ( evc, int_vect )
   !
   ! Setup the variables that describe the FFT grid in use.
   IF(gamma_only) THEN
-     ALLOCATE( fac(exx_fft%ngmt) )
-     nrxxs= exx_fft%dfftt%nnr
+     ALLOCATE( fac(dfftt%ngm) )
+     nrxxs= dfftt%nnr
   ELSE
      ALLOCATE( fac(ngm) )
      nxxs=dffts%nr1x * dffts%nr2x * dffts%nr3x
@@ -384,8 +427,7 @@ SUBROUTINE lr_exx_kernel_noint ( evc, int_vect )
      ! Put the appropriate interaction in fac(). Note g2_convolution respects
      ! the choice of divergence treatment etc set in the initial PWscf run.
      !
-     CALL g2_convolution(exx_fft%ngmt, exx_fft%gt, xk(:,1),&
-          & xk(:,1), fac)
+     CALL g2_convolution(dfftt%ngm, gt, xk(:,1), xk(:,1), fac)
      !
      ALLOCATE(revc_int(nrxxs,nbnd))
      !
@@ -402,8 +444,8 @@ SUBROUTINE lr_exx_kernel_noint ( evc, int_vect )
      !
      DO ibnd=ibnd_start_gamma,ibnd_end_gamma,2
         !
-        CALL invfft_orbital_custom_gamma(evc(:,:,1), ibnd, nbnd,&
-             & exx_fft)
+        CALL invfft_orbital_custom_gamma(evc(:,:,1), ibnd, nbnd, &
+             npwt, dfftt)
         !
         w1=wg(ibnd,1)/omega
         !
@@ -413,9 +455,9 @@ SUBROUTINE lr_exx_kernel_noint ( evc, int_vect )
            w2=0.0d0
         ENDIF
         ! Update the container with the actual interaction for this band(s).
-        revc_int(1:exx_fft%dfftt%nnr,:)= revc_int(1:exx_fft%dfftt%nnr,:) & 
-             & -1.d0 * scale * k1d_term_gamma(w1,w2,psic,fac,ibnd) &
-             & +1.d0 * scale * k2d_term_gamma(w1,w2,psic,fac,ibnd)
+        revc_int(1:dfftt%nnr,:)= revc_int(1:dfftt%nnr,:) &
+             & -1.d0 * scale * k1d_term_gamma(w1,w2,psic,fac,ibnd,evc(:,:,1)) & 
+             & +1.d0 * scale * k2d_vexx_term_gamma(w1,w2,psic,fac,ibnd,.false.)
         !
      ENDDO
      !
@@ -433,8 +475,8 @@ SUBROUTINE lr_exx_kernel_noint ( evc, int_vect )
         IF (ibnd==nbnd) psic(1:nrxxs)=CMPLX(revc_int(1:nrxxs,ibnd)&
              &,0.d0,dp)
         !
-        CALL fwfft_orbital_custom_gamma (int_vect(:,:,1), ibnd, nbnd,&
-             & exx_fft) 
+        CALL fwfft_orbital_custom_gamma (int_vect(:,:,1), ibnd, nbnd, &
+             npwt, dfftt) 
         !
      ENDDO
      !
@@ -575,8 +617,9 @@ SUBROUTINE lr_exx_kernel_int ( orbital, ibnd, nbnd, ikk )
   USE kinds,                  ONLY : DP
   USE klist,                  ONLY : xk
   USE io_global,              ONLY : stdout  
-  USE exx,                    ONLY : exxalfa, g2_convolution, nqs,&
-                                   & index_sym, index_xkq, index_xk, rir, nkqs
+  USE exx,                    ONLY : exxalfa
+  USE exx_base,               ONLY : g2_convolution, nqs, index_sym, &
+                                     index_xkq, index_xk, rir, nkqs
   USE symm_base,              ONLY : s
   USE cell_base,              ONLY : bg, at
   USE funct,                  ONLY : exx_is_active
@@ -612,8 +655,8 @@ SUBROUTINE lr_exx_kernel_int ( orbital, ibnd, nbnd, ikk )
   IF (lr_verbosity > 5 ) WRITE(stdout,'("<lr_exx_kernel_int>")')
 
   IF(gamma_only) THEN
-     ALLOCATE( fac(exx_fft%ngmt) )
-     nrxxs= exx_fft%dfftt%nnr
+     ALLOCATE( fac(dfftt%ngm) )
+     nrxxs= dfftt%nnr
   ELSE
      ALLOCATE( fac(ngm) )
      nxxs=dffts%nr1x * dffts%nr2x * dffts%nr3x
@@ -631,7 +674,7 @@ SUBROUTINE lr_exx_kernel_int ( orbital, ibnd, nbnd, ikk )
   !
   IF( gamma_only ) THEN
      !
-     CALL invfft_orbital_custom_gamma( orbital, ibnd, nbnd, exx_fft )
+     CALL invfft_orbital_custom_gamma( orbital, ibnd, nbnd, npwt, dfftt )
      !
      w1=wg(ibnd,1)/omega
      !
@@ -641,13 +684,12 @@ SUBROUTINE lr_exx_kernel_int ( orbital, ibnd, nbnd, ikk )
         w2=0.0d0
      ENDIF
      !
-     CALL g2_convolution(exx_fft%ngmt, exx_fft%gt,&
-          & xk(:,1), xk(:,1), fac)
+     CALL g2_convolution(dfftt%ngm, gt,xk(:,1), xk(:,1), fac)
      !
      IF (.NOT.ltammd) THEN
-        revc_int(1:exx_fft%dfftt%nnr,:)= revc_int(1:exx_fft%dfftt%nnr,:)&
-             & -1.d0 * scale * k1d_term_gamma(w1,w2,psic,fac,ibnd) &
-             & -1.d0 * scale * k2d_term_gamma(w1,w2,psic,fac,ibnd)
+        revc_int(1:dfftt%nnr,:)= revc_int(1:dfftt%nnr,:)&
+             & -1.d0 * scale * k1d_term_gamma(w1,w2,psic,fac,ibnd,orbital) &
+             & -1.d0 * scale * k2d_vexx_term_gamma(w1,w2,psic,fac,ibnd,.true.)
      ELSE
         !
         ! Slightly different interaction in the Tamm--Dancoff case.
@@ -655,8 +697,8 @@ SUBROUTINE lr_exx_kernel_int ( orbital, ibnd, nbnd, ikk )
         ! scales the *whole interaction term* by a factor of 0.5 in the TD
         ! case.
         !
-        revc_int(1:exx_fft%dfftt%nnr,:)= revc_int(1:exx_fft%dfftt%nnr,:)&
-             & -2.d0 * scale * k1d_term_gamma(w1,w2,psic,fac,ibnd)
+        revc_int(1:dfftt%nnr,:)= revc_int(1:dfftt%nnr,:)&
+             & -2.d0 * scale * k1d_term_gamma(w1,w2,psic,fac,ibnd,orbital)
      ENDIF
      !
   ELSE
@@ -748,7 +790,7 @@ SUBROUTINE lr_exx_kernel_int ( orbital, ibnd, nbnd, ikk )
   
 END SUBROUTINE lr_exx_kernel_int
 !
-FUNCTION k1d_term_gamma(w1, w2, psi, fac_in, ibnd) RESULT (psi_int)
+FUNCTION k1d_term_gamma(w1, w2, psi, fac_in, ibnd, orbital) RESULT (psi_int)
   !------------------------------------------------------------------
   !
   !   This routine computes the  K^1d term, Eq (21) from Eq Ref (1).
@@ -759,7 +801,13 @@ FUNCTION k1d_term_gamma(w1, w2, psi, fac_in, ibnd) RESULT (psi_int)
   !   psi contains two bands of |a> with w1, w2 the associated weights
   !   fac_in contains the interaction W(G) and ibnd the band index n'.
   !
+  !   in gamma, bands are real, so is not necessary computes vhart 
+  !   integrals for each couple of bands (integral in eq(21), 
+  !   because v, v' is the same of v', v, so only nbnd(nbnd+1)/2 
+  !   couple are computed    
+  !
   !   (1)  Rocca, Lu and Galli, J. Chem. Phys., 133, 164109 (2010)
+  !
   !------------------------------------------------------------------
   !
 
@@ -770,69 +818,250 @@ FUNCTION k1d_term_gamma(w1, w2, psi, fac_in, ibnd) RESULT (psi_int)
   REAL(kind=DP), INTENT(IN)  :: w1, w2
   REAL(KIND=DP), ALLOCATABLE :: psi_int(:,:)
   INTEGER, INTENT(IN) :: ibnd
+  COMPLEX(DP), INTENT(IN) :: orbital(:,:)
+  COMPLEX(DP) :: psitemp(dfftt%nnr)
   !
   ! Workspaces
   !
   INTEGER                  :: ibnd2, is, npw_, ngm_, nnr_
+  INTEGER                  :: nrec
   !
-  npw_=exx_fft%npwt
-  ngm_=exx_fft%ngmt
-  nnr_=exx_fft%dfftt%nnr
+  npw_=npwt
+  ngm_=dfftt%ngm
+  nnr_=dfftt%nnr
   !  
   ALLOCATE(psi_int(nnr_, nbnd))
   psi_int = 0.d0
   !
   !
-  DO ibnd2=1,nbnd,1
-     !
-     !
-     ! Set up the pseudo density for this Hartree like interaction.
+  IF (ibnd < nbnd) then
      !
      vhart(:,:) = (0.d0,0.d0)
      pseudo_dens_c(:) = (0.d0,0.d0)
      !
-     ! The following code is to check if the value of ibnd2 is even or odd
-     ! and therefore whether the REAL or IMAGINARY part of red_revc0 is to be
-     ! used. This is because red_revc0 is stored using gamma_tricks.
+     ! calculate vhart for couples ibnd,ibnd and ibnd+1,ibnd
      !
-     IF (MOD(ibnd2,2)==1) THEN
-        pseudo_dens_c(1:nnr_) = CMPLX( w1*DBLE(red_revc0(1:nnr_,ibnd,1)) *&
-             & DBLE(red_revc0(1:nnr_,ibnd2,1)), &
-             & w2*AIMAG(red_revc0(1:nnr_,ibnd, 1)) *&
-             & DBLE(red_revc0(1:nnr_,ibnd2,1)), kind=DP )
-     ELSE
-        pseudo_dens_c(1:nnr_) = CMPLX( w1*DBLE(red_revc0(1:nnr_,ibnd,1)) *&
-             &AIMAG(red_revc0(1:nnr_,ibnd2-1,1)),&
-             & w2*AIMAG(red_revc0(1:nnr_,ibnd,1)) *&
-             &AIMAG(red_revc0(1:nnr_,ibnd2-1,1)), kind=DP )
-     ENDIF
+     pseudo_dens_c(1:nnr_) = CMPLX( w1*DBLE(red_revc0(1:nnr_,ibnd,1)) *&
+          & DBLE(red_revc0(1:nnr_,ibnd,1)), &
+          & w2*AIMAG(red_revc0(1:nnr_,ibnd, 1)) *&
+          & DBLE(red_revc0(1:nnr_,ibnd,1)), kind=DP )
      !
-     CALL fwfft ('Custom', pseudo_dens_c, exx_fft%dfftt)
-     !
-     ! hartree contribution is computed in reciprocal space
+     CALL fwfft ('Rho', pseudo_dens_c, dfftt)
      !
      DO is = 1, nspin
+           !
+           vhart(dfftt%nl(1:ngm_),is) =&
+                & pseudo_dens_c(dfftt%nl(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           IF (gamma_only) vhart(dfftt%nlm(1:ngm_),is) = &
+                & pseudo_dens_c(dfftt%nlm(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           !
+           !  and transformed back to real space
+           !
+           CALL invfft ('Rho', vhart (:, is), dfftt)
+     ENDDO
+     !
+     ! Finally return the interaction psi_int for terms:
+     ! ibnd,   ibnd,   ibnd 
+     ! ibnd+1, ibnd+1, ibnd 
+     ! ibnd,   ibnd,   ibnd+1
+     !
+     psi_int(1:nnr_,ibnd) = psi_int(1:nnr_,ibnd) &
+          & + DBLE(vhart(1:nnr_, 1)) * DBLE(psi(1:nnr_)) &
+          & + AIMAG(vhart(1:nnr_,1)) * AIMAG(psi(1:nnr_))
+     !
+     psi_int(1:nnr_,ibnd+1) = psi_int(1:nnr_,ibnd+1) &
+           & + AIMAG(vhart(1:nnr_,1)) * DBLE(psi(1:nnr_))
+     !
+     ! calculate vhart for couple ibnd+1,ibnd+1
+     !
+     vhart(:,:) = (0.d0,0.d0)
+     pseudo_dens_c(:) = (0.d0,0.d0)
+     !
+     pseudo_dens_c(1:nnr_) = CMPLX( w2*AIMAG(red_revc0(1:nnr_,ibnd,1)) *&
+          & AIMAG(red_revc0(1:nnr_,ibnd,1)), 0.0d0, kind=DP )
+     !
+     CALL fwfft ('Rho', pseudo_dens_c, dfftt)
+     !
+     DO is = 1, nspin
+           !
+           vhart(dfftt%nl(1:ngm_),is) =&
+                & pseudo_dens_c(dfftt%nl(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           IF (gamma_only) vhart(dfftt%nlm(1:ngm_),is) = &
+                & pseudo_dens_c(dfftt%nlm(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           !
+           !  and transformed back to real space
+           !
+           CALL invfft ('Rho', vhart (:, is), dfftt)
+     ENDDO
+     !
+     ! Finally return the interaction psi_int for term:
+     ! ibnd+1, ibnd+1, ibnd+1
+     ! 
+     psi_int(1:nnr_,ibnd+1) = psi_int(1:nnr_,ibnd+1) &
+           & + DBLE(vhart(1:nnr_,1)) * AIMAG(psi(1:nnr_))
+     !
+     !
+     ! start second loop over bands
+     !
+     DO ibnd2=1,ibnd-1,1  
         !
-        vhart(exx_fft%nlt(1:ngm_),is) =&
-             & pseudo_dens_c(exx_fft%nlt(1:ngm_)) *&
-             & fac_in(1:ngm_) 
-        IF (gamma_only) vhart(exx_fft%nltm(1:ngm_),is) = &
-             & pseudo_dens_c(exx_fft%nltm(1:ngm_)) *&
-             & fac_in(1:ngm_) 
+        ! calculate vhart for couples ibnd,ibnd2 and ibnd+1,ibnd2
         !
-        !  and transformed back to real space
+        vhart(:,:) = (0.d0,0.d0)
+        pseudo_dens_c(:) = (0.d0,0.d0)
         !
-        CALL invfft ('Custom', vhart (:, is), exx_fft%dfftt)
+        ! The following code is to check if the value of ibnd2 is even or odd
+        ! and therefore whether the REAL or IMAGINARY part of red_revc0 is to be
+        ! used. This is because red_revc0 is stored using gamma_tricks.
+        !
+        IF (MOD(ibnd2,2)==1) THEN
+           pseudo_dens_c(1:nnr_) = CMPLX( w1*DBLE(red_revc0(1:nnr_,ibnd,1)) *&
+                & DBLE(red_revc0(1:nnr_,ibnd2,1)), &
+                & w2*AIMAG(red_revc0(1:nnr_,ibnd, 1)) *&
+                & DBLE(red_revc0(1:nnr_,ibnd2,1)), kind=DP )
+        ELSE
+           pseudo_dens_c(1:nnr_) = CMPLX( w1*DBLE(red_revc0(1:nnr_,ibnd,1)) *&
+                &AIMAG(red_revc0(1:nnr_,ibnd2-1,1)),&
+                &w2*AIMAG(red_revc0(1:nnr_,ibnd,1)) *&
+                &AIMAG(red_revc0(1:nnr_,ibnd2-1,1)), kind=DP )
+        ENDIF
+        !
+        CALL fwfft ('Rho', pseudo_dens_c, dfftt)
+        !
+        ! hartree contribution is computed in reciprocal space
+        !
+        DO is = 1, nspin
+           !
+           vhart(dfftt%nl(1:ngm_),is) =&
+                & pseudo_dens_c(dfftt%nl(1:ngm_)) *&
+                & fac_in(1:ngm_) 
+           IF (gamma_only) vhart(dfftt%nlm(1:ngm_),is) = &
+                & pseudo_dens_c(dfftt%nlm(1:ngm_)) *&
+                & fac_in(1:ngm_) 
+           !
+           !  and transformed back to real space
+           !
+           CALL invfft ('Rho', vhart (:, is), dfftt)
+           !
+        ENDDO
+        !
+        ! Finally return the interaction psi_int for terms:
+        ! ibnd,   ibnd,   ibnd2
+        ! ibnd+1, ibnd+1, ibnd2
+        ! ibnd2,  ibnd2,  ibnd
+        ! ibnd2,  ibnd2,  ibnd+1
+        !
+        psi_int(1:nnr_,ibnd2) = psi_int(1:nnr_,ibnd2) &
+             & + DBLE(vhart(1:nnr_, 1)) * DBLE(psi(1:nnr_)) &
+             & + AIMAG(vhart(1:nnr_,1)) * AIMAG(psi(1:nnr_))
+        !
+        CALL invfft_orbital_ibnd2_gamma(orbital(:,:), psitemp, ibnd2, npw_, dfftt)
+        !
+        !
+        psi_int(1:nnr_,ibnd) = psi_int(1:nnr_,ibnd) &
+             & + DBLE(vhart(1:nnr_, 1)) * DBLE(psitemp(1:nnr_))
+        !
+        psi_int(1:nnr_,ibnd+1) = psi_int(1:nnr_,ibnd+1) &
+             & + AIMAG(vhart(1:nnr_, 1)) * DBLE(psitemp(1:nnr_))
         !
      ENDDO
      !
-     ! Finally return the interaction
+  ELSE
      !
-     psi_int(1:nnr_,ibnd2) = psi_int(1:nnr_,ibnd2) &
-          & +DBLE(vhart(1:nnr_, 1)) * DBLE(psi(1:nnr_)) &
-          & +AIMAG(vhart(1:nnr_,1)) * AIMAG(psi(1:nnr_))
-  ENDDO
+     ! calculate vhart for couple ibnd,ibnd 
+     !
+     vhart(:,:) = (0.d0,0.d0)
+     pseudo_dens_c(:) = (0.d0,0.d0)
+     !
+     pseudo_dens_c(1:nnr_) = CMPLX( w1*DBLE(red_revc0(1:nnr_,ibnd,1)) *&
+          & DBLE(red_revc0(1:nnr_,ibnd,1)), 0.0d0, kind=DP )
+     !
+     CALL fwfft ('Rho', pseudo_dens_c, dfftt)
+     !
+     DO is = 1, nspin
+           !
+           vhart(dfftt%nl(1:ngm_),is) =&
+                & pseudo_dens_c(dfftt%nl(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           IF (gamma_only) vhart(dfftt%nlm(1:ngm_),is) = &
+                & pseudo_dens_c(dfftt%nlm(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           !
+           !  and transformed back to real space
+           !
+           CALL invfft ('Rho', vhart (:, is), dfftt)
+     ENDDO
+     !
+     ! Finally return the interaction psi_int for term:
+     ! ibnd,   ibnd,   ibnd
+     ! 
+     psi_int(1:nnr_,ibnd) = psi_int(1:nnr_,ibnd) &
+           & + DBLE(vhart(1:nnr_,1)) * DBLE(psi(1:nnr_))
+     !
+     ! start second loop over bands
+     !
+     DO ibnd2=1,ibnd-1,1
+        !
+        !
+        ! Set up the pseudo density for this Hartree like interaction.
+        ! calculate vhart for couple ibnd,ibnd2
+        !
+        vhart(:,:) = (0.d0,0.d0)
+        pseudo_dens_c(:) = (0.d0,0.d0)
+        !
+        ! The following code is to check if the value of ibnd2 is even or odd
+        ! and therefore whether the REAL or IMAGINARY part of red_revc0 is to be
+        ! used. This is because red_revc0 is stored using gamma_tricks.
+        !
+        IF (MOD(ibnd2,2)==1) THEN
+           pseudo_dens_c(1:nnr_) = CMPLX( w1*DBLE(red_revc0(1:nnr_,ibnd,1)) *&
+                & DBLE(red_revc0(1:nnr_,ibnd2,1)), 0.0d0, kind=DP )
+        ELSE
+           pseudo_dens_c(1:nnr_) = CMPLX( w1*DBLE(red_revc0(1:nnr_,ibnd,1)) *&
+                & AIMAG(red_revc0(1:nnr_,ibnd2-1,1)), 0.0d0, kind=DP )
+        ENDIF
+        !
+        CALL fwfft ('Rho', pseudo_dens_c, dfftt)
+        !
+        ! hartree contribution is computed in reciprocal space
+        !
+        DO is = 1, nspin
+           !
+           vhart(dfftt%nl(1:ngm_),is) =&
+                & pseudo_dens_c(dfftt%nl(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           IF (gamma_only) vhart(dfftt%nlm(1:ngm_),is) = &
+                & pseudo_dens_c(dfftt%nlm(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           !
+           !  and transformed back to real space
+           !
+           CALL invfft ('Rho', vhart (:, is), dfftt)
+           !
+        ENDDO
+        !
+        ! Finally return the interaction psi_int for terms:
+        ! ibnd, ibnd,  ibnd2
+        ! ibn2, ibnd2, ibnd
+        !
+        psi_int(1:nnr_,ibnd2) = psi_int(1:nnr_,ibnd2) &
+             & + DBLE(vhart(1:nnr_, 1)) * DBLE(psi(1:nnr_))
+        !
+        CALL invfft_orbital_ibnd2_gamma(orbital(:,:), psitemp, ibnd2, npw_, dfftt)
+        !
+        !
+        psi_int(1:nnr_,ibnd) = psi_int(1:nnr_,ibnd) &
+             & + DBLE(vhart(1:nnr_, 1)) * DBLE(psitemp(1:nnr_))
+        !
+     ENDDO
+     !
+  ENDIF
   !
+  RETURN
   !
 END FUNCTION k1d_term_gamma
 !
@@ -877,18 +1106,18 @@ FUNCTION k1d_term_k(w1, psi, fac_in, ibnd, ik,ikq) RESULT (psi_int)
      pseudo_dens_c(:) = CONJG(red_revc0(:,ibnd,ikq))*&
           &red_revc0(:,ibnd2,k2q(ik))/omega
      !
-     CALL fwfft ('Smooth', pseudo_dens_c, dffts)
+     CALL fwfft ('Rho', pseudo_dens_c, dffts)
      !
      ! hartree contribution is computed in reciprocal space
      !
      DO is = 1, nspin
         !
-        vhart(nls(1:ngm),is) = w1*pseudo_dens_c(nls(1:ngm)) *&
+        vhart(dffts%nl(1:ngm),is) = w1*pseudo_dens_c(dffts%nl(1:ngm)) *&
              & fac_in(1:ngm) 
         !
         !  and transformed back to real space
         !
-        CALL invfft ('Smooth', vhart (:, is), dffts)
+        CALL invfft ('Rho', vhart (:, is), dffts)
         !
      ENDDO
      !
@@ -900,14 +1129,20 @@ FUNCTION k1d_term_k(w1, psi, fac_in, ibnd, ik,ikq) RESULT (psi_int)
   !
 END FUNCTION k1d_term_k
 !
-FUNCTION k2d_term_gamma(w1, w2, psi, fac_in, ibnd) RESULT (psi_int)
+FUNCTION k2d_vexx_term_gamma(w1, w2, psi, fac_in, ibnd, interaction) RESULT (psi_int)
   !------------------------------------------------------------------
   !
   !   This routine computes the  K^2d term, Eq (22) from Eq Ref (1).
   !   psi contains two bands of |b> with w1, w2 the associated weights
   !   fac_in contains the interaction W(G) and ibnd the band index n'.
   !
+  !   Also computes the Vexx term, Eq (15) from Eq Ref (2). 
+  !   So in gamma calculations, h_psi soubrite doesn't call anymore 
+  !   vexx routine 
+  !
   !   (1)  Rocca, Lu and Galli, J. Chem. Phys., 133, 164109 (2010)
+  !   (2)  Ge, Binnie, Rocca, Gebauer, Baroni, Comp. Phys. Comm., 
+  !        185, 2080 (2014)
   !------------------------------------------------------------------
   !
 
@@ -918,14 +1153,15 @@ FUNCTION k2d_term_gamma(w1, w2, psi, fac_in, ibnd) RESULT (psi_int)
   REAL(kind=DP), INTENT(IN)  :: w1, w2
   INTEGER, INTENT(IN) :: ibnd
   REAL(KIND=DP), ALLOCATABLE :: psi_int(:,:)
+  LOGICAL, INTENT(IN) :: interaction
   !
   ! Workspaces
   !
   INTEGER                  :: ibnd2, is, npw_,ngm_, nnr_
   !
-  nnr_ = exx_fft%dfftt%nnr
-  npw_ = exx_fft%npwt
-  ngm_ = exx_fft%ngmt
+  nnr_ = dfftt%nnr
+  ngm_ = dfftt%ngm
+  npw_ = npwt
   !
   ALLOCATE(psi_int(nnr_, nbnd))
   psi_int = 0.d0
@@ -946,41 +1182,103 @@ FUNCTION k2d_term_gamma(w1, w2, psi, fac_in, ibnd) RESULT (psi_int)
         pseudo_dens_c(1:nnr_) = CMPLX( &
              & w1*DBLE(psi(1:nnr_))*DBLE(red_revc0(1:nnr_,ibnd2,1)),&
              & w2*AIMAG(psi(1:nnr_))*DBLE(red_revc0(1:nnr_,ibnd2,1)), kind=DP )
+        !
+        CALL fwfft ('Rho', pseudo_dens_c, dfftt)
+        !
+        ! hartree contribution is computed in reciprocal space
+        !
+        DO is = 1, nspin
+           !
+           vhart(dfftt%nl(1:ngm_),is) = pseudo_dens_c(dfftt%nl(1:ngm_)) * fac_in(1:ngm_)
+           IF (gamma_only) vhart(dfftt%nlm(1:ngm_),is) = &
+                & pseudo_dens_c(dfftt%nlm(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           !
+           !  and transformed back to real space
+           !
+           CALL invfft ('Rho', vhart (:, is), dfftt)
+           !
+        ENDDO
+        !
+        !
+        ! Finally return the interaction
+        !
+        psi_int(1:nnr_,ibnd2) = psi_int(1:nnr_,ibnd2) &
+             & +DBLE(vhart(1:nnr_,1)) * DBLE(red_revc0(1:nnr_,ibnd,1)) &
+             & +AIMAG(vhart(1:nnr_,1)) * AIMAG(red_revc0(1:nnr_,ibnd,1))
+        !
+        IF (interaction) then 
+           psi_int(1:nnr_,ibnd) = psi_int(1:nnr_,ibnd) &
+                & +DBLE(vhart(1:nnr_,1)) * DBLE(red_revc0(1:nnr_,ibnd2,1))
+           IF (ibnd < nbnd) then
+              psi_int(1:nnr_,ibnd+1) = psi_int(1:nnr_,ibnd+1) &
+                   & +AIMAG(vhart(1:nnr_,1)) * DBLE(red_revc0(1:nnr_,ibnd2,1))
+           ENDIF
+        ELSE
+           psi_int(1:nnr_,ibnd) = psi_int(1:nnr_,ibnd) &
+                & -DBLE(vhart(1:nnr_,1)) * DBLE(red_revc0(1:nnr_,ibnd2,1)) 
+           IF (ibnd < nbnd) then
+              psi_int(1:nnr_,ibnd+1) = psi_int(1:nnr_,ibnd+1) &
+                   & -AIMAG(vhart(1:nnr_,1)) * DBLE(red_revc0(1:nnr_,ibnd2,1))
+           ENDIF
+        ENDIF
+        !
      ELSE
         pseudo_dens_c(1:nnr_) = CMPLX( &
              & w1*DBLE(psi(1:nnr_))*AIMAG(red_revc0(1:nnr_,ibnd2-1,1)),&
              & w2*AIMAG(psi(1:nnr_))*AIMAG(red_revc0(1:nnr_,ibnd2-1,1)), kind=DP )
+        !
+        CALL fwfft ('Rho', pseudo_dens_c, dfftt)
+        !
+        ! hartree contribution is computed in reciprocal space
+        !
+        DO is = 1, nspin
+           !
+           vhart(dfftt%nl(1:ngm_),is) = pseudo_dens_c(dfftt%nl(1:ngm_)) * fac_in(1:ngm_)
+           IF (gamma_only) vhart(dfftt%nlm(1:ngm_),is) = &
+                & pseudo_dens_c(dfftt%nlm(1:ngm_)) *&
+                & fac_in(1:ngm_)
+           !
+           !  and transformed back to real space
+           !
+           CALL invfft ('Rho', vhart (:, is), dfftt)
+           !
+        ENDDO
+        !
+        !
+        ! Finally return the interaction
+        !
+        psi_int(1:nnr_,ibnd2) = psi_int(1:nnr_,ibnd2) &
+             & +DBLE(vhart(1:nnr_,1)) * DBLE(red_revc0(1:nnr_,ibnd,1)) &
+             & +AIMAG(vhart(1:nnr_,1)) * AIMAG(red_revc0(1:nnr_,ibnd,1))
+        !
+        ! 
+        ! and interaction for Vexx term
+        !
+        IF (interaction) then
+           psi_int(1:nnr_,ibnd) = psi_int(1:nnr_,ibnd) &
+                & +DBLE(vhart(1:nnr_,1)) * AIMAG(red_revc0(1:nnr_,ibnd2-1,1)) 
+           IF (ibnd < nbnd) then
+              psi_int(1:nnr_,ibnd+1) = psi_int(1:nnr_,ibnd+1) &
+                   & +AIMAG(vhart(1:nnr_,1)) * AIMAG(red_revc0(1:nnr_,ibnd2-1,1))
+           ENDIF
+        ELSE
+           psi_int(1:nnr_,ibnd) = psi_int(1:nnr_,ibnd) &
+                & -DBLE(vhart(1:nnr_,1)) * AIMAG(red_revc0(1:nnr_,ibnd2-1,1))
+           IF (ibnd < nbnd) then
+              psi_int(1:nnr_,ibnd+1) = psi_int(1:nnr_,ibnd+1) &
+                   & -AIMAG(vhart(1:nnr_,1)) * AIMAG(red_revc0(1:nnr_,ibnd2-1,1))
+           ENDIF
+        ENDIF
+        !
      ENDIF
      !
-     CALL fwfft ('Custom', pseudo_dens_c, exx_fft%dfftt)
      !
-     ! hartree contribution is computed in reciprocal space
-     !
-     DO is = 1, nspin
-        !
-        vhart(exx_fft%nlt(1:ngm_),is) = pseudo_dens_c(exx_fft&
-             &%nlt(1:ngm_)) * fac_in(1:ngm_)
-        IF (gamma_only) vhart(exx_fft%nltm(1:ngm_),is) = &
-             & pseudo_dens_c(exx_fft%nltm(1:ngm_)) *&
-             & fac_in(1:ngm_)
-        !
-        !  and transformed back to real space
-        !
-        CALL invfft ('Custom', vhart (:, is), exx_fft%dfftt)
-        !
-     ENDDO
-     !
-     !
-     ! Finally return the interaction
-     !
-     psi_int(1:nnr_,ibnd2) = psi_int(1:nnr_,ibnd2) &
-          & +DBLE(vhart(1:nnr_,1)) * DBLE(red_revc0(1:nnr_,ibnd,1)) &
-          & +AIMAG(vhart(1:nnr_,1)) * AIMAG(red_revc0(1:nnr_,ibnd,1))
-     ! 
   ENDDO
   !
+  RETURN
   !
-END FUNCTION k2d_term_gamma
+END FUNCTION k2d_vexx_term_gamma
 !
 FUNCTION k2d_term_k(w1, psi, fac_in, ibnd, ik, ikq) RESULT (psi_int)
   !------------------------------------------------------------------
@@ -1017,18 +1315,18 @@ FUNCTION k2d_term_k(w1, psi, fac_in, ibnd, ik, ikq) RESULT (psi_int)
      !
      pseudo_dens_c(:) =  CONJG(psi(:))*red_revc0(:,ibnd2,k2q(ik))/omega
      !
-     CALL fwfft ('Smooth', pseudo_dens_c, dffts)
+     CALL fwfft ('Rho', pseudo_dens_c, dffts)
      !
      ! hartree contribution is computed in reciprocal space
      !
      DO is = 1, nspin
         !
-        vhart(nls(1:ngm),is) = w1*pseudo_dens_c(nls(1:ngm)) *&
+        vhart(dffts%nl(1:ngm),is) = w1*pseudo_dens_c(dffts%nl(1:ngm)) *&
              & fac_in(1:ngm) 
         !
         !  and transformed back to real space
         !
-        CALL invfft ('Smooth', vhart (:, is), dffts)
+        CALL invfft ('Rho', vhart (:, is), dffts)
         !
      ENDDO
      !
@@ -1050,70 +1348,70 @@ END FUNCTION k2d_term_k
 !! moved somewhere else but for now they live here.
 !!
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-SUBROUTINE invfft_orbital_custom_gamma(orbital, ibnd, nbnd, g2r)
+SUBROUTINE invfft_orbital_custom_gamma(orbital, ibnd, nbnd, npwt, dfftt)
 
   USE kinds,        ONLY : DP
-  USE fft_custom,   ONLY : fft_cus
+  USE fft_types,    ONLY : fft_type_descriptor
 
   IMPLICIT NONE
 
   COMPLEX(DP), INTENT(IN)    :: orbital(:,:)
-  INTEGER, INTENT(IN)        :: ibnd, nbnd
-  TYPE(fft_cus), INTENT(IN)  :: g2r
+  INTEGER, INTENT(IN)        :: ibnd, nbnd, npwt
+  TYPE(fft_type_descriptor), INTENT(IN)  :: dfftt
   !
   psic=(0.0_dp, 0.0_dp)
   !
   IF (ibnd < nbnd) THEN
      !
-     psic(g2r%nlt(1:g2r%npwt)) = orbital(1:g2r%npwt,ibnd) + &
-          &(0.0_dp, 1.0_dp) * orbital(1:g2r%npwt,ibnd+1) 
-     psic(g2r%nltm(1:g2r%npwt)) = CONJG(orbital(1:g2r%npwt,ibnd) - &   
-          &(0.0_dp, 1.0_dp) * orbital(1:g2r%npwt,ibnd+1))
+     psic(dfftt%nl(1:npwt)) = orbital(1:npwt,ibnd) + &
+          &(0.0_dp, 1.0_dp) * orbital(1:npwt,ibnd+1) 
+     psic(dfftt%nlm(1:npwt)) = CONJG(orbital(1:npwt,ibnd) - &   
+          &(0.0_dp, 1.0_dp) * orbital(1:npwt,ibnd+1))
      !
   ELSE
      !
-     psic(g2r%nlt(1:g2r%npwt))  = orbital(1:g2r%npwt,ibnd)
-     psic(g2r%nltm(1:g2r%npwt)) =CONJG(orbital(1:g2r%npwt,ibnd))
+     psic(dfftt%nl(1:npwt))  = orbital(1:npwt,ibnd)
+     psic(dfftt%nlm(1:npwt)) =CONJG(orbital(1:npwt,ibnd))
      !
   ENDIF
   !
-  CALL invfft ('CustomWave', psic, exx_fft%dfftt)
+  CALL invfft ('Wave', psic, dfftt)
   !
   RETURN
   !
 END SUBROUTINE invfft_orbital_custom_gamma
 
-SUBROUTINE fwfft_orbital_custom_gamma(orbital, ibnd, nbnd, g2r)
+SUBROUTINE fwfft_orbital_custom_gamma(orbital, ibnd, nbnd, npwt, dfftt)
 
   USE kinds,        ONLY : DP
-  USE fft_custom,   ONLY : fft_cus
+  USE fft_types,    ONLY : fft_type_descriptor
 
   IMPLICIT NONE
 
   COMPLEX(DP), INTENT(INOUT)    :: orbital(:,:)
-  INTEGER, INTENT(IN)        :: ibnd, nbnd
-  TYPE(fft_cus), INTENT(IN)  :: g2r
+  INTEGER, INTENT(IN)        :: ibnd, nbnd, npwt
+  TYPE(fft_type_descriptor), INTENT(IN)  :: dfftt
 
   ! Workspaces
   COMPLEX(DP) :: fp, fm
   ! Counters
   INTEGER :: j
   !
-  CALL fwfft ('CustomWave', psic(:), g2r%dfftt)   
+  CALL fwfft ('Wave', psic(:), dfftt)
   !
   IF (ibnd < nbnd) THEN
      !
      ! two ffts at the same time
-     DO j = 1, g2r%npwt
-        fp = (psic(g2r%nlt(j)) + psic(g2r%nltm(j)))*0.5d0
-        fm = (psic(g2r%nlt(j)) - psic(g2r%nltm(j)))*0.5d0
+     DO j = 1, npwt
+        fp = (psic(dfftt%nl(j)) + psic(dfftt%nlm(j)))*0.5d0
+        fm = (psic(dfftt%nl(j)) - psic(dfftt%nlm(j)))*0.5d0
         orbital( j, ibnd)   = CMPLX( DBLE(fp), AIMAG(fm),kind=DP)
         orbital( j, ibnd+1) = CMPLX(AIMAG(fp),- DBLE(fm),kind=DP)
      ENDDO
      !
   ELSE
      !
-     orbital(1:g2r%npwt,ibnd)=psic(g2r%nlt(1:g2r%npwt))
+     orbital(1:npwt,ibnd)=psic(dfftt%nl(1:npwt))
      !
   ENDIF
   !
@@ -1121,4 +1419,27 @@ SUBROUTINE fwfft_orbital_custom_gamma(orbital, ibnd, nbnd, g2r)
   !
 END SUBROUTINE fwfft_orbital_custom_gamma
 
+SUBROUTINE invfft_orbital_ibnd2_gamma(orbital, psitemp, ibnd2, npwt, dfftt)
+
+  USE kinds,        ONLY : DP
+  USE fft_types,    ONLY : fft_type_descriptor
+
+  IMPLICIT NONE
+
+  COMPLEX(DP), INTENT(IN)    :: orbital(:,:)
+  INTEGER, INTENT(IN)        :: ibnd2, npwt
+  TYPE(fft_type_descriptor), INTENT(IN)  :: dfftt
+  COMPLEX(DP), INTENT(OUT)   :: psitemp(:) 
+  !
+  psitemp=(0.0_dp, 0.0_dp)
+  !
+  psitemp(dfftt%nl(1:npwt))  = orbital(1:npwt,ibnd2)
+  psitemp(dfftt%nlm(1:npwt)) = CONJG(orbital(1:npwt,ibnd2))
+  !
+  CALL invfft ('Wave', psitemp, dfftt)
+  !
+  RETURN
+  !
+END SUBROUTINE invfft_orbital_ibnd2_gamma
+
 END MODULE lr_exx_kernel
