@@ -34,17 +34,18 @@ SUBROUTINE lr_read_wf()
   USE uspp,                 ONLY : vkb, nkb, okvan
   USE becmod,               ONLY : bec_type, becp, calbec
   USE realus,               ONLY : real_space, invfft_orbital_gamma,&
-                                 & initialisation_level,&
-                                 & fwfft_orbital_gamma, calbec_rs_gamma,&
-                                 & add_vuspsir_gamma, v_loc_psir,&
-                                 & s_psir_gamma, real_space_debug
+                                   initialisation_level,&
+                                   fwfft_orbital_gamma, calbec_rs_gamma,&
+                                   add_vuspsir_gamma, v_loc_psir,&
+                                   s_psir_gamma
   USE funct,                ONLY : dft_is_hybrid
   USE lr_exx_kernel,        ONLY : lr_exx_revc0_init, lr_exx_alloc, &
                                    lr_exx_restart
-  USE wavefunctions_module, ONLY : evc
+  USE wavefunctions,        ONLY : evc
   USE buffers,              ONLY : open_buffer
   USE qpoint,               ONLY : nksq
   USE noncollin_module,     ONLY : npol
+  USE symm_base,            ONLY : fft_fact
   USE fft_helper_subroutines
   !
   IMPLICIT NONE
@@ -70,6 +71,15 @@ SUBROUTINE lr_read_wf()
   IF (.NOT.eels) evc(:,:) = evc0(:,:,1)
   !
   IF ( dft_is_hybrid() ) THEN
+     !
+     ! Initialize fft_fact
+     ! Warning: If there are fractional translations and 
+     ! they are not commensurate with the dfftt grid, then
+     ! fft_fact is different from (1,1,1) and it must be
+     ! properly initialized. This matters when a symmetrization
+     ! is real space is used.
+     !
+     fft_fact(:) = 1
      !
      CALL open_buffer ( iunwfc, 'wfc', nwordwfc, io_level, exst ) 
      !
@@ -104,7 +114,7 @@ SUBROUTINE normal_read()
   ! The usual way of reading wavefunctions
   !
   USE lr_variables,             ONLY : tg_revc0
-  USE wavefunctions_module,     ONLY : psic
+  USE wavefunctions,     ONLY : psic
   USE realus,                   ONLY : tg_psic
   USE mp_global,                ONLY : me_bgrp
   !
@@ -172,7 +182,7 @@ SUBROUTINE normal_read()
         !
         CALL init_us_2(ngk(1),igk_k(:,1),xk(1,1),vkb)
         !
-        IF (real_space_debug>0) THEN
+        IF (real_space) THEN
            !
            DO ibnd = 1, nbnd, 2
               !
@@ -271,9 +281,7 @@ SUBROUTINE normal_read()
      !
   ENDIF
   !
-  ! OBM: Last minute check for real space implementation.
-  !
-  IF ( real_space_debug > 0 .AND. .NOT. gamma_only ) &
+  IF ( real_space .AND. .NOT. gamma_only ) &
            CALL errore( ' iosys ', ' Linear response calculation ' // &
            & 'real space algorithms with k-points not implemented', 1 )
   !
@@ -387,7 +395,7 @@ SUBROUTINE virt_read()
         !
         CALL init_us_2(ngk(1),igk_k(:,1),xk(1,1),vkb)
         !    
-        IF (real_space_debug>0) THEN
+        IF (real_space) THEN
            !
            DO ibnd=1,nbnd,2
               CALL invfft_orbital_gamma(evc_all(:,:,1),ibnd,nbnd)
@@ -532,9 +540,7 @@ SUBROUTINE virt_read()
   DEALLOCATE(sevc_all)
   DEALLOCATE(revc_all)
   !
-  ! OBM: Last minute check for real space implementation.
-  !
-  IF ( real_space_debug > 0 .and. .not. gamma_only ) &
+  IF ( real_space .and. .not. gamma_only ) &
            & CALL errore( ' iosys ', ' Linear response calculation ' // &
            & 'real space algorithms with k-points not implemented', 1 )
   !
