@@ -1,29 +1,73 @@
+*> \brief \b ZERRPOX
+*
+*  =========== DOCUMENTATION ===========
+*
+* Online html documentation available at 
+*            http://www.netlib.org/lapack/explore-html/ 
+*
+*  Definition:
+*  ===========
+*
+*       SUBROUTINE ZERRPO( PATH, NUNIT )
+* 
+*       .. Scalar Arguments ..
+*       CHARACTER*3        PATH
+*       INTEGER            NUNIT
+*       ..
+*  
+*
+*> \par Purpose:
+*  =============
+*>
+*> \verbatim
+*>
+*> ZERRPO tests the error exits for the COMPLEX*16 routines
+*> for Hermitian positive definite matrices.
+*>
+*> Note that this file is used only when the XBLAS are available,
+*> otherwise zerrpo.f defines this subroutine.
+*> \endverbatim
+*
+*  Arguments:
+*  ==========
+*
+*> \param[in] PATH
+*> \verbatim
+*>          PATH is CHARACTER*3
+*>          The LAPACK path name for the routines to be tested.
+*> \endverbatim
+*>
+*> \param[in] NUNIT
+*> \verbatim
+*>          NUNIT is INTEGER
+*>          The unit number for output.
+*> \endverbatim
+*
+*  Authors:
+*  ========
+*
+*> \author Univ. of Tennessee 
+*> \author Univ. of California Berkeley 
+*> \author Univ. of Colorado Denver 
+*> \author NAG Ltd. 
+*
+*> \date November 2015
+*
+*> \ingroup complex16_lin
+*
+*  =====================================================================
       SUBROUTINE ZERRPO( PATH, NUNIT )
 *
-*  -- LAPACK test routine (version 3.1) --
-*     Univ. of Tennessee, Univ. of California Berkeley and NAG Ltd..
-*     November 2006
+*  -- LAPACK test routine (version 3.6.0) --
+*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
+*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
+*     November 2015
 *
 *     .. Scalar Arguments ..
       CHARACTER*3        PATH
       INTEGER            NUNIT
 *     ..
 *
-*  Purpose
-*  =======
-*
-*  ZERRPO tests the error exits for the COMPLEX*16 routines
-*  for Hermitian positive definite matrices.
-*
-*  Arguments
-*  =========
-*
-*  PATH    (input) CHARACTER*3
-*          The LAPACK path name for the routines to be tested.
-*
-*  NUNIT   (input) INTEGER
-*          The unit number for output.
-*
 *  =====================================================================
 *
 *     .. Parameters ..
@@ -39,7 +83,7 @@
 *     .. Local Arrays ..
       DOUBLE PRECISION   S( NMAX ), R( NMAX ), R1( NMAX ), R2( NMAX ),
      $                   ERR_BNDS_N( NMAX, 3 ), ERR_BNDS_C( NMAX, 3 ),
-     $                   PARAMS 
+     $                   PARAMS( 1 )
       COMPLEX*16         A( NMAX, NMAX ), AF( NMAX, NMAX ), B( NMAX ),
      $                   W( 2*NMAX ), X( NMAX )
 *     ..
@@ -187,49 +231,49 @@
 *
 *        ZPORFSX
 *
-         n_err_bnds = 3
-         nparams = 0
+         N_ERR_BNDS = 3
+         NPARAMS = 0
          SRNAMT = 'ZPORFSX'
          INFOT = 1
-         CALL ZPORFSX( '/', EQ, 0, 0, A, 1, AF, 1, S, B, 1, X, 1, 
-     $        RCOND, berr, n_err_bnds, err_bnds_n, err_bnds_c, nparams,
-     $        params, W, R, INFO )
+         CALL ZPORFSX( '/', EQ, 0, 0, A, 1, AF, 1, S, B, 1, X, 1,
+     $        RCOND, BERR, N_ERR_BNDS, ERR_BNDS_N, ERR_BNDS_C, NPARAMS,
+     $        PARAMS, W, R, INFO )
          CALL CHKXER( 'ZPORFSX', INFOT, NOUT, LERR, OK )
          INFOT = 2
-         CALL ZPORFSX( 'U', EQ, -1, 0, A, 1, AF, 1, S, B, 1, X, 1,
-     $        RCOND, berr, n_err_bnds, err_bnds_n, err_bnds_c, nparams,
-     $        params, W, R, INFO )
+         CALL ZPORFSX( 'U', "/", -1, 0, A, 1, AF, 1, S, B, 1, X, 1,
+     $        RCOND, BERR, N_ERR_BNDS, ERR_BNDS_N, ERR_BNDS_C, NPARAMS,
+     $        PARAMS, W, R, INFO )
          CALL CHKXER( 'ZPORFSX', INFOT, NOUT, LERR, OK )
          EQ = 'N'
          INFOT = 3
          CALL ZPORFSX( 'U', EQ, -1, 0, A, 1, AF, 1, S, B, 1, X, 1,
-     $        RCOND, berr, n_err_bnds, err_bnds_n, err_bnds_c, nparams,
-     $        params, W, R, INFO )
+     $        RCOND, BERR, N_ERR_BNDS, ERR_BNDS_N, ERR_BNDS_C, NPARAMS,
+     $        PARAMS, W, R, INFO )
          CALL CHKXER( 'ZPORFSX', INFOT, NOUT, LERR, OK )
          INFOT = 4
-         CALL ZPORFSX( 'U', EQ, 0, -1, A, 1, AF, 1, S, B, 1, X, 1, 
-     $        RCOND, berr, n_err_bnds, err_bnds_n, err_bnds_c, nparams,
-     $        params, W, R, INFO )
+         CALL ZPORFSX( 'U', EQ, 0, -1, A, 1, AF, 1, S, B, 1, X, 1,
+     $        RCOND, BERR, N_ERR_BNDS, ERR_BNDS_N, ERR_BNDS_C, NPARAMS,
+     $        PARAMS, W, R, INFO )
          CALL CHKXER( 'ZPORFSX', INFOT, NOUT, LERR, OK )
          INFOT = 6
-         CALL ZPORFSX( 'U', EQ, 2, 1, A, 1, AF, 2, S, B, 2, X, 2, 
-     $        RCOND, berr, n_err_bnds, err_bnds_n, err_bnds_c, nparams,
-     $        params, W, R, INFO )
+         CALL ZPORFSX( 'U', EQ, 2, 1, A, 1, AF, 2, S, B, 2, X, 2,
+     $        RCOND, BERR, N_ERR_BNDS, ERR_BNDS_N, ERR_BNDS_C, NPARAMS,
+     $        PARAMS, W, R, INFO )
          CALL CHKXER( 'ZPORFSX', INFOT, NOUT, LERR, OK )
          INFOT = 8
-         CALL ZPORFSX( 'U', EQ, 2, 1, A, 2, AF, 1, S, B, 2, X, 2, 
-     $        RCOND, berr, n_err_bnds, err_bnds_n, err_bnds_c, nparams,
-     $        params, W, R, INFO )
+         CALL ZPORFSX( 'U', EQ, 2, 1, A, 2, AF, 1, S, B, 2, X, 2,
+     $        RCOND, BERR, N_ERR_BNDS, ERR_BNDS_N, ERR_BNDS_C, NPARAMS,
+     $        PARAMS, W, R, INFO )
          CALL CHKXER( 'ZPORFSX', INFOT, NOUT, LERR, OK )
          INFOT = 11
-         CALL ZPORFSX( 'U', EQ, 2, 1, A, 2, AF, 2, S, B, 1, X, 2, 
-     $        RCOND, berr, n_err_bnds, err_bnds_n, err_bnds_c, nparams,
-     $        params, W, R, INFO )
+         CALL ZPORFSX( 'U', EQ, 2, 1, A, 2, AF, 2, S, B, 1, X, 2,
+     $        RCOND, BERR, N_ERR_BNDS, ERR_BNDS_N, ERR_BNDS_C, NPARAMS,
+     $        PARAMS, W, R, INFO )
          CALL CHKXER( 'ZPORFSX', INFOT, NOUT, LERR, OK )
          INFOT = 13
-         CALL ZPORFSX( 'U', EQ, 2, 1, A, 2, AF, 2, S, B, 2, X, 1, 
-     $        RCOND, berr, n_err_bnds, err_bnds_n, err_bnds_c, nparams,
-     $        params, W, R, INFO )
+         CALL ZPORFSX( 'U', EQ, 2, 1, A, 2, AF, 2, S, B, 2, X, 1,
+     $        RCOND, BERR, N_ERR_BNDS, ERR_BNDS_N, ERR_BNDS_C, NPARAMS,
+     $        PARAMS, W, R, INFO )
          CALL CHKXER( 'ZPORFSX', INFOT, NOUT, LERR, OK )
 *
 *        ZPOCON
