@@ -25,6 +25,29 @@
 #define __USE_BARRIER
 
 
+!=----------------------------------------------------------------------------=!
+!
+! These routines allocate buffer spaces used in reduce_base_real_gpu.
+! These should be in data_buffer.f90 but need to be here becouse size is
+! depends on the __MSGSIZ_MAX definition
+SUBROUTINE allocate_buffers
+    USE data_buffer
+    IMPLICIT NONE
+    INTEGER, PARAMETER :: maxb = __MSGSIZ_MAX
+    !
+    IF (.NOT. ALLOCATED(mp_buff_r)) ALLOCATE(mp_buff_r(maxb))
+    IF (.NOT. ALLOCATED(mp_buff_i)) ALLOCATE(mp_buff_i(maxb))
+    !
+END SUBROUTINE allocate_buffers
+
+SUBROUTINE deallocate_buffers
+    USE data_buffer
+    IMPLICIT NONE
+    !
+    DEALLOCATE(mp_buff_r, mp_buff_i)
+    !
+END SUBROUTINE deallocate_buffers
+
 !=----------------------------------------------------------------------------=!
 !
 
@@ -92,53 +115,110 @@ END SUBROUTINE mp_synchronize
         RETURN
    END SUBROUTINE bcast_real 
 
-
-   SUBROUTINE bcast_integer( array, n, root, gid )
-        USE parallel_include  
-        IMPLICIT NONE
-        INTEGER, INTENT(IN) :: n, root, gid
-        INTEGER :: array( n )
+   !------------------------------------------------------------------------------!
+   SUBROUTINE bcast_integer(array, n, root, gid)
+   !------------------------------------------------------------------------------!
+   !! 
+   !! Broadcast integers
+   !!   
+   USE parallel_include  
+   ! 
+   IMPLICIT NONE
+   INTEGER, INTENT(in) :: n, root, gid
+   INTEGER :: array(n)
 #if defined __MPI
-        INTEGER :: msgsiz_max = __MSGSIZ_MAX
-        INTEGER :: nblk, blksiz, iblk, istart, ierr
-
+   INTEGER :: msgsiz_max = __MSGSIZ_MAX
+   INTEGER :: nblk, blksiz, iblk, istart, ierr
 #if defined __TRACE
-        write(*,*) 'BCAST_INTEGER IN'
+   WRITE(*, *) 'BCAST_INTEGER IN'
 #endif
-
-        IF( n <= 0 ) GO TO 1
-
+   IF(n <= 0) GOTO 1
+   ! 
 #if defined __USE_BARRIER
-        CALL mp_synchronize( gid )
+   CALL mp_synchronize(gid)
 #endif
-
-        IF( n <= msgsiz_max ) THEN
-           CALL MPI_BCAST( array, n, MPI_INTEGER, root, gid, ierr )
-           IF( ierr /= 0 ) CALL errore( ' bcast_integer ', ' error in mpi_bcast 1 ', ierr )
-        ELSE
-           nblk   = n / msgsiz_max
-           blksiz = msgsiz_max
-           DO iblk = 1, nblk
-              istart = (iblk-1)*msgsiz_max + 1
-              CALL MPI_BCAST( array( istart ), blksiz, MPI_INTEGER, root, gid, ierr )
-              IF( ierr /= 0 ) CALL errore( ' bcast_integer ', ' error in mpi_bcast 2 ', ierr )
-           END DO
-           blksiz = MOD( n, msgsiz_max )
-           IF( blksiz > 0 ) THEN
-              istart = nblk * msgsiz_max + 1
-              CALL MPI_BCAST( array( istart ), blksiz, MPI_INTEGER, root, gid, ierr )
-              IF( ierr /= 0 ) CALL errore( ' bcast_integer ', ' error in mpi_bcast 3 ', ierr )
-           END IF
-        END IF
-1       CONTINUE
+   ! 
+   IF(n <= msgsiz_max) THEN
+     CALL MPI_BCAST(array, n, MPI_INTEGER, root, gid, ierr)
+     IF(ierr /= 0) CALL errore(' bcast_integer ', ' error in mpi_bcast 1 ', ierr)
+   ELSE
+     nblk   = n / msgsiz_max
+     blksiz = msgsiz_max
+     DO iblk = 1, nblk
+       istart = (iblk - 1) * msgsiz_max + 1
+       CALL MPI_BCAST(array(istart), blksiz, MPI_INTEGER, root, gid, ierr )
+       IF(ierr /= 0) CALL errore(' bcast_integer ', ' error in mpi_bcast 2 ', ierr)
+     ENDDO
+     blksiz = MOD( n, msgsiz_max )
+     IF(blksiz > 0) THEN
+       istart = nblk * msgsiz_max + 1
+       CALL MPI_BCAST(array( istart ), blksiz, MPI_INTEGER, root, gid, ierr)
+       IF(ierr /= 0) CALL errore(' bcast_integer ', ' error in mpi_bcast 3 ', ierr)
+     ENDIF
+   ENDIF
+1  CONTINUE
 #if defined __TRACE
-        write(*,*) 'BCAST_INTEGER OUT'
+   WRITE(*, *) 'BCAST_INTEGER OUT'
 #endif
 #endif
-        RETURN
+   RETURN
+   !------------------------------------------------------------------------------!
    END SUBROUTINE bcast_integer
-
-
+   !------------------------------------------------------------------------------!
+   ! 
+   !------------------------------------------------------------------------------!
+   SUBROUTINE bcast_integer8(array, n, root, gid)
+   !------------------------------------------------------------------------------!
+   !! 
+   !! Broadcast integers
+   !!   
+   USE util_param,     ONLY : i8b
+   USE parallel_include
+   ! 
+   IMPLICIT NONE
+   INTEGER, INTENT(in) :: n, root, gid
+   INTEGER(KIND = i8b) :: array(n)
+#if defined __MPI
+   INTEGER :: msgsiz_max = __MSGSIZ_MAX
+   INTEGER :: nblk, blksiz, iblk, istart, ierr
+#if defined __TRACE
+   WRITE(*, *) 'BCAST_INTEGER IN'
+#endif
+   IF(n <= 0) GOTO 1
+   ! 
+#if defined __USE_BARRIER
+   CALL mp_synchronize(gid)
+#endif
+   ! 
+   IF(n <= msgsiz_max) THEN
+     CALL MPI_BCAST(array, n, MPI_INTEGER8, root, gid, ierr)
+     IF(ierr /= 0) CALL errore(' bcast_integer8 ', ' error in mpi_bcast 1 ', ierr)
+   ELSE
+     nblk   = n / msgsiz_max
+     blksiz = msgsiz_max
+     DO iblk = 1, nblk
+       istart = (iblk - 1) * msgsiz_max + 1
+       CALL MPI_BCAST(array(istart), blksiz, MPI_INTEGER8, root, gid, ierr )
+       IF(ierr /= 0) CALL errore(' bcast_integer8 ', ' error in mpi_bcast 2 ', ierr)
+     ENDDO
+     blksiz = MOD( n, msgsiz_max )
+     IF(blksiz > 0) THEN
+       istart = nblk * msgsiz_max + 1
+       CALL MPI_BCAST(array( istart ), blksiz, MPI_INTEGER8, root, gid, ierr)
+       IF(ierr /= 0) CALL errore(' bcast_integer8 ', ' error in mpi_bcast 3 ', ierr)
+     ENDIF
+   ENDIF
+1  CONTINUE
+#if defined __TRACE
+   WRITE(*, *) 'BCAST_INTEGER OUT'
+#endif
+#endif
+   RETURN
+   !------------------------------------------------------------------------------!
+   END SUBROUTINE bcast_integer8
+   !------------------------------------------------------------------------------!
+   ! 
+   !------------------------------------------------------------------------------!
    SUBROUTINE bcast_logical( array, n, root, gid )
         USE parallel_include  
         IMPLICIT NONE
@@ -211,7 +291,7 @@ SUBROUTINE reduce_base_real( dim, ps, comm, root )
   !
 #if defined (__MPI)  
   !
-  INTEGER            :: info
+  INTEGER            :: info, myid
   !
 #if defined __TRACE
   write(*,*) 'reduce_base_real IN'
@@ -226,8 +306,15 @@ SUBROUTINE reduce_base_real( dim, ps, comm, root )
 #endif
   !
   IF( root >= 0 ) THEN
-     CALL MPI_REDUCE( MPI_IN_PLACE, ps, dim, MPI_DOUBLE_PRECISION, MPI_SUM, root, comm, info )
-     IF( info /= 0 ) CALL errore( 'reduce_base_real', 'error in mpi_reduce 1', info )
+     CALL mpi_comm_rank( comm, myid, info )
+     IF( info /= 0 ) CALL errore( 'reduce_base_real', 'error in mpi_comm_rank', info )
+     IF (myid == root) THEN
+        CALL MPI_REDUCE( MPI_IN_PLACE, ps, dim, MPI_DOUBLE_PRECISION, MPI_SUM, root, comm, info )
+        IF( info /= 0 ) CALL errore( 'reduce_base_real', 'error in mpi_reduce 1', info )
+     ELSE
+        CALL MPI_REDUCE( ps, ps, dim, MPI_DOUBLE_PRECISION, MPI_SUM, root, comm, info )
+        IF( info /= 0 ) CALL errore( 'reduce_base_real', 'error in mpi_reduce 1', info )
+     ENDIF
   ELSE
      CALL MPI_ALLREDUCE( MPI_IN_PLACE, ps, dim, MPI_DOUBLE_PRECISION, MPI_SUM, comm, info )
      IF( info /= 0 ) CALL errore( 'reduce_base_real', 'error in mpi_allreduce 1', info )
@@ -255,6 +342,7 @@ SUBROUTINE reduce_base_real( dim, ps, comm, root )
   ! ... This version uses a fixed-length buffer of appropriate (?) dim
   !
   USE util_param, ONLY : DP
+  USE data_buffer, ONLY: buff => mp_buff_r
   USE parallel_include  
   !
   IMPLICIT NONE
@@ -270,9 +358,6 @@ SUBROUTINE reduce_base_real( dim, ps, comm, root )
   INTEGER            :: info, n, nbuf, nproc, myid
   INTEGER, PARAMETER :: maxb = __MSGSIZ_MAX
   !
-  REAL(DP) :: buff(maxb)  
-  ! the use of the common here could help the transfer of data to the network device
-  COMMON / mp_base_real / buff
   !
 #if defined __TRACE
   write(*,*) 'reduce_base_real IN'
@@ -348,161 +433,320 @@ END SUBROUTINE reduce_base_real
 !
 !
 #if defined (__USE_INPLACE_MPI)
-!
-!----------------------------------------------------------------------------
-SUBROUTINE reduce_base_integer( dim, ps, comm, root )
+  !
   !----------------------------------------------------------------------------
-  !
-  ! ... sums a distributed variable ps(dim) over the processors.
-  ! ... This version uses a fixed-length buffer of appropriate (?) dim
-  !
-  USE util_param, ONLY : DP
+  SUBROUTINE reduce_base_integer(dim, ps, comm, root)
+  !----------------------------------------------------------------------------
+  !!
+  !! Sums a distributed variable ps(dim) over the processors.
+  !! This version uses a fixed-length buffer of appropriate (?) dim
+  !!
+  USE util_param,       ONLY : DP
   USE parallel_include  
   !
   IMPLICIT NONE
   !
-  INTEGER,  INTENT(IN)    :: dim
-  INTEGER                 :: ps(dim)
-  INTEGER,  INTENT(IN)    :: comm    ! communicator
-  INTEGER,  INTENT(IN)    :: root    ! if root <  0 perform a reduction to all procs
-                                     ! if root >= 0 perform a reduce only to root proc.
+  INTEGER, INTENT(in)    :: dim
+  INTEGER, INTENT(inout) :: ps(dim)
+  INTEGER, INTENT(in)    :: comm    ! communicator
+  INTEGER, INTENT(in)    :: root    ! if root <  0 perform a reduction to all procs
+                                    ! if root >= 0 perform a reduce only to root proc.
   !
 #if defined (__MPI)  
   !
-  INTEGER            :: info
+  INTEGER            :: info, myid
   !
 #if defined __TRACE
-  write(*,*) 'reduce_base_integer IN'
+  WRITE(*, *) 'reduce_base_integer IN'
 #endif
   !
   ! ... synchronize processes
   !
 #if defined __USE_BARRIER
-  CALL mp_synchronize( comm )
+  CALL mp_synchronize(comm)
 #endif
   !
-  IF( root >= 0 ) THEN
-     CALL MPI_REDUCE( MPI_IN_PLACE, ps, dim, MPI_INTEGER, MPI_SUM, root, comm, info )
-     IF( info /= 0 ) CALL errore( 'reduce_base_integer', 'error in mpi_reduce 1', info )
+  IF(root >= 0) THEN
+    CALL mpi_comm_rank( comm, myid, info )
+    IF( info /= 0 ) CALL errore( 'reduce_base_integer', 'error in mpi_comm_rank', info )
+    IF (myid == root) THEN
+      CALL MPI_REDUCE(MPI_IN_PLACE, ps, dim, MPI_INTEGER, MPI_SUM, root, comm, info)
+      IF(info /= 0) CALL errore('reduce_base_integer', 'error in mpi_reduce 1', info)
+    ELSE
+      CALL MPI_REDUCE(ps, ps, dim, MPI_INTEGER, MPI_SUM, root, comm, info)
+      IF(info /= 0) CALL errore('reduce_base_integer', 'error in mpi_reduce 1', info)
+    ENDIF
   ELSE
-     CALL MPI_ALLREDUCE( MPI_IN_PLACE, ps, dim, MPI_INTEGER, MPI_SUM, comm, info )
-     IF( info /= 0 ) CALL errore( 'reduce_base_integer', 'error in mpi_allreduce 1', info )
-  END IF
+    CALL MPI_ALLREDUCE(MPI_IN_PLACE, ps, dim, MPI_INTEGER, MPI_SUM, comm, info)
+    IF(info /= 0 ) CALL errore('reduce_base_integer', 'error in mpi_allreduce 1', info)
+  ENDIF
   !
 #if defined __TRACE
-  write(*,*) 'reduce_base_integer OUT'
+  WRITE(*, *) 'reduce_base_integer OUT'
 #endif
   !
 #endif
   !
   RETURN
-  !
-END SUBROUTINE reduce_base_integer
-!
-#else
-!
-!----------------------------------------------------------------------------
-SUBROUTINE reduce_base_integer( dim, ps, comm, root )
+  !----------------------------------------------------------------------------
+  END SUBROUTINE reduce_base_integer
   !----------------------------------------------------------------------------
   !
-  ! ... sums a distributed variable ps(dim) over the processors.
-  ! ... This version uses a fixed-length buffer of appropriate (?) dim
+  !----------------------------------------------------------------------------
+  SUBROUTINE reduce_base_integer8(dim, ps, comm, root)
+  !----------------------------------------------------------------------------
+  !!
+  !! Sums a distributed variable ps(dim) over the processors.
+  !! This version uses a fixed-length buffer of appropriate (?) dim
+  !!
+  USE util_param,       ONLY : DP, i8b
+  USE parallel_include
   !
-  USE util_param, ONLY : DP
+  IMPLICIT NONE
+  !
+  INTEGER, INTENT(in)  :: dim
+  INTEGER(KIND = i8b), INTENT(inout) :: ps(dim)
+  INTEGER, INTENT(in)  :: comm    ! communicator
+  INTEGER, INTENT(in)  :: root    ! if root <  0 perform a reduction to all procs
+                                  ! if root >= 0 perform a reduce only to root proc.
+  !   
+#if defined (__MPI)  
+  !
+  INTEGER            :: info, myid
+  !
+#if defined __TRACE
+  WRITE(*, *) 'reduce_base_integer IN'
+#endif
+  !
+  ! ... synchronize processes
+  !
+#if defined __USE_BARRIER
+  CALL mp_synchronize(comm)
+#endif
+  !
+  IF(root >= 0) THEN
+    CALL mpi_comm_rank( comm, myid, info )
+    IF( info /= 0 ) CALL errore( 'reduce_base_integer8', 'error in mpi_comm_rank', info )
+    IF (myid == root) THEN
+      CALL MPI_REDUCE(MPI_IN_PLACE, ps, dim, MPI_INTEGER8, MPI_SUM, root, comm, info)
+      IF(info /= 0) CALL errore('reduce_base_integer8', 'error in mpi_reduce 1', info)
+    ELSE
+      CALL MPI_REDUCE(ps, ps, dim, MPI_INTEGER8, MPI_SUM, root, comm, info)
+      IF(info /= 0) CALL errore('reduce_base_integer8', 'error in mpi_reduce 1', info)
+    ENDIF
+  ELSE
+    CALL MPI_ALLREDUCE(MPI_IN_PLACE, ps, dim, MPI_INTEGER8, MPI_SUM, comm, info)
+    IF(info /= 0 ) CALL errore('reduce_base_integer8', 'error in mpi_allreduce 1', info)
+  ENDIF
+  !
+#if defined __TRACE
+  WRITE(*, *) 'reduce_base_integer OUT'
+#endif
+  !
+#endif
+  !
+  RETURN
+  !----------------------------------------------------------------------------
+  END SUBROUTINE reduce_base_integer8
+  !----------------------------------------------------------------------------
+#else
+  !
+  !----------------------------------------------------------------------------
+  SUBROUTINE reduce_base_integer(dim, ps, comm, root)
+  !----------------------------------------------------------------------------
+  !!
+  !! Sums a distributed variable ps(dim) over the processors.
+  !! This version uses a fixed-length buffer of appropriate (?) dim
+  !!
+  USE util_param,  ONLY : DP
+  USE data_buffer, ONLY : buff => mp_buff_i
   USE parallel_include  
   !
   IMPLICIT NONE
   !
-  INTEGER,  INTENT(IN)    :: dim
-  INTEGER                 :: ps(dim)
-  INTEGER,  INTENT(IN)    :: comm    ! communicator
-  INTEGER,  INTENT(IN)    :: root    ! if root <  0 perform a reduction to all procs
-                                     ! if root >= 0 perform a reduce only to root proc.
+  INTEGER, INTENT(in)    :: dim
+  INTEGER, INTENT(inout) :: ps(dim)
+  INTEGER, INTENT(in)    :: comm    ! communicator
+  INTEGER, INTENT(in)    :: root    ! if root <  0 perform a reduction to all procs
+                                    ! if root >= 0 perform a reduce only to root proc.
   !
 #if defined (__MPI)  
   !
   INTEGER            :: info, n, nbuf, nproc, myid
   INTEGER, PARAMETER :: maxb = __MSGSIZ_MAX
   !
-  INTEGER :: buff(maxb)  
-  COMMON / mp_base_integer / buff
-  !
 #if defined __TRACE
-  write(*,*) 'reduce_base_integer IN'
+  WRITE(*, *) 'reduce_base_integer IN'
 #endif
   !
-  CALL mpi_comm_size( comm, nproc, info )
-  IF( info /= 0 ) CALL errore( 'reduce_base_integer', 'error in mpi_comm_size', info )
-
-  CALL mpi_comm_rank( comm, myid, info )
-  IF( info /= 0 ) CALL errore( 'reduce_base_integer', 'error in mpi_comm_rank', info )
+  CALL mpi_comm_size(comm, nproc, info)
+  IF(info /= 0) CALL errore('reduce_base_integer', 'error in mpi_comm_size', info)
+  ! 
+  CALL mpi_comm_rank(comm, myid, info)
+  IF(info /= 0) CALL errore('reduce_base_integer', 'error in mpi_comm_rank', info)
   !
-  IF ( dim <= 0 .OR. nproc <= 1 ) GO TO 1  ! go to the end of the subroutine
+  IF (dim <= 0 .OR. nproc <= 1) GOTO 1  ! go to the end of the subroutine
   !
   ! ... synchronize processes
   !
 #if defined __USE_BARRIER
-  CALL mp_synchronize( comm )
+  CALL mp_synchronize(comm)
 #endif
   !
   nbuf = dim / maxb
   !
   DO n = 1, nbuf
-     !
-     IF( root >= 0 ) THEN
-        CALL MPI_REDUCE( ps(1+(n-1)*maxb), buff, maxb, MPI_INTEGER, MPI_SUM, root, comm, info )
-        IF( info /= 0 ) CALL errore( 'reduce_base_integer', 'error in mpi_reduce 1', info )
-     ELSE
-        CALL MPI_ALLREDUCE( ps(1+(n-1)*maxb), buff, maxb, MPI_INTEGER, MPI_SUM, comm, info )
-        IF( info /= 0 ) CALL errore( 'reduce_base_integer', 'error in mpi_allreduce 1', info )
-     END IF
-     !
-     IF( root < 0 ) THEN
-        ps((1+(n-1)*maxb):(n*maxb)) = buff(1:maxb)
-     ELSE IF( root == myid ) THEN
-        ps((1+(n-1)*maxb):(n*maxb)) = buff(1:maxb)
-     END IF
-     !
-  END DO
+    !
+    IF (root >= 0) THEN
+      CALL MPI_REDUCE(ps(1 + (n - 1) * maxb), buff, maxb, MPI_INTEGER, MPI_SUM, root, comm, info)
+      IF (info /= 0 ) CALL errore('reduce_base_integer', 'error in mpi_reduce 1', info)
+    ELSE
+      CALL MPI_ALLREDUCE(ps(1 + (n - 1) * maxb), buff, maxb, MPI_INTEGER, MPI_SUM, comm, info)
+      IF (info /= 0) CALL errore('reduce_base_integer', 'error in mpi_allreduce 1', info)
+    ENDIF
+    !
+    IF (root < 0 ) THEN
+      ps((1 + (n - 1) * maxb):(n * maxb)) = buff(1:maxb)
+    ELSEIF( root == myid) THEN
+      ps((1 + (n - 1) * maxb):(n * maxb)) = buff(1:maxb)
+    ENDIF
+    !
+  ENDDO
   !
   ! ... possible remaining elements < maxb
   !
-  IF ( ( dim - nbuf * maxb ) > 0 ) THEN
-     !
-     IF( root >= 0 ) THEN
-        CALL MPI_REDUCE( ps(1+nbuf*maxb), buff, (dim-nbuf*maxb), MPI_INTEGER, MPI_SUM, root, comm, info )
-        IF( info /= 0 ) CALL errore( 'reduce_base_integer', 'error in mpi_reduce 2', info )
-     ELSE
-        CALL MPI_ALLREDUCE( ps(1+nbuf*maxb), buff, (dim-nbuf*maxb), MPI_INTEGER, MPI_SUM, comm, info )
-        IF( info /= 0 ) CALL errore( 'reduce_base_integer', 'error in mpi_allreduce 2', info )
-     END IF
-     !
-     IF( root < 0 ) THEN
-        ps((1+nbuf*maxb):dim) = buff(1:(dim-nbuf*maxb))
-     ELSE IF( root == myid ) THEN
-        ps((1+nbuf*maxb):dim) = buff(1:(dim-nbuf*maxb))
-     END IF
-     !
-  END IF
+  IF ((dim - nbuf * maxb) > 0) THEN
+    !
+    IF (root >= 0) THEN
+      CALL MPI_REDUCE(ps(1 + nbuf * maxb), buff, (dim - nbuf * maxb), MPI_INTEGER, MPI_SUM, root, comm, info)
+      IF(info /= 0) CALL errore('reduce_base_integer', 'error in mpi_reduce 2', info)
+    ELSE
+      CALL MPI_ALLREDUCE(ps(1 + nbuf * maxb), buff, (dim - nbuf * maxb), MPI_INTEGER, MPI_SUM, comm, info)
+      IF(info /= 0) CALL errore('reduce_base_integer', 'error in mpi_allreduce 2', info)
+    ENDIF
+    !
+    IF(root < 0) THEN
+      ps((1 + nbuf * maxb):dim) = buff(1:(dim - nbuf * maxb))
+    ELSEIF(root == myid) THEN
+      ps((1 + nbuf * maxb):dim) = buff(1:(dim - nbuf * maxb))
+    ENDIF
+    !
+  ENDIF
   !
 1 CONTINUE
   !
 #if defined __TRACE
-  write(*,*) 'reduce_base_integer OUT'
+  WRITE(*, *) 'reduce_base_integer OUT'
 #endif
   !
 #endif
   !
   RETURN
+  !----------------------------------------------------------------------------
+  END SUBROUTINE reduce_base_integer
+  !----------------------------------------------------------------------------
   !
-END SUBROUTINE reduce_base_integer
-!
+  !----------------------------------------------------------------------------
+  SUBROUTINE reduce_base_integer8(dim, ps, comm, root)
+  !----------------------------------------------------------------------------
+  !!
+  !! Sums a distributed variable ps(dim) over the processors.
+  !! This version uses a fixed-length buffer of appropriate (?) dim
+  !!
+  USE util_param,  ONLY : DP, i8b
+  USE data_buffer, ONLY : buff => mp_buff_i
+  USE parallel_include  
+  !
+  IMPLICIT NONE
+  !
+  INTEGER, INTENT(in)    :: dim
+  INTEGER(KIND = i8b), INTENT(inout) :: ps(dim)
+  INTEGER, INTENT(in)    :: comm    ! communicator
+  INTEGER, INTENT(in)    :: root    ! if root <  0 perform a reduction to all procs
+                                    ! if root >= 0 perform a reduce only to root proc.
+  !
+#if defined (__MPI)  
+  !
+  INTEGER            :: info, n, nbuf, nproc, myid
+  INTEGER, PARAMETER :: maxb = __MSGSIZ_MAX
+  !
+#if defined __TRACE
+  WRITE(*, *) 'reduce_base_integer8 IN'
 #endif
-!
-! ... "reduce"-like subroutines
-!
-!----------------------------------------------------------------------------
-SUBROUTINE reduce_base_real_to( dim, ps, psout, comm, root )
+  !
+  CALL mpi_comm_size(comm, nproc, info)
+  IF(info /= 0) CALL errore('reduce_base_integer8', 'error in mpi_comm_size', info)
+  ! 
+  CALL mpi_comm_rank(comm, myid, info)
+  IF(info /= 0) CALL errore('reduce_base_integer8', 'error in mpi_comm_rank', info)
+  !
+  IF (dim <= 0 .OR. nproc <= 1) GOTO 1  ! go to the end of the subroutine
+  !
+  ! ... synchronize processes
+  !
+#if defined __USE_BARRIER
+  CALL mp_synchronize(comm)
+#endif
+  !
+  nbuf = dim / maxb
+  !
+  DO n = 1, nbuf
+    !
+    IF (root >= 0) THEN
+      CALL MPI_REDUCE(ps(1 + (n - 1) * maxb), buff, maxb, MPI_INTEGER8, MPI_SUM, root, comm, info)
+      IF (info /= 0 ) CALL errore('reduce_base_integer8', 'error in mpi_reduce 1', info)
+    ELSE
+      CALL MPI_ALLREDUCE(ps(1 + (n - 1) * maxb), buff, maxb, MPI_INTEGER8, MPI_SUM, comm, info)
+      IF (info /= 0) CALL errore('reduce_base_integer8', 'error in mpi_allreduce 1', info)
+    ENDIF
+    !
+    IF (root < 0 ) THEN
+      ps((1 + (n - 1) * maxb):(n * maxb)) = buff(1:maxb)
+    ELSEIF( root == myid) THEN
+      ps((1 + (n - 1) * maxb):(n * maxb)) = buff(1:maxb)
+    ENDIF
+    !
+  ENDDO
+  !
+  ! ... possible remaining elements < maxb
+  !
+  IF ((dim - nbuf * maxb) > 0) THEN
+    !
+    IF (root >= 0) THEN
+      CALL MPI_REDUCE(ps(1 + nbuf * maxb), buff, (dim - nbuf * maxb), MPI_INTEGER8, MPI_SUM, root, comm, info)
+      IF(info /= 0) CALL errore('reduce_base_integer', 'error in mpi_reduce 2', info)
+    ELSE
+      CALL MPI_ALLREDUCE(ps(1 + nbuf * maxb), buff, (dim - nbuf * maxb), MPI_INTEGER8, MPI_SUM, comm, info)
+      IF(info /= 0) CALL errore('reduce_base_integer', 'error in mpi_allreduce 2', info)
+    ENDIF
+    !
+    IF(root < 0) THEN
+      ps((1 + nbuf * maxb):dim) = buff(1:(dim - nbuf * maxb))
+    ELSEIF(root == myid) THEN
+      ps((1 + nbuf * maxb):dim) = buff(1:(dim - nbuf * maxb))
+    ENDIF
+    !
+  ENDIF
+  !
+1 CONTINUE
+  !
+#if defined __TRACE
+  WRITE(*, *) 'reduce_base_integer8 OUT'
+#endif
+  !
+#endif
+  !
+  RETURN
+  !----------------------------------------------------------------------------
+  END SUBROUTINE reduce_base_integer8
+  !----------------------------------------------------------------------------
+  !
+#endif
+  !
+  ! ... "reduce"-like subroutines
+  !
+  !----------------------------------------------------------------------------
+  SUBROUTINE reduce_base_real_to( dim, ps, psout, comm, root )
   !----------------------------------------------------------------------------
   !
   ! ... sums a distributed variable ps(dim) over the processors,
@@ -687,6 +931,7 @@ SUBROUTINE parallel_min_integer( dim, ps, comm, root )
   ! ... This version uses a fixed-length buffer of appropriate (?) dim
   !
   USE util_param, ONLY : DP
+  USE data_buffer, ONLY : buff => mp_buff_i
   USE parallel_include  
   !
   IMPLICIT NONE
@@ -702,9 +947,6 @@ SUBROUTINE parallel_min_integer( dim, ps, comm, root )
   INTEGER            :: info, n, nbuf, nproc, myid
   INTEGER, PARAMETER :: maxb = __MSGSIZ_MAX
   !
-  INTEGER :: buff(maxb)  
-  COMMON / mp_base_integer / buff
-  !
 #if defined __TRACE
   write(*,*) 'parallel_min_integer IN'
 #endif
@@ -783,7 +1025,8 @@ SUBROUTINE parallel_max_integer( dim, ps, comm, root )
   ! ... compute the maximum of a distributed variable ps(dim) over the processors.
   ! ... This version uses a fixed-length buffer of appropriate (?) dim
   !
-  USE util_param, ONLY : DP
+  USE util_param,  ONLY : DP
+  USE data_buffer, ONLY : buff => mp_buff_i
   USE parallel_include  
   !
   IMPLICIT NONE
@@ -799,9 +1042,6 @@ SUBROUTINE parallel_max_integer( dim, ps, comm, root )
   INTEGER            :: info, n, nbuf, nproc, myid
   INTEGER, PARAMETER :: maxb = __MSGSIZ_MAX
   !
-  INTEGER :: buff(maxb)  
-  COMMON / mp_base_integer / buff
-  !
 #if defined __TRACE
   write(*,*) 'parallel_max_integer IN'
 #endif
@@ -879,6 +1119,7 @@ SUBROUTINE parallel_min_real( dim, ps, comm, root )
   ! ... This version uses a fixed-length buffer of appropriate (?) dim
   !
   USE util_param, ONLY : DP
+  USE data_buffer, ONLY : buff => mp_buff_r
   USE parallel_include  
   !
   IMPLICIT NONE
@@ -894,9 +1135,6 @@ SUBROUTINE parallel_min_real( dim, ps, comm, root )
   INTEGER            :: info, n, nbuf, nproc, myid
   INTEGER, PARAMETER :: maxb = __MSGSIZ_MAX
   !
-  REAL(DP) :: buff(maxb)  
-  COMMON / mp_base_real / buff
-  !
 #if defined __TRACE
   write(*,*) 'parallel_min_real IN'
 #endif
@@ -974,6 +1212,7 @@ SUBROUTINE parallel_max_real( dim, ps, comm, root )
   ! ... This version uses a fixed-length buffer of appropriate (?) dim
   !
   USE util_param, ONLY : DP
+  USE data_buffer, ONLY : buff => mp_buff_r
   USE parallel_include  
   !
   IMPLICIT NONE
@@ -989,9 +1228,6 @@ SUBROUTINE parallel_max_real( dim, ps, comm, root )
   INTEGER            :: info, n, nbuf, nproc, myid
   INTEGER, PARAMETER :: maxb = __MSGSIZ_MAX
   !
-  REAL(DP) :: buff(maxb)  
-  COMMON / mp_base_real / buff
-  !
 #if defined __TRACE
   write(*,*) 'parallel_max_real IN'
 #endif
