@@ -12,7 +12,8 @@ MODULE gipaw_module
   ! ... This module contains the variables used for GIPAW calculations
   !
   USE kinds, ONLY : DP
-  USE parameters, ONLY : npk, ntypx, lmaxx
+  USE parameters, ONLY : npk, ntypx
+  USE upf_params, ONLY : lmaxx
 
   IMPLICIT NONE
   SAVE
@@ -111,7 +112,7 @@ CONTAINS
   !         /
   !-----------------------------------------------------------------------
   SUBROUTINE gipaw_readin()
-    USE io_files,      ONLY : nd_nmbr, prefix, tmp_dir
+    USE io_files,      ONLY : prefix, tmp_dir
     USE io_global,     ONLY : ionode
     USE us,            ONLY : spline_ps
     IMPLICIT NONE
@@ -249,7 +250,7 @@ CONTAINS
     nwordwfc = nbnd*npwx*npol
     CALL open_buffer( iunwfc, 'wfc', nwordwfc, io_level, exst )
 
-    ! ... Needed for LDA+U
+    ! ... Needed for DFT+U(+V)
     ! ... iunhub contains the (orthogonalized) atomic wfcs * S
     !
     nwordwfcU = npwx*nwfcU*npol
@@ -321,7 +322,7 @@ CONTAINS
                               read_recon, set_paw_upf
     USE symm_base,     ONLY : nsym, s
     USE uspp_param,    ONLY : upf
-    USE mp_global,     ONLY : inter_pool_comm
+    USE mp_pools,      ONLY : inter_pool_comm
     USE mp,            ONLY : mp_max, mp_min
     USE dfunct,                 only : newd
 
@@ -354,9 +355,6 @@ CONTAINS
 
 !*apsi*    CALL test_symmetries ( s, nsym )
 
-    ! initialize pseudopotentials
-    call init_us_1
-
     ! initialise data, also for the case that no GIPAW is present
     IF ( .NOT. ALLOCATED ( paw_recon ) ) ALLOCATE ( paw_recon(ntyp) )
     paw_recon(:)%gipaw_data_in_upf_file = .FALSE.
