@@ -34,7 +34,6 @@ SUBROUTINE sm1_psi( recalc, lda, n, m, psi, spsi)
   USE control_flags,      ONLY : gamma_only
   USE uspp,         ONLY : okvan, vkb, nkb, qq_nt
   USE uspp_param, ONLY : upf, nh
-  USE ldaU,       ONLY : lda_plus_u
   USE ions_base,   ONLY : nat, ntyp => nsp, ityp  
   use becmod, only : calbec
   !
@@ -190,8 +189,8 @@ CONTAINS
     !
     USE becmod,  ONLY : becp
     USE klist, only :         xk
-    USE mp,         only :  mp_sum  ! CG
-    USE mp_global,  ONLY : intra_pool_comm ! CG
+    USE mp,        ONLY : mp_sum  ! CG
+    USE mp_pools,  ONLY : intra_pool_comm ! CG
     !
     IMPLICIT NONE
     !
@@ -425,8 +424,8 @@ SUBROUTINE pw_gemm( sum_over_nodes, na, nb, n, a, lda, b, ldb, c, ldc )
   !
   USE kinds, ONLY : DP
   USE gvect, ONLY : gstart
-  USE mp,         only :  mp_sum  ! CG
-  USE mp_global,  ONLY : intra_pool_comm ! CG
+  USE mp,        ONLy : mp_sum  ! CG
+  USE mp_pools,  ONLY : intra_pool_comm ! CG
   !
   IMPLICIT NONE
   !
