@@ -21,8 +21,8 @@ subroutine read_input_and_bcast(filerecon, r_paw)
   USE io_global,       ONLY : stdout,ionode,ionode_id   ! Modules/io_global.f90
   USE mp,              ONLY : mp_bcast, mp_sum             !parallelization
   USE mp_world,        ONLY : nproc, world_comm
-  USE parameters,      ONLY : ntypx,lmaxx,lqmax
-  USE control_flags, ONLY : twfcollect
+  USE parameters,      ONLY : ntypx
+  USE upf_params,      ONLY : lmaxx
   USE klist, ONLY : nelup, neldw, nelec
 
   IMPLICIT NONE
@@ -58,7 +58,6 @@ subroutine read_input_and_bcast(filerecon, r_paw)
        xcheck_conv, &
        show_status, &
        nelup,neldw, &
-       wf_collect,&
        U_projection_type,&
        time_limit,&
        restart_mode,&
@@ -193,8 +192,6 @@ subroutine read_input_and_bcast(filerecon, r_paw)
   CALL mp_bcast( xe0,  ionode_id, world_comm )
   CALL mp_bcast( cut_occ_states, ionode_id, world_comm )
   CALL mp_bcast( terminator, ionode_id, world_comm )
-  CALL mp_bcast( wf_collect, ionode_id, world_comm )
-  CALL mp_bcast( twfcollect, ionode_id, world_comm )
   CALL mp_bcast( xanes_file,  ionode_id, world_comm )
 
   CALL mp_bcast( U_projection_type, ionode_id, world_comm )
@@ -238,19 +235,6 @@ subroutine read_input_and_bcast(filerecon, r_paw)
      calculation='xanes'
   ENDIF
 
-
-
-
-  ! $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
-  ! $   check on wfcollect
-  ! $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
-
-  IF(xread_wf.AND.wf_collect) THEN
-     CALL errore ('main','incompatibility xread_wf and wf_collect',1)
-  ENDIF
-
-  twfcollect=wf_collect
-
   IF(trim(adjustl(edge)).NE.'K'.AND. &
        trim(adjustl(edge)).NE.'L1'.AND. &
        trim(adjustl(edge)).NE.'L2'.AND. &
