@@ -15,10 +15,9 @@ SUBROUTINE xanes_dipole(a,b,ncalcv,xnorm,core_wfn,paw_iltonhb,&
   USE wvfct,           ONLY : npwx, nbnd, et, current_k
   USE gvecw,           ONLY : gcutw
   USE symm_base,       ONLY : d1,d2,d3
-  USE noncollin_module,ONLY : noncolin
   USE lsda_mod,        ONLY : nspin,lsda,isk,current_spin
   USE cell_base,       ONLY : tpiba2, bg
-  USE wavefunctions_module, ONLY: evc
+  USE wavefunctions, ONLY: evc
   USE klist,           ONLY : &
        nkstot,                & ! total number of k-points
        nks,                   & ! number of k-points per pool
@@ -44,15 +43,11 @@ SUBROUTINE xanes_dipole(a,b,ncalcv,xnorm,core_wfn,paw_iltonhb,&
                               xnitermax, xepsilon,time_limit,calculated,&
                               save_file_kind
   USE atom,            ONLY : rgrid, msh
-  !  use atom,        ONLY : &
-  !       mesh,     &!mesh(ntypx) number of mesh points
-  !       msh ,     &!msh(ntypx)the point at rcut=end of radial integration
-  !       r   
   USE radin_mod
   USE basis,           ONLY : natomwfc
   USE uspp,            ONLY : vkb, nkb, okvan !CG
   USE uspp_param,      ONLY : upf
-  USE ldaU,            ONLY : lda_plus_u, init_lda_plus_u 
+  USE ldaU,            ONLY : lda_plus_u, init_lda_plus_u, lda_plus_u_kind 
   !<CG>
   USE xspectra_paw_variables, ONLY : xspectra_paw_nhm
   !</CG>
@@ -76,13 +71,12 @@ SUBROUTINE xanes_dipole(a,b,ncalcv,xnorm,core_wfn,paw_iltonhb,&
   REAL(dp) :: pref,prefb,v_of_0,xnorm_partial
   REAL(dp) :: norm, normps
   REAL(dp), ALLOCATABLE :: aux(:)
-  COMPLEX(KIND=DP), EXTERNAL :: zdotc
   COMPLEX(dp), ALLOCATABLE :: paw_vkb_cplx(:,:)
   COMPLEX(dp), ALLOCATABLE :: psiwfc(:), spsiwfc(:)
   CHARACTER(LEN=4) :: verbosity
 
   REAL(dp) :: timenow 
-  REAL(DP), EXTERNAL ::  get_clock
+  REAL(DP), EXTERNAL :: ddot, get_clock
   EXTERNAL :: zdscal
 
   timenow=0
@@ -231,9 +225,15 @@ SUBROUTINE xanes_dipole(a,b,ncalcv,xnorm,core_wfn,paw_iltonhb,&
      !<CG>        
      CALL init_gipaw_2(npw,igk_k(1,ik),xk(1,ik),paw_vkb)
      !</CG>
-     IF (.NOT.lda_plus_u) CALL init_us_2(npw,igk_k(1,ik),xk(1,ik),vkb)
-     IF (lda_plus_u) CALL orthoUwfc_k(ik)
-
+     
+     IF (lda_plus_u) THEN
+        CALL orthoUwfc_k(ik)
+        ! Compute the phase factor for each k point in the case of DFT+U+V
+        IF (lda_plus_u_kind.EQ.2) CALL phase_factor(ik) 
+     ELSE
+        CALL init_us_2(npw,igk_k(1,ik),xk(1,ik),vkb)
+     ENDIF
+     ! 
      ! Angular Matrix element
      !
      !... Calculates the complex PAW projectors, paw_vkb_cplx, from
@@ -327,14 +327,14 @@ SUBROUTINE xanes_dipole(a,b,ncalcv,xnorm,core_wfn,paw_iltonhb,&
         spsiwfc(:)=(0.d0,0.d0)
         recalc=.true.
         CALL sm1_psi(recalc,npwx, npw, 1, psiwfc, spsiwfc)
-        xnorm_partial=zdotc(npw,psiwfc,1,spsiwfc,1)
+        xnorm_partial=ddot(2*npw,psiwfc,1,spsiwfc,1)
         DEALLOCATE(spsiwfc)
      ELSE
 !        xnorm_partial=0.d0
 !        do ip=1,npw
 !          xnorm_partial=xnorm_partial+conjg(psiwfc(ip))*psiwfc(ip)
 !       enddo
-        xnorm_partial=real(zdotc(npw,psiwfc,1,psiwfc,1),dp)
+        xnorm_partial=ddot(2*npw,psiwfc,1,psiwfc,1)
 
      ENDIF
      !</CG>
