@@ -7,7 +7,7 @@
 destination="$PWD/src"
 
 # definitions specific to this code
-CODE="lammps_15-01-2010"
+CODE="lammps_27-10-2011"
 LINKED_FILES="$plumedir/common_files/*.h $plumedir/common_files/*.c"
 WHERE_LINKS="./USER-PLUMED"
 
@@ -35,77 +35,15 @@ cat > ./Install.sh << \EOF
 
 if (test $1 = 1) then
 
-  cp -p style_user_plumed.h ../
-  cp -p metadyn.h ../
-  cp -p fix_plumed.h ../
-  cp -p testderivatives.cpp ../
-  cp -p restraint_zpath.cpp ../
-  cp -p restraint_waterbridge.cpp ../
-  cp -p restraint_torsion.cpp ../
-  cp -p restraint_spath.cpp ../
-  cp -p restraint_rmsdtor.cpp ../
-  cp -p restraint_rgyr.cpp ../
-  cp -p restraint_puckering.cpp ../
-  cp -p restraint_position.cpp ../
-  cp -p restraint_parabetarmsd.cpp ../
-  cp -p restraint_mindist.cpp ../
-  cp -p restraint_helix.cpp ../
-  cp -p restraint_hbonds.cpp ../
-  cp -p restraint_energy.cpp ../
-  cp -p restraint_elstpot.cpp ../
-  cp -p restraint_dist.cpp ../
-  cp -p restraint_dipole.cpp ../
-  cp -p restraint_dihcor.cpp ../
-  cp -p restraint_coord.cpp ../
-  cp -p restraint_cmap.cpp ../
-  cp -p restraint_antibetarmsd.cpp ../
-  cp -p restraint_angle.cpp ../
-  cp -p restraint_alpharmsd.cpp ../
-  cp -p restraint_alfabeta.cpp ../
-  cp -p restraint.cpp ../
-  cp -p read_restraint.cpp ../
-  cp -p ptmetad.cpp ../
-  cp -p metadyn.cpp ../
-  cp -p hills.cpp ../
-  cp -p fix_plumed.cpp ../
-  cp -p biasexchange.cpp ../
+  for file in *.cpp *h;do
+    cp -p $file ../
+  done
 
 elif (test $1 = 0) then
 
-  rm ../style_user_plumed.h
-  rm ../metadyn.h
-  rm ../fix_plumed.h
-  rm ../testderivatives.cpp
-  rm ../restraint_zpath.cpp
-  rm ../restraint_waterbridge.cpp
-  rm ../restraint_torsion.cpp
-  rm ../restraint_spath.cpp
-  rm ../restraint_rmsdtor.cpp
-  rm ../restraint_rgyr.cpp
-  rm ../restraint_puckering.cpp
-  rm ../restraint_position.cpp
-  rm ../restraint_parabetarmsd.cpp
-  rm ../restraint_mindist.cpp
-  rm ../restraint_helix.cpp
-  rm ../restraint_hbonds.cpp
-  rm ../restraint_energy.cpp
-  rm ../restraint_elstpot.cpp
-  rm ../restraint_dist.cpp
-  rm ../restraint_dipole.cpp
-  rm ../restraint_dihcor.cpp
-  rm ../restraint_coord.cpp
-  rm ../restraint_cmap.cpp
-  rm ../restraint_antibetarmsd.cpp
-  rm ../restraint_angle.cpp
-  rm ../restraint_alpharmsd.cpp
-  rm ../restraint_alfabeta.cpp
-  rm ../restraint.cpp
-  rm ../read_restraint.cpp
-  rm ../ptmetad.cpp
-  rm ../metadyn.cpp
-  rm ../hills.cpp
-  rm ../fix_plumed.cpp
-  rm ../biasexchange.cpp
+  for file in *.cpp *h;do
+    rm ../$file
+  done
 
 fi
 EOF
@@ -114,77 +52,15 @@ cat > ./Install.csh << \EOF
 
 if ($1 == 1) then
 
-  cp -p style_user_plumed.h ../
-  cp -p metadyn.h ../
-  cp -p fix_plumed.h ../
-  cp -p testderivatives.cpp ../
-  cp -p restraint_zpath.cpp ../
-  cp -p restraint_waterbridge.cpp ../
-  cp -p restraint_torsion.cpp ../
-  cp -p restraint_spath.cpp ../
-  cp -p restraint_rmsdtor.cpp ../
-  cp -p restraint_rgyr.cpp ../
-  cp -p restraint_puckering.cpp ../
-  cp -p restraint_position.cpp ../
-  cp -p restraint_parabetarmsd.cpp ../
-  cp -p restraint_mindist.cpp ../
-  cp -p restraint_helix.cpp ../
-  cp -p restraint_hbonds.cpp ../
-  cp -p restraint_energy.cpp ../
-  cp -p restraint_elstpot.cpp ../
-  cp -p restraint_dist.cpp ../
-  cp -p restraint_dipole.cpp ../
-  cp -p restraint_dihcor.cpp ../
-  cp -p restraint_coord.cpp ../
-  cp -p restraint_cmap.cpp ../
-  cp -p restraint_antibetarmsd.cpp ../
-  cp -p restraint_angle.cpp ../
-  cp -p restraint_alpharmsd.cpp ../
-  cp -p restraint_alfabeta.cpp ../
-  cp -p restraint.cpp ../
-  cp -p read_restraint.cpp ../
-  cp -p ptmetad.cpp ../
-  cp -p metadyn.cpp ../
-  cp -p hills.cpp ../
-  cp -p fix_plumed.cpp ../
-  cp -p biasexchange.cpp ../
+  foreach file (*cpp *.h)
+    cp $file ../
+  end
 
 else if ($1 == 0) then
 
-  rm ../style_user_plumed.h
-  rm ../metadyn.h
-  rm ../fix_plumed.h
-  rm ../testderivatives.cpp
-  rm ../restraint_zpath.cpp
-  rm ../restraint_waterbridge.cpp
-  rm ../restraint_torsion.cpp
-  rm ../restraint_spath.cpp
-  rm ../restraint_rmsdtor.cpp
-  rm ../restraint_rgyr.cpp
-  rm ../restraint_puckering.cpp
-  rm ../restraint_position.cpp
-  rm ../restraint_parabetarmsd.cpp
-  rm ../restraint_mindist.cpp
-  rm ../restraint_helix.cpp
-  rm ../restraint_hbonds.cpp
-  rm ../restraint_energy.cpp
-  rm ../restraint_elstpot.cpp
-  rm ../restraint_dist.cpp
-  rm ../restraint_dipole.cpp
-  rm ../restraint_dihcor.cpp
-  rm ../restraint_coord.cpp
-  rm ../restraint_cmap.cpp
-  rm ../restraint_antibetarmsd.cpp
-  rm ../restraint_angle.cpp
-  rm ../restraint_alpharmsd.cpp
-  rm ../restraint_alfabeta.cpp
-  rm ../restraint.cpp
-  rm ../read_restraint.cpp
-  rm ../ptmetad.cpp
-  rm ../metadyn.cpp
-  rm ../hills.cpp
-  rm ../fix_plumed.cpp
-  rm ../biasexchange.cpp
+  foreach file (*cpp *.h)
+    rm ../$file
+  end
 
 endif
 EOF
@@ -194,12 +70,12 @@ cat >./fix_plumed.cpp << \EOF
 *                                                                             *
 *                                PLUMED                                       *
 *   A Portable Plugin for Free Energy Calculations with Molecular Dynamics    *
-*                              VERSION 1.3                                    *
+*                              VERSION 1.2                                    *
 *                                                                             *
 *******************************************************************************
 *
 *  
-*  Copyright (c) 2008-2011 The PLUMED team.
+*  Copyright (c) 2010 The PLUMED team.
 *  See http://www.plumed-code.org for more information. 
 *
 *  This file is part of PLUMED.
@@ -236,6 +112,8 @@ cat >./fix_plumed.cpp << \EOF
 #include "group.h"
 #include "fix_plumed.h"
 
+static const double PLUMED_UNSET=-1.23456789e12;
+
 using namespace LAMMPS_NS;
 
 /* ---------------------------------------------------------------------- */
@@ -248,10 +126,20 @@ FixPlumed::FixPlumed(LAMMPS *lmp, int narg, char **arg) :
   int i,j,k,i_c,nn,mm;
   int me;
 
-  if (narg != 7) error->all("Illegal fix plumed command");
+  if (narg != 7) error->all(FLERR,"Illegal fix plumed command");
 
   MPI_Comm_rank(world,&me);
   MPI_Comm_size(world,&size);
+
+  if (!atom->tag_enable)
+    error->all(FLERR,"fix plumed requires atom tags");
+
+  if (!atom->tag_consecutive())
+    error->all(FLERR,"fix plumed requires consecutive atom tags");
+
+  if (strcmp(arg[1],"all") != 0)
+    error->all(FLERR,"fix plumed requires using group all");
+
 //  printf("PROCESSOR %d SIZE %d\n",me,size);
 
   if (strcmp(arg[3],"plumedfile") == 0) {
@@ -259,7 +147,7 @@ FixPlumed::FixPlumed(LAMMPS *lmp, int narg, char **arg) :
      if (strcmp(arg[5],"outfile") == 0) {
      printf("PLUMED: FOUND THE OUTPUT FILE %s\n",arg[6]);
      } else {
-          error->all("Illegal fix plumed command");
+          error->all(FLERR,"Illegal fix plumed command");
      }
      // detect the number of atoms, mass and charge
      // number of atoms in this group (default must be all )
@@ -271,12 +159,11 @@ FixPlumed::FixPlumed(LAMMPS *lmp, int narg, char **arg) :
   //   printf("FOUND NATOMS FOR GROUP %d : %d\n",0,nn);
   //   printf("FOUND MASS  %f DT %f\n",group->mass(0),update->dt);
      //for(i=0;i<nn;i++){chg[i]=atom->q[i];} 
-     for(i=0;i<nn;i++){chg[i]=-100.0;} 
-     for(i=0;i<nn;i++){mss[i]=-100.0;} 
+     for(i=0;i<nn;i++){chg[i]=PLUMED_UNSET;} 
+     for(i=0;i<nn;i++){mss[i]=PLUMED_UNSET;} 
      //use the groups for massses
      MPI_Request request;
-     int groupbit = group->bitmask[0]; //get the groupbit of all atoms
-     
+     int groupbit = group->bitmask[igroup]; //get the groupbit of all atoms
      // retrieve masses 
      if (atom->rmass) {
       for (int i = 0; i < atom->nlocal; i++){
@@ -284,7 +171,7 @@ FixPlumed::FixPlumed(LAMMPS *lmp, int narg, char **arg) :
              mm=atom->tag[i]-1; 
              mss[mm] = atom->rmass[i];
 //             printf("I AM PROC %d AND I OWN AT %d R-MASS %f TAG %d \n",me,mm,mss[mm],atom->tag[i]);
-             MPI_Isend(&(mss[mm]),1,MPI_DOUBLE,0,mm,world,&request);
+             if (me) MPI_Isend(&(mss[mm]),1,MPI_DOUBLE,0,mm,world,&request);
         }
       }
      } else {
@@ -293,7 +180,7 @@ FixPlumed::FixPlumed(LAMMPS *lmp, int narg, char **arg) :
                mm=atom->tag[i]-1; 
                mss[mm] = atom->mass[atom->type[i]];
 //               printf("I AM PROC %d AND I OWN AT %d MASS %f TAG %d \n",me,mm,mss[mm],atom->tag[i]);
-               MPI_Isend(&(mss[mm]),1,MPI_DOUBLE,0,mm,world,&request);
+               if (me) MPI_Isend(&(mss[mm]),1,MPI_DOUBLE,0,mm,world,&request);
           }
         }
      }
@@ -303,7 +190,7 @@ FixPlumed::FixPlumed(LAMMPS *lmp, int narg, char **arg) :
      if(!me){
         MPI_Status status;
         for (i=0;i<nn;i++){
-          if(mss[i]==-100.0)MPI_Recv(&(mss[i]),1,MPI_DOUBLE,MPI_ANY_SOURCE,i,world,&status); 
+          if(mss[i]==PLUMED_UNSET)MPI_Recv(&(mss[i]),1,MPI_DOUBLE,MPI_ANY_SOURCE,i,world,&status); 
         }
      }
 
@@ -320,7 +207,7 @@ FixPlumed::FixPlumed(LAMMPS *lmp, int narg, char **arg) :
                chg[mm] = 0.0 ;
              }
 //             printf("I AM PROC %d AND I OWN AT %d CHG %f \n",me,mm,chg[mm]);
-             MPI_Isend(&(chg[mm]),1,MPI_DOUBLE,0,mm,world,&request);
+             if (me) MPI_Isend(&(chg[mm]),1,MPI_DOUBLE,0,mm,world,&request);
         }
      }
 
@@ -329,7 +216,7 @@ FixPlumed::FixPlumed(LAMMPS *lmp, int narg, char **arg) :
      if(!me){
         MPI_Status status;
         for (i=0;i<nn;i++){
-          if(chg[i]==-100.0)MPI_Recv(&(chg[i]),1,MPI_DOUBLE,MPI_ANY_SOURCE,i,world,&status); 
+          if(chg[i]==PLUMED_UNSET)MPI_Recv(&(chg[i]),1,MPI_DOUBLE,MPI_ANY_SOURCE,i,world,&status); 
         }
      }
    
@@ -361,12 +248,12 @@ FixPlumed::FixPlumed(LAMMPS *lmp, int narg, char **arg) :
      group->create_plumed("plumed",int_p);
      plumedgroup=group->find("plumed");
      if (plumedgroup == -1) 
-       error->all("Fix plumed group ID does not exist"); 
+       error->all(FLERR,"Fix plumed group ID does not exist"); 
       plumedgroup2bit = group->bitmask[plumedgroup];
      //for (int iarg = 0; iarg < narg_p; iarg++) { fprintf(screen,"ARG %s ",arg_p[iarg]);}fprintf(screen,"\n");
      //printf("ASSIGNMENT DONE TO INDEX %d\n",plumedgroup); 
   }
-  else error->all("Illegal fix plumed command");
+  else error->all(FLERR,"Illegal fix plumed command");
   // commit my special mpi type rvec
   MPI_Type_contiguous( 3 ,MPI_DOUBLE,&MPI_RVEC);
   MPI_Type_commit(&MPI_RVEC); 
@@ -481,9 +368,10 @@ void FixPlumed::plumed_interface()
     }
   }
   // gather all the infos    
-  int sum_allatoms;
-  MPI_Allreduce(&my_atoms[me],&sum_allatoms,1,MPI_INT,MPI_SUM,world);  
-  MPI_Allgather(&my_atoms[me],1,MPI_INT,my_atoms,1,MPI_INT,world);  
+  int sum_allatoms, meatoms;
+  meatoms=my_atoms[me];
+  MPI_Allreduce(&meatoms,&sum_allatoms,1,MPI_INT,MPI_SUM,world);
+  MPI_Allgather(&meatoms,1,MPI_INT,my_atoms,1,MPI_INT,world);
   // test the allgather 
 
 //   for (i=0;i<nprocs;i++){
@@ -552,8 +440,7 @@ void FixPlumed::plumed_interface()
       f[j][1]+=my_force[i][1];
       f[j][2]+=my_force[i][2];
   }
- 
-   
+
   // free all the vectors
   free(allpos); 
   free(allforce); 
@@ -563,6 +450,7 @@ void FixPlumed::plumed_interface()
   free(my_force);
   free(my_idx);
   free(my_backtable);
+  free(incr_pos);
 //  MPI_Finalize();
 //  abort();
 
@@ -591,12 +479,12 @@ cat >./fix_plumed.h  << \EOF
 *                                                                             *
 *                                PLUMED                                       *
 *   A Portable Plugin for Free Energy Calculations with Molecular Dynamics    *
-*                              VERSION 1.2                                    *
+*                              VERSION 1.3                                    *
 *                                                                             *
 *******************************************************************************
 *
 *  
-*  Copyright (c) 2010 The PLUMED team.
+*  Copyright (c) 2008-2011 The PLUMED team.
 *  See http://www.plumed-code.org for more information. 
 *
 *  This file is part of PLUMED.
@@ -663,16 +551,7 @@ class FixPlumed : public Fix {
 
 #endif
 #endif
-EOF
 
-cat >./style_user_plumed.h << \EOF
-#ifdef FixInclude
-#include "fix_plumed.h"
-#endif
-
-#ifdef FixClass
-FixStyle(plumed,FixPlumed)
-#endif
 EOF
 
 cd ../
